
/* Attribution-NonCommercial-ShareAlike 4.0 International
Attribution - You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.
NonCommercial - You may not use the material for commercial purposes.
ShareAlike - If you remix, transform, or build upon the material, you must distribute your contributions under the same license as the original.
https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode */

; iFade1 and iFade2 always zero...


; PadSynthChoir.csd
; Written by Iain McCurdy, 2015

; Voices are created from wavetables created using the "padsynth" GEN routine.
; Each time a new voice is selected or if 'Bandwidth', 'Partial Scaling', 'Table Size', or either of the keyboard mapping tables are modified, the tables require recalculation which will take a little time.
;  The Csound Output window should indicate this activity. If it doesn't, please be patient before assuming it has crashed!

; Keygroups are used (21 in total), each group extending across 6 notes and crossfading into the next or previous wavetable stepping up or down the keyboard.
; The complete note range for each voice and vowel is from note number 24 to note number 84.
; Notes played above or below these limits will simply employ the same wavetable scaled up or down. 
; Using more wavetables which extend across fewer keys would result in very long load times.
; Using fewer wavetables which extend across more keys would result in poor results as wavetables will be scaled excessively.

; Voice        --    Choose from 5 different voice types and for each of those from 5 different vowels. 
;            'Mixed' voice types use a combination of all five voice types with the appropriate voice type chosen the various pitch ranges.
; Bandwidth    --    effectively controls modulation of the partial frequencies 
;             (amplitude and phase modulation)
;             Increasing this increases the amount a noise component that will be preset in all partials.       
; B.width.Kybd    --    This editable function table allows keyboard tracking according to the map defined by this table to influence bandwidth values.
;            The value derived from mapping the note played through this table is multiplied to the rotary slider value for bandwidth. 
; Part.Scal.    --    scales the modulations from partial to partial, raising this beyond 1 will result in increasing     
;             excursive modulations in higher partials - this effect can be likened to adding 'air' into 
;             the pad texture whilst retaining clarity in the lower partials. Experimentation with this 
;             parameter should help in gaining an understanding of its effect.                                     
;            This control will be less effective in sounds with just a small number of partials defined. In this example this means that
;            higher notes (which call tables with fewer partials) will be less affected.
; P.scal.Kybd    --    This editable function table allows keyboard tracking according to the map defined by this table to influence partial modulation scaling.
;            The value derived from mapping the note played through this table is multiplied to the rotary slider value for partial modulation scaling. 
; Table Size    --    Size of the table created by padsynth. (Actual size will be 2 ^ 'Table Size'.)
;             Reducing the table size will reduce the load time and the time it takes to recalculate the 
;             table whenever a change is made to 'padsynth's parameters, but if the table becomes too small, 
;             looping will become obvious.                          

; Filter    --    The filter section feature three stages in series: low cut (highpass), bandpass and high cut (lowpass)

<Cabbage>
form caption("Padsynth Choir"), size(1060, 485), pluginId("PSCh")

#define PlantBoxAppearance # shape("rounded"), colour( 30, 20, 20), outlineColour("silver"), outlineThickness(3) #

image    bounds(  5,  5,580,130), plant("Voice 1"), $PlantBoxAppearance {
label    bounds(  5,  4,580, 15), text("Main")
label    bounds( 15, 26,100, 13), text("Voice")
combobox bounds( 15, 40,100, 25), channel("Voice"), text("Bass Ah","Bass Eh","Bass Eee","Bass Oh","Bass Ooh","Tenor Ah","Tenor Eh","Tenor Eee","Tenor Oh","Tenor Ooh","C.tenor Ah","C.tenor Eh","C.tenor Eee","C.tenor Oh","C.tenor Ooh","Alto Ah","Alto Eh","Alto Eee","Alto Oh","Alto Ooh","Soprano Ah","Soprano Eh","Soprano Eee","Soprano Oh","Soprano Ooh","Mixed Ah","Mixed Eh","Mixed Eee","Mixed Oh","Mixed Ooh"), value(26)
rslider  bounds(130, 25, 90, 90), channel("BW"),   text("Bandwidth"),  range(1,999,100,0.25,0.1), valueTextBox(1), textBox(1), popupText("Bandwidth of Modulation")
label    bounds(220, 29, 90, 13), text("B.width Kybd.")                                                            
gentable bounds(220, 45, 90, 70), tableNumber(51), identChannel(BW_Kybd), ampRange(0.001,2,51), zoom(-1), tableColour(turquoise), active(1), fill(0)
rslider  bounds(310, 25, 90, 90), channel("PartScal"), text("Part.Scal."),  range(1,30,1,0.5,0.01), valueTextBox(1), textBox(1), popupText("Partial Scaling")
label    bounds(400, 29, 90, 13), text("P.scal.Kybd.")
gentable bounds(400, 45, 90, 70), tableNumber(52), identChannel(PS_Kybd), ampRange(0,2,52), zoom(-1), tableColour(turquoise), active(1), fill(0)
rslider  bounds(490, 25, 90, 90), channel("TabSize"), text("Table Size"),  range(1,24,16,1,1), valueTextBox(1), textBox(1), popupText("Larger values slow down table rebuilds!")
}

image    bounds(590,  5,270,130), plant("Voice2"), $PlantBoxAppearance {
label    bounds(  5,  4,270, 15), text("Voice 2")
checkbox bounds( 15, 33, 80, 15), channel("V2OnOff"), text("On/Off") value(1)
label    bounds( 15, 56,100, 13), text("Voice 2")
combobox bounds( 15, 70,100, 25), channel("Voice2"), text("Bass Ah","Bass Eh","Bass Eee","Bass Oh","Bass Ooh","Tenor Ah","Tenor Eh","Tenor Eee","Tenor Oh","Tenor Ooh","C.tenor Ah","C.tenor Eh","C.tenor Eee","C.tenor Oh","C.tenor Ooh","Alto Ah","Alto Eh","Alto Eee","Alto Oh","Alto Ooh","Soprano Ah","Soprano Eh","Soprano Eee","Soprano Oh","Soprano Ooh","Mixed Ah","Mixed Eh","Mixed Eee","Mixed Oh","Mixed Ooh"), value(30)
rslider  bounds(110, 25, 90, 90), channel("BW_V2"), text("Bandwidth"), range(1,999,200,0.25,0.1), valueTextBox(1), textBox(1), popupText("Bandwidth of Modulation")
rslider  bounds(180, 25, 90, 90), channel("VoiceMix"), text("Manual"),  range(0,1,0), valueTextBox(1) textBox(1)
}

image    bounds(865,  5,190,130), plant("Exciter"), identChannel("ExciterID") alpha(0.5) $PlantBoxAppearance {
label    bounds(  5,  4,190, 15), text("Exciter")
rslider  bounds( 10, 25, 90, 90), channel("ExciterAmount"), text("Amount"),  range(0,100,0,1,0.1), valueTextBox(1), textBox(1)
rslider  bounds( 90, 25, 90, 90), channel("ExciterFreq"), text("Freq."),  range(1000,10000,3000,1,1), valueTextBox(1), textBox(1)
}

image    bounds(  5,140,320,130), plant("AmplitudeEnvelope") $PlantBoxAppearance {
label    bounds(  5,  4,320, 15), text("Amplitude Envelope")
rslider  bounds( 10, 30, 90, 90), channel("AAtt"), text("Attack"), range(0.001,5,0.5,0.5,0.001), valueTextBox(1), textBox(1)
rslider  bounds( 80, 30, 90, 90), channel("ADec"), text("Decay"),  range(0.001,5,0.01,0.5,0.001), valueTextBox(1), textBox(1)
rslider  bounds(150, 30, 90, 90), channel("ASus"), text("Sustain"),range(0,    1,1,0.5,0.001), valueTextBox(1), textBox(1)
rslider  bounds(220, 30, 90, 90), channel("ARel"), text("Release"),range(0.001,5,0.5,0.5,0.001), valueTextBox(1), textBox(1)
}

image    bounds(330,140,460,130), plant("FilterEnvelope"), $PlantBoxAppearance {
label    bounds(  0,  4,460, 15), text("Filter Envelope")
checkbox bounds( 20, 33, 80, 15), channel("FOnOff"), text("On/Off") value(1)
rslider  bounds( 80, 30, 90, 90), channel("FiltLowCut"), text("Low Cut"), range(20,8000,20,0.5,1), valueTextBox(1), textBox(1)
rslider  bounds(150, 30, 90, 90), channel("FiltCF"), text("Freq."), range(20,8000,150,0.5,1), valueTextBox(1), textBox(1)
rslider  bounds(220, 30, 90, 90), channel("FiltBW"), text("Bandwidth"), range(0.1,10,1,0.5), valueTextBox(1), textBox(1)
rslider  bounds(290, 30, 90, 90), channel("FiltGain"), text("Gain"), range(-24,24,12,1,0.1), valueTextBox(1), textBox(1)
rslider  bounds(360, 30, 90, 90), channel("FiltHighCut"), text("High Cut"), range(100,20000,20000,0.25,1), valueTextBox(1), textBox(1)
}

image    bounds(800,140,255,130), plant("Reverb"), $PlantBoxAppearance {
label    bounds(  0,  4,255, 15), text("Reverb")
rslider  bounds( 10, 30, 90, 90), channel("RSend"), text("Send"), range(0,1,0.5), valueTextBox(1), textBox(1)
rslider  bounds( 80, 30, 90, 90), channel("RSize"), text("Size"),  range(0,0.99,0.85), valueTextBox(1), textBox(1)
rslider  bounds(150, 30, 90, 90), channel("R__CF"), text("Damping"),range(20,20000,8000,0.5,1), valueTextBox(1), textBox(1)
}

image    bounds(  5,275,230,120), plant("Legato"), $PlantBoxAppearance {
label    bounds(  0,  4,230, 15), text("Legato")
checkbox bounds( 10, 10, 60, 12), channel("LegatoOnOff"), text("On/Off"), value(0)
combobox bounds( 10, 50, 60, 22), channel("LegType"), text("Line","Curve"), value(1)
rslider  bounds( 70, 30, 80, 80), channel("LegTime"), text("Legato Time"),  range(0,3,0.2,0.5), valueTextBox(1), textBox(1), popupText("Partial Scaling")
rslider  bounds(140, 30, 80, 80), channel("PBRange"), text("Bend Range"),  range(0,48,2,1,1), valueTextBox(1), textBox(1), popupText("Partial Scaling")
}

image    bounds(240,275,815,120), plant("OutputMeter"), $PlantBoxAppearance 
{
label    bounds(  0,  4,815, 15), text("Meter")
rslider  bounds(330, 30, 80, 80), channel("AGain"),text("Gain"),   range(0,  5,0.4,0.5,0.001), valueTextBox(1), textBox(1)
vmeter   bounds(430, 30, 20, 80) channel("vMeter1") value(0) overlayColour(0, 0, 0, 255) meterColour:0(255, 255, 0) meterColour:1(255, 103, 171) meterColour:2(250,250, 0) outlineThickness(1)
}

keyboard bounds(  5,400,1050, 80)
</Cabbage>

<CsoundSynthesizer>

<CsOptions>
-dm0 -n -+rtmidi=NULL -M0
</CsOptions>

<CsInstruments>

; sr set by host
ksmps    =    32
nchnls    =    2
0dbfs    =    1

massign    0, 3

gaRvbL,gaRvbR    init    0

giBW_Kybd    ftgen    51,0,128,-5,1,127,1
giPS_Kybd    ftgen    52,0,128,-5,1,127,1

giNoteMap    ftgen    53,0,128,-17,   0,24,   24,24,   30,30,    36,36,    42,42,    48,48,    54,54,  60,60,  66,66,  72,72,   78,78,  84,84,  127,84
iCurve            =    -1
giFadeMap    ftgen    54,0, 6,-16,   0,  3,-iCurve,1,  3,iCurve,0

instr    1
 gkLegatoOnOff chnget  "LegatoOnOff"
 gkLegType     chnget  "LegType"
 gkLegTime     chnget  "LegTime"
 gkPBRange  chnget  "PBRange"

 kMOUSE_DOWN_LEFT    chnget    "MOUSE_DOWN_LEFT"        ; left click status (only allowing a reinit when the mouse button is released smooths interaction)
 if timeinstk()==1 then
  kMOUSE_DOWN_LEFT    =    1
 endif
 gkVoice     chnget    "Voice"  
 iVoice2     chnget    "Voice2"
 gkVoice2    init    iVoice2
 gkVoiceMix  chnget  "VoiceMix"
 if trigger(kMOUSE_DOWN_LEFT,0.5,1)==1 then            ; when left click is released update these values
  gkBW       chnget    "BW"
  gkBW_V2    chnget    "BW_V2"
  gkPartScal chnget    "PartScal"
  gkHarmStr  chnget    "HarmStr"
  gkTabSize  chnget    "TabSize"
  gkBW1      =         limit(gkBW*table:k(24,giBW_Kybd),1,999)    
  gkBW2      =         limit(gkBW*table:k(27,giBW_Kybd),1,999)    
  gkBW3      =         limit(gkBW*table:k(30,giBW_Kybd),1,999)    
  gkBW4      =         limit(gkBW*table:k(33,giBW_Kybd),1,999)    
  gkBW5      =         limit(gkBW*table:k(36,giBW_Kybd),1,999)    
  gkBW6      =         limit(gkBW*table:k(39,giBW_Kybd),1,999)    
  gkBW7      =         limit(gkBW*table:k(42,giBW_Kybd),1,999)    
  gkBW8      =         limit(gkBW*table:k(45,giBW_Kybd),1,999)    
  gkBW9      =         limit(gkBW*table:k(48,giBW_Kybd),1,999)    
  gkBW10     =         limit(gkBW*table:k(51,giBW_Kybd),1,999)    
  gkBW11     =         limit(gkBW*table:k(54,giBW_Kybd),1,999)    
  gkBW12     =         limit(gkBW*table:k(57,giBW_Kybd),1,999)
  gkBW13     =         limit(gkBW*table:k(60,giBW_Kybd),1,999)    
  gkBW14     =         limit(gkBW*table:k(63,giBW_Kybd),1,999)    
  gkBW15     =         limit(gkBW*table:k(66,giBW_Kybd),1,999)    
  gkBW16     =         limit(gkBW*table:k(69,giBW_Kybd),1,999)    
  gkBW17     =         limit(gkBW*table:k(72,giBW_Kybd),1,999)    
  gkBW18     =         limit(gkBW*table:k(75,giBW_Kybd),1,999)    
  gkBW19     =         limit(gkBW*table:k(78,giBW_Kybd),1,999)    
  gkBW20     =         limit(gkBW*table:k(81,giBW_Kybd),1,999)    
  gkBW21     =         limit(gkBW*table:k(84,giBW_Kybd),1,999)    

  gkBW1_V2   =         limit(gkBW_V2*table:k(24,giBW_Kybd),1,999)    
  gkBW2_V2   =         limit(gkBW_V2*table:k(27,giBW_Kybd),1,999)    
  gkBW3_V2   =         limit(gkBW_V2*table:k(30,giBW_Kybd),1,999)    
  gkBW4_V2   =         limit(gkBW_V2*table:k(33,giBW_Kybd),1,999)    
  gkBW5_V2   =         limit(gkBW_V2*table:k(36,giBW_Kybd),1,999)    
  gkBW6_V2   =         limit(gkBW_V2*table:k(39,giBW_Kybd),1,999)    
  gkBW7_V2   =         limit(gkBW_V2*table:k(42,giBW_Kybd),1,999)    
  gkBW8_V2   =         limit(gkBW_V2*table:k(45,giBW_Kybd),1,999)    
  gkBW9_V2   =         limit(gkBW_V2*table:k(48,giBW_Kybd),1,999)    
  gkBW10_V2  =         limit(gkBW_V2*table:k(51,giBW_Kybd),1,999)    
  gkBW11_V2  =         limit(gkBW_V2*table:k(54,giBW_Kybd),1,999)    
  gkBW12_V2  =         limit(gkBW_V2*table:k(57,giBW_Kybd),1,999)
  gkBW13_V2  =         limit(gkBW_V2*table:k(60,giBW_Kybd),1,999)    
  gkBW14_V2  =         limit(gkBW_V2*table:k(63,giBW_Kybd),1,999)    
  gkBW15_V2  =         limit(gkBW_V2*table:k(66,giBW_Kybd),1,999)    
  gkBW16_V2  =         limit(gkBW_V2*table:k(69,giBW_Kybd),1,999)    
  gkBW17_V2  =         limit(gkBW_V2*table:k(72,giBW_Kybd),1,999)    
  gkBW18_V2  =         limit(gkBW_V2*table:k(75,giBW_Kybd),1,999)    
  gkBW19_V2  =         limit(gkBW_V2*table:k(78,giBW_Kybd),1,999)    
  gkBW20_V2  =         limit(gkBW_V2*table:k(81,giBW_Kybd),1,999)    
  gkBW21_V2  =         limit(gkBW_V2*table:k(84,giBW_Kybd),1,999)    

  gkPS1      =         limit(gkPartScal*table:k(24,giPS_Kybd),1,30)    
  gkPS2      =         limit(gkPartScal*table:k(27,giPS_Kybd),1,30)    
  gkPS3      =         limit(gkPartScal*table:k(30,giPS_Kybd),1,30)    
  gkPS4      =         limit(gkPartScal*table:k(33,giPS_Kybd),1,30)    
  gkPS5      =         limit(gkPartScal*table:k(36,giPS_Kybd),1,30)    
  gkPS6      =         limit(gkPartScal*table:k(39,giPS_Kybd),1,30)    
  gkPS7      =         limit(gkPartScal*table:k(42,giPS_Kybd),1,30)    
  gkPS8      =         limit(gkPartScal*table:k(45,giPS_Kybd),1,30)    
  gkPS9      =         limit(gkPartScal*table:k(48,giPS_Kybd),1,30)    
  gkPS10     =         limit(gkPartScal*table:k(51,giPS_Kybd),1,30)    
  gkPS11     =         limit(gkPartScal*table:k(54,giPS_Kybd),1,30)    
  gkPS12     =         limit(gkPartScal*table:k(57,giPS_Kybd),1,30)    
  gkPS13     =         limit(gkPartScal*table:k(60,giPS_Kybd),1,30)    
  gkPS14     =         limit(gkPartScal*table:k(63,giPS_Kybd),1,30)    
  gkPS15     =         limit(gkPartScal*table:k(66,giPS_Kybd),1,30)    
  gkPS16     =         limit(gkPartScal*table:k(69,giPS_Kybd),1,30)    
  gkPS17     =         limit(gkPartScal*table:k(72,giPS_Kybd),1,30)    
  gkPS18     =         limit(gkPartScal*table:k(75,giPS_Kybd),1,30)    
  gkPS19     =         limit(gkPartScal*table:k(78,giPS_Kybd),1,30)    
  gkPS20     =         limit(gkPartScal*table:k(81,giPS_Kybd),1,30)    
  gkPS21     =         limit(gkPartScal*table:k(84,giPS_Kybd),1,30)    

 endif
  
 gkAAtt        chnget   "AAtt"
 gkADec        chnget   "ADec"    
 gkASus        chnget   "ASus"
 gkARel        chnget   "ARel"    
 gkAGain       chnget   "AGain"

 gkFOnOff      chnget   "FOnOff"
 gkFiltLowCut  chnget   "FiltLowCut"
 gkFiltCF      chnget   "FiltCF"
 gkFiltBW      chnget   "FiltBW"
 gkFiltGain    chnget   "FiltGain"
 gkFiltHighCut chnget   "FiltHighCut"

 gkRSend       chnget   "RSend"
 gkRSize       chnget   "RSize"
 gkR__CF       chnget   "R__CF"

 gkMonoStereo  init    1       

 gkExciterAmount chnget "ExciterAmount"    
 if trigger:k(gkExciterAmount,0.1,2)==1 then
  reinit UPDATE_EXCITER
 endif 
 UPDATE_EXCITER:
 if i(gkExciterAmount)>0 then
  chnset    "alpha(1)", "ExciterID"
 else
  chnset    "alpha(.5)", "ExciterID"
 endif
 gkExciterFreq  chnget  "ExciterFreq"

 gkV2OnOff    chnget    "V2OnOff"
 if gkV2OnOff==1 then
  gkVoiceMix  chnget    "VoiceMix"
  gkVoice2    chnget    "Voice2"
 endif
endin

instr    2    ; Always on. Updates PadSynth table when relevant widgets are changed.
 ktrig        changed   gkBW,gkBW_V2,gkPartScal,gkVoice,gkVoice2,gkTabSize,gkBW1,gkBW2,gkBW3,gkBW4,gkBW5,gkBW6,gkBW7,gkBW8,gkBW9,gkBW10,gkBW11,gkBW12,gkBW13,gkBW14,gkBW15,gkBW16,gkBW17,gkBW18,gkBW19,gkBW20,gkBW21,gkPS1,gkPS2,gkPS3,gkPS4,gkPS5,gkPS6,gkPS7,gkPS8,gkPS9,gkPS10,gkPS11,gkPS12,gkPS13,gkPS14,gkPS15,gkPS16,gkPS17,gkPS18,gkPS19,gkPS20,gkPS21
 
 if ktrig==1 then
  reinit    UpdateTable
 endif
 UpdateTable:
 giBW        =    i(gkBW)
 giPartScal  =    i(gkPartScal)
 iVoice      =    i(gkVoice)
 giTabLen    =    (2^i(gkTabSize))    ; IF THIS IS TOO SMALL, LOOPING OF THE SPECTRUM WILL BECOME AUDIBLE
 
 /* BASS */
 if iVoice==1 then
  giBassAh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.000412,0.000370,0.000337,0.000371,0.000555,0.000859,0.001305,0.001896,0.002601,0.003585,0.004729,0.006226,0.008387,0.010759,0.015604,0.021153,0.036276,0.057553,0.053066,0.030189,0.020360,0.014722,0.011349,0.009414,0.008036,0.007520,0.007518,0.008204,0.010188,0.013445,0.021468,0.024378,0.017218,0.010300,0.007574,0.005320,0.004125,0.003105,0.002351,0.001777,0.001280,0.000908,0.000603,0.000362,0.000194,0.000113,0.000107,0.000165,0.000269,0.000410,0.000591,0.000829,0.001124,0.001497,0.001964,0.002540,0.003281,0.004214,0.005465,0.007172,0.009487,0.012221,0.012290,0.008928,0.005554,0.004575,0.006297,0.009642,0.012002,0.011210,0.008864,0.007021,0.005649,0.004715,0.004092,0.003711,0.003540,0.003269,0.002720,0.002116,0.001696,0.001392,0.001163,0.000978,0.000817,0.000679,0.000554,0.000445,0.000349,0.000267,0.000201,0.000148,0.000112
  giBassAh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.000484,0.000423,0.000410,0.000587,0.001000,0.001640,0.002515,0.003647,0.005207,0.007188,0.010221,0.013861,0.021835,0.034169,0.065821,0.062536,0.031595,0.021762,0.014578,0.011706,0.009599,0.008934,0.009048,0.011008,0.014402,0.024811,0.028584,0.017346,0.010562,0.007037,0.005089,0.003642,0.002621,0.001845,0.001237,0.000781,0.000431,0.000198,0.000132,0.000128,0.000107,0.000154,0.000279,0.000469,0.000726,0.001077,0.001533,0.002136,0.002909,0.003944,0.005309,0.007297,0.010064,0.014010,0.014733,0.009883,0.005835,0.006234,0.010391,0.014244,0.013109,0.009848,0.007528,0.005944,0.004974,0.004420,0.004181,0.003659,0.002784,0.002107,0.001664,0.001346,0.001093,0.000881,0.000697,0.000538,0.000403,0.000292,0.000206,0.000145,0.000112
  giBassAh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.000560,0.000482,0.000562,0.001054,0.001899,0.003144,0.004913,0.007274,0.010934,0.015824,0.026652,0.050310,0.084386,0.048694,0.027304,0.017198,0.013209,0.010896,0.010619,0.012323,0.016925,0.031534,0.030303,0.014903,0.009410,0.006192,0.004171,0.002825,0.001804,0.001089,0.000559,0.000223,0.000135,0.000159,0.000132,0.000111,0.000205,0.000408,0.000717,0.001165,0.001784,0.002644,0.003807,0.005452,0.007846,0.011507,0.016933,0.016329,0.008970,0.006585,0.011345,0.016921,0.014673,0.010437,0.007690,0.006096,0.005243,0.004888,0.003982,0.002810,0.002096,0.001620,0.001264,0.000975,0.000730,0.000526,0.000361,0.000238,0.000159,0.000123,0.000113,0.000108
  giBassAh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.000645,0.000569,0.000955,0.001964,0.003652,0.006202,0.009824,0.015928,0.027700,0.054004,0.099838,0.045530,0.026122,0.017357,0.013212,0.012640,0.015734,0.025801,0.041244,0.020359,0.011360,0.006947,0.004321,0.002669,0.001503,0.000708,0.000231,0.000110,0.000186,0.000170,0.000130,0.000254,0.000564,0.001062,0.001820,0.002933,0.004538,0.006945,0.010838,0.017334,0.020883,0.011091,0.008199,0.016262,0.019876,0.013707,0.009453,0.007159,0.006126,0.005456,0.003786,0.002617,0.001920,0.001428,0.001039,0.000721,0.000469,0.000288,0.000180,0.000140,0.000132,0.000122,0.000102
  giBassAh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.000741,0.000742,0.001718,0.003793,0.007172,0.012458,0.021527,0.042327,0.115172,0.060415,0.029465,0.018842,0.015053,0.016424,0.026918,0.048812,0.020626,0.010654,0.006220,0.003560,0.001819,0.000726,0.000149,0.000187,0.000214,0.000156,0.000332,0.000822,0.001662,0.003004,0.005098,0.008459,0.014399,0.024542,0.017933,0.009191,0.019377,0.022533,0.014117,0.009483,0.007465,0.006578,0.004260,0.002803,0.001970,0.001367,0.000896,0.000539,0.000299,0.000185,0.000160,0.000148,0.000121
  giBassAh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.000846,0.001173,0.003275,0.007285,0.014357,0.027687,0.068261,0.124920,0.043474,0.023386,0.017847,0.021990,0.049566,0.034657,0.014061,0.007278,0.003688,0.001560,0.000396,0.000192,0.000256,0.000187,0.000558,0.001451,0.003062,0.005814,0.010610,0.020116,0.029454,0.011668,0.020774,0.026213,0.015055,0.009950,0.008362,0.005568,0.003330,0.002188,0.001396,0.000808,0.000411,0.000223,0.000189,0.000167,0.000123
  giBassAh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.000963,0.002108,0.006288,0.014551,0.031661,0.100695,0.097406,0.034417,0.021810,0.024675,0.063158,0.029838,0.012403,0.005661,0.002185,0.000448,0.000270,0.000265,0.000411,0.001440,0.003583,0.007649,0.015770,0.034040,0.018012,0.022841,0.029509,0.015471,0.010557,0.008019,0.004223,0.002548,0.001472,0.000730,0.000322,0.000229,0.000199,0.000135
  giBassAh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.001138,0.003929,0.012405,0.031859,0.108048,0.091053,0.034718,0.025288,0.051642,0.040725,0.013900,0.005342,0.001417,0.000220,0.000340,0.000509,0.002128,0.005880,0.013925,0.034767,0.022222,0.032638,0.027491,0.014365,0.010950,0.005254,0.002868,0.001449,0.000579,0.000282,0.000244,0.000156
  giBassAh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.001482,0.007584,0.024916,0.084669,0.120825,0.037691,0.032860,0.097651,0.021318,0.007128,0.001455,0.000373,0.000311,0.001646,0.006016,0.016943,0.049158,0.018410,0.045129,0.018996,0.013181,0.005616,0.002740,0.001081,0.000371,0.000296,0.000174,0.000109
  giBassAh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.002346,0.014577,0.055410,0.249952,0.046797,0.044016,0.069343,0.014565,0.003123,0.000384,0.000374,0.002906,0.011647,0.040309,0.023365,0.052509,0.019934,0.011156,0.004385,0.001619,0.000449,0.000335,0.000163,0.000126
  giBassAh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.004218,0.029108,0.201426,0.068825,0.049355,0.059660,0.011314,0.000892,0.000531,0.002882,0.015302,0.068104,0.045706,0.030943,0.016039,0.005097,0.001458,0.000457,0.000270,0.000155
  giBassAh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.007858,0.063721,0.182088,0.050577,0.081435,0.010683,0.000440,0.001018,0.011764,0.069560,0.065307,0.028732,0.010508,0.002898,0.000565,0.000313,0.000177
  giBassAh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.015174,0.169369,0.075380,0.195316,0.014245,0.000747,0.003294,0.033895,0.036836,0.037995,0.011232,0.002161,0.000593,0.000220,0.000121
  giBassAh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.029160,0.499980,0.088056,0.029134,0.000770,0.005816,0.080662,0.105049,0.022319,0.003239,0.000671,0.000252,0.000122
  giBassAh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.058218,0.137644,0.119309,0.001783,0.005763,0.136221,0.061878,0.010194,0.000915,0.000309,0.000144
  giBassAh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.127432,0.101146,0.021362,0.002037,0.139115,0.057450,0.005793,0.000626,0.000185
  giBassAh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.338719,0.390591,0.001495,0.067786,0.075975,0.004319,0.000439,0.000166
  giBassAh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,1.000000,0.058271,0.011635,0.210114,0.006478,0.000504,0.000138
  giBassAh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.275285,0.003566,0.272436,0.020387,0.000618,0.000157
  giBassAh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.202299,0.004074,0.114905,0.001251,0.000191
  giBassAh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.781181,0.135572,0.008638,0.000333
 elseif iVoice==2 then
  giBassEh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.002549,0.003134,0.003974,0.005478,0.007017,0.009323,0.012997,0.015800,0.025880,0.031561,0.069262,0.116966,0.096839,0.041388,0.030133,0.019258,0.014690,0.010922,0.008058,0.006191,0.004473,0.003237,0.002267,0.001451,0.000894,0.000442,0.000126,0.000210,0.000263,0.000265,0.000240,0.000174,0.000100,0.000185,0.000338,0.000545,0.000796,0.001110,0.001516,0.002015,0.002652,0.003519,0.004603,0.006308,0.008742,0.013114,0.019649,0.020048,0.013747,0.009105,0.006633,0.004842,0.003722,0.002793,0.002088,0.001484,0.000943,0.000453,0.000147,0.000600,0.001155,0.001779,0.002508,0.003367,0.004414,0.005764,0.007472,0.010017,0.013566,0.019406,0.025684,0.025076,0.018739,0.013927,0.011049,0.009282,0.008375,0.008024,0.008229,0.009065,0.010646,0.013251,0.016228,0.016401,0.013297,0.010080,0.008006,0.006682,0.006049,0.005955,0.006476,0.007437,0.007523,0.006180,0.004613,0.003551,0.002779,0.002235
  giBassEh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.003129,0.004092,0.005657,0.007860,0.010921,0.016087,0.021139,0.035868,0.062023,0.136227,0.109956,0.040456,0.031049,0.018374,0.013762,0.009515,0.006960,0.004597,0.003159,0.001921,0.001079,0.000444,0.000177,0.000312,0.000316,0.000280,0.000177,0.000171,0.000376,0.000664,0.001031,0.001514,0.002167,0.002999,0.004195,0.005777,0.008581,0.012777,0.021830,0.024145,0.015340,0.009845,0.006641,0.004815,0.003434,0.002428,0.001585,0.000853,0.000201,0.000618,0.001398,0.002299,0.003391,0.004727,0.006500,0.008859,0.012573,0.018214,0.027784,0.031145,0.022997,0.016195,0.012414,0.010478,0.009647,0.009763,0.010916,0.013349,0.017534,0.020106,0.016727,0.012049,0.009207,0.007617,0.007098,0.007517,0.008848,0.008867,0.006730,0.004810,0.003566,0.002737,0.002122,0.001644,0.001252,0.000929,0.000662,0.000447,0.000285,0.000180,0.000139,0.000140,0.000147,0.000144,0.000130,0.000107
  giBassEh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.003867,0.005384,0.008278,0.011948,0.018781,0.027378,0.043927,0.124923,0.154756,0.052079,0.035175,0.020879,0.013635,0.009198,0.005837,0.003709,0.002010,0.000982,0.000220,0.000161,0.000370,0.000376,0.000299,0.000147,0.000155,0.000424,0.000832,0.001379,0.002152,0.003202,0.004758,0.006982,0.011177,0.019325,0.029701,0.020304,0.011682,0.007361,0.005041,0.003356,0.002114,0.001056,0.000187,0.001017,0.002186,0.003609,0.005412,0.007905,0.011471,0.017562,0.028740,0.037414,0.026954,0.018054,0.013544,0.011691,0.011440,0.012869,0.016457,0.022485,0.022347,0.015528,0.011112,0.008894,0.008412,0.009625,0.010828,0.008346,0.005596,0.003939,0.002900,0.002141,0.001562,0.001098,0.000729,0.000449,0.000257,0.000169,0.000165,0.000175,0.000167,0.000140,0.000106
  giBassEh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.004867,0.007460,0.011945,0.019946,0.031209,0.054453,0.186212,0.117747,0.049349,0.025385,0.015675,0.009940,0.005668,0.003035,0.001342,0.000247,0.000261,0.000440,0.000408,0.000208,0.000179,0.000569,0.001196,0.002093,0.003439,0.005483,0.008701,0.015148,0.030996,0.029556,0.014724,0.008576,0.005437,0.003329,0.001707,0.000348,0.001213,0.002897,0.005055,0.008006,0.012446,0.020472,0.036633,0.042329,0.025547,0.017331,0.014092,0.013706,0.016241,0.022858,0.028248,0.019753,0.013157,0.010345,0.010349,0.012777,0.010845,0.006746,0.004465,0.003112,0.002165,0.001447,0.000902,0.000506,0.000263,0.000193,0.000207,0.000202,0.000165,0.000114
  giBassEh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.006269,0.010960,0.018651,0.031612,0.063145,0.234031,0.082814,0.038536,0.021856,0.012390,0.006478,0.002903,0.000884,0.000151,0.000526,0.000481,0.000201,0.000371,0.001091,0.002222,0.004038,0.007052,0.012642,0.026285,0.040191,0.018257,0.009711,0.005602,0.002977,0.000908,0.001205,0.003571,0.006760,0.011575,0.020120,0.038989,0.050394,0.027994,0.018663,0.016137,0.018236,0.026663,0.033012,0.020295,0.013457,0.011996,0.014986,0.012457,0.007161,0.004508,0.002937,0.001849,0.001061,0.000524,0.000256,0.000239,0.000246,0.000199,0.000126
  giBassEh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.008173,0.015702,0.032144,0.071650,0.272194,0.080839,0.036707,0.019016,0.009188,0.003840,0.000888,0.000353,0.000631,0.000355,0.000343,0.001326,0.003025,0.005992,0.011542,0.025532,0.048254,0.019676,0.009624,0.004854,0.001705,0.001234,0.004596,0.009451,0.017714,0.036427,0.062296,0.032396,0.020962,0.019531,0.026707,0.040231,0.024114,0.015246,0.015050,0.017756,0.009634,0.005482,0.003293,0.001861,0.000896,0.000362,0.000281,0.000290,0.000215,0.000115,0.000101
  giBassEh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.010770,0.023902,0.054774,0.249962,0.104159,0.041772,0.018402,0.007420,0.001965,0.000321,0.000751,0.000293,0.000854,0.002767,0.006422,0.014005,0.038772,0.040687,0.014759,0.006732,0.002120,0.002044,0.007252,0.015890,0.035324,0.075238,0.036306,0.023528,0.025922,0.045315,0.031277,0.017930,0.019424,0.016834,0.007953,0.004326,0.002219,0.000907,0.000341,0.000353,0.000285,0.000144,0.000103,0.000123
  giBassEh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.014919,0.039894,0.108912,0.235444,0.050771,0.019880,0.006071,0.000493,0.000880,0.000415,0.001140,0.004191,0.010980,0.030342,0.059163,0.017172,0.006666,0.000696,0.005809,0.016058,0.041074,0.084879,0.034762,0.027511,0.045921,0.039646,0.020777,0.025682,0.013556,0.006258,0.002911,0.001018,0.000389,0.000407,0.000231,0.000117,0.000145
  giBassEh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.021924,0.063232,0.468119,0.077063,0.024778,0.005806,0.000302,0.000961,0.000747,0.004455,0.014122,0.052633,0.036527,0.011211,0.001822,0.007148,0.023183,0.078131,0.056059,0.032330,0.053450,0.040659,0.024048,0.024961,0.009037,0.003707,0.001049,0.000481,0.000402,0.000146,0.000173,0.000107
  giBassEh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.031418,0.143370,0.161725,0.038046,0.007682,0.000705,0.000709,0.002656,0.011998,0.051134,0.039384,0.009717,0.002474,0.018934,0.072998,0.064883,0.039139,0.080623,0.030556,0.035586,0.010987,0.003730,0.000725,0.000582,0.000232,0.000203,0.000114
  giBassEh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.047801,0.499973,0.083544,0.014840,0.000643,0.000586,0.005528,0.028009,0.081360,0.013462,0.004094,0.031800,0.150499,0.047062,0.090670,0.035868,0.033668,0.008649,0.001814,0.000708,0.000286,0.000243,0.000113
  giBassEh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.079790,0.470826,0.039758,0.000985,0.000829,0.008383,0.060699,0.034340,0.001390,0.032127,0.169760,0.055035,0.079288,0.051382,0.012516,0.002036,0.000814,0.000234,0.000188,0.000140
  giBassEh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.126470,0.154125,0.011612,0.001926,0.008899,0.105301,0.022420,0.014314,0.156341,0.064675,0.081320,0.049928,0.007414,0.000961,0.000290,0.000213,0.000158
  giBassEh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.286799,0.076099,0.001412,0.005314,0.102313,0.019436,0.037889,0.129794,0.161320,0.071197,0.007462,0.001164,0.000406,0.000186,0.000100
  giBassEh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.999999,0.029677,0.001175,0.056021,0.026921,0.063607,0.094120,0.071732,0.017296,0.001415,0.000487,0.000226,0.000113
  giBassEh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.941545,0.001970,0.016767,0.068666,0.064251,0.110055,0.102753,0.004071,0.000467,0.000280,0.000117
  giBassEh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.308230,0.003850,0.210611,0.028628,0.129335,0.099831,0.001921,0.000427,0.000186
  giBassEh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.152203,0.010631,0.038873,0.259601,0.142407,0.002328,0.000372,0.000102
  giBassEh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.059354,0.112042,0.127210,0.143460,0.002830,0.000452,0.000116
  giBassEh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.003941,0.137332,0.220127,0.008140,0.000560
  giBassEh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.007699,0.057256,0.199662,0.000854,0.000109
 elseif iVoice==3 then
  giBassEee24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.010612,0.014495,0.019809,0.028477,0.041398,0.063942,0.114518,0.124942,0.078855,0.046228,0.033959,0.023394,0.018143,0.013375,0.010099,0.007577,0.005431,0.003901,0.002611,0.001632,0.000932,0.000386,0.000171,0.000289,0.000422,0.000452,0.000449,0.000398,0.000315,0.000237,0.000144,0.000104,0.000105,0.000153,0.000215,0.000290,0.000380,0.000487,0.000633,0.000816,0.001099,0.001507,0.002203,0.003126,0.003165,0.002277,0.001567,0.001153,0.000855,0.000658,0.000500,0.000378,0.000277,0.000186,0.000102,0.000154,0.000300,0.000490,0.000733,0.001033,0.001413,0.001885,0.002475,0.003256,0.004250,0.005714,0.007781,0.011083,0.015021,0.015148,0.011378,0.008165,0.006212,0.004895,0.004095,0.003568,0.003306,0.003268,0.003450,0.003938,0.004750,0.006020,0.007017,0.006500,0.005013,0.003837,0.003082,0.002641,0.002479,0.002562
  giBassEee27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.013361,0.019632,0.028170,0.046001,0.073446,0.142620,0.135320,0.067419,0.045253,0.028671,0.021074,0.014727,0.010611,0.007252,0.004920,0.003071,0.001747,0.000807,0.000249,0.000303,0.000500,0.000540,0.000520,0.000422,0.000309,0.000180,0.000119,0.000127,0.000103,0.000168,0.000256,0.000366,0.000497,0.000673,0.000915,0.001288,0.001877,0.003013,0.003942,0.003063,0.001933,0.001339,0.000949,0.000692,0.000498,0.000349,0.000219,0.000102,0.000105,0.000283,0.000531,0.000867,0.001300,0.001875,0.002609,0.003613,0.004968,0.007064,0.010239,0.015759,0.018744,0.014260,0.009604,0.006959,0.005391,0.004464,0.003989,0.003908,0.004207,0.005028,0.006490,0.008241,0.007854,0.005766,0.004257,0.003368,0.002997,0.003039,0.003534,0.003719,0.002952,0.002107,0.001565,0.001195,0.000929,0.000719,0.000551,0.000410,0.000295,0.000202,0.000133
  giBassEee30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.016942,0.026728,0.044993,0.073582,0.166083,0.150187,0.067471,0.042023,0.027730,0.018157,0.012270,0.007756,0.004771,0.002544,0.001124,0.000288,0.000418,0.000628,0.000639,0.000532,0.000370,0.000189,0.000133,0.000151,0.000118,0.000167,0.000288,0.000442,0.000634,0.000915,0.001325,0.002073,0.003575,0.004636,0.003020,0.001869,0.001215,0.000836,0.000565,0.000361,0.000186,0.000308,0.000658,0.001154,0.001829,0.002760,0.004060,0.005937,0.009069,0.014428,0.021876,0.018429,0.011479,0.007850,0.005939,0.004916,0.004612,0.004928,0.006092,0.008369,0.009952,0.007626,0.005224,0.003945,0.003510,0.003799,0.004447,0.003710,0.002491,0.001752,0.001285,0.000950,0.000696,0.000493,0.000332,0.000209,0.000125
  giBassEee33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.021903,0.037076,0.070524,0.170727,0.184856,0.075853,0.042006,0.026533,0.016384,0.009694,0.005423,0.002512,0.000745,0.000367,0.000734,0.000760,0.000589,0.000358,0.000113,0.000157,0.000177,0.000123,0.000164,0.000331,0.000549,0.000839,0.001298,0.002144,0.003934,0.005481,0.003133,0.001834,0.001150,0.000725,0.000430,0.000185,0.000178,0.000585,0.001216,0.002131,0.003468,0.005492,0.008727,0.014780,0.025548,0.021159,0.012320,0.008120,0.006189,0.005514,0.005907,0.007722,0.011241,0.010580,0.006710,0.004738,0.004184,0.004875,0.005074,0.003298,0.002150,0.001485,0.001035,0.000705,0.000453,0.000265,0.000143
  giBassEee36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.028999,0.056971,0.127925,0.249973,0.092491,0.046808,0.026763,0.015161,0.007806,0.003265,0.000772,0.000577,0.000905,0.000797,0.000474,0.000128,0.000118,0.000208,0.000175,0.000190,0.000430,0.000762,0.001269,0.002203,0.004418,0.006347,0.003143,0.001714,0.001003,0.000555,0.000205,0.000308,0.000984,0.002075,0.003788,0.006543,0.011486,0.022285,0.030464,0.016426,0.009849,0.007181,0.006579,0.007931,0.012129,0.013098,0.007735,0.005325,0.005167,0.006343,0.004251,0.002537,0.001641,0.001065,0.000654,0.000360,0.000176,0.000109,0.000108
  giBassEee39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.039208,0.091871,0.284850,0.134665,0.057264,0.029417,0.014487,0.006136,0.001614,0.000605,0.001080,0.000843,0.000360,0.000237,0.000205,0.000101,0.000336,0.000730,0.001345,0.002573,0.006021,0.006121,0.002677,0.001384,0.000697,0.000205,0.000565,0.001734,0.003749,0.007227,0.014133,0.031533,0.028537,0.013927,0.008934,0.007821,0.010066,0.016499,0.011547,0.006746,0.006088,0.007451,0.004223,0.002395,0.001442,0.000822,0.000405,0.000174,0.000127,0.000124
  giBassEee42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.053470,0.147208,0.300415,0.084059,0.036324,0.015517,0.005091,0.000576,0.001256,0.001064,0.000377,0.000160,0.000303,0.000150,0.000341,0.000891,0.001840,0.004161,0.009292,0.003746,0.001676,0.000724,0.000174,0.001325,0.003681,0.008171,0.018259,0.044043,0.023100,0.011960,0.009296,0.012291,0.020077,0.010540,0.007087,0.008984,0.005033,0.002599,0.001411,0.000675,0.000255,0.000150,0.000143
  giBassEee45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.074148,0.341465,0.151691,0.053062,0.019387,0.005023,0.000735,0.001519,0.000715,0.000141,0.000355,0.000141,0.000664,0.001681,0.004296,0.010976,0.003672,0.001452,0.000370,0.001174,0.004275,0.011021,0.029674,0.042441,0.016298,0.011074,0.015522,0.021253,0.009522,0.009803,0.006632,0.002990,0.001422,0.000537,0.000189,0.000178,0.000110
  giBassEee48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.113952,0.499961,0.093618,0.030324,0.006532,0.001153,0.001593,0.000254,0.000416,0.000182,0.000869,0.002549,0.008846,0.006282,0.002003,0.000401,0.001977,0.007592,0.023020,0.061020,0.019732,0.013187,0.024324,0.015505,0.010364,0.008518,0.003290,0.001312,0.000353,0.000214,0.000143
  giBassEee51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.183849,0.269450,0.058864,0.012277,0.001210,0.001686,0.000115,0.000411,0.000673,0.002694,0.012061,0.005358,0.001397,0.001131,0.007512,0.028323,0.057149,0.017903,0.020182,0.023136,0.012207,0.008465,0.002892,0.000813,0.000255,0.000186
  giBassEee54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.294425,0.168105,0.031031,0.001152,0.002129,0.000319,0.000295,0.001773,0.008324,0.007496,0.001451,0.002648,0.016348,0.088119,0.023921,0.024596,0.021078,0.017984,0.005195,0.001344,0.000303,0.000185,0.000105
  giBassEee57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.682961,0.106120,0.010046,0.003039,0.000281,0.000282,0.003363,0.021952,0.002902,0.002350,0.022051,0.084871,0.022153,0.042507,0.019617,0.005980,0.001075,0.000355,0.000117
  giBassEee60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.999999,0.060648,0.002312,0.000510,0.000354,0.005083,0.012579,0.000821,0.015185,0.122052,0.026381,0.031005,0.017044,0.002622,0.000436,0.000138
  giBassEee63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.538975,0.024555,0.003372,0.000823,0.005390,0.010718,0.002266,0.056680,0.035812,0.046282,0.016933,0.001625,0.000371,0.000151
  giBassEee66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.336196,0.002303,0.000636,0.003546,0.014990,0.005298,0.176248,0.049197,0.035968,0.002688,0.000369,0.000111
  giBassEee69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.212220,0.006076,0.000565,0.043897,0.004701,0.169692,0.084989,0.011954,0.000713,0.000170
  giBassEee72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.121291,0.001021,0.010166,0.001642,0.244060,0.061995,0.005244,0.000275,0.000113
  giBassEee75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.049112,0.001647,0.021440,0.113388,0.092568,0.003249,0.000304,0.000100
  giBassEee78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.004609,0.007093,0.010596,0.098393,0.005375,0.000222
  giBassEee81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.012152,0.087798,0.339376,0.023913,0.000340
  giBassEee84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.002039,0.003283,0.123990,0.000551,0.000112
 elseif iVoice==4 then
  giBassOh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.002656,0.003252,0.004105,0.005622,0.007160,0.009459,0.013108,0.015856,0.025894,0.031448,0.069195,0.117293,0.097983,0.042979,0.032065,0.021929,0.018078,0.015475,0.014066,0.014550,0.016289,0.021526,0.023582,0.017583,0.010850,0.007974,0.005821,0.004584,0.003615,0.002841,0.002234,0.001707,0.001268,0.000905,0.000600,0.000361,0.000186,0.000124,0.000163,0.000187,0.000186,0.000166,0.000139,0.000101,0.000100,0.000137,0.000189,0.000263,0.000361,0.000489,0.000654,0.000858,0.001117,0.001451,0.001869,0.002471,0.003247,0.004440,0.005393,0.004629,0.002917,0.002066,0.002532,0.003852,0.005391,0.005792,0.004814,0.003675,0.002842,0.002214,0.001763,0.001401,0.001091,0.000746,0.000345,0.000127,0.000165,0.000194,0.000194,0.000185,0.000171,0.000154,0.000137,0.000119,0.000101
  giBassOh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.003258,0.004239,0.005824,0.008031,0.011084,0.016213,0.021193,0.035813,0.061897,0.136532,0.111390,0.042393,0.033786,0.022028,0.018961,0.016732,0.017591,0.021496,0.028305,0.022987,0.013061,0.009039,0.006327,0.004802,0.003617,0.002714,0.001972,0.001369,0.000897,0.000513,0.000245,0.000108,0.000159,0.000210,0.000226,0.000203,0.000166,0.000112,0.000138,0.000201,0.000297,0.000435,0.000622,0.000874,0.001199,0.001639,0.002216,0.003089,0.004305,0.006101,0.005978,0.003630,0.002459,0.003410,0.005563,0.006988,0.005953,0.004329,0.003190,0.002395,0.001829,0.001373,0.000891,0.000334,0.000160,0.000222,0.000234,0.000222,0.000202,0.000178,0.000153,0.000127,0.000102
  giBassOh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.004023,0.005567,0.008482,0.012147,0.018935,0.027436,0.043784,0.124923,0.156060,0.054359,0.038530,0.025628,0.020731,0.020302,0.024164,0.033681,0.024398,0.013158,0.008555,0.006066,0.004329,0.003073,0.002076,0.001307,0.000716,0.000311,0.000128,0.000209,0.000264,0.000252,0.000205,0.000130,0.000103,0.000161,0.000252,0.000400,0.000621,0.000939,0.001374,0.001992,0.002862,0.004244,0.006522,0.007371,0.004231,0.002979,0.004965,0.007920,0.007484,0.005169,0.003596,0.002580,0.001862,0.001227,0.000441,0.000198,0.000273,0.000271,0.000248,0.000214,0.000179,0.000142,0.000109
  giBassOh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.005059,0.007690,0.012187,0.020150,0.031282,0.054249,0.186526,0.120147,0.052781,0.030786,0.024379,0.024804,0.034918,0.035401,0.016807,0.010063,0.006707,0.004475,0.002912,0.001727,0.000868,0.000313,0.000184,0.000294,0.000312,0.000251,0.000145,0.000162,0.000275,0.000476,0.000802,0.001297,0.002024,0.003113,0.004960,0.008196,0.007816,0.003687,0.004946,0.009187,0.008701,0.005588,0.003662,0.002498,0.001599,0.000503,0.000264,0.000332,0.000310,0.000265,0.000215,0.000163,0.000117
  giBassOh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.006506,0.011247,0.018924,0.031724,0.062920,0.234685,0.085997,0.043880,0.030968,0.029117,0.043082,0.035194,0.015962,0.009176,0.005688,0.003419,0.001811,0.000723,0.000184,0.000326,0.000373,0.000279,0.000129,0.000146,0.000275,0.000527,0.000981,0.001724,0.002915,0.004964,0.008921,0.009303,0.004152,0.007745,0.011649,0.007393,0.004456,0.002821,0.001501,0.000256,0.000391,0.000373,0.000311,0.000239,0.000169,0.000110
  giBassOh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.008468,0.016044,0.032394,0.071542,0.272804,0.084708,0.044003,0.033428,0.042935,0.045900,0.018050,0.009589,0.005421,0.002735,0.001024,0.000216,0.000420,0.000405,0.000225,0.000125,0.000277,0.000594,0.001244,0.002397,0.004430,0.008609,0.011959,0.004919,0.011123,0.011908,0.006381,0.003660,0.001783,0.000320,0.000469,0.000405,0.000305,0.000205,0.000123
  giBassOh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.011135,0.024299,0.054889,0.249961,0.108722,0.051276,0.040626,0.067415,0.026335,0.012142,0.006153,0.002616,0.000622,0.000415,0.000502,0.000257,0.000119,0.000122,0.000324,0.000801,0.001884,0.004002,0.008536,0.014820,0.005995,0.015949,0.010404,0.005196,0.002469,0.000400,0.000547,0.000433,0.000288,0.000164
  giBassOh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.015381,0.040301,0.108505,0.240245,0.061575,0.049617,0.070803,0.020128,0.008953,0.003454,0.000627,0.000589,0.000502,0.000134,0.000183,0.000551,0.001609,0.004060,0.009951,0.015670,0.009932,0.017456,0.007348,0.003209,0.000530,0.000621,0.000431,0.000236,0.000113
  giBassOh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.022496,0.063453,0.469427,0.087754,0.058244,0.070388,0.018356,0.006838,0.001446,0.000651,0.000556,0.000143,0.000122,0.000550,0.001961,0.005833,0.017875,0.008317,0.023338,0.008927,0.003005,0.000784,0.000625,0.000341,0.000143
  giBassOh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.032102,0.143154,0.169468,0.066893,0.091855,0.019191,0.005474,0.000433,0.000811,0.000162,0.000249,0.001190,0.004802,0.017251,0.009855,0.023855,0.007334,0.000641,0.000812,0.000411,0.000150,0.000101
  giBassOh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.048599,0.499974,0.102552,0.134829,0.024280,0.005230,0.000837,0.000520,0.000237,0.000241,0.001607,0.008013,0.029637,0.031910,0.010392,0.000800,0.000865,0.000325,0.000132
  giBassOh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.080605,0.480428,0.099238,0.040254,0.006909,0.001178,0.000182,0.000102,0.001103,0.008123,0.031336,0.034912,0.006417,0.001243,0.000473,0.000155
  giBassOh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.126917,0.175507,0.140772,0.013677,0.001307,0.000241,0.003934,0.035768,0.046682,0.006011,0.001247,0.000283,0.000138
  giBassOh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.286365,0.133810,0.038389,0.000867,0.000107,0.000499,0.009611,0.019717,0.014670,0.001624,0.000299,0.000129
  giBassOh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000002,0.269661,0.010462,0.001040,0.000483,0.016028,0.063825,0.001601,0.000650,0.000177
  giBassOh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.960747,0.080495,0.002355,0.000205,0.016245,0.069803,0.002487,0.000311,0.000112
  giBassOh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.350990,0.027349,0.007868,0.093348,0.002493,0.000277
  giBassOh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.267641,0.001732,0.001001,0.039440,0.003247,0.000257
  giBassOh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.539313,0.002079,0.032056,0.003202,0.000354
  giBassOh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.160992,0.000409,0.139609,0.000620,0.000116
  giBassOh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.054697,0.015735,0.004986,0.000134
 elseif iVoice==5 then
  giBassOoh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.004170,0.004767,0.005809,0.007125,0.009423,0.012338,0.016616,0.025281,0.036773,0.084223,0.097509,0.055644,0.026776,0.021850,0.013883,0.011537,0.008646,0.005282,0.002155,0.001654,0.001414,0.001239,0.001040,0.000878,0.000785,0.000712,0.000629,0.000575,0.000483,0.000406,0.000327,0.000237,0.000173,0.000104,0.000121,0.000160,0.000207,0.000263,0.000334,0.000421,0.000551,0.000726,0.001018,0.001329,0.001313,0.001049,0.000891,0.000885,0.000983,0.001199,0.001540,0.001977,0.002121,0.001784,0.001340,0.001034,0.000826,0.000705,0.000652,0.000680,0.000715,0.000646,0.000503,0.000390,0.000309,0.000249,0.000204,0.000167,0.000137,0.000111
  giBassOoh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.005030,0.006191,0.007651,0.010372,0.014571,0.021434,0.030781,0.071907,0.119545,0.072776,0.031574,0.022556,0.015399,0.011394,0.007041,0.002518,0.001911,0.001634,0.001295,0.001096,0.000937,0.000834,0.000721,0.000630,0.000508,0.000397,0.000272,0.000181,0.000107,0.000153,0.000213,0.000285,0.000379,0.000500,0.000688,0.000967,0.001445,0.001618,0.001277,0.001057,0.001077,0.001285,0.001687,0.002301,0.002526,0.001994,0.001425,0.001068,0.000863,0.000780,0.000826,0.000841,0.000676,0.000495,0.000375,0.000290,0.000229,0.000180,0.000141,0.000108
  giBassOoh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.006083,0.008003,0.010711,0.016374,0.024519,0.037449,0.112833,0.124998,0.041417,0.028771,0.018019,0.012343,0.005402,0.002386,0.001972,0.001531,0.001231,0.001066,0.000905,0.000784,0.000607,0.000449,0.000285,0.000153,0.000101,0.000109,0.000153,0.000230,0.000327,0.000460,0.000642,0.000947,0.001505,0.001937,0.001502,0.001242,0.001345,0.001774,0.002567,0.002998,0.002232,0.001518,0.001113,0.000935,0.000970,0.000984,0.000727,0.000511,0.000372,0.000279,0.000210,0.000157,0.000113
  giBassOoh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.007502,0.010380,0.016096,0.024230,0.043922,0.147876,0.113247,0.043397,0.022792,0.015355,0.005760,0.002715,0.002152,0.001629,0.001298,0.001071,0.000882,0.000647,0.000405,0.000215,0.000125,0.000127,0.000114,0.000198,0.000315,0.000473,0.000703,0.001106,0.001912,0.002216,0.001551,0.001530,0.002044,0.003150,0.003426,0.002208,0.001462,0.001135,0.001161,0.001127,0.000745,0.000502,0.000355,0.000253,0.000178,0.000120
  giBassOoh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.009537,0.014254,0.024685,0.050579,0.168510,0.111335,0.043720,0.023086,0.010571,0.003311,0.002480,0.001757,0.001426,0.001152,0.000813,0.000474,0.000209,0.000141,0.000155,0.000117,0.000124,0.000242,0.000415,0.000671,0.001107,0.002045,0.002638,0.001790,0.001976,0.003097,0.004267,0.002696,0.001662,0.001314,0.001439,0.001013,0.000623,0.000411,0.000275,0.000178,0.000106
  giBassOoh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.012372,0.020730,0.042850,0.143721,0.145460,0.045094,0.022771,0.005016,0.003262,0.002189,0.001666,0.001260,0.000793,0.000362,0.000117,0.000168,0.000179,0.000112,0.000138,0.000307,0.000569,0.001000,0.001934,0.003236,0.002114,0.002570,0.004603,0.003988,0.002137,0.001561,0.001683,0.000990,0.000581,0.000361,0.000217,0.000117
  giBassOoh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.016008,0.032755,0.074919,0.250025,0.057545,0.024692,0.004776,0.003064,0.002133,0.001569,0.000898,0.000306,0.000146,0.000215,0.000146,0.000188,0.000459,0.000921,0.001903,0.003897,0.002498,0.003572,0.006036,0.003057,0.001884,0.001984,0.001031,0.000565,0.000317,0.000160
  giBassOoh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.020759,0.048464,0.295785,0.086792,0.030710,0.005430,0.003258,0.002143,0.001295,0.000430,0.000194,0.000255,0.000111,0.000112,0.000397,0.000949,0.002219,0.004444,0.003070,0.006325,0.004431,0.002278,0.002265,0.001008,0.000508,0.000241
  giBassOoh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.028510,0.101166,0.222637,0.046174,0.006623,0.003513,0.002303,0.000949,0.000184,0.000305,0.000110,0.000102,0.000478,0.001339,0.004097,0.003587,0.006207,0.005401,0.002632,0.002031,0.000827,0.000359,0.000115
  giBassOoh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.041474,0.287568,0.090200,0.010044,0.004379,0.002520,0.000724,0.000336,0.000224,0.000275,0.001139,0.003874,0.004235,0.009226,0.004283,0.003373,0.001165,0.000434,0.000107
  giBassOoh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.065511,0.500015,0.049381,0.006128,0.003138,0.000611,0.000437,0.000383,0.001850,0.007794,0.007149,0.006113,0.003967,0.001128,0.000317
  giBassOoh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.096931,0.173581,0.010859,0.004286,0.000861,0.000510,0.000225,0.001899,0.008888,0.012656,0.004558,0.002017,0.000482
  giBassOoh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.202350,0.092351,0.007027,0.001897,0.000620,0.000129,0.000970,0.008202,0.012422,0.005266,0.001649,0.000227
  giBassOoh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.575236,0.020098,0.005041,0.000672,0.000138,0.002281,0.008473,0.008567,0.002331,0.000214
  giBassOoh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000000,0.012255,0.001224,0.000167,0.000119,0.003699,0.014299,0.007933,0.000634,0.000110
  giBassOoh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.347133,0.008571,0.001019,0.003798,0.025311,0.004032,0.000169
  giBassOoh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.184689,0.003794,0.000258,0.001940,0.024843,0.003298,0.000182
  giBassOoh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.040199,0.001344,0.000201,0.016948,0.004662,0.000147
  giBassOoh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.024509,0.000334,0.007398,0.015866,0.000219
  giBassOoh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.017142,0.050630,0.000336
  giBassOoh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.007586,0.003880,0.006596
 /* TENOR */
 elseif iVoice==6 then
  giTenorAh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.000583,0.000538,0.000460,0.000380,0.000388,0.000576,0.000951,0.001458,0.002147,0.003005,0.004083,0.005479,0.007166,0.009451,0.012430,0.016493,0.023155,0.032764,0.051969,0.061659,0.047828,0.030764,0.022969,0.017260,0.014303,0.012261,0.011262,0.011171,0.011848,0.014033,0.017624,0.024861,0.029338,0.024087,0.015948,0.011498,0.008368,0.006382,0.004890,0.003733,0.002846,0.002112,0.001533,0.001072,0.000708,0.000443,0.000260,0.000174,0.000165,0.000177,0.000179,0.000163,0.000135,0.000113,0.000143,0.000167,0.000180,0.000184,0.000185,0.000201,0.000268,0.000401,0.000606,0.000882,0.001240,0.001688,0.002239,0.002922,0.003753,0.004799,0.006128,0.007860,0.010282,0.013541,0.017840,0.020070,0.017502,0.013105,0.010276,0.009233,0.009930,0.012379,0.015621,0.016680,0.014400,0.011294,0.008942,0.007204,0.005991,0.005111,0.004508,0.004114,0.003872,0.003547,0.002939
  giTenorAh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.000687,0.000612,0.000487,0.000443,0.000667,0.001204,0.001979,0.003057,0.004439,0.006332,0.008723,0.012150,0.016682,0.024614,0.036410,0.062524,0.071897,0.047198,0.030625,0.021709,0.017111,0.014322,0.013279,0.013564,0.015875,0.020510,0.030613,0.034231,0.023560,0.015114,0.010394,0.007498,0.005453,0.003961,0.002827,0.001944,0.001277,0.000775,0.000433,0.000237,0.000194,0.000210,0.000212,0.000185,0.000139,0.000103,0.000152,0.000190,0.000213,0.000219,0.000222,0.000264,0.000405,0.000664,0.001044,0.001561,0.002232,0.003095,0.004195,0.005621,0.007518,0.010165,0.014016,0.019655,0.023962,0.020629,0.014615,0.011499,0.011311,0.014111,0.018689,0.019687,0.015745,0.011810,0.009057,0.007235,0.006000,0.005218,0.004763,0.004396,0.003608,0.002646,0.002014,0.001621,0.001344,0.001117,0.000916,0.000734,0.000568,0.000422,0.000298,0.000203,0.000142,0.000117,0.000116,0.000118,0.000115
  giTenorAh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.000802,0.000674,0.000523,0.000689,0.001389,0.002504,0.004118,0.006355,0.009422,0.013917,0.020353,0.032454,0.055699,0.087276,0.061357,0.036058,0.024181,0.018765,0.016031,0.015921,0.018738,0.025814,0.040021,0.034794,0.019857,0.012723,0.008611,0.005883,0.004012,0.002612,0.001611,0.000893,0.000439,0.000242,0.000241,0.000254,0.000220,0.000152,0.000106,0.000178,0.000230,0.000257,0.000260,0.000288,0.000455,0.000818,0.001385,0.002191,0.003277,0.004737,0.006712,0.009481,0.013715,0.020280,0.027934,0.024626,0.016448,0.013135,0.014739,0.020750,0.023477,0.018047,0.012882,0.009552,0.007507,0.006271,0.005611,0.005025,0.003750,0.002614,0.001991,0.001589,0.001273,0.000999,0.000753,0.000538,0.000359,0.000228,0.000153,0.000136,0.000139,0.000137,0.000122
  giTenorAh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.000932,0.000718,0.000659,0.001407,0.002892,0.005202,0.008652,0.013651,0.021662,0.036789,0.070120,0.102139,0.054521,0.032839,0.023181,0.019038,0.019227,0.024920,0.041242,0.045371,0.023935,0.014094,0.008980,0.005704,0.003505,0.002002,0.000991,0.000417,0.000274,0.000304,0.000270,0.000176,0.000182,0.000265,0.000306,0.000311,0.000394,0.000759,0.001464,0.002554,0.004117,0.006330,0.009569,0.014606,0.023150,0.033635,0.026442,0.017079,0.016240,0.023483,0.027945,0.019964,0.013524,0.009824,0.007738,0.006717,0.005807,0.003917,0.002676,0.002012,0.001549,0.001160,0.000819,0.000533,0.000316,0.000191,0.000162,0.000167,0.000158,0.000129
  giTenorAh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.001076,0.000760,0.001152,0.002918,0.006013,0.010963,0.018910,0.033003,0.065564,0.123397,0.061570,0.034546,0.024544,0.022364,0.028096,0.049779,0.048235,0.023028,0.012783,0.007477,0.004231,0.002147,0.000887,0.000349,0.000355,0.000327,0.000195,0.000152,0.000288,0.000361,0.000371,0.000539,0.001217,0.002491,0.004498,0.007542,0.012318,0.020674,0.035880,0.035208,0.020677,0.019986,0.031450,0.029001,0.018014,0.012074,0.009087,0.007807,0.005927,0.003587,0.002503,0.001834,0.001291,0.000830,0.000470,0.000249,0.000194,0.000200,0.000178,0.000130
  giTenorAh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.001221,0.000884,0.002403,0.006103,0.012641,0.024258,0.049147,0.124843,0.094249,0.043351,0.028599,0.027085,0.040952,0.068353,0.030182,0.014975,0.007910,0.003883,0.001548,0.000473,0.000421,0.000370,0.000161,0.000304,0.000426,0.000443,0.000809,0.002086,0.004462,0.008387,0.015033,0.028032,0.047931,0.029239,0.022629,0.037393,0.031509,0.018127,0.012012,0.009537,0.007224,0.004034,0.002693,0.001836,0.001138,0.000598,0.000284,0.000232,0.000231,0.000177,0.000105
  giTenorAh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.001347,0.001378,0.005009,0.012714,0.027848,0.064949,0.174667,0.072154,0.037557,0.031873,0.051698,0.069662,0.025483,0.011788,0.005236,0.001791,0.000485,0.000510,0.000306,0.000357,0.000517,0.000580,0.001645,0.004407,0.009531,0.019083,0.040846,0.049561,0.026461,0.041849,0.036370,0.019264,0.012655,0.010146,0.005280,0.003211,0.002020,0.001088,0.000460,0.000274,0.000277,0.000199,0.000106,0.000102
  giTenorAh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.001435,0.002814,0.010404,0.027304,0.073592,0.204283,0.065680,0.038086,0.049867,0.090768,0.028199,0.011415,0.004006,0.000835,0.000609,0.000353,0.000145,0.000530,0.000623,0.001521,0.005124,0.012704,0.029322,0.067515,0.034280,0.047183,0.040088,0.019737,0.013502,0.007873,0.004048,0.002335,0.001073,0.000384,0.000337,0.000259,0.000126,0.000121,0.000110
  giTenorAh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.001521,0.005836,0.021928,0.066019,0.246823,0.069099,0.044735,0.099607,0.046071,0.014964,0.004298,0.000698,0.000655,0.000576,0.000744,0.002438,0.009011,0.024682,0.071912,0.041426,0.063055,0.036096,0.018215,0.011882,0.005020,0.002589,0.000943,0.000389,0.000358,0.000166,0.000147,0.000118
  giTenorAh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.001771,0.012217,0.048556,0.249892,0.086762,0.054219,0.136820,0.029974,0.007772,0.000947,0.000741,0.000186,0.000854,0.001622,0.008944,0.030132,0.096048,0.045359,0.063137,0.024077,0.014480,0.005399,0.002282,0.000570,0.000463,0.000211,0.000176,0.000111
  giTenorAh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.002757,0.025433,0.129910,0.144292,0.063753,0.139297,0.023565,0.003578,0.001018,0.000106,0.001032,0.003292,0.019074,0.081740,0.052932,0.072742,0.025316,0.010562,0.004040,0.000921,0.000554,0.000212,0.000203
  giTenorAh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.005629,0.054612,0.408549,0.076173,0.181522,0.022828,0.001670,0.000706,0.001061,0.003044,0.025417,0.135064,0.094412,0.039477,0.015746,0.004670,0.000769,0.000519,0.000243,0.000120,0.000116
  giTenorAh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.011675,0.132064,0.138210,0.199271,0.029918,0.001395,0.000113,0.001485,0.018035,0.143887,0.126163,0.036439,0.010040,0.001886,0.000716,0.000291,0.000124
  giTenorAh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.024440,0.499943,0.108469,0.059961,0.001897,0.000373,0.003245,0.060299,0.090767,0.048171,0.010803,0.001140,0.000421,0.000222,0.000136
  giTenorAh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.050867,0.288566,0.278569,0.007157,0.000212,0.006586,0.163506,0.145461,0.021119,0.001842,0.000424,0.000177,0.000102
  giTenorAh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.109214,0.152326,0.045647,0.001411,0.006088,0.270092,0.078932,0.009337,0.001038,0.000241,0.000131
  giTenorAh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.264112,0.398522,0.002789,0.002968,0.287762,0.072864,0.003770,0.000582,0.000183,0.000108
  giTenorAh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,1.000000,0.119936,0.000746,0.120626,0.096351,0.002278,0.000445,0.000181
  giTenorAh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.577120,0.014313,0.013172,0.290914,0.003683,0.000354,0.000139
  giTenorAh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.304668,0.002822,0.540234,0.018674,0.000482,0.000178
  giTenorAh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.797043,0.005936,0.145727,0.001164,0.000216
 elseif iVoice==7 then
  giTenorEh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.003711,0.004448,0.005620,0.007420,0.009664,0.012761,0.017128,0.022353,0.032172,0.044202,0.073680,0.104950,0.091828,0.054791,0.037776,0.026500,0.019680,0.014875,0.011123,0.008435,0.006205,0.004488,0.003152,0.002073,0.001292,0.000697,0.000324,0.000231,0.000316,0.000377,0.000378,0.000345,0.000277,0.000200,0.000121,0.000127,0.000231,0.000372,0.000572,0.000833,0.001169,0.001615,0.002156,0.002881,0.003837,0.005088,0.007184,0.010048,0.015987,0.019744,0.015794,0.009935,0.007135,0.005064,0.003843,0.002896,0.002183,0.001646,0.001193,0.000842,0.000546,0.000290,0.000179,0.000420,0.000705,0.001053,0.001472,0.002002,0.002658,0.003482,0.004575,0.005975,0.008042,0.010971,0.015644,0.021201,0.021315,0.015876,0.011198,0.008260,0.006172,0.004757,0.003667,0.002902,0.002421,0.002278,0.002500,0.003021,0.003820,0.004887,0.006330,0.008360,0.011171,0.014732,0.016196,0.013872,0.010516
  giTenorEh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.004552,0.005760,0.007814,0.010792,0.014955,0.021165,0.029612,0.045235,0.074156,0.122974,0.105870,0.057214,0.038741,0.025625,0.018589,0.013142,0.009439,0.006465,0.004364,0.002718,0.001559,0.000735,0.000308,0.000321,0.000440,0.000451,0.000405,0.000302,0.000194,0.000158,0.000311,0.000537,0.000866,0.001309,0.001925,0.002713,0.003839,0.005325,0.007903,0.011718,0.020082,0.022933,0.014817,0.009328,0.006254,0.004484,0.003195,0.002291,0.001599,0.001065,0.000642,0.000286,0.000381,0.000762,0.001246,0.001850,0.002651,0.003674,0.005077,0.006985,0.009944,0.014441,0.022249,0.026413,0.019930,0.013155,0.009153,0.006590,0.004826,0.003613,0.002894,0.002744,0.003188,0.004118,0.005508,0.007468,0.010395,0.014756,0.019114,0.017765,0.012963,0.009621,0.007541,0.006420,0.005922,0.006058,0.006871,0.008554,0.009629,0.007928,0.005529,0.003971,0.002926,0.002205,0.001657,0.001233,0.000892
  giTenorEh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.005607,0.007614,0.011207,0.016365,0.024712,0.037348,0.060375,0.121842,0.141500,0.072108,0.044425,0.028047,0.018770,0.012615,0.008179,0.005124,0.002879,0.001420,0.000505,0.000350,0.000521,0.000532,0.000443,0.000289,0.000131,0.000204,0.000437,0.000812,0.001364,0.002176,0.003280,0.004954,0.007398,0.011851,0.021792,0.027213,0.015773,0.009598,0.006088,0.004119,0.002767,0.001798,0.001092,0.000534,0.000408,0.000945,0.001653,0.002596,0.003890,0.005707,0.008347,0.012773,0.020361,0.030867,0.025823,0.015735,0.010227,0.007000,0.004859,0.003600,0.003233,0.003864,0.005306,0.007526,0.011040,0.016627,0.022728,0.019669,0.013379,0.009687,0.007778,0.007000,0.007281,0.008747,0.011171,0.010086,0.006655,0.004490,0.003153,0.002248,0.001593,0.001092,0.000713,0.000433,0.000247,0.000157,0.000144,0.000149,0.000141,0.000119
  giTenorEh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.007001,0.010427,0.016456,0.026630,0.043168,0.076989,0.169617,0.125049,0.061026,0.034673,0.021596,0.013484,0.007893,0.004258,0.001928,0.000584,0.000462,0.000640,0.000585,0.000382,0.000160,0.000329,0.000737,0.001424,0.002500,0.004083,0.006621,0.011222,0.020906,0.033101,0.018125,0.010139,0.006169,0.003823,0.002339,0.001300,0.000525,0.000152,0.000829,0.001747,0.003025,0.004885,0.007717,0.012276,0.020847,0.036056,0.029616,0.016722,0.010179,0.006565,0.004476,0.003867,0.004909,0.007299,0.011244,0.018113,0.027071,0.022114,0.014163,0.010128,0.008501,0.008626,0.010768,0.013620,0.009945,0.006082,0.003944,0.002638,0.001744,0.001093,0.000630,0.000328,0.000188,0.000175,0.000178,0.000156,0.000116
  giTenorEh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.008898,0.014845,0.025531,0.044722,0.088437,0.209987,0.109632,0.053027,0.029768,0.016881,0.008983,0.004150,0.001395,0.000463,0.000755,0.000691,0.000400,0.000255,0.000745,0.001668,0.003236,0.005773,0.010198,0.020143,0.039588,0.019923,0.010157,0.005810,0.003302,0.001689,0.000582,0.000359,0.001415,0.002957,0.005341,0.009195,0.016165,0.031454,0.042867,0.022527,0.012419,0.007380,0.004874,0.005036,0.007696,0.012757,0.022519,0.032658,0.021211,0.013339,0.010317,0.010331,0.013816,0.015314,0.008765,0.005134,0.003184,0.001938,0.001090,0.000535,0.000250,0.000207,0.000210,0.000173,0.000113
  giTenorEh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.011502,0.021549,0.042263,0.090322,0.245564,0.114253,0.051172,0.026245,0.012912,0.005428,0.001469,0.000641,0.000901,0.000604,0.000162,0.000315,0.001073,0.002614,0.005421,0.010640,0.023418,0.045836,0.018645,0.008963,0.004582,0.002130,0.000572,0.000762,0.002491,0.005301,0.010156,0.019894,0.044518,0.039884,0.018319,0.009661,0.005793,0.006382,0.011028,0.020816,0.038283,0.025968,0.015109,0.011870,0.013774,0.019307,0.011088,0.005870,0.003324,0.001790,0.000830,0.000329,0.000246,0.000244,0.000178
  giTenorEh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.015233,0.032741,0.074724,0.243797,0.144246,0.056112,0.025240,0.010252,0.002839,0.000700,0.001067,0.000580,0.000104,0.000883,0.002741,0.006585,0.014845,0.043733,0.031620,0.012214,0.005557,0.002198,0.000195,0.001893,0.005217,0.011479,0.025712,0.062141,0.031662,0.014093,0.007252,0.007795,0.015192,0.033589,0.039695,0.019565,0.014153,0.017708,0.020404,0.009087,0.004554,0.002215,0.000878,0.000317,0.000301,0.000241,0.000125
  giTenorEh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.020852,0.053258,0.153984,0.250056,0.069342,0.026968,0.008516,0.001168,0.001282,0.000765,0.000162,0.001475,0.005006,0.013261,0.041892,0.036286,0.012356,0.004686,0.001052,0.001663,0.006070,0.015487,0.041856,0.059403,0.020427,0.008986,0.009865,0.022609,0.054432,0.028468,0.017103,0.021683,0.020009,0.007941,0.003514,0.001272,0.000379,0.000357,0.000233,0.000104,0.000116
  giTenorEh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.029695,0.089457,0.420024,0.106052,0.033762,0.008298,0.000925,0.001385,0.000186,0.001501,0.006486,0.020422,0.079237,0.020329,0.006617,0.001174,0.002828,0.010697,0.032394,0.085861,0.024877,0.009765,0.015434,0.045175,0.042511,0.020691,0.027733,0.017578,0.006386,0.002185,0.000498,0.000422,0.000230,0.000123,0.000124
  giTenorEh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.043127,0.180772,0.228631,0.052523,0.010863,0.001282,0.001207,0.000630,0.005235,0.021308,0.091756,0.017942,0.004263,0.001528,0.010624,0.039871,0.079869,0.019355,0.012797,0.041749,0.052038,0.023798,0.038715,0.011768,0.003590,0.000660,0.000489,0.000194,0.000164
  giTenorEh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.065477,0.487624,0.112219,0.020505,0.001401,0.001155,0.001754,0.013158,0.087497,0.024420,0.004380,0.003803,0.022978,0.124331,0.028184,0.015603,0.067238,0.039134,0.035441,0.018175,0.004427,0.000636,0.000482,0.000172,0.000145
  giTenorEh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.106518,0.500069,0.053933,0.002334,0.001530,0.002950,0.026528,0.072557,0.009371,0.003329,0.030986,0.118789,0.017972,0.045243,0.056938,0.043394,0.015883,0.002543,0.000714,0.000208,0.000179,0.000107
  giTenorEh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.178931,0.212116,0.016600,0.002767,0.002985,0.040842,0.040651,0.002326,0.021412,0.171741,0.019535,0.090410,0.041394,0.035157,0.004372,0.000844,0.000247,0.000120
  giTenorEh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.361640,0.105064,0.002567,0.001261,0.042631,0.035890,0.003057,0.079786,0.038715,0.083552,0.047617,0.023541,0.001321,0.000388,0.000190,0.000108
  giTenorEh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.975277,0.041007,0.002314,0.026319,0.048835,0.007606,0.248676,0.031209,0.078257,0.036342,0.001274,0.000344,0.000170
  giTenorEh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.999999,0.004669,0.005900,0.145077,0.006659,0.237507,0.090489,0.086786,0.005082,0.000415,0.000215,0.000103
  giTenorEh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.424187,0.005531,0.081687,0.004650,0.343420,0.180822,0.070293,0.001686,0.000240,0.000139
  giTenorEh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.210142,0.002525,0.071786,0.159607,0.167145,0.047085,0.000779,0.000220
  giTenorEh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.082013,0.052640,0.015212,0.062417,0.072683,0.000688,0.000200
  giTenorEh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.009338,0.290149,0.475002,0.173594,0.000829,0.000206
  giTenorEh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.011062,0.009300,0.361645,0.003372,0.000278
 elseif iVoice==8 then
  giTenorEee24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.007836,0.009017,0.011311,0.014842,0.018645,0.029937,0.038490,0.092836,0.120812,0.077127,0.033458,0.027877,0.016830,0.013887,0.009932,0.007423,0.005705,0.003965,0.002903,0.001947,0.001187,0.000715,0.000276,0.000168,0.000280,0.000294,0.000295,0.000264,0.000200,0.000154,0.000111,0.000202,0.000314,0.000470,0.000672,0.000922,0.001256,0.001662,0.002194,0.002908,0.003823,0.005290,0.007297,0.010980,0.014156,0.012522,0.008422,0.005960,0.004302,0.003220,0.002441,0.001839,0.001385,0.001017,0.000728,0.000498,0.000315,0.000172,0.000153,0.000269,0.000415,0.000594,0.000822,0.001105,0.001457,0.001919,0.002506,0.003348,0.004555,0.006428,0.008898,0.009355,0.007247,0.005218,0.004033,0.003245,0.002812,0.002568,0.002513,0.002633,0.002927,0.003483,0.004336,0.005623,0.006787,0.006501
  giTenorEee27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.009549,0.011595,0.015950,0.020100,0.035178,0.051554,0.130198,0.125062,0.047558,0.034534,0.019595,0.015158,0.010326,0.007489,0.004997,0.003422,0.002102,0.001160,0.000517,0.000131,0.000196,0.000347,0.000349,0.000335,0.000261,0.000188,0.000102,0.000117,0.000241,0.000405,0.000645,0.000960,0.001392,0.001953,0.002719,0.003785,0.005367,0.007876,0.012538,0.016966,0.013606,0.008442,0.005765,0.004012,0.002889,0.002064,0.001464,0.000997,0.000649,0.000381,0.000181,0.000125,0.000280,0.000481,0.000736,0.001075,0.001512,0.002098,0.002890,0.004070,0.005876,0.009027,0.011425,0.009230,0.006235,0.004602,0.003652,0.003170,0.003002,0.003138,0.003573,0.004460,0.005915,0.007838,0.007914,0.005903,0.004220,0.003145,0.002512,0.002171,0.002123,0.002075,0.001670,0.001214,0.000931,0.000732,0.000589,0.000472,0.000374,0.000289,0.000216
  giTenorEee30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.011630,0.015290,0.021759,0.037725,0.057788,0.163156,0.117975,0.044909,0.026599,0.018812,0.011933,0.008144,0.004975,0.003001,0.001508,0.000594,0.000130,0.000311,0.000418,0.000409,0.000314,0.000210,0.000107,0.000263,0.000494,0.000849,0.001339,0.002047,0.003027,0.004491,0.006866,0.010927,0.018703,0.017814,0.010239,0.006481,0.004290,0.002881,0.001930,0.001234,0.000740,0.000381,0.000122,0.000299,0.000577,0.000950,0.001469,0.002176,0.003199,0.004773,0.007413,0.012158,0.012680,0.008124,0.005577,0.004243,0.003652,0.003588,0.004011,0.005100,0.007116,0.009602,0.008448,0.005627,0.003915,0.002969,0.002529,0.002523,0.002225,0.001542,0.001115,0.000840,0.000647,0.000496,0.000369,0.000263,0.000176,0.000109
  giTenorEee33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.014340,0.021213,0.032104,0.055421,0.189690,0.120221,0.050550,0.026047,0.016045,0.010125,0.005723,0.003021,0.001308,0.000246,0.000324,0.000496,0.000471,0.000309,0.000160,0.000115,0.000295,0.000626,0.001166,0.001976,0.003173,0.005084,0.008393,0.014976,0.024026,0.015041,0.008440,0.005111,0.003182,0.001956,0.001114,0.000544,0.000160,0.000136,0.000453,0.000897,0.001549,0.002490,0.003935,0.006420,0.011227,0.016103,0.010451,0.006593,0.004861,0.004256,0.004516,0.005730,0.008419,0.011629,0.008791,0.005493,0.003779,0.003042,0.003014,0.002416,0.001563,0.001093,0.000802,0.000584,0.000408,0.000266,0.000158
  giTenorEee36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.018039,0.029693,0.059894,0.185737,0.154314,0.055778,0.027787,0.014854,0.007935,0.003897,0.001432,0.000187,0.000561,0.000590,0.000401,0.000185,0.000117,0.000134,0.000109,0.000405,0.000942,0.001848,0.003332,0.005831,0.010609,0.022023,0.025121,0.011959,0.006463,0.003691,0.002042,0.001000,0.000346,0.000540,0.001193,0.002221,0.003859,0.006735,0.012934,0.018831,0.010505,0.006536,0.005174,0.005305,0.007021,0.011338,0.013114,0.007641,0.004738,0.003619,0.003554,0.002391,0.001498,0.001024,0.000703,0.000458,0.000268,0.000136
  giTenorEee39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.023171,0.040165,0.103046,0.249899,0.069009,0.030296,0.014964,0.006837,0.002317,0.000262,0.000693,0.000670,0.000375,0.000134,0.000157,0.000234,0.000809,0.001919,0.003902,0.007563,0.015739,0.033909,0.016874,0.008020,0.004127,0.001993,0.000762,0.000560,0.001472,0.003025,0.005784,0.011760,0.022871,0.012484,0.007312,0.006012,0.007155,0.011847,0.015854,0.008455,0.005033,0.004257,0.003348,0.001866,0.001182,0.000751,0.000433,0.000208
  giTenorEee42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.030585,0.075466,0.326397,0.089842,0.037631,0.016293,0.006004,0.001189,0.000623,0.000819,0.000421,0.000187,0.000122,0.000220,0.000997,0.002690,0.006079,0.013783,0.037543,0.020537,0.008608,0.003873,0.001483,0.000243,0.000604,0.001913,0.004382,0.009617,0.024507,0.016361,0.008552,0.007239,0.010299,0.019396,0.011363,0.006002,0.005105,0.003117,0.001699,0.001004,0.000533,0.000219,0.000103,0.000108
  giTenorEee45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.042426,0.110844,0.240398,0.052093,0.020251,0.006042,0.000491,0.000993,0.000619,0.000230,0.000590,0.002336,0.006356,0.016818,0.048133,0.016906,0.006376,0.002233,0.000321,0.000910,0.003109,0.007901,0.022561,0.020975,0.009763,0.009076,0.016934,0.017670,0.007601,0.006066,0.003146,0.001615,0.000823,0.000318,0.000124,0.000127
  giTenorEee48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.059390,0.371533,0.111553,0.029708,0.007793,0.000375,0.001180,0.000370,0.000232,0.000146,0.000819,0.003706,0.011680,0.044106,0.023926,0.007382,0.001997,0.000198,0.002392,0.007734,0.025933,0.021044,0.010370,0.014081,0.026288,0.009501,0.007130,0.003002,0.001408,0.000534,0.000152,0.000151
  giTenorEee51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.080361,0.499941,0.060609,0.013679,0.000525,0.001340,0.000163,0.000315,0.000469,0.003843,0.015148,0.067904,0.016057,0.003990,0.002951,0.011593,0.045827,0.014651,0.014346,0.031778,0.010090,0.006709,0.002369,0.000868,0.000190,0.000173
  giTenorEee54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.150937,0.179680,0.032584,0.002376,0.001637,0.000158,0.000252,0.001981,0.012152,0.075113,0.017221,0.002972,0.001205,0.008768,0.049043,0.017107,0.020616,0.022725,0.010214,0.003400,0.001068,0.000208,0.000166
  giTenorEee57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.221694,0.104185,0.012082,0.001986,0.000188,0.000185,0.004673,0.033646,0.033807,0.004465,0.001822,0.015810,0.041944,0.018160,0.035337,0.012137,0.003231,0.000635,0.000256
  giTenorEee60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.743154,0.059418,0.000747,0.000739,0.000301,0.007400,0.088245,0.014776,0.000388,0.015476,0.042088,0.028181,0.019002,0.006011,0.001075,0.000301,0.000101
  giTenorEee63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.999999,0.027360,0.002680,0.000630,0.007690,0.135857,0.007982,0.005905,0.091694,0.028709,0.020187,0.004740,0.000381,0.000133
  giTenorEee66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.359360,0.004751,0.000315,0.003964,0.150241,0.005941,0.017540,0.034212,0.045442,0.006799,0.000416,0.000140
  giTenorEee69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.208351,0.003970,0.000371,0.067291,0.008928,0.031620,0.036318,0.024271,0.001270,0.000155
  giTenorEee72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.118830,0.001480,0.014801,0.029546,0.030951,0.056360,0.012020,0.000602
  giTenorEee75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.054723,0.001258,0.271741,0.011812,0.057430,0.009479,0.000266
  giTenorEee78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.009503,0.007927,0.011882,0.068422,0.013599,0.000280
  giTenorEee81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.007941,0.134597,0.063248,0.048544,0.000311
  giTenorEee84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.002957,0.059093,0.112719,0.001203,0.000131
 elseif iVoice==9 then
  giTenorOh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.002609,0.003212,0.004071,0.005593,0.007140,0.009447,0.013108,0.015876,0.025921,0.031520,0.069218,0.117217,0.097674,0.042519,0.031624,0.021238,0.017323,0.014353,0.012627,0.012174,0.012732,0.014614,0.019478,0.026308,0.024844,0.016400,0.010959,0.008002,0.005958,0.004623,0.003544,0.002734,0.002055,0.001508,0.001067,0.000705,0.000436,0.000235,0.000143,0.000144,0.000177,0.000192,0.000182,0.000161,0.000124,0.000104,0.000128,0.000145,0.000153,0.000156,0.000157,0.000181,0.000242,0.000356,0.000521,0.000741,0.001027,0.001380,0.001829,0.002381,0.003069,0.003965,0.005082,0.006691,0.008822,0.012074,0.015298,0.014009,0.009283,0.006092,0.006121,0.008720,0.012324,0.013604,0.011387,0.008379,0.005974,0.003793,0.001511,0.000709,0.001826,0.001937,0.001735,0.001456,0.001193,0.000965,0.000774,0.000620
  giTenorOh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.003203,0.004193,0.005787,0.008004,0.011069,0.016214,0.021222,0.035866,0.061952,0.136461,0.110996,0.041891,0.033083,0.021225,0.017671,0.014999,0.014528,0.015884,0.020089,0.030066,0.029796,0.018039,0.011760,0.008010,0.005921,0.004311,0.003164,0.002231,0.001534,0.000966,0.000557,0.000270,0.000164,0.000189,0.000228,0.000219,0.000192,0.000139,0.000133,0.000165,0.000180,0.000186,0.000188,0.000227,0.000336,0.000535,0.000821,0.001216,0.001726,0.002403,0.003262,0.004422,0.005950,0.008247,0.011486,0.016679,0.017937,0.011848,0.007184,0.007840,0.012354,0.016239,0.014304,0.010016,0.006636,0.003533,0.000306,0.002001,0.002325,0.002034,0.001635,0.001282,0.000989,0.000760,0.000584,0.000453,0.000355,0.000283,0.000227,0.000183,0.000149,0.000123,0.000103
  giTenorOh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.003960,0.005516,0.008444,0.012125,0.018936,0.027471,0.043879,0.124923,0.155727,0.053732,0.037654,0.024575,0.019046,0.017222,0.018408,0.024108,0.037434,0.030007,0.016142,0.010296,0.007092,0.004871,0.003350,0.002173,0.001311,0.000693,0.000293,0.000193,0.000251,0.000266,0.000236,0.000161,0.000147,0.000194,0.000215,0.000221,0.000238,0.000349,0.000604,0.001006,0.001590,0.002388,0.003477,0.004987,0.007107,0.010506,0.015883,0.021912,0.015844,0.008572,0.009907,0.016773,0.018799,0.013123,0.008157,0.003769,0.000813,0.002743,0.002542,0.001996,0.001496,0.001097,0.000802,0.000589,0.000439,0.000334,0.000257,0.000200,0.000157,0.000127,0.000107
  giTenorOh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.004986,0.007636,0.012153,0.020144,0.031323,0.054374,0.186452,0.119440,0.051970,0.029606,0.022330,0.020530,0.024026,0.039836,0.039006,0.018975,0.011161,0.007228,0.004658,0.002839,0.001584,0.000735,0.000266,0.000262,0.000323,0.000286,0.000181,0.000138,0.000212,0.000253,0.000263,0.000299,0.000510,0.000968,0.001707,0.002787,0.004353,0.006688,0.010300,0.016543,0.025924,0.017916,0.009680,0.015265,0.023074,0.016822,0.009587,0.003290,0.002408,0.003229,0.002528,0.001800,0.001245,0.000857,0.000598,0.000428,0.000313,0.000233,0.000176,0.000139,0.000117,0.000102
  giTenorOh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.006425,0.011190,0.018900,0.031763,0.063063,0.234533,0.085077,0.042498,0.028723,0.024364,0.029248,0.052657,0.032828,0.016019,0.009256,0.005473,0.003020,0.001411,0.000472,0.000289,0.000385,0.000324,0.000169,0.000209,0.000292,0.000313,0.000363,0.000715,0.001488,0.002773,0.004783,0.007968,0.013450,0.024276,0.028174,0.012256,0.017548,0.027382,0.016871,0.007639,0.001429,0.003903,0.002935,0.001946,0.001251,0.000808,0.000539,0.000371,0.000260,0.000189,0.000148,0.000125,0.000105
  giTenorOh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.008376,0.015990,0.032397,0.071647,0.272662,0.083705,0.042399,0.029965,0.031725,0.060042,0.036024,0.015995,0.008610,0.004457,0.001931,0.000539,0.000377,0.000438,0.000278,0.000188,0.000329,0.000372,0.000455,0.001070,0.002432,0.004805,0.008844,0.016499,0.033376,0.023715,0.015683,0.032489,0.020044,0.007073,0.004006,0.004073,0.002568,0.001523,0.000908,0.000566,0.000367,0.000246,0.000180,0.000147,0.000118
  giTenorOh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.011034,0.024255,0.054958,0.249960,0.107468,0.049168,0.034461,0.048258,0.060049,0.020609,0.009752,0.004351,0.001387,0.000382,0.000531,0.000321,0.000134,0.000183,0.000390,0.000442,0.000696,0.002019,0.004800,0.010033,0.021149,0.044106,0.017250,0.033814,0.026431,0.007582,0.005537,0.004027,0.002216,0.001191,0.000676,0.000405,0.000256,0.000192,0.000151,0.000111
  giTenorOh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.015273,0.040290,0.108755,0.238831,0.059213,0.041067,0.079700,0.037954,0.014459,0.005681,0.001470,0.000525,0.000572,0.000143,0.000149,0.000276,0.000507,0.000599,0.001942,0.005592,0.013421,0.033213,0.035928,0.030674,0.033762,0.006591,0.006489,0.003618,0.001724,0.000862,0.000468,0.000280,0.000206,0.000147,0.000101
  giTenorOh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.022381,0.063531,0.469122,0.084990,0.048733,0.105348,0.032042,0.010949,0.002821,0.000574,0.000647,0.000166,0.000185,0.000584,0.000720,0.002976,0.009583,0.026955,0.056424,0.035183,0.033796,0.002879,0.005885,0.002510,0.001081,0.000521,0.000299,0.000213,0.000130
  giTenorOh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.031994,0.143365,0.167462,0.059961,0.120183,0.032013,0.008919,0.001080,0.000876,0.000112,0.000190,0.000377,0.000745,0.002145,0.009630,0.033066,0.047478,0.065117,0.014161,0.008162,0.003051,0.001136,0.000494,0.000295,0.000181,0.000121
  giTenorOh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.048511,0.499973,0.098337,0.096526,0.041209,0.008700,0.000772,0.000645,0.000267,0.000365,0.000886,0.004049,0.020077,0.088236,0.067669,0.015154,0.008053,0.002378,0.000809,0.000383,0.000218,0.000140
  giTenorOh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.080583,0.477600,0.082135,0.075901,0.011361,0.001051,0.000287,0.001014,0.003886,0.026853,0.071839,0.067519,0.012982,0.003449,0.000937,0.000412,0.000202,0.000131
  giTenorOh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.127073,0.169977,0.210725,0.021898,0.001155,0.000367,0.001455,0.019176,0.112848,0.067589,0.011765,0.002161,0.000595,0.000260,0.000153
  giTenorOh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.286787,0.119943,0.064037,0.002159,0.000224,0.000754,0.004292,0.066166,0.130304,0.016330,0.002272,0.000591,0.000243,0.000124
  giTenorOh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000000,0.193060,0.017402,0.001290,0.000730,0.008099,0.176475,0.030291,0.004757,0.000766,0.000282,0.000138
  giTenorOh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.955092,0.151776,0.002102,0.000192,0.007772,0.143628,0.025957,0.001874,0.000402,0.000171
  giTenorOh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.339931,0.043789,0.000154,0.002909,0.225646,0.023525,0.001190,0.000305,0.000120
  giTenorOh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.239903,0.004318,0.001511,0.132362,0.032662,0.001182,0.000249
  giTenorOh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.386114,0.002580,0.016199,0.060581,0.001531,0.000277
  giTenorOh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.303561,0.000384,0.287242,0.003746,0.000342
  giTenorOh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.087577,0.005818,0.047050,0.000610,0.000103
 elseif iVoice==10 then
  giTenorOoh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.004160,0.004756,0.005796,0.007110,0.009404,0.012319,0.016588,0.025253,0.036732,0.084172,0.097499,0.055655,0.026840,0.021900,0.014064,0.011724,0.009450,0.006251,0.002854,0.002079,0.001522,0.001320,0.001012,0.000844,0.000726,0.000657,0.000578,0.000532,0.000449,0.000378,0.000306,0.000221,0.000161,0.000112,0.000145,0.000197,0.000271,0.000371,0.000502,0.000666,0.000873,0.001133,0.001454,0.001875,0.002392,0.003125,0.004070,0.005451,0.006740,0.005949,0.003815,0.003032,0.004216,0.006465,0.009166,0.010218,0.008738,0.006687,0.005210,0.004127,0.003395,0.002882,0.002550,0.002381,0.002374,0.002539,0.002842
  giTenorOoh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.005018,0.006177,0.007634,0.010351,0.014548,0.021399,0.030750,0.071843,0.119517,0.072786,0.031652,0.022635,0.015612,0.012023,0.008250,0.003348,0.002272,0.001786,0.001280,0.001064,0.000866,0.000770,0.000661,0.000586,0.000473,0.000372,0.000254,0.000169,0.000102,0.000123,0.000163,0.000233,0.000341,0.000493,0.000700,0.000970,0.001326,0.001786,0.002410,0.003241,0.004476,0.006249,0.008081,0.006509,0.003821,0.004319,0.007145,0.010967,0.012075,0.009457,0.006928,0.005206,0.004101,0.003382,0.002964,0.002823,0.002955,0.003370,0.003360,0.002535,0.001765,0.001296,0.000988,0.000767,0.000592,0.000451,0.000332,0.000236,0.000160,0.000105
  giTenorOoh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.006069,0.007985,0.010689,0.016347,0.024478,0.037411,0.112759,0.124998,0.041485,0.028849,0.018264,0.013449,0.006554,0.003139,0.002141,0.001509,0.001181,0.000981,0.000833,0.000728,0.000565,0.000421,0.000265,0.000141,0.000101,0.000117,0.000142,0.000198,0.000307,0.000479,0.000736,0.001090,0.001584,0.002260,0.003208,0.004668,0.006836,0.009554,0.007100,0.004250,0.006687,0.011831,0.014395,0.010804,0.007515,0.005449,0.004235,0.003554,0.003339,0.003592,0.004112,0.003422,0.002223,0.001532,0.001113,0.000822,0.000601,0.000424,0.000284,0.000179,0.000109
  giTenorOoh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.007484,0.010358,0.016064,0.024189,0.043878,0.147795,0.113259,0.043500,0.023110,0.016473,0.007061,0.003259,0.002323,0.001591,0.001196,0.000986,0.000820,0.000603,0.000378,0.000199,0.000110,0.000113,0.000104,0.000123,0.000148,0.000202,0.000327,0.000558,0.000928,0.001473,0.002265,0.003459,0.005340,0.008451,0.011341,0.006178,0.006471,0.013053,0.017137,0.011865,0.007810,0.005560,0.004372,0.003989,0.004406,0.004836,0.003238,0.002015,0.001363,0.000950,0.000651,0.000420,0.000250,0.000139
  giTenorOoh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.009514,0.014225,0.024646,0.050523,0.168409,0.111356,0.043820,0.023460,0.012509,0.004160,0.002642,0.001689,0.001315,0.001065,0.000757,0.000443,0.000192,0.000123,0.000137,0.000101,0.000120,0.000148,0.000186,0.000291,0.000545,0.001008,0.001755,0.002924,0.004809,0.008185,0.013559,0.007678,0.008486,0.018454,0.017597,0.010496,0.006842,0.005140,0.004787,0.005732,0.004640,0.002578,0.001610,0.001047,0.000655,0.000372,0.000189,0.000109,0.000102
  giTenorOoh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.012343,0.020687,0.042780,0.143593,0.145481,0.045253,0.024030,0.006681,0.003567,0.002127,0.001539,0.001172,0.000743,0.000337,0.000147,0.000158,0.000139,0.000180,0.000245,0.000466,0.000987,0.001939,0.003573,0.006484,0.012507,0.013031,0.008641,0.021946,0.018929,0.010423,0.006772,0.005654,0.006751,0.005079,0.002597,0.001537,0.000903,0.000473,0.000212,0.000124,0.000121
  giTenorOoh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.015973,0.032700,0.074843,0.250024,0.057702,0.026906,0.006281,0.003020,0.001964,0.001456,0.000842,0.000281,0.000120,0.000190,0.000125,0.000117,0.000172,0.000230,0.000397,0.000965,0.002195,0.004549,0.009401,0.019240,0.008562,0.023865,0.021772,0.010988,0.007172,0.007255,0.006908,0.003095,0.001662,0.000857,0.000363,0.000158,0.000146,0.000111
  giTenorOoh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.020715,0.048381,0.295623,0.086997,0.032947,0.006518,0.003184,0.001972,0.001207,0.000399,0.000165,0.000225,0.000166,0.000247,0.000405,0.001120,0.002956,0.006943,0.016973,0.012394,0.026234,0.023823,0.011169,0.008018,0.009723,0.004053,0.001912,0.000846,0.000280,0.000172,0.000144
  giTenorOoh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.028452,0.101054,0.222679,0.046921,0.008320,0.003378,0.002130,0.000886,0.000146,0.000269,0.000181,0.000290,0.000582,0.002022,0.005860,0.016406,0.015371,0.037003,0.021032,0.010304,0.011498,0.005167,0.002100,0.000748,0.000221,0.000191
  giTenorOoh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.041389,0.287313,0.090518,0.013371,0.004256,0.002343,0.000674,0.000295,0.000189,0.000130,0.000278,0.000491,0.001979,0.007162,0.025068,0.017327,0.037924,0.013572,0.013536,0.005205,0.001811,0.000424,0.000243,0.000123
  giTenorOoh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.065400,0.500015,0.053810,0.006040,0.002913,0.000562,0.000387,0.000141,0.000350,0.000798,0.004390,0.018813,0.017129,0.043544,0.014348,0.013820,0.003325,0.000724,0.000290,0.000124
  giTenorOoh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.096766,0.173992,0.013034,0.003944,0.000798,0.000451,0.000332,0.000810,0.005915,0.033962,0.052501,0.022340,0.019449,0.003824,0.000560,0.000287,0.000114
  giTenorOoh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.202127,0.093846,0.006755,0.001771,0.000549,0.000186,0.000594,0.004041,0.032826,0.074042,0.020610,0.010337,0.001493,0.000384,0.000136
  giTenorOoh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.574725,0.026750,0.004688,0.000591,0.000260,0.000983,0.014330,0.034680,0.027154,0.010414,0.000848,0.000246,0.000117
  giTenorOoh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.999999,0.012080,0.001126,0.000101,0.000282,0.001597,0.037631,0.087072,0.027634,0.001448,0.000249
  giTenorOoh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.347954,0.007887,0.000902,0.000132,0.001620,0.067927,0.044669,0.007647,0.000575,0.000129
  giTenorOoh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.187680,0.003542,0.000182,0.001188,0.065653,0.041213,0.002984,0.000272
  giTenorOoh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.053502,0.001181,0.000524,0.028666,0.054314,0.001696,0.000234
  giTenorOoh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.024159,0.000201,0.003194,0.174139,0.002895,0.000179
  giTenorOoh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.015774,0.000264,0.135872,0.015294,0.000258
  giTenorOoh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.007082,0.002375,0.082425,0.000545
 /* COUNTERTENOR */
 elseif iVoice==11 then
  giCountertenorAh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.000622,0.000579,0.000505,0.000416,0.000404,0.000563,0.000935,0.001450,0.002158,0.003063,0.004178,0.005657,0.007437,0.009784,0.012967,0.016948,0.023788,0.032799,0.051846,0.069434,0.060781,0.039066,0.027913,0.020569,0.016254,0.013456,0.011669,0.010899,0.010953,0.011875,0.014299,0.018107,0.025757,0.032911,0.029411,0.019830,0.013946,0.010125,0.007600,0.005820,0.004423,0.003377,0.002523,0.001848,0.001319,0.000899,0.000595,0.000380,0.000265,0.000228,0.000227,0.000228,0.000214,0.000192,0.000160,0.000127,0.000135,0.000199,0.000281,0.000383,0.000508,0.000661,0.000851,0.001091,0.001400,0.001829,0.002402,0.003168,0.003603,0.003179,0.002384,0.001859,0.001660,0.001771,0.002195,0.002778,0.002998,0.002611,0.002049,0.001621,0.001304,0.001084,0.000925,0.000816,0.000745
  giCountertenorAh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.000735,0.000662,0.000536,0.000470,0.000654,0.001187,0.001971,0.003095,0.004543,0.006527,0.009051,0.012579,0.017418,0.025067,0.036897,0.062520,0.083382,0.061395,0.037338,0.026171,0.019458,0.015588,0.013449,0.012898,0.013720,0.016734,0.022266,0.033995,0.038925,0.027154,0.017317,0.011854,0.008501,0.006153,0.004459,0.003184,0.002202,0.001467,0.000918,0.000551,0.000338,0.000273,0.000271,0.000267,0.000243,0.000203,0.000155,0.000111,0.000121,0.000200,0.000308,0.000447,0.000626,0.000852,0.001149,0.001540,0.002109,0.002917,0.003991,0.004205,0.003195,0.002308,0.001985,0.002187,0.002895,0.003567,0.003282,0.002484,0.001880,0.001465,0.001192,0.001010,0.000899,0.000836,0.000741,0.000567,0.000414,0.000324,0.000266,0.000223,0.000186,0.000152,0.000121
  giCountertenorAh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.000859,0.000737,0.000570,0.000685,0.001368,0.002498,0.004192,0.006507,0.009781,0.014407,0.021215,0.033314,0.054502,0.095648,0.079270,0.043918,0.028744,0.021058,0.016749,0.015325,0.016241,0.020739,0.030451,0.046307,0.036821,0.021184,0.013513,0.009191,0.006237,0.004238,0.002750,0.001701,0.000967,0.000518,0.000340,0.000320,0.000317,0.000281,0.000219,0.000154,0.000105,0.000201,0.000341,0.000534,0.000794,0.001139,0.001619,0.002329,0.003422,0.004882,0.004650,0.003126,0.002393,0.002545,0.003536,0.004253,0.003418,0.002429,0.001787,0.001395,0.001156,0.001026,0.000931,0.000712,0.000489,0.000368,0.000293,0.000237,0.000189,0.000145,0.000106
  giCountertenorAh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.001001,0.000790,0.000685,0.001374,0.002883,0.005307,0.008904,0.014153,0.022631,0.037604,0.068470,0.117881,0.069973,0.039907,0.026187,0.019843,0.018237,0.020805,0.030099,0.052526,0.044464,0.023490,0.013922,0.008867,0.005619,0.003393,0.001911,0.000964,0.000476,0.000379,0.000381,0.000331,0.000241,0.000152,0.000120,0.000258,0.000474,0.000785,0.001223,0.001861,0.002867,0.004562,0.006080,0.004315,0.002924,0.003059,0.004530,0.004842,0.003317,0.002269,0.001676,0.001348,0.001192,0.000989,0.000643,0.000449,0.000343,0.000266,0.000200,0.000142
  giCountertenorAh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.001159,0.000833,0.001127,0.002901,0.006128,0.011319,0.019577,0.033913,0.065634,0.138955,0.078186,0.041169,0.026935,0.021819,0.023774,0.036256,0.065905,0.039715,0.020280,0.011658,0.006765,0.003703,0.001801,0.000761,0.000458,0.000456,0.000385,0.000254,0.000139,0.000103,0.000271,0.000566,0.001022,0.001711,0.002815,0.004833,0.007250,0.004799,0.003342,0.004420,0.006040,0.004129,0.002629,0.001866,0.001503,0.001298,0.000824,0.000531,0.000388,0.000284,0.000196,0.000121
  giCountertenorAh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.001323,0.000939,0.002370,0.006179,0.013031,0.025116,0.050054,0.124839,0.122598,0.052264,0.031128,0.025756,0.033416,0.067885,0.054229,0.023674,0.012290,0.006360,0.002931,0.001101,0.000546,0.000534,0.000406,0.000222,0.000118,0.000103,0.000242,0.000615,0.001251,0.002297,0.004218,0.007984,0.006394,0.003973,0.005794,0.006570,0.003764,0.002387,0.001801,0.001484,0.000829,0.000532,0.000372,0.000243,0.000138
  giCountertenorAh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.001473,0.001369,0.004998,0.013019,0.028826,0.066670,0.191435,0.087882,0.042145,0.030678,0.041530,0.092746,0.042423,0.018412,0.008493,0.003408,0.001039,0.000642,0.000564,0.000309,0.000143,0.000118,0.000210,0.000683,0.001594,0.003257,0.006894,0.009365,0.004822,0.007137,0.006892,0.003605,0.002334,0.001880,0.000988,0.000593,0.000383,0.000215
  giCountertenorAh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.001580,0.002749,0.010614,0.028309,0.075224,0.235778,0.079815,0.039694,0.041631,0.105124,0.046998,0.017743,0.006790,0.001929,0.000758,0.000664,0.000304,0.000154,0.000113,0.000242,0.000953,0.002455,0.005759,0.012206,0.005872,0.009102,0.006662,0.003369,0.002397,0.001292,0.000690,0.000403,0.000185
  giCountertenorAh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.001665,0.005802,0.022639,0.067838,0.277959,0.082343,0.043643,0.072542,0.079439,0.023326,0.007411,0.001524,0.000914,0.000509,0.000186,0.000132,0.000538,0.002041,0.005636,0.014527,0.006699,0.012109,0.005270,0.003013,0.001652,0.000778,0.000392,0.000133
  giCountertenorAh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.001879,0.012367,0.050272,0.249889,0.104596,0.051557,0.135917,0.047385,0.012730,0.002203,0.001070,0.000444,0.000205,0.000486,0.002508,0.008455,0.012808,0.011617,0.007544,0.003612,0.001661,0.000745,0.000277
  giCountertenorAh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.002739,0.026042,0.133355,0.175744,0.061363,0.185512,0.036813,0.006813,0.001283,0.000619,0.000238,0.000104,0.001374,0.006525,0.018731,0.014280,0.007212,0.003761,0.001187,0.000430,0.000106
  giCountertenorAh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.005498,0.056620,0.471547,0.079386,0.210272,0.035484,0.003858,0.001328,0.000308,0.000117,0.001906,0.011522,0.011745,0.013323,0.004796,0.001380,0.000371,0.000133
  giCountertenorAh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.011607,0.135706,0.164696,0.145131,0.046646,0.003046,0.001018,0.000264,0.001087,0.011288,0.013399,0.010542,0.003304,0.000785,0.000154
  giCountertenorAh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.024741,0.499940,0.103142,0.094789,0.004409,0.000888,0.000177,0.005018,0.025621,0.015091,0.003323,0.000554,0.000160
  giCountertenorAh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.052086,0.351464,0.371032,0.013625,0.001238,0.000209,0.013050,0.028566,0.007522,0.000861,0.000171
  giCountertenorAh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.113230,0.158750,0.070954,0.002655,0.000236,0.023043,0.026638,0.002759,0.000266
  giCountertenorAh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.271393,0.290247,0.006089,0.000528,0.022576,0.021078,0.001570,0.000118
  giCountertenorAh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,1.000001,0.189594,0.001776,0.010036,0.030183,0.001107,0.000150
  giCountertenorAh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.702916,0.027248,0.000418,0.057129,0.001722,0.000133
  giCountertenorAh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.317514,0.005311,0.046093,0.005518,0.000190
  giCountertenorAh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.580494,0.001056,0.042157,0.000235
 elseif iVoice==12 then
  giCountertenorEh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.001407,0.001909,0.002654,0.003601,0.004888,0.006467,0.008499,0.011368,0.014805,0.020766,0.028754,0.045419,0.070436,0.068915,0.043266,0.028132,0.020165,0.014552,0.011119,0.008287,0.006270,0.004661,0.003363,0.002384,0.001583,0.000990,0.000554,0.000266,0.000187,0.000237,0.000286,0.000284,0.000259,0.000207,0.000143,0.000115,0.000212,0.000361,0.000554,0.000797,0.001116,0.001495,0.001997,0.002660,0.003508,0.004943,0.006866,0.010908,0.013782,0.011286,0.007103,0.005073,0.003604,0.002726,0.002060,0.001553,0.001173,0.000855,0.000607,0.000405,0.000237,0.000102,0.000142,0.000257,0.000388,0.000538,0.000722,0.000944,0.001216,0.001571,0.002018,0.002669,0.003593,0.005060,0.006932,0.007192,0.005615,0.004200,0.003465,0.003069,0.003039,0.003286,0.003878,0.004703,0.004898,0.004167,0.003337,0.002887,0.002710,0.002818,0.003204,0.003908,0.004910
  giCountertenorEh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.001760,0.002580,0.003766,0.005400,0.007590,0.010513,0.014826,0.020586,0.031392,0.050841,0.085318,0.074894,0.040539,0.027387,0.018076,0.013133,0.009279,0.006665,0.004564,0.003073,0.001908,0.001086,0.000511,0.000236,0.000261,0.000339,0.000336,0.000293,0.000209,0.000123,0.000106,0.000226,0.000430,0.000709,0.001084,0.001575,0.002218,0.003126,0.004372,0.006522,0.010363,0.016022,0.014016,0.008187,0.005534,0.003746,0.002718,0.001937,0.001388,0.000948,0.000622,0.000358,0.000154,0.000191,0.000360,0.000557,0.000798,0.001104,0.001493,0.002025,0.002744,0.003904,0.005749,0.008373,0.008259,0.005844,0.004423,0.003722,0.003634,0.004027,0.005025,0.005913,0.005254,0.004033,0.003408,0.003246,0.003572,0.004377,0.005760,0.006334,0.005015,0.003516,0.002533,0.001874,0.001408,0.001059,0.000788,0.000573,0.000405,0.000272,0.000175,0.000109
  giCountertenorEh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.002230,0.003552,0.005479,0.008344,0.012266,0.018381,0.028255,0.045291,0.091892,0.093320,0.046217,0.028853,0.018680,0.012419,0.008371,0.005365,0.003320,0.001818,0.000856,0.000315,0.000307,0.000407,0.000387,0.000299,0.000173,0.000214,0.000472,0.000862,0.001419,0.002166,0.003268,0.004826,0.007739,0.013719,0.019452,0.012299,0.007218,0.004524,0.003078,0.002064,0.001362,0.000843,0.000456,0.000166,0.000314,0.000574,0.000894,0.001319,0.001890,0.002704,0.004006,0.006196,0.009856,0.009370,0.006160,0.004704,0.004267,0.004706,0.006105,0.006965,0.005453,0.004205,0.003832,0.004261,0.005524,0.007380,0.006685,0.004432,0.002994,0.002099,0.001494,0.001061,0.000733,0.000488,0.000304,0.000177,0.000104
  giCountertenorEh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.002887,0.005002,0.008336,0.013250,0.021432,0.036173,0.067184,0.125012,0.067454,0.036388,0.021564,0.013251,0.008195,0.004726,0.002405,0.000993,0.000329,0.000429,0.000479,0.000385,0.000209,0.000200,0.000531,0.001087,0.001932,0.003147,0.005052,0.008695,0.016946,0.022167,0.011132,0.006361,0.003959,0.002459,0.001486,0.000816,0.000333,0.000360,0.000731,0.001217,0.001892,0.002907,0.004565,0.007708,0.012368,0.009215,0.006087,0.005095,0.005647,0.007722,0.007793,0.005459,0.004595,0.005065,0.006975,0.009015,0.006440,0.003971,0.002578,0.001720,0.001140,0.000724,0.000427,0.000229,0.000121,0.000102
  giCountertenorEh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.003820,0.007205,0.012939,0.022744,0.041548,0.090875,0.137894,0.056293,0.029121,0.016585,0.009329,0.004772,0.001982,0.000532,0.000474,0.000568,0.000415,0.000167,0.000107,0.000231,0.000723,0.001597,0.002997,0.005333,0.009912,0.021876,0.022637,0.010177,0.005470,0.003116,0.001715,0.000813,0.000206,0.000285,0.000781,0.001450,0.002444,0.004058,0.007225,0.013945,0.011299,0.006974,0.006118,0.007810,0.009866,0.006724,0.005463,0.006461,0.009905,0.009117,0.005103,0.003040,0.001880,0.001137,0.000638,0.000315,0.000147,0.000119,0.000122,0.000104
  giCountertenorEh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.005152,0.010784,0.020993,0.041111,0.101534,0.149567,0.054696,0.026229,0.013311,0.006139,0.002170,0.000472,0.000676,0.000586,0.000246,0.000127,0.000452,0.001417,0.003147,0.006248,0.013035,0.032027,0.016366,0.007490,0.003873,0.001896,0.000717,0.000720,0.001596,0.002988,0.005491,0.011506,0.016531,0.008854,0.007274,0.010059,0.010519,0.006824,0.007155,0.011539,0.010048,0.005075,0.002823,0.001580,0.000811,0.000351,0.000153,0.000145,0.000134
  giCountertenorEh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.007108,0.016693,0.036775,0.090623,0.186687,0.057721,0.024848,0.010735,0.003637,0.000631,0.000815,0.000597,0.000102,0.000150,0.000433,0.001732,0.004346,0.009685,0.027540,0.024661,0.009079,0.004147,0.001697,0.000337,0.000629,0.001796,0.003803,0.008068,0.019855,0.012401,0.008599,0.012317,0.010998,0.007737,0.011164,0.013499,0.006048,0.003019,0.001484,0.000615,0.000211,0.000173,0.000156
  giCountertenorEh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.010002,0.026500,0.072350,0.250024,0.072769,0.026502,0.009452,0.001985,0.000858,0.000771,0.000180,0.001063,0.003870,0.010120,0.033965,0.022292,0.007930,0.002977,0.000666,0.000722,0.002442,0.005835,0.015483,0.018490,0.010231,0.015520,0.010969,0.010186,0.018128,0.007983,0.003461,0.001456,0.000460,0.000199,0.000194,0.000107
  giCountertenorEh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.014412,0.045494,0.181791,0.112588,0.033171,0.009544,0.001064,0.001135,0.000329,0.000220,0.001449,0.006000,0.019846,0.045292,0.010951,0.003439,0.000417,0.001563,0.004898,0.014483,0.022627,0.012262,0.019775,0.010955,0.019872,0.010229,0.003771,0.001279,0.000296,0.000245,0.000142
  giCountertenorEh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.021582,0.082278,0.299289,0.052488,0.012284,0.000944,0.001172,0.000108,0.000906,0.006303,0.026106,0.032757,0.007753,0.001434,0.001444,0.005988,0.023064,0.017739,0.020167,0.013678,0.023142,0.010173,0.003168,0.000704,0.000290,0.000184
  giCountertenorEh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.033385,0.181253,0.115431,0.021468,0.001260,0.001196,0.000294,0.003460,0.019374,0.049307,0.008286,0.000663,0.003599,0.016148,0.024802,0.024651,0.015477,0.027005,0.006039,0.001230,0.000344,0.000180,0.000106
  giCountertenorEh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.053000,0.500048,0.052999,0.003969,0.001543,0.000360,0.007742,0.067956,0.015859,0.001332,0.004887,0.030983,0.020466,0.021939,0.036267,0.006921,0.000920,0.000388,0.000110
  giCountertenorEh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.090997,0.225190,0.019088,0.002273,0.000427,0.012001,0.090580,0.006866,0.003130,0.028980,0.024535,0.021910,0.020460,0.002558,0.000490,0.000131
  giCountertenorEh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.164597,0.104995,0.001890,0.000217,0.012610,0.065521,0.002869,0.011981,0.035487,0.027366,0.020351,0.001408,0.000368,0.000158
  giCountertenorEh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.362512,0.042934,0.002393,0.006921,0.098595,0.001326,0.032300,0.049308,0.054000,0.002460,0.000362,0.000116
  giCountertenorEh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.999999,0.007938,0.000720,0.135918,0.002663,0.061969,0.043865,0.013840,0.000774,0.000180
  giCountertenorEh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.450330,0.004546,0.024003,0.013728,0.057960,0.043814,0.005114,0.000263,0.000126
  giCountertenorEh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.210003,0.000432,0.131045,0.023965,0.054738,0.002817,0.000316
  giCountertenorEh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.085867,0.013843,0.002652,0.098614,0.004920,0.000232
  giCountertenorEh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.015875,0.271870,0.123957,0.027679,0.000360
  giCountertenorEh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.009092,0.027457,0.087628,0.000525
 elseif iVoice==13 then
  giCountertenorEee24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.008519,0.010447,0.014054,0.016869,0.027416,0.033625,0.072566,0.124934,0.105409,0.045534,0.032582,0.021052,0.015918,0.011883,0.008760,0.006708,0.004852,0.003497,0.002443,0.001561,0.000949,0.000462,0.000135,0.000131,0.000257,0.000327,0.000331,0.000315,0.000264,0.000201,0.000142,0.000104,0.000153,0.000219,0.000301,0.000411,0.000551,0.000728,0.000969,0.001276,0.001735,0.002395,0.003484,0.004988,0.005127,0.003704,0.002506,0.001822,0.001332,0.001016,0.000765,0.000579,0.000431,0.000311,0.000217,0.000141,0.000129,0.000204,0.000297,0.000417,0.000566,0.000751,0.000993,0.001302,0.001740,0.002374,0.003337,0.004655,0.004963,0.003852,0.002706,0.002017,0.001532,0.001224,0.000999,0.000848,0.000756,0.000711,0.000725,0.000791
  giCountertenorEee27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.010469,0.013785,0.018825,0.028603,0.039687,0.096960,0.152046,0.086764,0.039736,0.026663,0.018685,0.012926,0.009150,0.006583,0.004367,0.002875,0.001660,0.000901,0.000280,0.000124,0.000303,0.000390,0.000394,0.000348,0.000255,0.000176,0.000123,0.000196,0.000297,0.000428,0.000613,0.000855,0.001201,0.001667,0.002454,0.003667,0.005797,0.006023,0.003933,0.002587,0.001760,0.001267,0.000906,0.000649,0.000452,0.000302,0.000186,0.000179,0.000295,0.000448,0.000651,0.000911,0.001270,0.001753,0.002529,0.003719,0.005568,0.005777,0.004005,0.002734,0.001948,0.001480,0.001166,0.000973,0.000869,0.000856,0.000935,0.001116,0.001223,0.000990,0.000701,0.000607,0.000678,0.000910,0.001178,0.001113,0.000812,0.000589,0.000435,0.000331,0.000252,0.000192,0.000143
  giCountertenorEee30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.012961,0.018883,0.025737,0.046165,0.109238,0.177786,0.070937,0.042673,0.023396,0.016018,0.010600,0.006942,0.004369,0.002442,0.001200,0.000319,0.000192,0.000435,0.000467,0.000430,0.000306,0.000189,0.000101,0.000137,0.000238,0.000385,0.000597,0.000894,0.001336,0.001977,0.003125,0.005266,0.007527,0.005281,0.003131,0.001989,0.001349,0.000906,0.000600,0.000374,0.000211,0.000133,0.000263,0.000450,0.000714,0.001069,0.001585,0.002351,0.003649,0.006036,0.006943,0.004523,0.002921,0.002003,0.001483,0.001174,0.001024,0.001026,0.001183,0.001440,0.001243,0.000829,0.000719,0.000881,0.001279,0.001377,0.000977,0.000667,0.000469,0.000340,0.000246,0.000175,0.000119
  giCountertenorEee33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.016304,0.025781,0.047131,0.093856,0.213577,0.070728,0.040472,0.024108,0.014169,0.008749,0.004988,0.002391,0.000889,0.000136,0.000481,0.000556,0.000488,0.000316,0.000133,0.000122,0.000156,0.000299,0.000521,0.000870,0.001395,0.002231,0.003813,0.007216,0.008330,0.004423,0.002570,0.001607,0.001002,0.000602,0.000328,0.000143,0.000109,0.000260,0.000510,0.000882,0.001420,0.002252,0.003738,0.006675,0.008239,0.004852,0.002954,0.001992,0.001464,0.001227,0.001245,0.001533,0.001684,0.001095,0.000857,0.001094,0.001647,0.001436,0.000900,0.000587,0.000399,0.000272,0.000179,0.000111
  giCountertenorEee36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.020901,0.033747,0.067272,0.249956,0.091103,0.042121,0.023777,0.013422,0.006998,0.003123,0.000925,0.000261,0.000655,0.000630,0.000402,0.000153,0.000132,0.000136,0.000207,0.000439,0.000824,0.001459,0.002557,0.004804,0.010004,0.007430,0.003657,0.002038,0.001162,0.000624,0.000283,0.000156,0.000410,0.000838,0.001510,0.002619,0.004778,0.009373,0.007757,0.004064,0.002466,0.001710,0.001434,0.001595,0.002067,0.001417,0.001024,0.001442,0.002020,0.001289,0.000763,0.000481,0.000304,0.000181
  giCountertenorEee39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.027550,0.057179,0.193782,0.173420,0.053305,0.025836,0.013158,0.005747,0.001800,0.000247,0.000779,0.000697,0.000352,0.000147,0.000159,0.000139,0.000391,0.000854,0.001707,0.003330,0.007329,0.012038,0.005171,0.002532,0.001299,0.000603,0.000200,0.000196,0.000590,0.001302,0.002541,0.005062,0.011149,0.008021,0.003901,0.002335,0.001741,0.001874,0.002450,0.001406,0.001360,0.002362,0.001629,0.000873,0.000506,0.000288,0.000146
  giCountertenorEee42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.037773,0.092350,0.355634,0.085357,0.032041,0.013887,0.004884,0.000639,0.000872,0.000861,0.000379,0.000202,0.000130,0.000134,0.000483,0.001202,0.002686,0.006275,0.015101,0.006277,0.002705,0.001203,0.000420,0.000268,0.000907,0.002154,0.004740,0.012176,0.009112,0.004038,0.002369,0.002070,0.002909,0.001676,0.001786,0.002785,0.001348,0.000686,0.000353,0.000154
  giCountertenorEee45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.051559,0.187745,0.141432,0.048215,0.017498,0.004781,0.000273,0.001114,0.000633,0.000244,0.000108,0.000313,0.001044,0.002794,0.007641,0.016682,0.005148,0.002007,0.000658,0.000106,0.000523,0.001771,0.004523,0.013415,0.009741,0.003999,0.002464,0.003082,0.002203,0.002205,0.002887,0.001181,0.000547,0.000222
  giCountertenorEee48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.067500,0.499957,0.084236,0.026845,0.006246,0.000524,0.001260,0.000307,0.000263,0.000154,0.000423,0.001657,0.005126,0.020030,0.007311,0.002320,0.000560,0.000313,0.001681,0.005251,0.018793,0.008143,0.003428,0.003199,0.002838,0.002897,0.002582,0.000962,0.000359
  giCountertenorEee51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.114392,0.346891,0.051686,0.011497,0.000495,0.001393,0.000319,0.000278,0.001711,0.006671,0.024100,0.005070,0.001207,0.000395,0.002611,0.010148,0.016063,0.004678,0.003757,0.002818,0.004738,0.001749,0.000577,0.000121
  giCountertenorEee54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.184699,0.170704,0.027773,0.001277,0.001722,0.000135,0.000260,0.000954,0.005364,0.030213,0.005416,0.000846,0.000161,0.001816,0.009486,0.018223,0.004741,0.005826,0.003571,0.002699,0.000709,0.000111
  giCountertenorEee57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.375525,0.096426,0.009561,0.002227,0.000108,0.000215,0.002089,0.015287,0.010294,0.001315,0.000212,0.003544,0.026847,0.007999,0.006168,0.004412,0.002362,0.000444,0.000109
  giCountertenorEee60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,1.000001,0.053693,0.001047,0.000612,0.000311,0.003299,0.040074,0.004652,0.000183,0.003363,0.037603,0.006857,0.005682,0.005170,0.000725,0.000129
  giCountertenorEee63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.693800,0.022994,0.002786,0.000638,0.003424,0.048210,0.002415,0.000789,0.020308,0.009360,0.005636,0.003499,0.000242
  giCountertenorEee66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.341398,0.002553,0.000269,0.001909,0.060427,0.001690,0.003632,0.036438,0.011652,0.005397,0.000221
  giCountertenorEee69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.192837,0.004454,0.000431,0.030573,0.002629,0.007088,0.015996,0.008825,0.000889
  giCountertenorEee72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.107381,0.001225,0.006599,0.009301,0.006726,0.013712,0.010336,0.000259
  giCountertenorEee75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.045989,0.001278,0.096423,0.001577,0.018720,0.006999,0.000144
  giCountertenorEee78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.005108,0.003817,0.003379,0.072873,0.010794
  giCountertenorEee81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.008908,0.061152,0.014177,0.017652,0.000134
  giCountertenorEee84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.002448,0.018603,0.027424,0.000518
 elseif iVoice==14 then
  giCountertenorOh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.000976,0.001418,0.001996,0.002649,0.003721,0.004755,0.006289,0.008700,0.010470,0.017279,0.020680,0.047585,0.075931,0.059717,0.025499,0.020090,0.013249,0.011161,0.009282,0.008320,0.008180,0.008813,0.010336,0.014497,0.017321,0.013886,0.008690,0.006221,0.004488,0.003467,0.002671,0.002061,0.001592,0.001183,0.000868,0.000603,0.000392,0.000239,0.000128,0.000106,0.000126,0.000132,0.000124,0.000110,0.000135,0.000180,0.000235,0.000301,0.000381,0.000484,0.000612,0.000797,0.001059,0.001479,0.002029,0.002140,0.001753,0.001427,0.001336,0.001376,0.001574,0.001924,0.002473,0.003150,0.003321,0.002752,0.002057,0.001579,0.001239,0.001017,0.000865,0.000776,0.000736
  giCountertenorOh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.001238,0.001930,0.002779,0.004109,0.005581,0.007795,0.011138,0.015868,0.023498,0.051584,0.090982,0.058872,0.024979,0.018745,0.013492,0.010997,0.009771,0.009964,0.011317,0.016000,0.020636,0.015422,0.009037,0.006223,0.004400,0.003252,0.002389,0.001745,0.001211,0.000817,0.000496,0.000277,0.000133,0.000115,0.000142,0.000158,0.000148,0.000126,0.000131,0.000189,0.000261,0.000353,0.000467,0.000621,0.000824,0.001156,0.001681,0.002451,0.002478,0.001885,0.001618,0.001623,0.001883,0.002404,0.003279,0.004007,0.003505,0.002498,0.001824,0.001384,0.001115,0.000952,0.000886,0.000783,0.000591,0.000435,0.000341,0.000275,0.000225,0.000183,0.000147,0.000116
  giCountertenorOh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.001604,0.002674,0.004072,0.006176,0.009086,0.013563,0.023369,0.035194,0.101582,0.077033,0.029449,0.018986,0.014865,0.011952,0.011694,0.013514,0.020789,0.023285,0.013030,0.008051,0.005265,0.003665,0.002550,0.001705,0.001091,0.000616,0.000306,0.000141,0.000150,0.000186,0.000177,0.000148,0.000124,0.000198,0.000294,0.000421,0.000588,0.000823,0.001193,0.001814,0.002885,0.002835,0.002063,0.001887,0.002113,0.002782,0.004010,0.004725,0.003536,0.002382,0.001692,0.001306,0.001100,0.001011,0.000790,0.000541,0.000403,0.000313,0.000246,0.000192,0.000145,0.000106
  giCountertenorOh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.002118,0.003720,0.006338,0.009787,0.015959,0.030101,0.065229,0.125015,0.039115,0.024298,0.017276,0.013843,0.014630,0.020706,0.028742,0.015233,0.008650,0.005463,0.003511,0.002232,0.001316,0.000660,0.000266,0.000162,0.000213,0.000212,0.000170,0.000102,0.000144,0.000251,0.000396,0.000597,0.000891,0.001362,0.002256,0.003634,0.002895,0.002272,0.002441,0.003352,0.005140,0.005242,0.003308,0.002161,0.001568,0.001294,0.001139,0.000764,0.000520,0.000381,0.000286,0.000212,0.000149
  giCountertenorOh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.002837,0.005300,0.009514,0.017406,0.034571,0.095209,0.119489,0.040201,0.022335,0.016651,0.017637,0.029015,0.027795,0.012453,0.006942,0.004127,0.002369,0.001209,0.000479,0.000192,0.000252,0.000249,0.000173,0.000136,0.000271,0.000472,0.000765,0.001230,0.002129,0.004081,0.003528,0.002689,0.003169,0.004981,0.006691,0.004146,0.002498,0.001745,0.001484,0.001074,0.000655,0.000451,0.000321,0.000221,0.000141
  giCountertenorOh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.003856,0.008213,0.015579,0.031723,0.103107,0.117667,0.037472,0.021976,0.019904,0.031955,0.030799,0.012429,0.006496,0.003487,0.001632,0.000553,0.000231,0.000316,0.000252,0.000119,0.000102,0.000262,0.000522,0.000934,0.001650,0.003365,0.004960,0.003239,0.003768,0.006564,0.007017,0.003653,0.002234,0.001775,0.001184,0.000682,0.000451,0.000296,0.000176
  giCountertenorOh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.005349,0.012355,0.027133,0.070422,0.154071,0.037978,0.023916,0.027050,0.046604,0.016115,0.007338,0.003414,0.001231,0.000279,0.000372,0.000296,0.000104,0.000102,0.000249,0.000591,0.001183,0.002404,0.005812,0.004154,0.004259,0.008091,0.007129,0.003415,0.002221,0.001594,0.000813,0.000497,0.000294,0.000145
  giCountertenorOh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.007441,0.019575,0.060208,0.250025,0.048593,0.027690,0.041429,0.030466,0.010928,0.004465,0.001320,0.000325,0.000425,0.000205,0.000122,0.000289,0.000795,0.001788,0.004530,0.005810,0.004904,0.010331,0.006646,0.003152,0.002288,0.001045,0.000576,0.000300,0.000117
  giCountertenorOh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.010601,0.034815,0.190470,0.080405,0.033305,0.058053,0.024910,0.008255,0.002415,0.000380,0.000499,0.000157,0.000134,0.000544,0.001534,0.004269,0.007067,0.006352,0.013410,0.005008,0.002977,0.001313,0.000644,0.000282
  giCountertenorOh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.016431,0.063467,0.235366,0.043971,0.063965,0.024873,0.006978,0.001106,0.000633,0.000237,0.000165,0.000526,0.001871,0.006744,0.006491,0.013161,0.007320,0.003558,0.001368,0.000593,0.000178
  giCountertenorOh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.024710,0.140863,0.075949,0.054103,0.032225,0.006828,0.000565,0.000591,0.000130,0.000122,0.000130,0.001184,0.004810,0.008308,0.016195,0.006829,0.003189,0.000995,0.000290
  giCountertenorOh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.039150,0.500037,0.055380,0.060924,0.008930,0.000650,0.000410,0.000244,0.001590,0.009065,0.009812,0.013291,0.004577,0.001153,0.000235,0.000102
  giCountertenorOh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.069637,0.160819,0.116129,0.016511,0.000768,0.000310,0.000199,0.000121,0.001088,0.008540,0.012711,0.010013,0.002627,0.000564,0.000125
  giCountertenorOh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.126948,0.087953,0.049754,0.002213,0.000474,0.000165,0.003744,0.012986,0.014644,0.002737,0.000355,0.000112
  giCountertenorOh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.281744,0.108208,0.013656,0.001182,0.000243,0.000261,0.009621,0.032394,0.006377,0.000580,0.000114
  giCountertenorOh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,1.000000,0.121825,0.001300,0.000487,0.000199,0.018133,0.026571,0.002305,0.000202
  giCountertenorOh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.321614,0.033016,0.000620,0.000243,0.017081,0.020022,0.001128
  giCountertenorOh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.175915,0.004425,0.000329,0.007489,0.029288,0.000710,0.000103
  giCountertenorOh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.216413,0.002364,0.000521,0.064787,0.001159
  giCountertenorOh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.243654,0.000974,0.036270,0.004610,0.000126
  giCountertenorOh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.066032,0.000487,0.040044,0.000145
 elseif iVoice==15 then
  giCountertenorOoh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.002930,0.003338,0.004209,0.005237,0.006779,0.009344,0.011428,0.018295,0.022905,0.047515,0.085264,0.075133,0.033639,0.022964,0.015677,0.011837,0.009420,0.008016,0.005598,0.002825,0.001635,0.001188,0.001026,0.000841,0.000709,0.000648,0.000588,0.000526,0.000477,0.000401,0.000336,0.000264,0.000191,0.000133,0.000108,0.000159,0.000223,0.000304,0.000406,0.000530,0.000690,0.000897,0.001168,0.001579,0.002149,0.003081,0.003723,0.003238,0.002269,0.001653,0.001261,0.001042,0.001008,0.001130,0.001209,0.001084,0.000893,0.000767,0.000687,0.000655,0.000659,0.000701,0.000788
  giCountertenorOoh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.003586,0.004256,0.005702,0.007556,0.010921,0.014024,0.024945,0.035150,0.092038,0.093788,0.037116,0.026081,0.015318,0.012347,0.009771,0.006560,0.002960,0.001559,0.001368,0.001026,0.000848,0.000758,0.000673,0.000602,0.000498,0.000407,0.000303,0.000209,0.000120,0.000160,0.000244,0.000355,0.000499,0.000686,0.000937,0.001278,0.001812,0.002618,0.003961,0.004359,0.003122,0.002085,0.001507,0.001221,0.001248,0.001432,0.001351,0.001078,0.000901,0.000805,0.000781,0.000820,0.000929,0.001142,0.001493,0.001896,0.001791,0.001284,0.000903,0.000650,0.000483,0.000359,0.000267,0.000194,0.000137
  giCountertenorOoh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.004387,0.005685,0.007842,0.011742,0.016404,0.030852,0.061026,0.124965,0.062106,0.031873,0.017790,0.013428,0.010387,0.004571,0.002046,0.001614,0.001183,0.000962,0.000842,0.000729,0.000595,0.000464,0.000313,0.000193,0.000161,0.000272,0.000424,0.000634,0.000922,0.001335,0.001984,0.003066,0.004916,0.004770,0.002924,0.001939,0.001459,0.001499,0.001713,0.001437,0.001123,0.000961,0.000924,0.000992,0.001191,0.001581,0.002177,0.002117,0.001415,0.000940,0.000647,0.000456,0.000320,0.000219,0.000143
  giCountertenorOoh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.005388,0.007726,0.011546,0.018473,0.036095,0.086804,0.138411,0.041289,0.024565,0.016650,0.011902,0.004266,0.001997,0.001497,0.001165,0.000998,0.000847,0.000647,0.000450,0.000263,0.000103,0.000118,0.000101,0.000207,0.000376,0.000627,0.000985,0.001532,0.002453,0.004202,0.006295,0.004173,0.002462,0.001735,0.001864,0.001976,0.001463,0.001175,0.001103,0.001211,0.001558,0.002280,0.002680,0.001752,0.001076,0.000697,0.000459,0.000297,0.000181,0.000101
  giCountertenorOoh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.006678,0.010477,0.018695,0.036602,0.095066,0.150330,0.045948,0.023685,0.016041,0.005653,0.002378,0.001682,0.001298,0.001053,0.000802,0.000528,0.000265,0.000105,0.000138,0.000118,0.000217,0.000449,0.000815,0.001387,0.002350,0.004323,0.007492,0.004566,0.002539,0.002030,0.002436,0.001799,0.001386,0.001329,0.001590,0.002352,0.003257,0.002059,0.001161,0.000702,0.000425,0.000243,0.000123
  giCountertenorOoh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.008507,0.015099,0.028021,0.070256,0.187408,0.052114,0.024673,0.013102,0.003116,0.002049,0.001515,0.001204,0.000813,0.000417,0.000115,0.000127,0.000167,0.000128,0.000197,0.000489,0.000998,0.001875,0.003627,0.007929,0.006251,0.003018,0.002497,0.002704,0.001804,0.001564,0.001861,0.002991,0.003590,0.001811,0.000967,0.000535,0.000275,0.000118
  giCountertenorOoh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.011372,0.023489,0.061720,0.249997,0.063763,0.026863,0.009143,0.003230,0.001926,0.001459,0.000929,0.000387,0.000186,0.000157,0.000177,0.000548,0.001276,0.002689,0.006178,0.009604,0.003905,0.003024,0.002899,0.001942,0.002006,0.003198,0.004277,0.001900,0.000922,0.000444,0.000179
  giCountertenorOoh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.015452,0.036944,0.173650,0.082571,0.033299,0.008530,0.002994,0.001997,0.001295,0.000526,0.000105,0.000236,0.000137,0.000202,0.000755,0.001978,0.004926,0.012637,0.004941,0.003746,0.002939,0.002218,0.003135,0.005390,0.002165,0.000924,0.000366,0.000102
  giCountertenorOoh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.020956,0.073214,0.300650,0.047374,0.011304,0.003365,0.002105,0.001056,0.000160,0.000271,0.000143,0.000105,0.000437,0.001635,0.004710,0.015016,0.005087,0.004883,0.002780,0.003191,0.006532,0.002329,0.000854,0.000249
  giCountertenorOoh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.030210,0.140577,0.104265,0.026214,0.004100,0.002408,0.000835,0.000254,0.000256,0.000123,0.000395,0.002001,0.007270,0.012518,0.005007,0.003616,0.003732,0.007195,0.001939,0.000551
  giCountertenorOoh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.046979,0.499997,0.053723,0.006460,0.002919,0.000773,0.000378,0.000118,0.000106,0.000152,0.001094,0.005381,0.019208,0.006051,0.003882,0.006400,0.003801,0.000886,0.000115
  giCountertenorOoh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.073889,0.165133,0.017058,0.003994,0.001052,0.000471,0.000177,0.001512,0.009857,0.009881,0.005879,0.006275,0.004329,0.000733,0.000125
  giCountertenorOoh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.146440,0.094756,0.006731,0.002111,0.000551,0.000141,0.000870,0.009425,0.010175,0.005557,0.013068,0.001707,0.000146
  giCountertenorOoh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.281199,0.052436,0.004817,0.000509,0.000247,0.004004,0.025038,0.007235,0.014393,0.001102,0.000174
  giCountertenorOoh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000000,0.012920,0.001547,0.000236,0.000305,0.010763,0.012104,0.012802,0.001771,0.000185
  giCountertenorOoh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.330234,0.007988,0.000942,0.000128,0.000355,0.019716,0.011755,0.008656,0.000251
  giCountertenorOoh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.189497,0.004222,0.000170,0.000283,0.018851,0.011112,0.003413,0.000105
  giCountertenorOoh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.104873,0.001016,0.008010,0.014471,0.002205,0.000124
  giCountertenorOoh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.025840,0.000472,0.000610,0.024208,0.003543,0.000116
  giCountertenorOoh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.015975,0.000256,0.039437,0.017313,0.000164
  giCountertenorOoh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.008443,0.000565,0.022224,0.000210
 /* ALTO */
 elseif iVoice==16 then
  giAltoAh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.000189,0.000296,0.000384,0.000470,0.000519,0.000516,0.000488,0.000371,0.000272,0.000362,0.000762,0.001312,0.002068,0.003050,0.004182,0.005758,0.007569,0.009859,0.013271,0.016801,0.024312,0.033001,0.056588,0.102975,0.102169,0.056712,0.036664,0.028737,0.023278,0.022005,0.021266,0.024719,0.028942,0.046575,0.063428,0.051044,0.027265,0.019362,0.013395,0.010268,0.007894,0.006039,0.004698,0.003520,0.002602,0.001851,0.001231,0.000766,0.000406,0.000199,0.000194,0.000263,0.000310,0.000314,0.000289,0.000244,0.000184,0.000124,0.000126,0.000177,0.000255,0.000364,0.000514,0.000708,0.000954,0.001263,0.001646,0.002126,0.002727,0.003491,0.004448,0.005556,0.006485,0.006605,0.005809,0.004697,0.003698,0.002896,0.002268,0.001771,0.001374,0.001053,0.000794,0.000590,0.000437,0.000344,0.000317,0.000349
  giAltoAh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.000244,0.000397,0.000518,0.000607,0.000616,0.000568,0.000396,0.000335,0.000726,0.001487,0.002548,0.003987,0.005730,0.008260,0.011227,0.015932,0.020978,0.034291,0.049287,0.113108,0.122662,0.059916,0.041411,0.029183,0.026718,0.025244,0.029994,0.038221,0.068475,0.069216,0.035306,0.022725,0.014644,0.011021,0.007946,0.005866,0.004193,0.002921,0.001910,0.001127,0.000578,0.000244,0.000241,0.000340,0.000374,0.000356,0.000298,0.000215,0.000128,0.000113,0.000169,0.000257,0.000396,0.000598,0.000875,0.001242,0.001721,0.002342,0.003155,0.004233,0.005634,0.007200,0.007995,0.007183,0.005618,0.004224,0.003160,0.002359,0.001750,0.001279,0.000914,0.000639,0.000456,0.000380,0.000412,0.000508,0.000638,0.000799,0.000989,0.001180,0.001248,0.001102,0.000867,0.000662,0.000507,0.000390,0.000299,0.000227,0.000169,0.000122
  giAltoAh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.000322,0.000523,0.000687,0.000739,0.000683,0.000446,0.000481,0.001318,0.002593,0.004536,0.007027,0.010628,0.015538,0.022888,0.037867,0.059838,0.147441,0.117395,0.052829,0.036972,0.031809,0.030303,0.037765,0.063847,0.088422,0.044709,0.025986,0.015831,0.011132,0.007664,0.005234,0.003397,0.002015,0.001038,0.000395,0.000267,0.000404,0.000445,0.000401,0.000296,0.000178,0.000161,0.000262,0.000438,0.000716,0.001120,0.001683,0.002450,0.003502,0.004969,0.006966,0.009036,0.009149,0.007148,0.005115,0.003622,0.002556,0.001780,0.001208,0.000794,0.000531,0.000449,0.000528,0.000692,0.000905,0.001168,0.001426,0.001435,0.001144,0.000835,0.000607,0.000444,0.000323,0.000231,0.000159,0.000104
  giAltoAh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.000435,0.000700,0.000875,0.000843,0.000544,0.000647,0.001988,0.004188,0.007324,0.012179,0.019122,0.029406,0.054206,0.141337,0.157466,0.062492,0.040883,0.036334,0.041267,0.064121,0.105743,0.045932,0.025684,0.015869,0.010059,0.006455,0.003844,0.001991,0.000808,0.000291,0.000460,0.000530,0.000454,0.000294,0.000125,0.000105,0.000104,0.000188,0.000335,0.000614,0.001080,0.001786,0.002812,0.004302,0.006516,0.009550,0.011271,0.008968,0.006048,0.004011,0.002642,0.001695,0.001040,0.000640,0.000541,0.000692,0.000956,0.001306,0.001686,0.001690,0.001250,0.000853,0.000587,0.000404,0.000271,0.000172,0.000103,0.000102,0.000112
  giAltoAh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.000591,0.000940,0.001033,0.000742,0.000724,0.002625,0.006104,0.011523,0.019729,0.033624,0.066046,0.206109,0.113522,0.057528,0.044055,0.049494,0.093266,0.102229,0.038781,0.020569,0.012100,0.007054,0.003709,0.001535,0.000398,0.000528,0.000629,0.000488,0.000249,0.000106,0.000124,0.000180,0.000356,0.000732,0.001422,0.002539,0.004275,0.007024,0.011180,0.013295,0.009456,0.005833,0.003569,0.002122,0.001189,0.000693,0.000704,0.001020,0.001484,0.002021,0.001923,0.001279,0.000814,0.000523,0.000327,0.000190,0.000101,0.000126,0.000129
  giAltoAh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.000792,0.001213,0.001135,0.000668,0.002969,0.007965,0.016502,0.031835,0.068519,0.226014,0.119754,0.058321,0.050446,0.076384,0.138316,0.045413,0.022028,0.011725,0.005840,0.002252,0.000488,0.000680,0.000712,0.000429,0.000104,0.000120,0.000144,0.000149,0.000337,0.000791,0.001747,0.003438,0.006303,0.011258,0.015979,0.011229,0.006318,0.003500,0.001828,0.000913,0.000824,0.001276,0.001980,0.002497,0.001737,0.001015,0.000599,0.000339,0.000171,0.000148,0.000149
  giAltoAh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.001046,0.001479,0.000892,0.002638,0.009076,0.021269,0.045807,0.119803,0.234872,0.073996,0.060678,0.127922,0.089484,0.031708,0.015355,0.006806,0.002080,0.000536,0.000893,0.000593,0.000132,0.000160,0.000143,0.000109,0.000328,0.000883,0.002256,0.004934,0.010012,0.018210,0.014399,0.007297,0.003587,0.001599,0.000905,0.001398,0.002364,0.002903,0.001690,0.000899,0.000467,0.000211,0.000109,0.000178,0.000165
  giAltoAh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.001400,0.001687,0.001294,0.008377,0.024362,0.058827,0.282834,0.125013,0.072697,0.128373,0.091880,0.031757,0.012918,0.003985,0.000583,0.001062,0.000588,0.000210,0.000100,0.000377,0.001233,0.003586,0.008637,0.019180,0.018001,0.008054,0.003404,0.001286,0.001393,0.002629,0.003400,0.001717,0.000813,0.000347,0.000124,0.000156,0.000228,0.000128
  giAltoAh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.001880,0.001482,0.005251,0.023053,0.067263,0.412366,0.115064,0.099033,0.204459,0.041147,0.014111,0.003072,0.001057,0.000977,0.000247,0.000164,0.000717,0.002848,0.008565,0.022410,0.018941,0.007145,0.002378,0.001413,0.002978,0.003857,0.001634,0.000655,0.000202,0.000113,0.000183,0.000258,0.000115
  giAltoAh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.002427,0.001338,0.015940,0.063709,0.452338,0.116697,0.152906,0.090878,0.023464,0.004506,0.001360,0.000857,0.000242,0.000142,0.000677,0.003504,0.012640,0.032038,0.012664,0.003664,0.001654,0.003974,0.003484,0.001202,0.000342,0.000133,0.000199,0.000300,0.000106
  giAltoAh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.002958,0.005276,0.042538,0.239659,0.147975,0.255925,0.063414,0.013610,0.001072,0.001187,0.000319,0.000216,0.001764,0.009875,0.036436,0.014599,0.003203,0.002798,0.005807,0.001797,0.000423,0.000157,0.000220,0.000330
  giAltoAh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.003374,0.016755,0.117663,0.250019,0.256814,0.063510,0.007969,0.002123,0.000140,0.000201,0.002468,0.017282,0.036001,0.006807,0.002788,0.006801,0.001627,0.000248,0.000137,0.000312,0.000255
  giAltoAh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.002967,0.046107,0.824869,0.198093,0.082301,0.006142,0.001955,0.000496,0.001429,0.017146,0.037895,0.004766,0.005960,0.003267,0.000407,0.000157,0.000155,0.000519
  giAltoAh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.002677,0.127443,0.233422,0.181784,0.009013,0.001717,0.000284,0.007015,0.064115,0.007331,0.007952,0.002405,0.000267,0.000125,0.000600
  giAltoAh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.010553,0.479368,0.511917,0.027219,0.002374,0.000432,0.019753,0.029194,0.005598,0.003593,0.000314,0.000115,0.000440,0.000188
  giAltoAh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.033509,0.499985,0.127000,0.004247,0.000403,0.034562,0.013610,0.013599,0.000498,0.000136,0.000625,0.000114
  giAltoAh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.092211,0.396173,0.012280,0.000993,0.034289,0.009528,0.006532,0.000315,0.000310,0.000185
  giAltoAh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.254908,0.363579,0.003434,0.014036,0.014660,0.004810,0.000173,0.001201
  giAltoAh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.958726,0.054439,0.000865,0.058385,0.007186,0.000230,0.000375
  giAltoAh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,1.000000,0.008493,0.069131,0.027200,0.000272,0.000228
  giAltoAh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.792347,0.001986,0.019056,0.000630,0.000371
 elseif iVoice==17 then
  giAltoEh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.003362,0.004063,0.005154,0.006879,0.008943,0.011836,0.016028,0.020650,0.030582,0.041256,0.073678,0.110417,0.094909,0.051940,0.035835,0.024628,0.018327,0.013819,0.010292,0.007833,0.005745,0.004159,0.002926,0.001916,0.001193,0.000629,0.000269,0.000196,0.000305,0.000382,0.000393,0.000375,0.000314,0.000239,0.000159,0.000177,0.000278,0.000395,0.000541,0.000706,0.000932,0.001227,0.001618,0.002273,0.003186,0.005042,0.006061,0.004724,0.002963,0.002132,0.001509,0.001144,0.000862,0.000649,0.000491,0.000359,0.000260,0.000181,0.000118,0.000128,0.000186,0.000258,0.000348,0.000460,0.000604,0.000788,0.001043,0.001394,0.001891,0.002418,0.002462,0.001970,0.001456,0.001095,0.000833,0.000649,0.000509,0.000407,0.000337,0.000299,0.000298,0.000331,0.000397,0.000491,0.000619,0.000790,0.001006,0.001234
  giAltoEh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.004127,0.005275,0.007209,0.009991,0.013867,0.019808,0.027445,0.042904,0.071987,0.129126,0.108909,0.053400,0.036759,0.023688,0.017299,0.012158,0.008774,0.005971,0.004053,0.002515,0.001440,0.000656,0.000247,0.000296,0.000443,0.000468,0.000441,0.000347,0.000241,0.000122,0.000217,0.000362,0.000543,0.000763,0.001053,0.001462,0.002056,0.003040,0.004969,0.007193,0.005760,0.003352,0.002274,0.001551,0.001119,0.000797,0.000571,0.000393,0.000264,0.000165,0.000115,0.000188,0.000283,0.000406,0.000567,0.000781,0.001077,0.001517,0.002169,0.002914,0.002838,0.002063,0.001457,0.001046,0.000776,0.000583,0.000451,0.000372,0.000353,0.000394,0.000488,0.000633,0.000840,0.001123,0.001453,0.001578,0.001331,0.000989,0.000724,0.000534,0.000397,0.000294,0.000216,0.000156,0.000110
  giAltoEh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.005092,0.006977,0.010396,0.015163,0.023130,0.034809,0.056474,0.124888,0.147945,0.067675,0.042041,0.026103,0.017387,0.011699,0.007558,0.004760,0.002659,0.001309,0.000425,0.000311,0.000523,0.000556,0.000492,0.000344,0.000181,0.000108,0.000276,0.000487,0.000768,0.001122,0.001665,0.002495,0.003962,0.007356,0.007971,0.004348,0.002698,0.001734,0.001166,0.000785,0.000511,0.000320,0.000182,0.000190,0.000315,0.000486,0.000720,0.001051,0.001562,0.002371,0.003434,0.003234,0.002153,0.001437,0.000994,0.000705,0.000521,0.000426,0.000432,0.000535,0.000722,0.001009,0.001423,0.001836,0.001720,0.001239,0.000855,0.000596,0.000418,0.000292,0.000200,0.000132
  giAltoEh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.006374,0.009593,0.015226,0.024859,0.040118,0.071823,0.177585,0.125051,0.058039,0.032167,0.019996,0.012533,0.007307,0.003947,0.001782,0.000489,0.000427,0.000651,0.000634,0.000446,0.000221,0.000149,0.000413,0.000750,0.001205,0.001923,0.003136,0.005583,0.010175,0.006418,0.003433,0.002024,0.001259,0.000784,0.000459,0.000244,0.000111,0.000114,0.000244,0.000436,0.000711,0.001122,0.001783,0.002933,0.004251,0.003241,0.001974,0.001256,0.000835,0.000588,0.000498,0.000577,0.000806,0.001196,0.001794,0.002240,0.001746,0.001130,0.000733,0.000481,0.000313,0.000196,0.000116,0.000107,0.000123,0.000104
  giAltoEh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.008128,0.013762,0.023679,0.041314,0.082543,0.220926,0.103928,0.049281,0.027654,0.015677,0.008325,0.003834,0.001260,0.000392,0.000764,0.000751,0.000478,0.000163,0.000354,0.000791,0.001415,0.002458,0.004555,0.010107,0.009469,0.004274,0.002294,0.001301,0.000721,0.000362,0.000148,0.000256,0.000519,0.000925,0.001584,0.002804,0.004864,0.003965,0.002203,0.001307,0.000820,0.000602,0.000668,0.000990,0.001593,0.002490,0.002373,0.001446,0.000864,0.000523,0.000310,0.000173,0.000135,0.000143,0.000103
  giAltoEh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.010535,0.019953,0.039560,0.085683,0.257901,0.106658,0.047311,0.024286,0.011928,0.005025,0.001312,0.000591,0.000935,0.000693,0.000244,0.000433,0.001084,0.002103,0.004108,0.009930,0.011512,0.004545,0.002237,0.001142,0.000527,0.000191,0.000230,0.000566,0.001133,0.002154,0.004340,0.005678,0.002915,0.001553,0.000903,0.000707,0.000977,0.001682,0.002909,0.002665,0.001449,0.000795,0.000433,0.000220,0.000158,0.000169,0.000109
  giAltoEh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.013958,0.030334,0.069643,0.249891,0.135371,0.052225,0.023407,0.009523,0.002620,0.000623,0.001115,0.000690,0.000220,0.000978,0.002252,0.005006,0.014753,0.008712,0.003477,0.001575,0.000644,0.000191,0.000198,0.000633,0.001450,0.003146,0.006918,0.004334,0.002002,0.001048,0.000872,0.001457,0.002874,0.003471,0.001726,0.000845,0.000403,0.000170,0.000107,0.000188,0.000185,0.000102
  giAltoEh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.019186,0.049717,0.143654,0.250057,0.064333,0.025066,0.007894,0.000978,0.001303,0.000893,0.000298,0.001501,0.003852,0.011185,0.012845,0.004053,0.001570,0.000488,0.000229,0.000874,0.002252,0.005891,0.006503,0.002521,0.001180,0.001158,0.002404,0.004502,0.002271,0.000968,0.000395,0.000131,0.000158,0.000250,0.000148
  giAltoEh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.027527,0.082639,0.441905,0.098560,0.031354,0.007669,0.000785,0.001502,0.000330,0.001589,0.004927,0.020236,0.008549,0.002604,0.000730,0.000119,0.000515,0.001854,0.005620,0.007941,0.002618,0.001208,0.001989,0.004995,0.002897,0.001049,0.000348,0.000188,0.000287,0.000130
  giAltoEh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.039930,0.171475,0.213415,0.048597,0.010054,0.001181,0.001385,0.000867,0.004211,0.019887,0.009096,0.002286,0.000383,0.000109,0.000463,0.002271,0.008699,0.005840,0.001810,0.001961,0.005836,0.002906,0.000868,0.000198,0.000198,0.000336,0.000125
  giAltoEh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.060669,0.499820,0.104443,0.019045,0.001245,0.001376,0.001951,0.010010,0.017427,0.003148,0.000373,0.001269,0.006296,0.008668,0.002099,0.002917,0.006949,0.001692,0.000339,0.000217,0.000372,0.000107
  giAltoEh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.099436,0.500065,0.050129,0.001955,0.001786,0.003004,0.022378,0.008105,0.000976,0.000133,0.001750,0.011786,0.005041,0.002318,0.009006,0.001935,0.000261,0.000101,0.000315,0.000297
  giAltoEh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.165294,0.197124,0.015337,0.003004,0.003167,0.040484,0.005206,0.000220,0.001029,0.011247,0.005236,0.003975,0.005800,0.000693,0.000124,0.000151,0.000577,0.000103
  giAltoEh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.343032,0.097209,0.002364,0.001736,0.039793,0.004573,0.000222,0.004544,0.011682,0.003924,0.005813,0.000396,0.000102,0.000125,0.000672
  giAltoEh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.999678,0.038087,0.002755,0.020021,0.006295,0.000187,0.012594,0.004197,0.013896,0.000677,0.000104,0.000433,0.000213
  giAltoEh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,1.000001,0.003910,0.006008,0.016207,0.000265,0.023574,0.004637,0.003870,0.000201,0.000631,0.000123
  giAltoEh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.394210,0.006006,0.080968,0.000440,0.022493,0.007950,0.001386,0.000302,0.000206
  giAltoEh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.194429,0.003472,0.009145,0.009090,0.007850,0.000792,0.001344
  giAltoEh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.076172,0.040042,0.000373,0.008393,0.001355,0.000427
  giAltoEh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.007820,0.032414,0.047156,0.007739,0.000246
  giAltoEh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.012011,0.000881,0.015900,0.000124,0.000412
 elseif iVoice==18 then
  giAltoEee24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.004734,0.005426,0.006616,0.008187,0.010807,0.014143,0.019214,0.028383,0.042150,0.084741,0.096689,0.058804,0.031217,0.023777,0.015637,0.012369,0.009000,0.006764,0.005109,0.003621,0.002624,0.001760,0.001102,0.000655,0.000294,0.000144,0.000174,0.000252,0.000263,0.000258,0.000224,0.000168,0.000120,0.000134,0.000222,0.000342,0.000495,0.000695,0.000938,0.001253,0.001659,0.002197,0.003006,0.004145,0.005948,0.006987,0.005870,0.004071,0.002952,0.002161,0.001639,0.001249,0.000949,0.000719,0.000532,0.000385,0.000268,0.000175,0.000107,0.000136,0.000197,0.000274,0.000368,0.000490,0.000641,0.000851,0.001137,0.001539,0.001960,0.001987,0.001583,0.001163,0.000868,0.000655,0.000503,0.000386,0.000296,0.000224,0.000165,0.000118
  giAltoEee27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.005733,0.007002,0.008794,0.011941,0.016656,0.024458,0.035967,0.075403,0.118292,0.075882,0.036455,0.024866,0.016911,0.011927,0.008415,0.006026,0.004028,0.002674,0.001591,0.000891,0.000358,0.000167,0.000242,0.000314,0.000312,0.000271,0.000193,0.000123,0.000109,0.000199,0.000350,0.000552,0.000829,0.001181,0.001665,0.002311,0.003324,0.004850,0.007381,0.008158,0.005824,0.003857,0.002663,0.001912,0.001382,0.000998,0.000708,0.000485,0.000316,0.000189,0.000101,0.000135,0.000213,0.000320,0.000454,0.000635,0.000878,0.001238,0.001765,0.002361,0.002288,0.001653,0.001159,0.000824,0.000602,0.000439,0.000319,0.000226,0.000154,0.000147,0.000209,0.000290,0.000395,0.000538,0.000733,0.000978,0.001145,0.001038,0.000788,0.000577,0.000424,0.000314
  giAltoEee30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.006959,0.009081,0.012315,0.018565,0.028153,0.044588,0.114187,0.124993,0.048859,0.031482,0.019683,0.012844,0.008644,0.005471,0.003421,0.001862,0.000902,0.000286,0.000233,0.000370,0.000371,0.000309,0.000202,0.000154,0.000316,0.000568,0.000934,0.001428,0.002143,0.003188,0.004956,0.008182,0.009679,0.006341,0.003982,0.002603,0.001775,0.001204,0.000799,0.000504,0.000291,0.000147,0.000134,0.000236,0.000382,0.000579,0.000856,0.001275,0.001931,0.002784,0.002605,0.001721,0.001138,0.000774,0.000533,0.000364,0.000239,0.000148,0.000108,0.000174,0.000263,0.000386,0.000556,0.000803,0.001139,0.001354,0.001132,0.000793,0.000548,0.000383,0.000267,0.000183,0.000121
  giAltoEee33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.008585,0.011910,0.018442,0.028601,0.051881,0.148152,0.116598,0.047937,0.025511,0.015999,0.009973,0.005757,0.003142,0.001432,0.000410,0.000282,0.000444,0.000423,0.000287,0.000140,0.000115,0.000271,0.000586,0.001072,0.001777,0.002864,0.004718,0.008269,0.011724,0.007336,0.004255,0.002631,0.001661,0.001032,0.000600,0.000308,0.000128,0.000173,0.000332,0.000566,0.000913,0.001456,0.002388,0.003440,0.002602,0.001568,0.000982,0.000630,0.000399,0.000238,0.000128,0.000101,0.000183,0.000304,0.000480,0.000742,0.001146,0.001590,0.001412,0.000931,0.000601,0.000393,0.000254,0.000158
  giAltoEee36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.010854,0.016378,0.028295,0.056786,0.169548,0.117657,0.047576,0.024750,0.013535,0.007247,0.003523,0.001311,0.000289,0.000504,0.000516,0.000336,0.000126,0.000268,0.000684,0.001392,0.002510,0.004405,0.008309,0.014010,0.008164,0.004335,0.002505,0.001443,0.000773,0.000352,0.000118,0.000180,0.000395,0.000740,0.001290,0.002287,0.003943,0.003185,0.001748,0.001013,0.000595,0.000332,0.000163,0.000200,0.000368,0.000630,0.001059,0.001722,0.001799,0.001123,0.000669,0.000403,0.000236,0.000128,0.000109,0.000115
  giAltoEee39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.013990,0.023858,0.048875,0.150665,0.151627,0.049694,0.023833,0.012042,0.005344,0.001779,0.000333,0.000629,0.000543,0.000246,0.000119,0.000397,0.001102,0.002359,0.004615,0.009688,0.016299,0.007706,0.003821,0.001994,0.000969,0.000378,0.000160,0.000427,0.000907,0.001756,0.003531,0.004578,0.002318,0.001204,0.000638,0.000309,0.000117,0.000128,0.000294,0.000580,0.001078,0.001961,0.002080,0.001158,0.000629,0.000339,0.000169,0.000128,0.000135
  giAltoEee42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.018166,0.037139,0.089205,0.250023,0.062972,0.025695,0.010945,0.003725,0.000572,0.000740,0.000619,0.000188,0.000311,0.001143,0.002867,0.006399,0.016417,0.012713,0.005223,0.002421,0.001017,0.000300,0.000144,0.000475,0.001166,0.002569,0.005607,0.003464,0.001560,0.000733,0.000297,0.000129,0.000351,0.000781,0.001627,0.002743,0.001605,0.000774,0.000369,0.000155,0.000152,0.000148
  giAltoEee45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.023819,0.057203,0.296329,0.095870,0.031997,0.011515,0.002863,0.000564,0.000846,0.000280,0.000125,0.000542,0.002146,0.005737,0.016566,0.014688,0.005270,0.002068,0.000618,0.000104,0.000168,0.000665,0.001832,0.004795,0.005221,0.001970,0.000800,0.000255,0.000106,0.000370,0.000967,0.002308,0.002840,0.001209,0.000509,0.000186,0.000129,0.000201,0.000118
  giAltoEee48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.032760,0.113585,0.235289,0.049500,0.014494,0.002623,0.001009,0.000673,0.000541,0.002795,0.008824,0.028041,0.008677,0.002894,0.000711,0.000113,0.000362,0.001483,0.004584,0.006379,0.002029,0.000664,0.000124,0.000401,0.001264,0.003457,0.002250,0.000805,0.000255,0.000153,0.000229,0.000104
  giAltoEee51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.047737,0.301484,0.099415,0.024093,0.003561,0.001256,0.000492,0.000239,0.002207,0.009245,0.032634,0.007649,0.001940,0.000195,0.000322,0.001819,0.007079,0.004645,0.001280,0.000235,0.000103,0.000590,0.002164,0.004171,0.001262,0.000339,0.000163,0.000268
  giAltoEee54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.074277,0.500015,0.051387,0.007447,0.001481,0.000373,0.000619,0.005727,0.032843,0.010441,0.002023,0.000168,0.000950,0.005141,0.006928,0.001467,0.000161,0.000256,0.001563,0.005490,0.001551,0.000312,0.000179,0.000297
  giAltoEee57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.114410,0.191733,0.023028,0.001128,0.000560,0.001084,0.011476,0.029371,0.004136,0.000208,0.001331,0.009595,0.003939,0.000510,0.000212,0.001935,0.005681,0.001019,0.000104,0.000257,0.000236
  giAltoEee60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.227192,0.099004,0.005244,0.001344,0.001073,0.017641,0.017350,0.001409,0.000721,0.009172,0.004060,0.000250,0.000806,0.006918,0.001617,0.000117,0.000126,0.000462
  giAltoEee63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.603091,0.048190,0.002514,0.000479,0.018496,0.015302,0.000391,0.003639,0.009291,0.000469,0.001181,0.008347,0.000679,0.000111,0.000537
  giAltoEee66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000000,0.014894,0.000748,0.011456,0.020880,0.000335,0.010283,0.002934,0.000513,0.010980,0.000625,0.000358,0.000173
  giAltoEee69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.383427,0.002256,0.002169,0.058726,0.000416,0.019191,0.001023,0.003871,0.002038,0.000111,0.000515,0.000103
  giAltoEee72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.197992,0.002686,0.035282,0.002817,0.018344,0.000500,0.013835,0.000234,0.000252,0.000171
  giAltoEee75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.096384,0.000955,0.030609,0.007281,0.000937,0.016694,0.000198,0.001073
  giAltoEee78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.029788,0.022912,0.000669,0.005868,0.021959,0.000130,0.000347
  giAltoEee81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.004512,0.117453,0.038387,0.007742,0.000221,0.000205
  giAltoEee84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.005371,0.005634,0.001000,0.000468,0.000341
 elseif iVoice==19 then
  giAltoOh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.001309,0.001824,0.002584,0.003538,0.004745,0.006302,0.008180,0.010795,0.014134,0.018933,0.026852,0.038995,0.065576,0.075348,0.053728,0.033361,0.025424,0.019181,0.016443,0.014528,0.014284,0.015282,0.019092,0.024569,0.022662,0.014749,0.009877,0.007312,0.005506,0.004369,0.003422,0.002692,0.002087,0.001569,0.001146,0.000793,0.000510,0.000298,0.000163,0.000137,0.000169,0.000197,0.000203,0.000190,0.000165,0.000129,0.000119,0.000199,0.000310,0.000458,0.000645,0.000885,0.001184,0.001561,0.002059,0.002697,0.003630,0.004955,0.007043,0.009589,0.009724,0.007266,0.005090,0.003734,0.002767,0.002112,0.001597,0.001205,0.000890,0.000637,0.000446,0.000328,0.000322,0.000409
  giAltoOh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.001648,0.002485,0.003697,0.005250,0.007398,0.010097,0.014102,0.019191,0.029184,0.043902,0.080410,0.085492,0.050018,0.033402,0.023668,0.019505,0.017033,0.017382,0.020190,0.028304,0.027191,0.016200,0.010650,0.007363,0.005554,0.004157,0.003124,0.002287,0.001614,0.001073,0.000647,0.000344,0.000174,0.000178,0.000225,0.000243,0.000227,0.000190,0.000138,0.000111,0.000208,0.000358,0.000570,0.000849,0.001223,0.001706,0.002371,0.003273,0.004688,0.006827,0.010463,0.011909,0.008681,0.005736,0.003964,0.002840,0.002045,0.001463,0.001019,0.000679,0.000447,0.000373,0.000475,0.000673,0.000930,0.001272,0.001741,0.002371,0.002729,0.002294,0.001637,0.001172,0.000856,0.000635,0.000471,0.000346,0.000248,0.000173,0.000114
  giAltoOh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.002103,0.003449,0.005368,0.008129,0.011793,0.017521,0.025816,0.041586,0.082368,0.104390,0.056599,0.036283,0.025378,0.020950,0.020421,0.024179,0.034910,0.027177,0.014524,0.009440,0.006646,0.004712,0.003347,0.002258,0.001434,0.000804,0.000377,0.000190,0.000239,0.000286,0.000274,0.000222,0.000148,0.000107,0.000224,0.000430,0.000739,0.001171,0.001769,0.002615,0.003842,0.005897,0.009422,0.014095,0.011544,0.006995,0.004512,0.003054,0.002058,0.001364,0.000851,0.000518,0.000455,0.000652,0.000977,0.001416,0.002057,0.002938,0.003127,0.002245,0.001501,0.001027,0.000720,0.000504,0.000346,0.000228,0.000141
  giAltoOh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.002740,0.004898,0.008092,0.012802,0.020408,0.032961,0.058600,0.125001,0.084118,0.045111,0.029687,0.024133,0.025379,0.037896,0.035451,0.017081,0.010266,0.006879,0.004582,0.002935,0.001729,0.000859,0.000329,0.000247,0.000333,0.000330,0.000255,0.000149,0.000102,0.000140,0.000335,0.000687,0.001224,0.002024,0.003222,0.005108,0.008605,0.015395,0.014809,0.008247,0.004922,0.003108,0.001934,0.001129,0.000626,0.000566,0.000903,0.001426,0.002224,0.003432,0.003660,0.002373,0.001484,0.000966,0.000632,0.000404,0.000244,0.000132,0.000134,0.000170,0.000126
  giAltoOh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.003650,0.007078,0.012608,0.021597,0.037879,0.078022,0.150765,0.066755,0.038384,0.029074,0.030586,0.049175,0.029523,0.014638,0.008747,0.005389,0.003142,0.001589,0.000598,0.000274,0.000395,0.000382,0.000259,0.000113,0.000103,0.000103,0.000112,0.000118,0.000144,0.000399,0.000921,0.001779,0.003139,0.005426,0.009970,0.019301,0.014629,0.007520,0.004255,0.002429,0.001283,0.000662,0.000825,0.001460,0.002472,0.004136,0.004073,0.002334,0.001367,0.000828,0.000489,0.000266,0.000125,0.000105,0.000173,0.000193,0.000114
  giAltoOh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.004962,0.010483,0.020163,0.038322,0.087672,0.170724,0.066701,0.038951,0.034708,0.056519,0.032353,0.014706,0.008302,0.004569,0.002143,0.000687,0.000355,0.000485,0.000379,0.000169,0.000126,0.000109,0.000144,0.000141,0.000416,0.001139,0.002447,0.004744,0.009383,0.020945,0.017383,0.007938,0.004095,0.002042,0.000896,0.000951,0.001864,0.003489,0.005470,0.003280,0.001715,0.000944,0.000498,0.000228,0.000109,0.000199,0.000222,0.000115
  giAltoOh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.006900,0.016262,0.035055,0.083207,0.208846,0.072590,0.041922,0.048400,0.054385,0.018894,0.009431,0.004519,0.001606,0.000377,0.000573,0.000445,0.000153,0.000115,0.000150,0.000151,0.000158,0.000452,0.001488,0.003562,0.007739,0.018995,0.023248,0.009091,0.004149,0.001716,0.000922,0.001977,0.004164,0.006325,0.003036,0.001458,0.000702,0.000287,0.000117,0.000240,0.000232,0.000105
  giAltoOh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.009796,0.025603,0.065926,0.250014,0.090217,0.048269,0.075817,0.034166,0.013762,0.005873,0.001719,0.000494,0.000660,0.000298,0.000114,0.000177,0.000166,0.000192,0.000672,0.002456,0.006469,0.017288,0.029726,0.009883,0.003885,0.001258,0.001818,0.004479,0.007363,0.002986,0.001274,0.000492,0.000134,0.000177,0.000345,0.000157
  giAltoOh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.014156,0.043198,0.156079,0.133513,0.058152,0.098380,0.029278,0.010778,0.003174,0.000545,0.000767,0.000229,0.000206,0.000126,0.000217,0.000289,0.001848,0.006291,0.019985,0.029302,0.008528,0.002573,0.001656,0.004961,0.008168,0.002742,0.000983,0.000252,0.000109,0.000211,0.000387,0.000132
  giAltoOh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.020980,0.076697,0.341641,0.077953,0.113133,0.029431,0.009144,0.001375,0.000970,0.000338,0.000253,0.000198,0.000284,0.002284,0.009509,0.041987,0.015901,0.004091,0.001908,0.007000,0.006575,0.001893,0.000457,0.000130,0.000219,0.000443,0.000123
  giAltoOh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.032525,0.166419,0.145169,0.096808,0.037784,0.009041,0.000762,0.000889,0.000227,0.000137,0.000311,0.000902,0.007130,0.038020,0.018180,0.003431,0.003954,0.012653,0.002915,0.000570,0.000161,0.000236,0.000466,0.000102
  giAltoOh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.051206,0.500042,0.096536,0.068326,0.011745,0.000989,0.000597,0.000354,0.000333,0.001345,0.012944,0.059447,0.007770,0.003639,0.014729,0.002548,0.000268,0.000152,0.000355,0.000315
  giAltoOh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.086407,0.267040,0.196788,0.021563,0.001095,0.000450,0.000254,0.000578,0.012590,0.058595,0.005142,0.009927,0.005483,0.000502,0.000175,0.000149,0.000775
  giAltoOh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.153433,0.155941,0.058873,0.002749,0.000677,0.000168,0.000395,0.004570,0.084026,0.008181,0.014008,0.003788,0.000261,0.000109,0.000887
  giAltoOh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.332841,0.193621,0.018082,0.001777,0.000274,0.000623,0.014262,0.036353,0.007910,0.005829,0.000322,0.000104,0.000472,0.000204
  giAltoOh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,1.000000,0.136629,0.001978,0.000708,0.000666,0.025888,0.015536,0.029450,0.000535,0.000119,0.000710,0.000111
  giAltoOh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.534019,0.043120,0.000901,0.000124,0.025180,0.010281,0.010963,0.000350,0.000297,0.000193
  giAltoOh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.311907,0.005497,0.000334,0.009143,0.016361,0.007576,0.000139,0.001773
  giAltoOh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.387236,0.003555,0.001246,0.072704,0.011658,0.000208,0.000409
  giAltoOh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.273266,0.001417,0.051783,0.058903,0.000238,0.000221
  giAltoOh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.086239,0.000247,0.020561,0.000699,0.000385
 elseif iVoice==20 then
  giAltoOoh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.006790,0.007793,0.009214,0.011657,0.015318,0.019466,0.030008,0.039968,0.083795,0.111525,0.078922,0.039204,0.030853,0.020835,0.017482,0.014376,0.012615,0.012668,0.013183,0.017897,0.025293,0.022448,0.012428,0.008118,0.005897,0.004389,0.003515,0.002702,0.002132,0.001638,0.001205,0.000879,0.000585,0.000364,0.000202,0.000102,0.000109,0.000138,0.000164,0.000161,0.000150,0.000126,0.000128,0.000187,0.000263,0.000356,0.000471,0.000613,0.000790,0.001010,0.001285,0.001628,0.002033,0.002437,0.002674,0.002567,0.002185,0.001749,0.001375,0.001076,0.000841,0.000653,0.000504,0.000384,0.000287,0.000212,0.000152,0.000108
  giAltoOoh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.008303,0.009791,0.012449,0.017203,0.022681,0.037000,0.060114,0.125058,0.111065,0.049238,0.035746,0.023284,0.018963,0.015616,0.014922,0.015724,0.023589,0.030602,0.019515,0.010173,0.007095,0.005069,0.003809,0.002822,0.002111,0.001470,0.001008,0.000607,0.000336,0.000144,0.000127,0.000167,0.000197,0.000186,0.000164,0.000119,0.000110,0.000180,0.000275,0.000399,0.000559,0.000764,0.001030,0.001376,0.001826,0.002391,0.002958,0.003205,0.002870,0.002255,0.001699,0.001269,0.000946,0.000699,0.000510,0.000363,0.000253,0.000171,0.000113
  giAltoOoh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.010169,0.012589,0.017650,0.024669,0.043280,0.081468,0.157756,0.089091,0.046827,0.028403,0.022088,0.017894,0.018301,0.023035,0.036512,0.022702,0.011378,0.007064,0.005144,0.003564,0.002530,0.001649,0.001018,0.000530,0.000218,0.000148,0.000204,0.000233,0.000212,0.000159,0.000100,0.000185,0.000312,0.000486,0.000722,0.001039,0.001472,0.002066,0.002849,0.003616,0.003709,0.002969,0.002140,0.001514,0.001066,0.000741,0.000504,0.000331,0.000209,0.000128
  giAltoOoh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.012545,0.016792,0.026024,0.045873,0.091044,0.186641,0.074850,0.043441,0.028672,0.021632,0.021853,0.032802,0.039616,0.015496,0.009086,0.006060,0.003944,0.002536,0.001505,0.000730,0.000267,0.000184,0.000267,0.000264,0.000210,0.000125,0.000135,0.000272,0.000482,0.000785,0.001216,0.001840,0.002746,0.003908,0.004522,0.003656,0.002483,0.001644,0.001079,0.000691,0.000424,0.000246,0.000138
  giAltoOoh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.015591,0.023322,0.038944,0.079963,0.223136,0.078440,0.041689,0.028767,0.025351,0.035817,0.044928,0.016249,0.008786,0.005409,0.003279,0.001761,0.000729,0.000205,0.000277,0.000322,0.000253,0.000133,0.000168,0.000376,0.000714,0.001232,0.002029,0.003271,0.004898,0.005161,0.003518,0.002165,0.001315,0.000773,0.000427,0.000218,0.000115
  giAltoOoh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.019557,0.034366,0.073907,0.249826,0.098371,0.046511,0.031196,0.031410,0.061129,0.020319,0.010124,0.005639,0.002938,0.001212,0.000288,0.000334,0.000372,0.000238,0.000103,0.000220,0.000550,0.001116,0.002056,0.003647,0.005908,0.005733,0.003394,0.001890,0.001019,0.000506,0.000227,0.000115
  giAltoOoh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.025184,0.049350,0.163002,0.178182,0.056821,0.035802,0.046106,0.045414,0.014134,0.007132,0.003299,0.001061,0.000296,0.000463,0.000314,0.000107,0.000120,0.000103,0.000103,0.000367,0.000973,0.002088,0.004156,0.007278,0.005975,0.003047,0.001492,0.000668,0.000258,0.000125,0.000108,0.000107
  giAltoOoh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.033583,0.091750,0.373265,0.086876,0.043265,0.065631,0.030991,0.012123,0.005072,0.001461,0.000369,0.000528,0.000250,0.000145,0.000109,0.000271,0.000967,0.002440,0.005510,0.009071,0.004981,0.002164,0.000851,0.000277,0.000138,0.000103,0.000143
  giAltoOoh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.046646,0.159951,0.156876,0.057537,0.071660,0.032501,0.010818,0.003521,0.000411,0.000638,0.000258,0.000149,0.000132,0.000138,0.000328,0.001425,0.004063,0.009814,0.007048,0.002634,0.000855,0.000228,0.000144,0.000107,0.000164
  giAltoOoh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.068767,0.499897,0.093067,0.062857,0.040662,0.011282,0.002425,0.000668,0.000477,0.000140,0.000137,0.000132,0.000162,0.001101,0.004121,0.011845,0.006803,0.002043,0.000456,0.000183,0.000114,0.000193
  giAltoOoh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.098701,0.356319,0.071604,0.090809,0.014265,0.002121,0.000932,0.000159,0.000235,0.000129,0.000214,0.001955,0.008318,0.011948,0.002984,0.000517,0.000201,0.000125,0.000215
  giAltoOoh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.183506,0.173742,0.131280,0.024244,0.002922,0.001057,0.000152,0.000219,0.001935,0.011024,0.009962,0.001703,0.000276,0.000113,0.000181,0.000173
  giAltoOoh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.319941,0.115082,0.065007,0.007044,0.001288,0.000283,0.000127,0.000673,0.008134,0.014095,0.001709,0.000289,0.000332
  giAltoOoh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,1.000001,0.125741,0.022569,0.001337,0.000279,0.000263,0.002205,0.023707,0.004085,0.000367,0.000387
  giAltoOoh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.712592,0.181596,0.004243,0.000318,0.000259,0.003910,0.023893,0.001033,0.000156,0.000248,0.000124
  giAltoOoh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.347452,0.048482,0.002113,0.000185,0.003869,0.019916,0.000554,0.000125,0.000363
  giAltoOoh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.230150,0.014084,0.000566,0.001346,0.028182,0.000579,0.000168,0.000119
  giAltoOoh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.251504,0.002674,0.000531,0.047427,0.000734,0.000774
  giAltoOoh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.363188,0.000636,0.007820,0.002067,0.000109,0.000249
  giAltoOoh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.096965,0.000370,0.039833,0.000247,0.000141
  giAltoOoh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.028166,0.002692,0.001157,0.000239
 /* SOPRANO */
 elseif iVoice==21 then
  giSopranoAh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.000243,0.000331,0.000420,0.000500,0.000551,0.000561,0.000535,0.000457,0.000407,0.000521,0.000895,0.001468,0.002246,0.003261,0.004498,0.006101,0.008059,0.010521,0.013877,0.018007,0.024724,0.033985,0.051610,0.076040,0.075738,0.052053,0.036243,0.028064,0.022858,0.020510,0.019512,0.020774,0.023901,0.031564,0.037490,0.031794,0.021090,0.015085,0.011109,0.008541,0.006680,0.005201,0.004060,0.003097,0.002311,0.001666,0.001140,0.000733,0.000435,0.000268,0.000244,0.000282,0.000311,0.000312,0.000289,0.000247,0.000195,0.000140,0.000112,0.000165,0.000229,0.000305,0.000401,0.000521,0.000683,0.000907,0.001221,0.001582,0.001659,0.001357,0.001002,0.000751,0.000566,0.000433,0.000328,0.000245,0.000178,0.000123
  giSopranoAh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.000306,0.000437,0.000557,0.000646,0.000669,0.000627,0.000515,0.000518,0.000901,0.001666,0.002762,0.004254,0.006165,0.008737,0.011991,0.016649,0.022748,0.033722,0.050609,0.085620,0.090728,0.057631,0.039541,0.029400,0.025061,0.023243,0.025034,0.030554,0.042366,0.041517,0.026592,0.017689,0.012328,0.009171,0.006810,0.005073,0.003690,0.002597,0.001732,0.001065,0.000592,0.000326,0.000296,0.000351,0.000375,0.000355,0.000301,0.000227,0.000150,0.000102,0.000108,0.000106,0.000176,0.000263,0.000371,0.000511,0.000698,0.000974,0.001379,0.001893,0.001941,0.001448,0.001016,0.000723,0.000525,0.000378,0.000266,0.000177,0.000111,0.000134,0.000234,0.000381,0.000580,0.000844,0.001189,0.001641,0.002250,0.003093,0.004320,0.006062,0.007661
  giSopranoAh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.000390,0.000574,0.000730,0.000796,0.000751,0.000597,0.000704,0.001497,0.002843,0.004841,0.007543,0.011311,0.016537,0.024179,0.037585,0.060986,0.108486,0.093219,0.053086,0.036955,0.029857,0.027640,0.031245,0.043881,0.052110,0.032914,0.020217,0.013324,0.009422,0.006627,0.004584,0.003021,0.001841,0.000997,0.000471,0.000343,0.000417,0.000445,0.000401,0.000308,0.000200,0.000106,0.000107,0.000127,0.000122,0.000173,0.000287,0.000435,0.000637,0.000926,0.001386,0.002094,0.002326,0.001646,0.001087,0.000731,0.000499,0.000330,0.000205,0.000116,0.000177,0.000335,0.000575,0.000913,0.001379,0.002023,0.002940,0.004333,0.006473,0.008929,0.008243,0.005616,0.003765,0.002575,0.001770,0.001205,0.000794,0.000500,0.000293,0.000161
  giSopranoAh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.000506,0.000759,0.000930,0.000920,0.000718,0.000914,0.002228,0.004508,0.007891,0.012923,0.020316,0.031841,0.054837,0.111349,0.120063,0.062505,0.041150,0.033769,0.034811,0.047594,0.062278,0.035528,0.020648,0.013186,0.008671,0.005635,0.003425,0.001850,0.000831,0.000409,0.000484,0.000529,0.000455,0.000312,0.000163,0.000128,0.000152,0.000137,0.000193,0.000356,0.000577,0.000900,0.001436,0.002358,0.002764,0.001789,0.001100,0.000697,0.000432,0.000248,0.000125,0.000200,0.000427,0.000795,0.001343,0.002141,0.003346,0.005301,0.008544,0.010941,0.007840,0.004856,0.003081,0.001971,0.001236,0.000735,0.000401,0.000199,0.000112,0.000101,0.000129,0.000193,0.000281,0.000339,0.000248,0.000156,0.000103
  giSopranoAh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.000663,0.001001,0.001123,0.000915,0.001042,0.002938,0.006525,0.012209,0.021053,0.036037,0.068016,0.152197,0.104196,0.056180,0.041062,0.041595,0.063208,0.063675,0.030215,0.017110,0.010421,0.006205,0.003340,0.001470,0.000537,0.000565,0.000626,0.000496,0.000281,0.000117,0.000145,0.000181,0.000152,0.000138,0.000333,0.000614,0.001047,0.001825,0.003186,0.002732,0.001512,0.000873,0.000495,0.000248,0.000109,0.000106,0.000245,0.000590,0.001185,0.002126,0.003633,0.006253,0.010976,0.012260,0.007210,0.004145,0.002442,0.001401,0.000743,0.000345,0.000154,0.000120,0.000112,0.000142,0.000230,0.000359,0.000380,0.000227,0.000136
  giSopranoAh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.000871,0.001289,0.001252,0.001035,0.003325,0.008494,0.017445,0.033243,0.067334,0.170973,0.115091,0.058712,0.046417,0.061016,0.082910,0.035326,0.018317,0.010133,0.005189,0.002129,0.000651,0.000702,0.000709,0.000454,0.000173,0.000166,0.000215,0.000159,0.000213,0.000526,0.001023,0.001949,0.003788,0.002898,0.001448,0.000757,0.000355,0.000135,0.000106,0.000155,0.000469,0.001164,0.002384,0.004512,0.008666,0.015371,0.010478,0.005371,0.002862,0.001463,0.000658,0.000247,0.000145,0.000134,0.000130,0.000242,0.000414,0.000435,0.000230,0.000129
  giSopranoAh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.001148,0.001591,0.001193,0.002995,0.009688,0.022636,0.048394,0.122093,0.186546,0.073972,0.055348,0.087905,0.065898,0.026687,0.013278,0.006054,0.001998,0.000689,0.000892,0.000619,0.000212,0.000215,0.000244,0.000119,0.000343,0.000873,0.001866,0.004223,0.003315,0.001471,0.000664,0.000232,0.000124,0.000185,0.000679,0.001851,0.004101,0.008791,0.018115,0.011387,0.005227,0.002448,0.001017,0.000329,0.000170,0.000149,0.000119,0.000263,0.000501,0.000460,0.000214,0.000113
  giSopranoAh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.001518,0.001840,0.001829,0.009018,0.025849,0.063699,0.222787,0.125025,0.067561,0.095260,0.071075,0.026387,0.011277,0.003702,0.000818,0.001059,0.000625,0.000181,0.000305,0.000184,0.000153,0.000714,0.001808,0.004736,0.003592,0.001400,0.000498,0.000149,0.000193,0.000861,0.002706,0.006746,0.017233,0.015795,0.006213,0.002495,0.000811,0.000226,0.000191,0.000161,0.000390,0.000688,0.000316,0.000144
  giSopranoAh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.002001,0.001828,0.005878,0.024425,0.072091,0.304494,0.112379,0.083227,0.127363,0.034228,0.012415,0.002941,0.001132,0.000993,0.000234,0.000361,0.000158,0.000116,0.000662,0.002099,0.006388,0.003027,0.000988,0.000219,0.000213,0.001186,0.004269,0.012560,0.024604,0.008321,0.002813,0.000693,0.000241,0.000162,0.000153,0.000462,0.000765,0.000273,0.000112
  giSopranoAh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.002580,0.002072,0.017002,0.066543,0.342244,0.117509,0.122162,0.070710,0.020284,0.004261,0.001405,0.000907,0.000333,0.000319,0.000105,0.001054,0.003908,0.005806,0.001517,0.000271,0.000310,0.002335,0.009056,0.030843,0.010772,0.002935,0.000495,0.000270,0.000120,0.000486,0.000873,0.000259
  giSopranoAh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.003183,0.005991,0.045273,0.244218,0.147927,0.175844,0.053372,0.012105,0.001376,0.001237,0.000431,0.000243,0.000165,0.000696,0.003738,0.006635,0.001332,0.000247,0.001360,0.008210,0.036260,0.010453,0.002030,0.000343,0.000153,0.000529,0.000919,0.000227
  giSopranoAh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.003681,0.018037,0.127409,0.250032,0.190549,0.052769,0.007402,0.002118,0.000363,0.000368,0.000180,0.001429,0.009476,0.002801,0.000297,0.001724,0.013504,0.031591,0.004990,0.000452,0.000189,0.000780,0.000631,0.000132
  giSopranoAh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.003660,0.048855,0.609099,0.166474,0.068463,0.005879,0.001985,0.000726,0.000219,0.001335,0.012780,0.001982,0.000427,0.008543,0.049218,0.005623,0.000484,0.000309,0.001530,0.000224
  giSopranoAh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.004145,0.133126,0.235070,0.141452,0.008522,0.001818,0.000636,0.000100,0.007822,0.003035,0.000622,0.018126,0.021549,0.000989,0.000240,0.001747,0.000173
  giSopranoAh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.011982,0.488465,0.351711,0.024209,0.002473,0.000485,0.001393,0.013267,0.000494,0.016424,0.020902,0.000686,0.001057,0.000454
  giSopranoAh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.036071,0.499985,0.105518,0.004235,0.000736,0.002858,0.005599,0.003447,0.063157,0.000904,0.001561,0.000264
  giSopranoAh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.097704,0.332922,0.011754,0.001452,0.002670,0.003963,0.017086,0.011243,0.000618,0.000448
  giSopranoAh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.266284,0.282922,0.003635,0.000202,0.006066,0.036263,0.001979,0.003494
  giSopranoAh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.976912,0.048417,0.000970,0.026534,0.032847,0.001372,0.000907
  giSopranoAh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,1.000001,0.008470,0.005716,0.006895,0.001809,0.000528
  giSopranoAh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.665845,0.002904,0.007927,0.022486,0.000897
 elseif iVoice==22 then
  giSopranoEh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.005375,0.006159,0.007497,0.009319,0.012241,0.016007,0.021786,0.031498,0.046972,0.085088,0.095763,0.062094,0.035620,0.026336,0.017966,0.013939,0.010264,0.007721,0.005804,0.004160,0.003000,0.002025,0.001285,0.000766,0.000371,0.000201,0.000222,0.000300,0.000319,0.000316,0.000283,0.000227,0.000174,0.000113,0.000102,0.000163,0.000248,0.000363,0.000507,0.000697,0.000930,0.001233,0.001635,0.002152,0.002976,0.004104,0.006179,0.007906,0.006942,0.004677,0.003337,0.002428,0.001839,0.001407,0.001067,0.000799,0.000561,0.000352,0.000151,0.000282,0.000528,0.000809,0.001145,0.001552,0.002050,0.002702,0.003535,0.004730,0.006447,0.009098,0.012551,0.013076,0.009939,0.006919,0.005067,0.003738,0.002853,0.002164,0.001644,0.001235,0.000905,0.000647,0.000439,0.000277,0.000158
  giSopranoEh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.006525,0.007911,0.010016,0.013570,0.018820,0.027493,0.040827,0.078633,0.116838,0.079159,0.041386,0.027974,0.019129,0.013581,0.009612,0.006849,0.004621,0.003064,0.001847,0.001036,0.000450,0.000231,0.000298,0.000377,0.000380,0.000341,0.000263,0.000186,0.000103,0.000174,0.000289,0.000453,0.000672,0.000967,0.001352,0.001893,0.002630,0.003869,0.005875,0.008999,0.008671,0.005518,0.003725,0.002567,0.001876,0.001352,0.000961,0.000627,0.000335,0.000265,0.000606,0.001006,0.001504,0.002127,0.002953,0.004076,0.005751,0.008319,0.012764,0.016038,0.012717,0.008268,0.005702,0.004030,0.002916,0.002101,0.001501,0.001036,0.000687,0.000418,0.000226,0.000113,0.000126,0.000192,0.000258,0.000326,0.000405,0.000509,0.000644,0.000773,0.000757,0.000593,0.000433,0.000318,0.000237,0.000178,0.000133
  giSopranoEh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.007934,0.010268,0.014014,0.020866,0.031775,0.051069,0.115300,0.124988,0.056004,0.035128,0.022210,0.014648,0.009836,0.006282,0.003915,0.002162,0.001056,0.000377,0.000301,0.000440,0.000452,0.000392,0.000282,0.000165,0.000151,0.000284,0.000497,0.000798,0.001223,0.001822,0.002695,0.004186,0.006781,0.010969,0.009099,0.005272,0.003380,0.002303,0.001560,0.001027,0.000581,0.000179,0.000255,0.000730,0.001308,0.002061,0.003062,0.004509,0.006743,0.010496,0.017164,0.017655,0.010910,0.006960,0.004610,0.003129,0.002119,0.001387,0.000863,0.000480,0.000225,0.000124,0.000196,0.000291,0.000386,0.000499,0.000657,0.000858,0.000920,0.000712,0.000490,0.000340,0.000241,0.000172,0.000121
  giSopranoEh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.009785,0.013551,0.020874,0.032859,0.059208,0.148132,0.120179,0.053098,0.029187,0.018238,0.011338,0.006607,0.003610,0.001666,0.000527,0.000362,0.000533,0.000521,0.000386,0.000227,0.000106,0.000248,0.000500,0.000893,0.001489,0.002395,0.003832,0.006593,0.012423,0.010837,0.005786,0.003451,0.002202,0.001367,0.000718,0.000151,0.000475,0.001206,0.002167,0.003503,0.005548,0.009076,0.015888,0.022570,0.014146,0.008203,0.005068,0.003190,0.001979,0.001150,0.000585,0.000233,0.000166,0.000296,0.000428,0.000584,0.000810,0.001083,0.001004,0.000662,0.000427,0.000283,0.000188,0.000123,0.000110,0.000151,0.000172,0.000144,0.000101
  giSopranoEh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.012322,0.018644,0.032024,0.063018,0.170241,0.124239,0.052696,0.027893,0.015451,0.008324,0.004052,0.001533,0.000402,0.000600,0.000632,0.000454,0.000226,0.000100,0.000205,0.000497,0.001017,0.001865,0.003279,0.005970,0.012399,0.013933,0.006699,0.003693,0.002143,0.001128,0.000303,0.000567,0.001626,0.003121,0.005435,0.009514,0.018307,0.026321,0.013931,0.007528,0.004359,0.002490,0.001304,0.000559,0.000184,0.000297,0.000486,0.000704,0.001040,0.001332,0.000946,0.000559,0.000342,0.000211,0.000126,0.000132,0.000189,0.000199,0.000143
  giSopranoEh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.015802,0.027106,0.054923,0.157076,0.158131,0.055887,0.027131,0.013682,0.006122,0.002071,0.000461,0.000753,0.000682,0.000372,0.000101,0.000111,0.000194,0.000578,0.001344,0.002701,0.005258,0.011744,0.017335,0.007449,0.003752,0.001922,0.000671,0.000529,0.002012,0.004255,0.008157,0.016650,0.032107,0.016554,0.008071,0.004209,0.002076,0.000838,0.000227,0.000384,0.000654,0.001019,0.001550,0.001189,0.000637,0.000357,0.000199,0.000105,0.000142,0.000222,0.000233,0.000149
  giSopranoEh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.020540,0.041742,0.102173,0.250022,0.070268,0.029304,0.012569,0.004326,0.000755,0.000881,0.000785,0.000331,0.000140,0.000302,0.000996,0.002454,0.005411,0.013623,0.018252,0.006779,0.003128,0.001162,0.000516,0.002634,0.006167,0.013587,0.034596,0.021969,0.009288,0.004271,0.001737,0.000452,0.000399,0.000782,0.001331,0.001862,0.000991,0.000488,0.000244,0.000113,0.000153,0.000264,0.000258,0.000143
  giSopranoEh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.027101,0.065718,0.296280,0.106187,0.036474,0.013213,0.003330,0.000725,0.001042,0.000455,0.000159,0.000213,0.001002,0.002983,0.007680,0.024903,0.011593,0.004412,0.001440,0.000954,0.004351,0.011141,0.031928,0.028388,0.010175,0.003972,0.001172,0.000336,0.000864,0.001632,0.002019,0.000860,0.000380,0.000158,0.000105,0.000224,0.000349,0.000206
  giSopranoEh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.037289,0.126049,0.248462,0.055787,0.016649,0.003067,0.001199,0.000908,0.000105,0.000195,0.000408,0.002033,0.006568,0.024837,0.013399,0.004282,0.000601,0.003260,0.010894,0.036706,0.027905,0.008733,0.002613,0.000372,0.000976,0.002088,0.001895,0.000683,0.000249,0.000103,0.000265,0.000399,0.000182
  giSopranoEh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.054241,0.314339,0.111819,0.027377,0.004143,0.001507,0.000744,0.000202,0.000389,0.002690,0.010531,0.034705,0.007511,0.001342,0.004034,0.016350,0.064332,0.016169,0.004160,0.000455,0.001312,0.003109,0.001277,0.000398,0.000117,0.000282,0.000466,0.000173
  giSopranoEh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.083485,0.500013,0.058605,0.008650,0.001763,0.000659,0.000290,0.000127,0.001996,0.010824,0.036509,0.006267,0.001028,0.012340,0.069232,0.018578,0.003478,0.000794,0.002663,0.001984,0.000490,0.000121,0.000310,0.000517,0.000148
  giSopranoEh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.131440,0.212366,0.026425,0.001451,0.000910,0.000318,0.000426,0.005968,0.049821,0.008823,0.001910,0.022293,0.056767,0.007944,0.000675,0.003266,0.001719,0.000315,0.000447,0.000411
  giSopranoEh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.252130,0.111578,0.006131,0.001816,0.000401,0.000821,0.013137,0.026816,0.001212,0.021797,0.055807,0.005226,0.001951,0.003795,0.000506,0.000104,0.000211,0.000803,0.000143
  giSopranoEh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.628823,0.054762,0.003015,0.000404,0.000779,0.021073,0.015027,0.008072,0.128717,0.008320,0.002625,0.002555,0.000234,0.000174,0.000934,0.000112
  giSopranoEh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000000,0.017299,0.001321,0.000255,0.021650,0.012533,0.024684,0.037149,0.001588,0.003967,0.000242,0.000619,0.000296
  giSopranoEh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.424682,0.002901,0.000637,0.011935,0.017641,0.044586,0.015886,0.006533,0.000631,0.000895,0.000169
  giSopranoEh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.223136,0.003631,0.001641,0.053621,0.043595,0.010450,0.007588,0.000208,0.000421,0.000286
  giSopranoEh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.109531,0.000806,0.042151,0.016149,0.016640,0.005110,0.000133,0.001866
  giSopranoEh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.034598,0.000509,0.025064,0.074296,0.007935,0.000130,0.000593
  giSopranoEh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.005801,0.023872,0.089183,0.013066,0.000176,0.000337
  giSopranoEh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.007260,0.107242,0.020899,0.000417,0.000572
 elseif iVoice==23 then
  giSopranoEee24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.010638,0.013301,0.017800,0.022937,0.034070,0.046177,0.082117,0.124931,0.109118,0.060110,0.041165,0.028412,0.021053,0.015886,0.011808,0.008953,0.006545,0.004708,0.003285,0.002126,0.001294,0.000655,0.000252,0.000225,0.000362,0.000444,0.000451,0.000423,0.000354,0.000271,0.000187,0.000100,0.000102,0.000106,0.000127,0.000140,0.000138,0.000125,0.000158,0.000300,0.000501,0.000769,0.001120,0.001555,0.002122,0.002825,0.003721,0.004928,0.006460,0.008829,0.012145,0.017913,0.024865,0.024284,0.017013,0.011692,0.008487,0.006258,0.004772,0.003600,0.002730,0.002031,0.001468,0.001022,0.000652,0.000355,0.000117,0.000141,0.000322,0.000509,0.000701,0.000919,0.001190,0.001528,0.002038,0.002743,0.003904,0.004865,0.004376,0.003081,0.002205,0.001615,0.001216,0.000928,0.000704,0.000533,0.000394,0.000284
  giSopranoEee27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.013143,0.017627,0.024484,0.036616,0.054010,0.106371,0.151537,0.098520,0.052871,0.035393,0.024586,0.017401,0.012353,0.008770,0.005907,0.003868,0.002280,0.001215,0.000447,0.000241,0.000427,0.000533,0.000532,0.000465,0.000348,0.000231,0.000110,0.000104,0.000121,0.000112,0.000126,0.000155,0.000168,0.000158,0.000125,0.000106,0.000207,0.000430,0.000749,0.001201,0.001793,0.002594,0.003630,0.005050,0.007012,0.009981,0.014625,0.023409,0.030837,0.024037,0.014956,0.010218,0.007150,0.005156,0.003700,0.002635,0.001801,0.001168,0.000663,0.000277,0.000105,0.000356,0.000621,0.000897,0.001231,0.001659,0.002305,0.003282,0.004967,0.005817,0.004341,0.002841,0.001963,0.001402,0.001015,0.000731,0.000517,0.000352,0.000228,0.000134,0.000126,0.000225,0.000358,0.000534,0.000764,0.001064,0.001470,0.002034,0.002872,0.004116,0.005339
  giSopranoEee30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.016328,0.023966,0.035128,0.059257,0.121512,0.177707,0.090670,0.053040,0.031736,0.021506,0.014296,0.009353,0.005850,0.003306,0.001623,0.000515,0.000325,0.000586,0.000639,0.000576,0.000417,0.000250,0.000114,0.000143,0.000131,0.000145,0.000186,0.000199,0.000175,0.000123,0.000211,0.000510,0.000977,0.001671,0.002629,0.003977,0.005882,0.008680,0.013457,0.021730,0.035492,0.029923,0.017181,0.010895,0.007320,0.004922,0.003309,0.002111,0.001243,0.000586,0.000119,0.000355,0.000727,0.001119,0.001622,0.002326,0.003519,0.005702,0.006806,0.004491,0.002815,0.001839,0.001255,0.000850,0.000559,0.000348,0.000196,0.000189,0.000349,0.000574,0.000885,0.001314,0.001924,0.002866,0.004356,0.006214,0.005671,0.003728,0.002462,0.001676,0.001149,0.000782,0.000516,0.000324,0.000187
  giSopranoEee33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.020626,0.032969,0.058370,0.115099,0.213096,0.095777,0.052675,0.031539,0.019117,0.011719,0.006681,0.003278,0.001235,0.000319,0.000658,0.000760,0.000651,0.000420,0.000175,0.000135,0.000172,0.000141,0.000168,0.000224,0.000231,0.000176,0.000187,0.000547,0.001202,0.002233,0.003774,0.006037,0.009634,0.016040,0.029029,0.043390,0.025586,0.014544,0.008907,0.005565,0.003428,0.001951,0.000904,0.000184,0.000459,0.000990,0.001589,0.002432,0.003929,0.006894,0.007727,0.004364,0.002585,0.001631,0.001026,0.000621,0.000346,0.000162,0.000243,0.000488,0.000854,0.001385,0.002187,0.003507,0.005798,0.007630,0.005258,0.003185,0.002005,0.001280,0.000803,0.000478,0.000258,0.000123,0.000170,0.000276,0.000437,0.000708,0.000961,0.000685,0.000413,0.000261,0.000170,0.000110
  giSopranoEee36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.026610,0.045888,0.092383,0.249950,0.120266,0.056848,0.031787,0.017916,0.009422,0.004255,0.001310,0.000450,0.000889,0.000846,0.000542,0.000201,0.000187,0.000191,0.000120,0.000112,0.000212,0.000280,0.000251,0.000180,0.000602,0.001541,0.003119,0.005667,0.009889,0.017719,0.035957,0.048756,0.023480,0.012570,0.007234,0.004081,0.002055,0.000713,0.000283,0.001024,0.001849,0.003076,0.005523,0.009798,0.006208,0.003254,0.001871,0.001075,0.000573,0.000258,0.000108,0.000344,0.000740,0.001368,0.002373,0.004137,0.007509,0.008478,0.004765,0.002699,0.001585,0.000909,0.000481,0.000219,0.000165,0.000304,0.000524,0.000930,0.001082,0.000610,0.000347,0.000208,0.000124
  giSopranoEee39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.035209,0.073147,0.212478,0.196804,0.070708,0.034761,0.017520,0.007728,0.002428,0.000481,0.001065,0.000929,0.000462,0.000208,0.000225,0.000124,0.000181,0.000310,0.000315,0.000211,0.000859,0.002401,0.005185,0.010096,0.019955,0.046808,0.048070,0.020438,0.010314,0.005271,0.002337,0.000554,0.000712,0.001795,0.003321,0.006570,0.011649,0.005690,0.002809,0.001465,0.000706,0.000269,0.000254,0.000717,0.001531,0.002949,0.005763,0.010715,0.006995,0.003499,0.001858,0.000950,0.000424,0.000150,0.000118,0.000278,0.000538,0.001060,0.001234,0.000610,0.000321,0.000174
  giSopranoEee42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.047942,0.118546,0.355496,0.106101,0.043023,0.018713,0.006614,0.001030,0.001174,0.001153,0.000500,0.000284,0.000191,0.000193,0.000380,0.000359,0.000423,0.001960,0.005278,0.011817,0.027041,0.071322,0.034504,0.014706,0.006650,0.002498,0.000238,0.001464,0.003270,0.007097,0.013721,0.005674,0.002532,0.001128,0.000397,0.000382,0.001165,0.002666,0.005818,0.012613,0.007557,0.003399,0.001587,0.000656,0.000201,0.000104,0.000255,0.000576,0.001288,0.001283,0.000555,0.000270,0.000125
  giSopranoEee45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.065934,0.230214,0.191525,0.063074,0.023438,0.006556,0.000638,0.001521,0.000841,0.000343,0.000179,0.000337,0.000464,0.000376,0.002409,0.007565,0.019312,0.058226,0.051263,0.017853,0.006873,0.001812,0.000923,0.003191,0.007896,0.015518,0.005189,0.002060,0.000693,0.000490,0.001722,0.004412,0.011699,0.010593,0.004042,0.001620,0.000520,0.000134,0.000111,0.000344,0.000887,0.001947,0.000836,0.000344,0.000137
  giSopranoEee48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.091783,0.499950,0.113693,0.035833,0.008510,0.000901,0.001693,0.000403,0.000370,0.000237,0.000436,0.000513,0.001200,0.006244,0.019806,0.072046,0.047005,0.014484,0.004113,0.000568,0.003707,0.011075,0.012437,0.003752,0.001151,0.000146,0.000689,0.002748,0.008313,0.017019,0.005417,0.001825,0.000438,0.000148,0.000332,0.001055,0.002175,0.000698,0.000249
  giSopranoEee51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.146370,0.393762,0.069555,0.015463,0.000962,0.001858,0.000450,0.000362,0.000631,0.001719,0.010386,0.039977,0.096231,0.020650,0.004678,0.001429,0.006658,0.023341,0.005628,0.001415,0.001439,0.005919,0.021497,0.007017,0.001904,0.000301,0.000156,0.000237,0.001080,0.002476,0.000648,0.000179
  giSopranoEee54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.237094,0.212185,0.037422,0.002059,0.002305,0.000169,0.000384,0.000744,0.000848,0.010559,0.054094,0.068997,0.013299,0.000476,0.006544,0.027445,0.005064,0.000793,0.000765,0.005332,0.025238,0.006800,0.001313,0.000208,0.000144,0.001157,0.002565,0.000537
  giSopranoEee57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.460456,0.126143,0.013110,0.003041,0.000110,0.000358,0.000928,0.004819,0.038635,0.102507,0.013746,0.001848,0.015801,0.010379,0.001387,0.000981,0.008831,0.021187,0.003241,0.000268,0.000155,0.001775,0.001673,0.000275
  giSopranoEee60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.999999,0.071669,0.001801,0.000804,0.000480,0.001006,0.012498,0.144146,0.028964,0.001138,0.022162,0.007499,0.000284,0.005499,0.034034,0.003651,0.000296,0.000666,0.004356,0.000498
  giSopranoEee63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.787626,0.030929,0.003718,0.000901,0.001263,0.020781,0.192483,0.009358,0.013320,0.011259,0.000185,0.011846,0.014037,0.000603,0.000473,0.004954,0.000357
  giSopranoEee66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.424357,0.004117,0.000335,0.001490,0.021121,0.137972,0.000951,0.054887,0.001585,0.010667,0.013598,0.000416,0.002313,0.001073
  giSopranoEee69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.252260,0.006081,0.000717,0.009639,0.204955,0.003697,0.020753,0.001961,0.042356,0.000535,0.003552,0.000551
  giSopranoEee72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.143330,0.001607,0.002011,0.288307,0.002276,0.014994,0.010998,0.007299,0.001333,0.000996
  giSopranoEee75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.061863,0.001803,0.041574,0.018718,0.022519,0.023699,0.001207,0.009907,0.000135
  giSopranoEee78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.008237,0.002981,0.275939,0.109772,0.021334,0.000832,0.002147
  giSopranoEee81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.012164,0.019280,0.007395,0.003926,0.001069,0.001101
  giSopranoEee84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.003212,0.576616,0.029989,0.014597,0.001991
 elseif iVoice==24 then
  giSopranoOh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.001294,0.001804,0.002559,0.003508,0.004711,0.006268,0.008148,0.010769,0.014119,0.018933,0.026874,0.039039,0.065610,0.075277,0.053501,0.033025,0.024955,0.018554,0.015599,0.013411,0.012721,0.013105,0.015740,0.019623,0.017772,0.011470,0.007689,0.005735,0.004357,0.003493,0.002766,0.002194,0.001717,0.001300,0.000955,0.000665,0.000428,0.000251,0.000137,0.000118,0.000148,0.000173,0.000179,0.000168,0.000147,0.000115,0.000151,0.000226,0.000321,0.000442,0.000592,0.000782,0.001031,0.001351,0.001818,0.002481,0.003526,0.004801,0.004868,0.003637,0.002548,0.001868,0.001383,0.001054,0.000797,0.000601,0.000445,0.000319,0.000220,0.000141
  giSopranoOh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.001630,0.002458,0.003663,0.005211,0.007357,0.010060,0.014075,0.019183,0.029201,0.043953,0.080445,0.085357,0.049683,0.032906,0.022953,0.018492,0.015615,0.015240,0.016916,0.022734,0.021331,0.012592,0.008310,0.005802,0.004428,0.003356,0.002549,0.001886,0.001341,0.000898,0.000544,0.000289,0.000147,0.000155,0.000197,0.000214,0.000201,0.000168,0.000123,0.000175,0.000282,0.000423,0.000611,0.000854,0.001187,0.001639,0.002348,0.003418,0.005238,0.005962,0.004346,0.002871,0.001982,0.001418,0.001020,0.000730,0.000509,0.000340,0.000212,0.000117,0.000123,0.000210,0.000333,0.000498,0.000717,0.001000,0.001380,0.001892,0.002632,0.003685,0.005013,0.005372,0.004237,0.003015
  giSopranoOh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.002079,0.003414,0.005324,0.008081,0.011748,0.017492,0.025815,0.041628,0.082430,0.104259,0.056186,0.035627,0.024387,0.019456,0.018039,0.020232,0.027855,0.021210,0.011299,0.007422,0.005297,0.003812,0.002741,0.001871,0.001197,0.000675,0.000317,0.000162,0.000209,0.000251,0.000242,0.000197,0.000132,0.000107,0.000210,0.000366,0.000584,0.000885,0.001310,0.001924,0.002953,0.004717,0.007057,0.005779,0.003501,0.002256,0.001525,0.001026,0.000681,0.000426,0.000243,0.000117,0.000130,0.000248,0.000426,0.000678,0.001026,0.001509,0.002196,0.003251,0.004858,0.006394,0.005489,0.003690,0.002478,0.001703,0.001172,0.000797,0.000524,0.000327,0.000190,0.000103
  giSopranoOh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.002709,0.004852,0.008035,0.012747,0.020371,0.032964,0.058669,0.125001,0.083706,0.044369,0.028467,0.022150,0.021874,0.030653,0.027728,0.013291,0.008093,0.005517,0.003734,0.002427,0.001444,0.000721,0.000277,0.000215,0.000292,0.000291,0.000227,0.000134,0.000161,0.000338,0.000610,0.001012,0.001614,0.002559,0.004309,0.007707,0.007414,0.004128,0.002461,0.001551,0.000965,0.000565,0.000295,0.000120,0.000108,0.000252,0.000490,0.000850,0.001378,0.002169,0.003420,0.005508,0.007680,0.005998,0.003714,0.002344,0.001506,0.000954,0.000577,0.000322,0.000161,0.000107,0.000158,0.000239,0.000301,0.000222,0.000137
  giSopranoOh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.003609,0.007017,0.012541,0.021545,0.037880,0.078111,0.150624,0.066083,0.037128,0.026838,0.026228,0.039276,0.022960,0.011480,0.006993,0.004394,0.002604,0.001332,0.000502,0.000236,0.000346,0.000337,0.000231,0.000102,0.000101,0.000192,0.000455,0.000888,0.001572,0.002718,0.004992,0.009663,0.007323,0.003762,0.002124,0.001211,0.000642,0.000285,0.000266,0.000594,0.001132,0.001982,0.003379,0.005927,0.009151,0.006623,0.003751,0.002204,0.001292,0.000718,0.000358,0.000154,0.000119,0.000190,0.000308,0.000340,0.000202,0.000117
  giSopranoOh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.004909,0.010405,0.020089,0.038305,0.087774,0.170455,0.065711,0.036928,0.030431,0.045396,0.025146,0.011587,0.006704,0.003768,0.001793,0.000577,0.000309,0.000427,0.000337,0.000152,0.000119,0.000109,0.000199,0.000563,0.001223,0.002376,0.004699,0.010486,0.008702,0.003969,0.002043,0.001020,0.000424,0.000103,0.000125,0.000420,0.000997,0.002005,0.003793,0.007390,0.010776,0.006049,0.003151,0.001678,0.000839,0.000359,0.000132,0.000102,0.000113,0.000198,0.000355,0.000391,0.000203,0.000109
  giSopranoOh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.006830,0.016166,0.034995,0.083292,0.208584,0.071275,0.038932,0.040498,0.042444,0.014854,0.007630,0.003744,0.001348,0.000320,0.000504,0.000396,0.000138,0.000106,0.000146,0.000215,0.000737,0.001782,0.003877,0.009510,0.011639,0.004546,0.002069,0.000859,0.000236,0.000109,0.000501,0.001372,0.003057,0.006592,0.012961,0.007475,0.003451,0.001615,0.000663,0.000208,0.000120,0.000112,0.000108,0.000215,0.000428,0.000411,0.000185
  giSopranoOh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.009704,0.025493,0.065932,0.250014,0.088732,0.044302,0.061324,0.026584,0.011036,0.004855,0.001443,0.000431,0.000582,0.000267,0.000105,0.000172,0.000323,0.001223,0.003240,0.008657,0.014882,0.004940,0.001937,0.000593,0.000507,0.001712,0.004370,0.011103,0.012077,0.004722,0.001922,0.000649,0.000167,0.000146,0.000143,0.000321,0.000611,0.000278,0.000117
  giSopranoOh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.014035,0.043095,0.156256,0.132167,0.053681,0.078574,0.022962,0.008787,0.002659,0.000470,0.000678,0.000207,0.000194,0.000136,0.000137,0.000913,0.003151,0.010006,0.014670,0.004259,0.001289,0.000160,0.000534,0.002274,0.006787,0.018371,0.007525,0.002591,0.000717,0.000172,0.000153,0.000144,0.000382,0.000684,0.000234
  giSopranoOh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.020824,0.076663,0.341102,0.073904,0.090867,0.023190,0.007540,0.001155,0.000855,0.000304,0.000238,0.000105,0.000141,0.001129,0.004763,0.021021,0.007952,0.002044,0.000206,0.000843,0.004024,0.014832,0.012128,0.003366,0.000720,0.000204,0.000117,0.000109,0.000398,0.000782,0.000218
  giSopranoOh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.032333,0.166589,0.142539,0.081002,0.029705,0.007490,0.000649,0.000790,0.000209,0.000157,0.000135,0.000430,0.003567,0.019036,0.009090,0.001717,0.000218,0.002747,0.013196,0.014956,0.003234,0.000418,0.000224,0.000433,0.000823,0.000182
  giSopranoOh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.050988,0.500042,0.088602,0.053164,0.009710,0.000862,0.000535,0.000344,0.000132,0.000646,0.006483,0.029761,0.003875,0.000144,0.003426,0.022224,0.009445,0.001297,0.000291,0.000642,0.000556
  giSopranoOh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.086202,0.264349,0.157171,0.017580,0.000946,0.000407,0.000273,0.000276,0.006305,0.029333,0.002574,0.001071,0.013581,0.015055,0.001438,0.000307,0.000288,0.001369,0.000173
  giSopranoOh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.153365,0.147839,0.046389,0.002309,0.000608,0.000209,0.000135,0.002259,0.042068,0.004087,0.001687,0.029687,0.006733,0.000409,0.000218,0.001565,0.000131
  giSopranoOh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.333182,0.162007,0.014981,0.001580,0.000313,0.000270,0.007135,0.018178,0.000437,0.026398,0.006466,0.000447,0.000865,0.000364
  giSopranoOh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.999999,0.106309,0.001723,0.000687,0.000264,0.012966,0.007747,0.006853,0.018884,0.000582,0.001285,0.000199
  giSopranoOh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.528638,0.035156,0.000814,0.000113,0.012612,0.005146,0.027163,0.002875,0.000576,0.000345
  giSopranoOh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.295700,0.004617,0.000416,0.004518,0.008174,0.059391,0.000819,0.003130
  giSopranoOh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.324009,0.003159,0.000539,0.036354,0.052795,0.000895,0.000729
  giSopranoOh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.212624,0.001375,0.025936,0.013708,0.001164,0.000397
  giSopranoOh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.070310,0.000226,0.010293,0.005749,0.000690
 elseif iVoice==25 then
  giSopranoOoh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.006736,0.007744,0.009173,0.011625,0.015301,0.019467,0.030031,0.040022,0.083847,0.111438,0.078627,0.038797,0.030293,0.020098,0.016517,0.013075,0.010944,0.010263,0.009904,0.012389,0.016364,0.013953,0.007558,0.004976,0.003673,0.002791,0.002294,0.001798,0.001446,0.001130,0.000839,0.000619,0.000412,0.000255,0.000142,0.000113,0.000132,0.000128,0.000119,0.000126,0.000173,0.000228,0.000299,0.000385,0.000495,0.000633,0.000806,0.001004,0.001164,0.001174,0.001025,0.000827,0.000650,0.000509,0.000398,0.000310,0.000240,0.000183,0.000137
  giSopranoOoh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.008240,0.009736,0.012406,0.017177,0.022681,0.037033,0.060187,0.125058,0.110778,0.048765,0.035061,0.022345,0.017647,0.013782,0.012248,0.011790,0.016043,0.019429,0.011945,0.006214,0.004417,0.003235,0.002505,0.001898,0.001450,0.001023,0.000710,0.000427,0.000235,0.000103,0.000106,0.000137,0.000159,0.000148,0.000130,0.000131,0.000191,0.000267,0.000368,0.000496,0.000668,0.000893,0.001169,0.001399,0.001375,0.001123,0.000853,0.000637,0.000476,0.000354,0.000259,0.000186,0.000129,0.000104,0.000147,0.000202,0.000272,0.000362,0.000479,0.000623,0.000759,0.000789,0.000681,0.000530
  giSopranoOoh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.010095,0.012529,0.017608,0.024663,0.043315,0.081558,0.157661,0.088611,0.046106,0.027324,0.020478,0.015548,0.014372,0.016229,0.023376,0.013886,0.006978,0.004439,0.003346,0.002384,0.001737,0.001149,0.000718,0.000372,0.000154,0.000123,0.000167,0.000187,0.000168,0.000125,0.000135,0.000212,0.000319,0.000466,0.000663,0.000940,0.001310,0.001651,0.001590,0.001211,0.000862,0.000610,0.000429,0.000296,0.000198,0.000125,0.000140,0.000209,0.000299,0.000421,0.000586,0.000792,0.000937,0.000852,0.000643,0.000461,0.000327,0.000231,0.000160,0.000107
  giSopranoOoh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.012459,0.016728,0.025997,0.045900,0.091150,0.186461,0.074200,0.042360,0.026982,0.018939,0.017010,0.022371,0.024731,0.009432,0.005684,0.003945,0.002654,0.001756,0.001058,0.000513,0.000188,0.000154,0.000216,0.000210,0.000166,0.000171,0.000292,0.000464,0.000712,0.001081,0.001602,0.002011,0.001703,0.001162,0.000770,0.000507,0.000326,0.000198,0.000111,0.000102,0.000176,0.000278,0.000423,0.000631,0.000914,0.001124,0.000963,0.000669,0.000448,0.000297,0.000192,0.000119,0.000112
  giSopranoOoh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.015493,0.023256,0.038946,0.080072,0.222961,0.077625,0.040215,0.026163,0.020537,0.024794,0.027925,0.009959,0.005586,0.003599,0.002262,0.001239,0.000511,0.000154,0.000227,0.000257,0.000200,0.000105,0.000183,0.000348,0.000602,0.000995,0.001621,0.002342,0.002063,0.001309,0.000802,0.000483,0.000275,0.000140,0.000124,0.000233,0.000396,0.000642,0.001014,0.001340,0.001090,0.000692,0.000426,0.000257,0.000148,0.000123,0.000130
  giSopranoOoh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.019447,0.034315,0.073971,0.249826,0.097427,0.044636,0.027534,0.023553,0.038811,0.012411,0.006462,0.003791,0.002045,0.000853,0.000206,0.000273,0.000295,0.000187,0.000173,0.000381,0.000736,0.001334,0.002335,0.002747,0.001704,0.000952,0.000519,0.000258,0.000107,0.000208,0.000405,0.000725,0.001247,0.001582,0.001061,0.000602,0.000333,0.000173,0.000145,0.000152
  giSopranoOoh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.025063,0.049337,0.163181,0.177222,0.054661,0.031107,0.032482,0.027778,0.008882,0.004771,0.002299,0.000744,0.000246,0.000371,0.000246,0.000109,0.000162,0.000428,0.000938,0.001893,0.003326,0.002439,0.001229,0.000596,0.000251,0.000103,0.000284,0.000607,0.001190,0.001900,0.001302,0.000662,0.000323,0.000141,0.000101,0.000172,0.000168
  giSopranoOoh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.033454,0.091804,0.372905,0.084714,0.037879,0.044758,0.018864,0.007892,0.003513,0.001026,0.000308,0.000420,0.000196,0.000129,0.000187,0.000586,0.001429,0.003217,0.003417,0.001546,0.000653,0.000221,0.000103,0.000354,0.000853,0.001843,0.001938,0.000899,0.000386,0.000141,0.000147,0.000226,0.000135
  giSopranoOoh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.046516,0.160169,0.155245,0.052329,0.049605,0.019920,0.007198,0.002478,0.000308,0.000509,0.000201,0.000135,0.000127,0.000370,0.001207,0.003248,0.004134,0.001606,0.000550,0.000125,0.000106,0.000468,0.001290,0.002692,0.001390,0.000516,0.000158,0.000174,0.000259,0.000118
  giSopranoOoh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.068664,0.499897,0.089313,0.047133,0.024836,0.007586,0.001706,0.000547,0.000375,0.000132,0.000136,0.000347,0.001476,0.004681,0.003416,0.001040,0.000214,0.000172,0.000813,0.002504,0.002129,0.000667,0.000162,0.000186,0.000304,0.000115
  giSopranoOoh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.098674,0.354398,0.062213,0.055545,0.009543,0.001487,0.000749,0.000134,0.000212,0.000123,0.000856,0.003788,0.004876,0.001194,0.000166,0.000569,0.002379,0.002608,0.000649,0.000109,0.000205,0.000338
  giSopranoOoh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.183614,0.169417,0.089526,0.015785,0.002053,0.000839,0.000154,0.000122,0.000129,0.001172,0.006436,0.003091,0.000442,0.000113,0.000708,0.003688,0.001800,0.000283,0.000294,0.000271
  giSopranoOoh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.320378,0.104663,0.039843,0.004956,0.001029,0.000257,0.000735,0.006501,0.003212,0.000252,0.000215,0.002579,0.002779,0.000315,0.000143,0.000522
  giSopranoOoh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,1.000001,0.094284,0.015175,0.001094,0.000262,0.000162,0.002953,0.006834,0.000428,0.000345,0.005011,0.001335,0.000107,0.000114,0.000609
  giSopranoOoh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.708750,0.111076,0.002977,0.000267,0.000246,0.007577,0.002388,0.000170,0.004759,0.001298,0.000408,0.000196
  giSopranoOoh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.338803,0.031564,0.001678,0.000244,0.000258,0.012872,0.000885,0.001416,0.003598,0.000134,0.000588,0.000112
  giSopranoOoh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.209313,0.009909,0.000515,0.000104,0.013000,0.000503,0.005157,0.000630,0.000287,0.000189
  giSopranoOoh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.188584,0.002188,0.000154,0.005909,0.000855,0.010025,0.000214,0.001217
  giSopranoOoh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.222149,0.000533,0.000493,0.004776,0.009517,0.000194,0.000393
  giSopranoOoh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.063129,0.000488,0.025748,0.002834,0.000268,0.000223
  giSopranoOoh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.019817,0.000207,0.001007,0.001261,0.000379
 elseif iVoice==26 then
  giMixedAh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24), i(gkBW1)  , i(gkPS1)  ,1,1,1    ,0.000412,0.000370,0.000337,0.000371,0.000555,0.000859,0.001305,0.001896,0.002601,0.003585,0.004729,0.006226,0.008387,0.010759,0.015604,0.021153,0.036276,0.057553,0.053066,0.030189,0.020360,0.014722,0.011349,0.009414,0.008036,0.007520,0.007518,0.008204,0.010188,0.013445,0.021468,0.024378,0.017218,0.010300,0.007574,0.005320,0.004125,0.003105,0.002351,0.001777,0.001280,0.000908,0.000603,0.000362,0.000194,0.000113,0.000107,0.000165,0.000269,0.000410,0.000591,0.000829,0.001124,0.001497,0.001964,0.002540,0.003281,0.004214,0.005465,0.007172,0.009487,0.012221,0.012290,0.008928,0.005554,0.004575,0.006297,0.009642,0.012002,0.011210,0.008864,0.007021,0.005649,0.004715,0.004092,0.003711,0.003540,0.003269,0.002720,0.002116,0.001696,0.001392,0.001163,0.000978,0.000817,0.000679,0.000554,0.000445,0.000349,0.000267,0.000201,0.000148,0.000112
  giMixedAh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27), i(gkBW2)  , i(gkPS2)  ,1,1,1    ,0.000484,0.000423,0.000410,0.000587,0.001000,0.001640,0.002515,0.003647,0.005207,0.007188,0.010221,0.013861,0.021835,0.034169,0.065821,0.062536,0.031595,0.021762,0.014578,0.011706,0.009599,0.008934,0.009048,0.011008,0.014402,0.024811,0.028584,0.017346,0.010562,0.007037,0.005089,0.003642,0.002621,0.001845,0.001237,0.000781,0.000431,0.000198,0.000132,0.000128,0.000107,0.000154,0.000279,0.000469,0.000726,0.001077,0.001533,0.002136,0.002909,0.003944,0.005309,0.007297,0.010064,0.014010,0.014733,0.009883,0.005835,0.006234,0.010391,0.014244,0.013109,0.009848,0.007528,0.005944,0.004974,0.004420,0.004181,0.003659,0.002784,0.002107,0.001664,0.001346,0.001093,0.000881,0.000697,0.000538,0.000403,0.000292,0.000206,0.000145,0.000112
  giMixedAh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30), i(gkBW3)  , i(gkPS3)  ,1,1,1    ,0.000560,0.000482,0.000562,0.001054,0.001899,0.003144,0.004913,0.007274,0.010934,0.015824,0.026652,0.050310,0.084386,0.048694,0.027304,0.017198,0.013209,0.010896,0.010619,0.012323,0.016925,0.031534,0.030303,0.014903,0.009410,0.006192,0.004171,0.002825,0.001804,0.001089,0.000559,0.000223,0.000135,0.000159,0.000132,0.000111,0.000205,0.000408,0.000717,0.001165,0.001784,0.002644,0.003807,0.005452,0.007846,0.011507,0.016933,0.016329,0.008970,0.006585,0.011345,0.016921,0.014673,0.010437,0.007690,0.006096,0.005243,0.004888,0.003982,0.002810,0.002096,0.001620,0.001264,0.000975,0.000730,0.000526,0.000361,0.000238,0.000159,0.000123,0.000113,0.000108
  giMixedAh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33), i(gkBW4)  , i(gkPS4)  ,1,1,1    ,0.000645,0.000569,0.000955,0.001964,0.003652,0.006202,0.009824,0.015928,0.027700,0.054004,0.099838,0.045530,0.026122,0.017357,0.013212,0.012640,0.015734,0.025801,0.041244,0.020359,0.011360,0.006947,0.004321,0.002669,0.001503,0.000708,0.000231,0.000110,0.000186,0.000170,0.000130,0.000254,0.000564,0.001062,0.001820,0.002933,0.004538,0.006945,0.010838,0.017334,0.020883,0.011091,0.008199,0.016262,0.019876,0.013707,0.009453,0.007159,0.006126,0.005456,0.003786,0.002617,0.001920,0.001428,0.001039,0.000721,0.000469,0.000288,0.000180,0.000140,0.000132,0.000122,0.000102
  giMixedAh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36), i(gkBW5)  , i(gkPS5)  ,1,1,1    ,0.001076,0.000760,0.001152,0.002918,0.006013,0.010963,0.018910,0.033003,0.065564,0.123397,0.061570,0.034546,0.024544,0.022364,0.028096,0.049779,0.048235,0.023028,0.012783,0.007477,0.004231,0.002147,0.000887,0.000349,0.000355,0.000327,0.000195,0.000152,0.000288,0.000361,0.000371,0.000539,0.001217,0.002491,0.004498,0.007542,0.012318,0.020674,0.035880,0.035208,0.020677,0.019986,0.031450,0.029001,0.018014,0.012074,0.009087,0.007807,0.005927,0.003587,0.002503,0.001834,0.001291,0.000830,0.000470,0.000249,0.000194,0.000200,0.000178,0.000130
  giMixedAh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39), i(gkBW6)  , i(gkPS6)  ,1,1,1    ,0.001221,0.000884,0.002403,0.006103,0.012641,0.024258,0.049147,0.124843,0.094249,0.043351,0.028599,0.027085,0.040952,0.068353,0.030182,0.014975,0.007910,0.003883,0.001548,0.000473,0.000421,0.000370,0.000161,0.000304,0.000426,0.000443,0.000809,0.002086,0.004462,0.008387,0.015033,0.028032,0.047931,0.029239,0.022629,0.037393,0.031509,0.018127,0.012012,0.009537,0.007224,0.004034,0.002693,0.001836,0.001138,0.000598,0.000284,0.000232,0.000231,0.000177,0.000105
  giMixedAh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42), i(gkBW7)  , i(gkPS7)  ,1,1,1    ,0.001347,0.001378,0.005009,0.012714,0.027848,0.064949,0.174667,0.072154,0.037557,0.031873,0.051698,0.069662,0.025483,0.011788,0.005236,0.001791,0.000485,0.000510,0.000306,0.000357,0.000517,0.000580,0.001645,0.004407,0.009531,0.019083,0.040846,0.049561,0.026461,0.041849,0.036370,0.019264,0.012655,0.010146,0.005280,0.003211,0.002020,0.001088,0.000460,0.000274,0.000277,0.000199,0.000106,0.000102
  giMixedAh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45), i(gkBW8)  , i(gkPS8)  ,1,1,1    ,0.001435,0.002814,0.010404,0.027304,0.073592,0.204283,0.065680,0.038086,0.049867,0.090768,0.028199,0.011415,0.004006,0.000835,0.000609,0.000353,0.000145,0.000530,0.000623,0.001521,0.005124,0.012704,0.029322,0.067515,0.034280,0.047183,0.040088,0.019737,0.013502,0.007873,0.004048,0.002335,0.001073,0.000384,0.000337,0.000259,0.000126,0.000121,0.000110
  giMixedAh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48), i(gkBW9)  , i(gkPS9)  ,1,1,1    ,0.001665,0.005802,0.022639,0.067838,0.277959,0.082343,0.043643,0.072542,0.079439,0.023326,0.007411,0.001524,0.000914,0.000509,0.000186,0.000132,0.000538,0.002041,0.005636,0.014527,0.006699,0.012109,0.005270,0.003013,0.001652,0.000778,0.000392,0.000133
  giMixedAh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51), i(gkBW10) , i(gkPS10) ,1,1,1    ,0.001879,0.012367,0.050272,0.249889,0.104596,0.051557,0.135917,0.047385,0.012730,0.002203,0.001070,0.000444,0.000205,0.000486,0.002508,0.008455,0.012808,0.011617,0.007544,0.003612,0.001661,0.000745,0.000277
  giMixedAh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54), i(gkBW11) , i(gkPS11) ,1,1,1    ,0.002739,0.026042,0.133355,0.175744,0.061363,0.185512,0.036813,0.006813,0.001283,0.000619,0.000238,0.000104,0.001374,0.006525,0.018731,0.014280,0.007212,0.003761,0.001187,0.000430,0.000106
  giMixedAh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57), i(gkBW12) , i(gkPS12) ,1,1,1    ,0.005498,0.056620,0.471547,0.079386,0.210272,0.035484,0.003858,0.001328,0.000308,0.000117,0.001906,0.011522,0.011745,0.013323,0.004796,0.001380,0.000371,0.000133
  giMixedAh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60), i(gkBW13) , i(gkPS13) ,1,1,1    ,0.002967,0.046107,0.824869,0.198093,0.082301,0.006142,0.001955,0.000496,0.001429,0.017146,0.037895,0.004766,0.005960,0.003267,0.000407,0.000157,0.000155,0.000519
  giMixedAh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63), i(gkBW14) , i(gkPS14) ,1,1,1    ,0.002677,0.127443,0.233422,0.181784,0.009013,0.001717,0.000284,0.007015,0.064115,0.007331,0.007952,0.002405,0.000267,0.000125,0.000600
  giMixedAh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66), i(gkBW15) , i(gkPS15) ,1,1,1    ,0.010553,0.479368,0.511917,0.027219,0.002374,0.000432,0.019753,0.029194,0.005598,0.003593,0.000314,0.000115,0.000440,0.000188
  giMixedAh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69), i(gkBW16) , i(gkPS16) ,1,1,1    ,0.033509,0.499985,0.127000,0.004247,0.000403,0.034562,0.013610,0.013599,0.000498,0.000136,0.000625,0.000114
  giMixedAh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72), i(gkBW17) , i(gkPS17) ,1,1,1    ,0.097704,0.332922,0.011754,0.001452,0.002670,0.003963,0.017086,0.011243,0.000618,0.000448
  giMixedAh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75), i(gkBW18) , i(gkPS18) ,1,1,1    ,0.266284,0.282922,0.003635,0.000202,0.006066,0.036263,0.001979,0.003494
  giMixedAh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78), i(gkBW19) , i(gkPS19) ,1,1,1    ,0.976912,0.048417,0.000970,0.026534,0.032847,0.001372,0.000907
  giMixedAh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81), i(gkBW20) , i(gkPS20) ,1,1,1    ,1.000001,0.008470,0.005716,0.006895,0.001809,0.000528
  giMixedAh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84), i(gkBW21) , i(gkPS21) ,1,1,1    ,0.665845,0.002904,0.007927,0.022486,0.000897
 elseif iVoice==27 then
  giMixedEh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.002549,0.003134,0.003974,0.005478,0.007017,0.009323,0.012997,0.015800,0.025880,0.031561,0.069262,0.116966,0.096839,0.041388,0.030133,0.019258,0.014690,0.010922,0.008058,0.006191,0.004473,0.003237,0.002267,0.001451,0.000894,0.000442,0.000126,0.000210,0.000263,0.000265,0.000240,0.000174,0.000100,0.000185,0.000338,0.000545,0.000796,0.001110,0.001516,0.002015,0.002652,0.003519,0.004603,0.006308,0.008742,0.013114,0.019649,0.020048,0.013747,0.009105,0.006633,0.004842,0.003722,0.002793,0.002088,0.001484,0.000943,0.000453,0.000147,0.000600,0.001155,0.001779,0.002508,0.003367,0.004414,0.005764,0.007472,0.010017,0.013566,0.019406,0.025684,0.025076,0.018739,0.013927,0.011049,0.009282,0.008375,0.008024,0.008229,0.009065,0.010646,0.013251,0.016228,0.016401,0.013297,0.010080,0.008006,0.006682,0.006049,0.005955,0.006476,0.007437,0.007523,0.006180,0.004613,0.003551,0.002779,0.002235
  giMixedEh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.003129,0.004092,0.005657,0.007860,0.010921,0.016087,0.021139,0.035868,0.062023,0.136227,0.109956,0.040456,0.031049,0.018374,0.013762,0.009515,0.006960,0.004597,0.003159,0.001921,0.001079,0.000444,0.000177,0.000312,0.000316,0.000280,0.000177,0.000171,0.000376,0.000664,0.001031,0.001514,0.002167,0.002999,0.004195,0.005777,0.008581,0.012777,0.021830,0.024145,0.015340,0.009845,0.006641,0.004815,0.003434,0.002428,0.001585,0.000853,0.000201,0.000618,0.001398,0.002299,0.003391,0.004727,0.006500,0.008859,0.012573,0.018214,0.027784,0.031145,0.022997,0.016195,0.012414,0.010478,0.009647,0.009763,0.010916,0.013349,0.017534,0.020106,0.016727,0.012049,0.009207,0.007617,0.007098,0.007517,0.008848,0.008867,0.006730,0.004810,0.003566,0.002737,0.002122,0.001644,0.001252,0.000929,0.000662,0.000447,0.000285,0.000180,0.000139,0.000140,0.000147,0.000144,0.000130,0.000107
  giMixedEh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.003867,0.005384,0.008278,0.011948,0.018781,0.027378,0.043927,0.124923,0.154756,0.052079,0.035175,0.020879,0.013635,0.009198,0.005837,0.003709,0.002010,0.000982,0.000220,0.000161,0.000370,0.000376,0.000299,0.000147,0.000155,0.000424,0.000832,0.001379,0.002152,0.003202,0.004758,0.006982,0.011177,0.019325,0.029701,0.020304,0.011682,0.007361,0.005041,0.003356,0.002114,0.001056,0.000187,0.001017,0.002186,0.003609,0.005412,0.007905,0.011471,0.017562,0.028740,0.037414,0.026954,0.018054,0.013544,0.011691,0.011440,0.012869,0.016457,0.022485,0.022347,0.015528,0.011112,0.008894,0.008412,0.009625,0.010828,0.008346,0.005596,0.003939,0.002900,0.002141,0.001562,0.001098,0.000729,0.000449,0.000257,0.000169,0.000165,0.000175,0.000167,0.000140,0.000106
  giMixedEh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.004867,0.007460,0.011945,0.019946,0.031209,0.054453,0.186212,0.117747,0.049349,0.025385,0.015675,0.009940,0.005668,0.003035,0.001342,0.000247,0.000261,0.000440,0.000408,0.000208,0.000179,0.000569,0.001196,0.002093,0.003439,0.005483,0.008701,0.015148,0.030996,0.029556,0.014724,0.008576,0.005437,0.003329,0.001707,0.000348,0.001213,0.002897,0.005055,0.008006,0.012446,0.020472,0.036633,0.042329,0.025547,0.017331,0.014092,0.013706,0.016241,0.022858,0.028248,0.019753,0.013157,0.010345,0.010349,0.012777,0.010845,0.006746,0.004465,0.003112,0.002165,0.001447,0.000902,0.000506,0.000263,0.000193,0.000207,0.000202,0.000165,0.000114
  giMixedEh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.008898,0.014845,0.025531,0.044722,0.088437,0.209987,0.109632,0.053027,0.029768,0.016881,0.008983,0.004150,0.001395,0.000463,0.000755,0.000691,0.000400,0.000255,0.000745,0.001668,0.003236,0.005773,0.010198,0.020143,0.039588,0.019923,0.010157,0.005810,0.003302,0.001689,0.000582,0.000359,0.001415,0.002957,0.005341,0.009195,0.016165,0.031454,0.042867,0.022527,0.012419,0.007380,0.004874,0.005036,0.007696,0.012757,0.022519,0.032658,0.021211,0.013339,0.010317,0.010331,0.013816,0.015314,0.008765,0.005134,0.003184,0.001938,0.001090,0.000535,0.000250,0.000207,0.000210,0.000173,0.000113
  giMixedEh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.011502,0.021549,0.042263,0.090322,0.245564,0.114253,0.051172,0.026245,0.012912,0.005428,0.001469,0.000641,0.000901,0.000604,0.000162,0.000315,0.001073,0.002614,0.005421,0.010640,0.023418,0.045836,0.018645,0.008963,0.004582,0.002130,0.000572,0.000762,0.002491,0.005301,0.010156,0.019894,0.044518,0.039884,0.018319,0.009661,0.005793,0.006382,0.011028,0.020816,0.038283,0.025968,0.015109,0.011870,0.013774,0.019307,0.011088,0.005870,0.003324,0.001790,0.000830,0.000329,0.000246,0.000244,0.000178
  giMixedEh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.015233,0.032741,0.074724,0.243797,0.144246,0.056112,0.025240,0.010252,0.002839,0.000700,0.001067,0.000580,0.000104,0.000883,0.002741,0.006585,0.014845,0.043733,0.031620,0.012214,0.005557,0.002198,0.000195,0.001893,0.005217,0.011479,0.025712,0.062141,0.031662,0.014093,0.007252,0.007795,0.015192,0.033589,0.039695,0.019565,0.014153,0.017708,0.020404,0.009087,0.004554,0.002215,0.000878,0.000317,0.000301,0.000241,0.000125
  giMixedEh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.020852,0.053258,0.153984,0.250056,0.069342,0.026968,0.008516,0.001168,0.001282,0.000765,0.000162,0.001475,0.005006,0.013261,0.041892,0.036286,0.012356,0.004686,0.001052,0.001663,0.006070,0.015487,0.041856,0.059403,0.020427,0.008986,0.009865,0.022609,0.054432,0.028468,0.017103,0.021683,0.020009,0.007941,0.003514,0.001272,0.000379,0.000357,0.000233,0.000104,0.000116
  giMixedEh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.014412,0.045494,0.181791,0.112588,0.033171,0.009544,0.001064,0.001135,0.000329,0.000220,0.001449,0.006000,0.019846,0.045292,0.010951,0.003439,0.000417,0.001563,0.004898,0.014483,0.022627,0.012262,0.019775,0.010955,0.019872,0.010229,0.003771,0.001279,0.000296,0.000245,0.000142
  giMixedEh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.021582,0.082278,0.299289,0.052488,0.012284,0.000944,0.001172,0.000108,0.000906,0.006303,0.026106,0.032757,0.007753,0.001434,0.001444,0.005988,0.023064,0.017739,0.020167,0.013678,0.023142,0.010173,0.003168,0.000704,0.000290,0.000184
  giMixedEh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.033385,0.181253,0.115431,0.021468,0.001260,0.001196,0.000294,0.003460,0.019374,0.049307,0.008286,0.000663,0.003599,0.016148,0.024802,0.024651,0.015477,0.027005,0.006039,0.001230,0.000344,0.000180,0.000106
  giMixedEh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.053000,0.500048,0.052999,0.003969,0.001543,0.000360,0.007742,0.067956,0.015859,0.001332,0.004887,0.030983,0.020466,0.021939,0.036267,0.006921,0.000920,0.000388,0.000110
  giMixedEh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.165294,0.197124,0.015337,0.003004,0.003167,0.040484,0.005206,0.000220,0.001029,0.011247,0.005236,0.003975,0.005800,0.000693,0.000124,0.000151,0.000577,0.000103
  giMixedEh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.343032,0.097209,0.002364,0.001736,0.039793,0.004573,0.000222,0.004544,0.011682,0.003924,0.005813,0.000396,0.000102,0.000125,0.000672
  giMixedEh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.999678,0.038087,0.002755,0.020021,0.006295,0.000187,0.012594,0.004197,0.013896,0.000677,0.000104,0.000433,0.000213
  giMixedEh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,1.000001,0.003910,0.006008,0.016207,0.000265,0.023574,0.004637,0.003870,0.000201,0.000631,0.000123
  giMixedEh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.223136,0.003631,0.001641,0.053621,0.043595,0.010450,0.007588,0.000208,0.000421,0.000286
  giMixedEh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.109531,0.000806,0.042151,0.016149,0.016640,0.005110,0.000133,0.001866
  giMixedEh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.034598,0.000509,0.025064,0.074296,0.007935,0.000130,0.000593
  giMixedEh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.005801,0.023872,0.089183,0.013066,0.000176,0.000337
  giMixedEh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.007260,0.107242,0.020899,0.000417,0.000572
 elseif iVoice==28 then
  giMixedEee24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.010612,0.014495,0.019809,0.028477,0.041398,0.063942,0.114518,0.124942,0.078855,0.046228,0.033959,0.023394,0.018143,0.013375,0.010099,0.007577,0.005431,0.003901,0.002611,0.001632,0.000932,0.000386,0.000171,0.000289,0.000422,0.000452,0.000449,0.000398,0.000315,0.000237,0.000144,0.000104,0.000105,0.000153,0.000215,0.000290,0.000380,0.000487,0.000633,0.000816,0.001099,0.001507,0.002203,0.003126,0.003165,0.002277,0.001567,0.001153,0.000855,0.000658,0.000500,0.000378,0.000277,0.000186,0.000102,0.000154,0.000300,0.000490,0.000733,0.001033,0.001413,0.001885,0.002475,0.003256,0.004250,0.005714,0.007781,0.011083,0.015021,0.015148,0.011378,0.008165,0.006212,0.004895,0.004095,0.003568,0.003306,0.003268,0.003450,0.003938,0.004750,0.006020,0.007017,0.006500,0.005013,0.003837,0.003082,0.002641,0.002479,0.002562
  giMixedEee27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.013361,0.019632,0.028170,0.046001,0.073446,0.142620,0.135320,0.067419,0.045253,0.028671,0.021074,0.014727,0.010611,0.007252,0.004920,0.003071,0.001747,0.000807,0.000249,0.000303,0.000500,0.000540,0.000520,0.000422,0.000309,0.000180,0.000119,0.000127,0.000103,0.000168,0.000256,0.000366,0.000497,0.000673,0.000915,0.001288,0.001877,0.003013,0.003942,0.003063,0.001933,0.001339,0.000949,0.000692,0.000498,0.000349,0.000219,0.000102,0.000105,0.000283,0.000531,0.000867,0.001300,0.001875,0.002609,0.003613,0.004968,0.007064,0.010239,0.015759,0.018744,0.014260,0.009604,0.006959,0.005391,0.004464,0.003989,0.003908,0.004207,0.005028,0.006490,0.008241,0.007854,0.005766,0.004257,0.003368,0.002997,0.003039,0.003534,0.003719,0.002952,0.002107,0.001565,0.001195,0.000929,0.000719,0.000551,0.000410,0.000295,0.000202,0.000133
  giMixedEee30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.016942,0.026728,0.044993,0.073582,0.166083,0.150187,0.067471,0.042023,0.027730,0.018157,0.012270,0.007756,0.004771,0.002544,0.001124,0.000288,0.000418,0.000628,0.000639,0.000532,0.000370,0.000189,0.000133,0.000151,0.000118,0.000167,0.000288,0.000442,0.000634,0.000915,0.001325,0.002073,0.003575,0.004636,0.003020,0.001869,0.001215,0.000836,0.000565,0.000361,0.000186,0.000308,0.000658,0.001154,0.001829,0.002760,0.004060,0.005937,0.009069,0.014428,0.021876,0.018429,0.011479,0.007850,0.005939,0.004916,0.004612,0.004928,0.006092,0.008369,0.009952,0.007626,0.005224,0.003945,0.003510,0.003799,0.004447,0.003710,0.002491,0.001752,0.001285,0.000950,0.000696,0.000493,0.000332,0.000209,0.000125
  giMixedEee33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.021903,0.037076,0.070524,0.170727,0.184856,0.075853,0.042006,0.026533,0.016384,0.009694,0.005423,0.002512,0.000745,0.000367,0.000734,0.000760,0.000589,0.000358,0.000113,0.000157,0.000177,0.000123,0.000164,0.000331,0.000549,0.000839,0.001298,0.002144,0.003934,0.005481,0.003133,0.001834,0.001150,0.000725,0.000430,0.000185,0.000178,0.000585,0.001216,0.002131,0.003468,0.005492,0.008727,0.014780,0.025548,0.021159,0.012320,0.008120,0.006189,0.005514,0.005907,0.007722,0.011241,0.010580,0.006710,0.004738,0.004184,0.004875,0.005074,0.003298,0.002150,0.001485,0.001035,0.000705,0.000453,0.000265,0.000143
  giMixedEee36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.018039,0.029693,0.059894,0.185737,0.154314,0.055778,0.027787,0.014854,0.007935,0.003897,0.001432,0.000187,0.000561,0.000590,0.000401,0.000185,0.000117,0.000134,0.000109,0.000405,0.000942,0.001848,0.003332,0.005831,0.010609,0.022023,0.025121,0.011959,0.006463,0.003691,0.002042,0.001000,0.000346,0.000540,0.001193,0.002221,0.003859,0.006735,0.012934,0.018831,0.010505,0.006536,0.005174,0.005305,0.007021,0.011338,0.013114,0.007641,0.004738,0.003619,0.003554,0.002391,0.001498,0.001024,0.000703,0.000458,0.000268,0.000136
  giMixedEee39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.023171,0.040165,0.103046,0.249899,0.069009,0.030296,0.014964,0.006837,0.002317,0.000262,0.000693,0.000670,0.000375,0.000134,0.000157,0.000234,0.000809,0.001919,0.003902,0.007563,0.015739,0.033909,0.016874,0.008020,0.004127,0.001993,0.000762,0.000560,0.001472,0.003025,0.005784,0.011760,0.022871,0.012484,0.007312,0.006012,0.007155,0.011847,0.015854,0.008455,0.005033,0.004257,0.003348,0.001866,0.001182,0.000751,0.000433,0.000208
  giMixedEee42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.030585,0.075466,0.326397,0.089842,0.037631,0.016293,0.006004,0.001189,0.000623,0.000819,0.000421,0.000187,0.000122,0.000220,0.000997,0.002690,0.006079,0.013783,0.037543,0.020537,0.008608,0.003873,0.001483,0.000243,0.000604,0.001913,0.004382,0.009617,0.024507,0.016361,0.008552,0.007239,0.010299,0.019396,0.011363,0.006002,0.005105,0.003117,0.001699,0.001004,0.000533,0.000219,0.000103,0.000108
  giMixedEee45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.042426,0.110844,0.240398,0.052093,0.020251,0.006042,0.000491,0.000993,0.000619,0.000230,0.000590,0.002336,0.006356,0.016818,0.048133,0.016906,0.006376,0.002233,0.000321,0.000910,0.003109,0.007901,0.022561,0.020975,0.009763,0.009076,0.016934,0.017670,0.007601,0.006066,0.003146,0.001615,0.000823,0.000318,0.000124,0.000127
  giMixedEee48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.067500,0.499957,0.084236,0.026845,0.006246,0.000524,0.001260,0.000307,0.000263,0.000154,0.000423,0.001657,0.005126,0.020030,0.007311,0.002320,0.000560,0.000313,0.001681,0.005251,0.018793,0.008143,0.003428,0.003199,0.002838,0.002897,0.002582,0.000962,0.000359
  giMixedEee51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.114392,0.346891,0.051686,0.011497,0.000495,0.001393,0.000319,0.000278,0.001711,0.006671,0.024100,0.005070,0.001207,0.000395,0.002611,0.010148,0.016063,0.004678,0.003757,0.002818,0.004738,0.001749,0.000577,0.000121
  giMixedEee54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.184699,0.170704,0.027773,0.001277,0.001722,0.000135,0.000260,0.000954,0.005364,0.030213,0.005416,0.000846,0.000161,0.001816,0.009486,0.018223,0.004741,0.005826,0.003571,0.002699,0.000709,0.000111
  giMixedEee57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.375525,0.096426,0.009561,0.002227,0.000108,0.000215,0.002089,0.015287,0.010294,0.001315,0.000212,0.003544,0.026847,0.007999,0.006168,0.004412,0.002362,0.000444,0.000109
  giMixedEee60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.227192,0.099004,0.005244,0.001344,0.001073,0.017641,0.017350,0.001409,0.000721,0.009172,0.004060,0.000250,0.000806,0.006918,0.001617,0.000117,0.000126,0.000462
  giMixedEee63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.603091,0.048190,0.002514,0.000479,0.018496,0.015302,0.000391,0.003639,0.009291,0.000469,0.001181,0.008347,0.000679,0.000111,0.000537
  giMixedEee66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,1.000000,0.014894,0.000748,0.011456,0.020880,0.000335,0.010283,0.002934,0.000513,0.010980,0.000625,0.000358,0.000173
  giMixedEee69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.383427,0.002256,0.002169,0.058726,0.000416,0.019191,0.001023,0.003871,0.002038,0.000111,0.000515,0.000103
  giMixedEee72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.143330,0.001607,0.002011,0.288307,0.002276,0.014994,0.010998,0.007299,0.001333,0.000996
  giMixedEee75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.061863,0.001803,0.041574,0.018718,0.022519,0.023699,0.001207,0.009907,0.000135
  giMixedEee78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.008237,0.002981,0.275939,0.109772,0.021334,0.000832,0.002147
  giMixedEee81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.012164,0.019280,0.007395,0.003926,0.001069,0.001101
  giMixedEee84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.003212,0.576616,0.029989,0.014597,0.001991
 elseif iVoice==29 then
  giMixedOh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.002656,0.003252,0.004105,0.005622,0.007160,0.009459,0.013108,0.015856,0.025894,0.031448,0.069195,0.117293,0.097983,0.042979,0.032065,0.021929,0.018078,0.015475,0.014066,0.014550,0.016289,0.021526,0.023582,0.017583,0.010850,0.007974,0.005821,0.004584,0.003615,0.002841,0.002234,0.001707,0.001268,0.000905,0.000600,0.000361,0.000186,0.000124,0.000163,0.000187,0.000186,0.000166,0.000139,0.000101,0.000100,0.000137,0.000189,0.000263,0.000361,0.000489,0.000654,0.000858,0.001117,0.001451,0.001869,0.002471,0.003247,0.004440,0.005393,0.004629,0.002917,0.002066,0.002532,0.003852,0.005391,0.005792,0.004814,0.003675,0.002842,0.002214,0.001763,0.001401,0.001091,0.000746,0.000345,0.000127,0.000165,0.000194,0.000194,0.000185,0.000171,0.000154,0.000137,0.000119,0.000101
  giMixedOh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.003258,0.004239,0.005824,0.008031,0.011084,0.016213,0.021193,0.035813,0.061897,0.136532,0.111390,0.042393,0.033786,0.022028,0.018961,0.016732,0.017591,0.021496,0.028305,0.022987,0.013061,0.009039,0.006327,0.004802,0.003617,0.002714,0.001972,0.001369,0.000897,0.000513,0.000245,0.000108,0.000159,0.000210,0.000226,0.000203,0.000166,0.000112,0.000138,0.000201,0.000297,0.000435,0.000622,0.000874,0.001199,0.001639,0.002216,0.003089,0.004305,0.006101,0.005978,0.003630,0.002459,0.003410,0.005563,0.006988,0.005953,0.004329,0.003190,0.002395,0.001829,0.001373,0.000891,0.000334,0.000160,0.000222,0.000234,0.000222,0.000202,0.000178,0.000153,0.000127,0.000102
  giMixedOh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.004023,0.005567,0.008482,0.012147,0.018935,0.027436,0.043784,0.124923,0.156060,0.054359,0.038530,0.025628,0.020731,0.020302,0.024164,0.033681,0.024398,0.013158,0.008555,0.006066,0.004329,0.003073,0.002076,0.001307,0.000716,0.000311,0.000128,0.000209,0.000264,0.000252,0.000205,0.000130,0.000103,0.000161,0.000252,0.000400,0.000621,0.000939,0.001374,0.001992,0.002862,0.004244,0.006522,0.007371,0.004231,0.002979,0.004965,0.007920,0.007484,0.005169,0.003596,0.002580,0.001862,0.001227,0.000441,0.000198,0.000273,0.000271,0.000248,0.000214,0.000179,0.000142,0.000109
  giMixedOh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.005059,0.007690,0.012187,0.020150,0.031282,0.054249,0.186526,0.120147,0.052781,0.030786,0.024379,0.024804,0.034918,0.035401,0.016807,0.010063,0.006707,0.004475,0.002912,0.001727,0.000868,0.000313,0.000184,0.000294,0.000312,0.000251,0.000145,0.000162,0.000275,0.000476,0.000802,0.001297,0.002024,0.003113,0.004960,0.008196,0.007816,0.003687,0.004946,0.009187,0.008701,0.005588,0.003662,0.002498,0.001599,0.000503,0.000264,0.000332,0.000310,0.000265,0.000215,0.000163,0.000117
  giMixedOh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.006425,0.011190,0.018900,0.031763,0.063063,0.234533,0.085077,0.042498,0.028723,0.024364,0.029248,0.052657,0.032828,0.016019,0.009256,0.005473,0.003020,0.001411,0.000472,0.000289,0.000385,0.000324,0.000169,0.000209,0.000292,0.000313,0.000363,0.000715,0.001488,0.002773,0.004783,0.007968,0.013450,0.024276,0.028174,0.012256,0.017548,0.027382,0.016871,0.007639,0.001429,0.003903,0.002935,0.001946,0.001251,0.000808,0.000539,0.000371,0.000260,0.000189,0.000148,0.000125,0.000105
  giMixedOh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.008376,0.015990,0.032397,0.071647,0.272662,0.083705,0.042399,0.029965,0.031725,0.060042,0.036024,0.015995,0.008610,0.004457,0.001931,0.000539,0.000377,0.000438,0.000278,0.000188,0.000329,0.000372,0.000455,0.001070,0.002432,0.004805,0.008844,0.016499,0.033376,0.023715,0.015683,0.032489,0.020044,0.007073,0.004006,0.004073,0.002568,0.001523,0.000908,0.000566,0.000367,0.000246,0.000180,0.000147,0.000118
  giMixedOh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.011034,0.024255,0.054958,0.249960,0.107468,0.049168,0.034461,0.048258,0.060049,0.020609,0.009752,0.004351,0.001387,0.000382,0.000531,0.000321,0.000134,0.000183,0.000390,0.000442,0.000696,0.002019,0.004800,0.010033,0.021149,0.044106,0.017250,0.033814,0.026431,0.007582,0.005537,0.004027,0.002216,0.001191,0.000676,0.000405,0.000256,0.000192,0.000151,0.000111
  giMixedOh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.015273,0.040290,0.108755,0.238831,0.059213,0.041067,0.079700,0.037954,0.014459,0.005681,0.001470,0.000525,0.000572,0.000143,0.000149,0.000276,0.000507,0.000599,0.001942,0.005592,0.013421,0.033213,0.035928,0.030674,0.033762,0.006591,0.006489,0.003618,0.001724,0.000862,0.000468,0.000280,0.000206,0.000147,0.000101
  giMixedOh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.010601,0.034815,0.190470,0.080405,0.033305,0.058053,0.024910,0.008255,0.002415,0.000380,0.000499,0.000157,0.000134,0.000544,0.001534,0.004269,0.007067,0.006352,0.013410,0.005008,0.002977,0.001313,0.000644,0.000282
  giMixedOh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.016431,0.063467,0.235366,0.043971,0.063965,0.024873,0.006978,0.001106,0.000633,0.000237,0.000165,0.000526,0.001871,0.006744,0.006491,0.013161,0.007320,0.003558,0.001368,0.000593,0.000178
  giMixedOh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.024710,0.140863,0.075949,0.054103,0.032225,0.006828,0.000565,0.000591,0.000130,0.000122,0.000130,0.001184,0.004810,0.008308,0.016195,0.006829,0.003189,0.000995,0.000290
  giMixedOh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.039150,0.500037,0.055380,0.060924,0.008930,0.000650,0.000410,0.000244,0.001590,0.009065,0.009812,0.013291,0.004577,0.001153,0.000235,0.000102
  giMixedOh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.086407,0.267040,0.196788,0.021563,0.001095,0.000450,0.000254,0.000578,0.012590,0.058595,0.005142,0.009927,0.005483,0.000502,0.000175,0.000149,0.000775
  giMixedOh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,0.153433,0.155941,0.058873,0.002749,0.000677,0.000168,0.000395,0.004570,0.084026,0.008181,0.014008,0.003788,0.000261,0.000109,0.000887
  giMixedOh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.332841,0.193621,0.018082,0.001777,0.000274,0.000623,0.014262,0.036353,0.007910,0.005829,0.000322,0.000104,0.000472,0.000204
  giMixedOh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,1.000000,0.136629,0.001978,0.000708,0.000666,0.025888,0.015536,0.029450,0.000535,0.000119,0.000710,0.000111
  giMixedOh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.528638,0.035156,0.000814,0.000113,0.012612,0.005146,0.027163,0.002875,0.000576,0.000345
  giMixedOh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.295700,0.004617,0.000416,0.004518,0.008174,0.059391,0.000819,0.003130
  giMixedOh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.324009,0.003159,0.000539,0.036354,0.052795,0.000895,0.000729
  giMixedOh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.212624,0.001375,0.025936,0.013708,0.001164,0.000397
  giMixedOh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.070310,0.000226,0.010293,0.005749,0.000690
 elseif iVoice==30 then
  giMixedOoh24 ftgen  1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1) , i(gkPS1)  ,1,1,1    ,0.004170,0.004767,0.005809,0.007125,0.009423,0.012338,0.016616,0.025281,0.036773,0.084223,0.097509,0.055644,0.026776,0.021850,0.013883,0.011537,0.008646,0.005282,0.002155,0.001654,0.001414,0.001239,0.001040,0.000878,0.000785,0.000712,0.000629,0.000575,0.000483,0.000406,0.000327,0.000237,0.000173,0.000104,0.000121,0.000160,0.000207,0.000263,0.000334,0.000421,0.000551,0.000726,0.001018,0.001329,0.001313,0.001049,0.000891,0.000885,0.000983,0.001199,0.001540,0.001977,0.002121,0.001784,0.001340,0.001034,0.000826,0.000705,0.000652,0.000680,0.000715,0.000646,0.000503,0.000390,0.000309,0.000249,0.000204,0.000167,0.000137,0.000111
  giMixedOoh27 ftgen  2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2) , i(gkPS2)  ,1,1,1    ,0.005030,0.006191,0.007651,0.010372,0.014571,0.021434,0.030781,0.071907,0.119545,0.072776,0.031574,0.022556,0.015399,0.011394,0.007041,0.002518,0.001911,0.001634,0.001295,0.001096,0.000937,0.000834,0.000721,0.000630,0.000508,0.000397,0.000272,0.000181,0.000107,0.000153,0.000213,0.000285,0.000379,0.000500,0.000688,0.000967,0.001445,0.001618,0.001277,0.001057,0.001077,0.001285,0.001687,0.002301,0.002526,0.001994,0.001425,0.001068,0.000863,0.000780,0.000826,0.000841,0.000676,0.000495,0.000375,0.000290,0.000229,0.000180,0.000141,0.000108
  giMixedOoh30 ftgen  3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3) , i(gkPS3)  ,1,1,1    ,0.006083,0.008003,0.010711,0.016374,0.024519,0.037449,0.112833,0.124998,0.041417,0.028771,0.018019,0.012343,0.005402,0.002386,0.001972,0.001531,0.001231,0.001066,0.000905,0.000784,0.000607,0.000449,0.000285,0.000153,0.000101,0.000109,0.000153,0.000230,0.000327,0.000460,0.000642,0.000947,0.001505,0.001937,0.001502,0.001242,0.001345,0.001774,0.002567,0.002998,0.002232,0.001518,0.001113,0.000935,0.000970,0.000984,0.000727,0.000511,0.000372,0.000279,0.000210,0.000157,0.000113
  giMixedOoh33 ftgen  4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4) , i(gkPS4)  ,1,1,1    ,0.007502,0.010380,0.016096,0.024230,0.043922,0.147876,0.113247,0.043397,0.022792,0.015355,0.005760,0.002715,0.002152,0.001629,0.001298,0.001071,0.000882,0.000647,0.000405,0.000215,0.000125,0.000127,0.000114,0.000198,0.000315,0.000473,0.000703,0.001106,0.001912,0.002216,0.001551,0.001530,0.002044,0.003150,0.003426,0.002208,0.001462,0.001135,0.001161,0.001127,0.000745,0.000502,0.000355,0.000253,0.000178,0.000120
  giMixedOoh36 ftgen  5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5) , i(gkPS5)  ,1,1,1    ,0.009514,0.014225,0.024646,0.050523,0.168409,0.111356,0.043820,0.023460,0.012509,0.004160,0.002642,0.001689,0.001315,0.001065,0.000757,0.000443,0.000192,0.000123,0.000137,0.000101,0.000120,0.000148,0.000186,0.000291,0.000545,0.001008,0.001755,0.002924,0.004809,0.008185,0.013559,0.007678,0.008486,0.018454,0.017597,0.010496,0.006842,0.005140,0.004787,0.005732,0.004640,0.002578,0.001610,0.001047,0.000655,0.000372,0.000189,0.000109,0.000102
  giMixedOoh39 ftgen  6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6) , i(gkPS6)  ,1,1,1    ,0.012343,0.020687,0.042780,0.143593,0.145481,0.045253,0.024030,0.006681,0.003567,0.002127,0.001539,0.001172,0.000743,0.000337,0.000147,0.000158,0.000139,0.000180,0.000245,0.000466,0.000987,0.001939,0.003573,0.006484,0.012507,0.013031,0.008641,0.021946,0.018929,0.010423,0.006772,0.005654,0.006751,0.005079,0.002597,0.001537,0.000903,0.000473,0.000212,0.000124,0.000121
  giMixedOoh42 ftgen  7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7) , i(gkPS7)  ,1,1,1    ,0.015973,0.032700,0.074843,0.250024,0.057702,0.026906,0.006281,0.003020,0.001964,0.001456,0.000842,0.000281,0.000120,0.000190,0.000125,0.000117,0.000172,0.000230,0.000397,0.000965,0.002195,0.004549,0.009401,0.019240,0.008562,0.023865,0.021772,0.010988,0.007172,0.007255,0.006908,0.003095,0.001662,0.000857,0.000363,0.000158,0.000146,0.000111
  giMixedOoh45 ftgen  8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8) , i(gkPS8)  ,1,1,1    ,0.020715,0.048381,0.295623,0.086997,0.032947,0.006518,0.003184,0.001972,0.001207,0.000399,0.000165,0.000225,0.000166,0.000247,0.000405,0.001120,0.002956,0.006943,0.016973,0.012394,0.026234,0.023823,0.011169,0.008018,0.009723,0.004053,0.001912,0.000846,0.000280,0.000172,0.000144
  giMixedOoh48 ftgen  9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9) , i(gkPS9)  ,1,1,1    ,0.020956,0.073214,0.300650,0.047374,0.011304,0.003365,0.002105,0.001056,0.000160,0.000271,0.000143,0.000105,0.000437,0.001635,0.004710,0.015016,0.005087,0.004883,0.002780,0.003191,0.006532,0.002329,0.000854,0.000249
  giMixedOoh51 ftgen 10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10), i(gkPS10) ,1,1,1    ,0.030210,0.140577,0.104265,0.026214,0.004100,0.002408,0.000835,0.000254,0.000256,0.000123,0.000395,0.002001,0.007270,0.012518,0.005007,0.003616,0.003732,0.007195,0.001939,0.000551
  giMixedOoh54 ftgen 11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11), i(gkPS11) ,1,1,1    ,0.046979,0.499997,0.053723,0.006460,0.002919,0.000773,0.000378,0.000118,0.000106,0.000152,0.001094,0.005381,0.019208,0.006051,0.003882,0.006400,0.003801,0.000886,0.000115
  giMixedOoh57 ftgen 12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12), i(gkPS12) ,1,1,1    ,0.073889,0.165133,0.017058,0.003994,0.001052,0.000471,0.000177,0.001512,0.009857,0.009881,0.005879,0.006275,0.004329,0.000733,0.000125
  giMixedOoh60 ftgen 13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13), i(gkPS13) ,1,1,1    ,0.319941,0.115082,0.065007,0.007044,0.001288,0.000283,0.000127,0.000673,0.008134,0.014095,0.001709,0.000289,0.000332
  giMixedOoh63 ftgen 14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14), i(gkPS14) ,1,1,1    ,1.000001,0.125741,0.022569,0.001337,0.000279,0.000263,0.002205,0.023707,0.004085,0.000367,0.000387
  giMixedOoh66 ftgen 15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15), i(gkPS15) ,1,1,1    ,0.712592,0.181596,0.004243,0.000318,0.000259,0.003910,0.023893,0.001033,0.000156,0.000248,0.000124
  giMixedOoh69 ftgen 16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16), i(gkPS16) ,1,1,1    ,0.347452,0.048482,0.002113,0.000185,0.003869,0.019916,0.000554,0.000125,0.000363
  giMixedOoh72 ftgen 17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17), i(gkPS17) ,1,1,1    ,0.209313,0.009909,0.000515,0.000104,0.013000,0.000503,0.005157,0.000630,0.000287,0.000189
  giMixedOoh75 ftgen 18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18), i(gkPS18) ,1,1,1    ,0.188584,0.002188,0.000154,0.005909,0.000855,0.010025,0.000214,0.001217
  giMixedOoh78 ftgen 19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19), i(gkPS19) ,1,1,1    ,0.222149,0.000533,0.000493,0.004776,0.009517,0.000194,0.000393
  giMixedOoh81 ftgen 20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20), i(gkPS20) ,1,1,1    ,0.063129,0.000488,0.025748,0.002834,0.000268,0.000223
  giMixedOoh84 ftgen 21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21), i(gkPS21) ,1,1,1    ,0.019817,0.000207,0.001007,0.001261,0.000379
 endif

 ktrig    changed    gkVoice2
 if ktrig==1 && gkV2OnOff==1 then
  reinit    UpdateTable2
 endif
 UpdateTable2:
 iVoice2    =    i(gkVoice2)
 ; BASS
 if iVoice2==1 then
  giBassAh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.000412,0.000370,0.000337,0.000371,0.000555,0.000859,0.001305,0.001896,0.002601,0.003585,0.004729,0.006226,0.008387,0.010759,0.015604,0.021153,0.036276,0.057553,0.053066,0.030189,0.020360,0.014722,0.011349,0.009414,0.008036,0.007520,0.007518,0.008204,0.010188,0.013445,0.021468,0.024378,0.017218,0.010300,0.007574,0.005320,0.004125,0.003105,0.002351,0.001777,0.001280,0.000908,0.000603,0.000362,0.000194,0.000113,0.000107,0.000165,0.000269,0.000410,0.000591,0.000829,0.001124,0.001497,0.001964,0.002540,0.003281,0.004214,0.005465,0.007172,0.009487,0.012221,0.012290,0.008928,0.005554,0.004575,0.006297,0.009642,0.012002,0.011210,0.008864,0.007021,0.005649,0.004715,0.004092,0.003711,0.003540,0.003269,0.002720,0.002116,0.001696,0.001392,0.001163,0.000978,0.000817,0.000679,0.000554,0.000445,0.000349,0.000267,0.000201,0.000148,0.000112
  giBassAh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.000484,0.000423,0.000410,0.000587,0.001000,0.001640,0.002515,0.003647,0.005207,0.007188,0.010221,0.013861,0.021835,0.034169,0.065821,0.062536,0.031595,0.021762,0.014578,0.011706,0.009599,0.008934,0.009048,0.011008,0.014402,0.024811,0.028584,0.017346,0.010562,0.007037,0.005089,0.003642,0.002621,0.001845,0.001237,0.000781,0.000431,0.000198,0.000132,0.000128,0.000107,0.000154,0.000279,0.000469,0.000726,0.001077,0.001533,0.002136,0.002909,0.003944,0.005309,0.007297,0.010064,0.014010,0.014733,0.009883,0.005835,0.006234,0.010391,0.014244,0.013109,0.009848,0.007528,0.005944,0.004974,0.004420,0.004181,0.003659,0.002784,0.002107,0.001664,0.001346,0.001093,0.000881,0.000697,0.000538,0.000403,0.000292,0.000206,0.000145,0.000112
  giBassAh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.000560,0.000482,0.000562,0.001054,0.001899,0.003144,0.004913,0.007274,0.010934,0.015824,0.026652,0.050310,0.084386,0.048694,0.027304,0.017198,0.013209,0.010896,0.010619,0.012323,0.016925,0.031534,0.030303,0.014903,0.009410,0.006192,0.004171,0.002825,0.001804,0.001089,0.000559,0.000223,0.000135,0.000159,0.000132,0.000111,0.000205,0.000408,0.000717,0.001165,0.001784,0.002644,0.003807,0.005452,0.007846,0.011507,0.016933,0.016329,0.008970,0.006585,0.011345,0.016921,0.014673,0.010437,0.007690,0.006096,0.005243,0.004888,0.003982,0.002810,0.002096,0.001620,0.001264,0.000975,0.000730,0.000526,0.000361,0.000238,0.000159,0.000123,0.000113,0.000108
  giBassAh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.000645,0.000569,0.000955,0.001964,0.003652,0.006202,0.009824,0.015928,0.027700,0.054004,0.099838,0.045530,0.026122,0.017357,0.013212,0.012640,0.015734,0.025801,0.041244,0.020359,0.011360,0.006947,0.004321,0.002669,0.001503,0.000708,0.000231,0.000110,0.000186,0.000170,0.000130,0.000254,0.000564,0.001062,0.001820,0.002933,0.004538,0.006945,0.010838,0.017334,0.020883,0.011091,0.008199,0.016262,0.019876,0.013707,0.009453,0.007159,0.006126,0.005456,0.003786,0.002617,0.001920,0.001428,0.001039,0.000721,0.000469,0.000288,0.000180,0.000140,0.000132,0.000122,0.000102
  giBassAh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.000741,0.000742,0.001718,0.003793,0.007172,0.012458,0.021527,0.042327,0.115172,0.060415,0.029465,0.018842,0.015053,0.016424,0.026918,0.048812,0.020626,0.010654,0.006220,0.003560,0.001819,0.000726,0.000149,0.000187,0.000214,0.000156,0.000332,0.000822,0.001662,0.003004,0.005098,0.008459,0.014399,0.024542,0.017933,0.009191,0.019377,0.022533,0.014117,0.009483,0.007465,0.006578,0.004260,0.002803,0.001970,0.001367,0.000896,0.000539,0.000299,0.000185,0.000160,0.000148,0.000121
  giBassAh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.000846,0.001173,0.003275,0.007285,0.014357,0.027687,0.068261,0.124920,0.043474,0.023386,0.017847,0.021990,0.049566,0.034657,0.014061,0.007278,0.003688,0.001560,0.000396,0.000192,0.000256,0.000187,0.000558,0.001451,0.003062,0.005814,0.010610,0.020116,0.029454,0.011668,0.020774,0.026213,0.015055,0.009950,0.008362,0.005568,0.003330,0.002188,0.001396,0.000808,0.000411,0.000223,0.000189,0.000167,0.000123
  giBassAh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.000963,0.002108,0.006288,0.014551,0.031661,0.100695,0.097406,0.034417,0.021810,0.024675,0.063158,0.029838,0.012403,0.005661,0.002185,0.000448,0.000270,0.000265,0.000411,0.001440,0.003583,0.007649,0.015770,0.034040,0.018012,0.022841,0.029509,0.015471,0.010557,0.008019,0.004223,0.002548,0.001472,0.000730,0.000322,0.000229,0.000199,0.000135
  giBassAh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.001138,0.003929,0.012405,0.031859,0.108048,0.091053,0.034718,0.025288,0.051642,0.040725,0.013900,0.005342,0.001417,0.000220,0.000340,0.000509,0.002128,0.005880,0.013925,0.034767,0.022222,0.032638,0.027491,0.014365,0.010950,0.005254,0.002868,0.001449,0.000579,0.000282,0.000244,0.000156
  giBassAh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.001482,0.007584,0.024916,0.084669,0.120825,0.037691,0.032860,0.097651,0.021318,0.007128,0.001455,0.000373,0.000311,0.001646,0.006016,0.016943,0.049158,0.018410,0.045129,0.018996,0.013181,0.005616,0.002740,0.001081,0.000371,0.000296,0.000174,0.000109
  giBassAh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.002346,0.014577,0.055410,0.249952,0.046797,0.044016,0.069343,0.014565,0.003123,0.000384,0.000374,0.002906,0.011647,0.040309,0.023365,0.052509,0.019934,0.011156,0.004385,0.001619,0.000449,0.000335,0.000163,0.000126
  giBassAh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.004218,0.029108,0.201426,0.068825,0.049355,0.059660,0.011314,0.000892,0.000531,0.002882,0.015302,0.068104,0.045706,0.030943,0.016039,0.005097,0.001458,0.000457,0.000270,0.000155
  giBassAh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.007858,0.063721,0.182088,0.050577,0.081435,0.010683,0.000440,0.001018,0.011764,0.069560,0.065307,0.028732,0.010508,0.002898,0.000565,0.000313,0.000177
  giBassAh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.015174,0.169369,0.075380,0.195316,0.014245,0.000747,0.003294,0.033895,0.036836,0.037995,0.011232,0.002161,0.000593,0.000220,0.000121
  giBassAh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.029160,0.499980,0.088056,0.029134,0.000770,0.005816,0.080662,0.105049,0.022319,0.003239,0.000671,0.000252,0.000122
  giBassAh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.058218,0.137644,0.119309,0.001783,0.005763,0.136221,0.061878,0.010194,0.000915,0.000309,0.000144
  giBassAh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.127432,0.101146,0.021362,0.002037,0.139115,0.057450,0.005793,0.000626,0.000185
  giBassAh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.338719,0.390591,0.001495,0.067786,0.075975,0.004319,0.000439,0.000166
  giBassAh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,1.000000,0.058271,0.011635,0.210114,0.006478,0.000504,0.000138
  giBassAh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.275285,0.003566,0.272436,0.020387,0.000618,0.000157
  giBassAh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.202299,0.004074,0.114905,0.001251,0.000191
  giBassAh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.781181,0.135572,0.008638,0.000333
 elseif iVoice2==2 then
  giBassEh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.002549,0.003134,0.003974,0.005478,0.007017,0.009323,0.012997,0.015800,0.025880,0.031561,0.069262,0.116966,0.096839,0.041388,0.030133,0.019258,0.014690,0.010922,0.008058,0.006191,0.004473,0.003237,0.002267,0.001451,0.000894,0.000442,0.000126,0.000210,0.000263,0.000265,0.000240,0.000174,0.000100,0.000185,0.000338,0.000545,0.000796,0.001110,0.001516,0.002015,0.002652,0.003519,0.004603,0.006308,0.008742,0.013114,0.019649,0.020048,0.013747,0.009105,0.006633,0.004842,0.003722,0.002793,0.002088,0.001484,0.000943,0.000453,0.000147,0.000600,0.001155,0.001779,0.002508,0.003367,0.004414,0.005764,0.007472,0.010017,0.013566,0.019406,0.025684,0.025076,0.018739,0.013927,0.011049,0.009282,0.008375,0.008024,0.008229,0.009065,0.010646,0.013251,0.016228,0.016401,0.013297,0.010080,0.008006,0.006682,0.006049,0.005955,0.006476,0.007437,0.007523,0.006180,0.004613,0.003551,0.002779,0.002235
  giBassEh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.003129,0.004092,0.005657,0.007860,0.010921,0.016087,0.021139,0.035868,0.062023,0.136227,0.109956,0.040456,0.031049,0.018374,0.013762,0.009515,0.006960,0.004597,0.003159,0.001921,0.001079,0.000444,0.000177,0.000312,0.000316,0.000280,0.000177,0.000171,0.000376,0.000664,0.001031,0.001514,0.002167,0.002999,0.004195,0.005777,0.008581,0.012777,0.021830,0.024145,0.015340,0.009845,0.006641,0.004815,0.003434,0.002428,0.001585,0.000853,0.000201,0.000618,0.001398,0.002299,0.003391,0.004727,0.006500,0.008859,0.012573,0.018214,0.027784,0.031145,0.022997,0.016195,0.012414,0.010478,0.009647,0.009763,0.010916,0.013349,0.017534,0.020106,0.016727,0.012049,0.009207,0.007617,0.007098,0.007517,0.008848,0.008867,0.006730,0.004810,0.003566,0.002737,0.002122,0.001644,0.001252,0.000929,0.000662,0.000447,0.000285,0.000180,0.000139,0.000140,0.000147,0.000144,0.000130,0.000107
  giBassEh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.003867,0.005384,0.008278,0.011948,0.018781,0.027378,0.043927,0.124923,0.154756,0.052079,0.035175,0.020879,0.013635,0.009198,0.005837,0.003709,0.002010,0.000982,0.000220,0.000161,0.000370,0.000376,0.000299,0.000147,0.000155,0.000424,0.000832,0.001379,0.002152,0.003202,0.004758,0.006982,0.011177,0.019325,0.029701,0.020304,0.011682,0.007361,0.005041,0.003356,0.002114,0.001056,0.000187,0.001017,0.002186,0.003609,0.005412,0.007905,0.011471,0.017562,0.028740,0.037414,0.026954,0.018054,0.013544,0.011691,0.011440,0.012869,0.016457,0.022485,0.022347,0.015528,0.011112,0.008894,0.008412,0.009625,0.010828,0.008346,0.005596,0.003939,0.002900,0.002141,0.001562,0.001098,0.000729,0.000449,0.000257,0.000169,0.000165,0.000175,0.000167,0.000140,0.000106
  giBassEh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.004867,0.007460,0.011945,0.019946,0.031209,0.054453,0.186212,0.117747,0.049349,0.025385,0.015675,0.009940,0.005668,0.003035,0.001342,0.000247,0.000261,0.000440,0.000408,0.000208,0.000179,0.000569,0.001196,0.002093,0.003439,0.005483,0.008701,0.015148,0.030996,0.029556,0.014724,0.008576,0.005437,0.003329,0.001707,0.000348,0.001213,0.002897,0.005055,0.008006,0.012446,0.020472,0.036633,0.042329,0.025547,0.017331,0.014092,0.013706,0.016241,0.022858,0.028248,0.019753,0.013157,0.010345,0.010349,0.012777,0.010845,0.006746,0.004465,0.003112,0.002165,0.001447,0.000902,0.000506,0.000263,0.000193,0.000207,0.000202,0.000165,0.000114
  giBassEh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.006269,0.010960,0.018651,0.031612,0.063145,0.234031,0.082814,0.038536,0.021856,0.012390,0.006478,0.002903,0.000884,0.000151,0.000526,0.000481,0.000201,0.000371,0.001091,0.002222,0.004038,0.007052,0.012642,0.026285,0.040191,0.018257,0.009711,0.005602,0.002977,0.000908,0.001205,0.003571,0.006760,0.011575,0.020120,0.038989,0.050394,0.027994,0.018663,0.016137,0.018236,0.026663,0.033012,0.020295,0.013457,0.011996,0.014986,0.012457,0.007161,0.004508,0.002937,0.001849,0.001061,0.000524,0.000256,0.000239,0.000246,0.000199,0.000126
  giBassEh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.008173,0.015702,0.032144,0.071650,0.272194,0.080839,0.036707,0.019016,0.009188,0.003840,0.000888,0.000353,0.000631,0.000355,0.000343,0.001326,0.003025,0.005992,0.011542,0.025532,0.048254,0.019676,0.009624,0.004854,0.001705,0.001234,0.004596,0.009451,0.017714,0.036427,0.062296,0.032396,0.020962,0.019531,0.026707,0.040231,0.024114,0.015246,0.015050,0.017756,0.009634,0.005482,0.003293,0.001861,0.000896,0.000362,0.000281,0.000290,0.000215,0.000115,0.000101
  giBassEh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.010770,0.023902,0.054774,0.249962,0.104159,0.041772,0.018402,0.007420,0.001965,0.000321,0.000751,0.000293,0.000854,0.002767,0.006422,0.014005,0.038772,0.040687,0.014759,0.006732,0.002120,0.002044,0.007252,0.015890,0.035324,0.075238,0.036306,0.023528,0.025922,0.045315,0.031277,0.017930,0.019424,0.016834,0.007953,0.004326,0.002219,0.000907,0.000341,0.000353,0.000285,0.000144,0.000103,0.000123
  giBassEh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.014919,0.039894,0.108912,0.235444,0.050771,0.019880,0.006071,0.000493,0.000880,0.000415,0.001140,0.004191,0.010980,0.030342,0.059163,0.017172,0.006666,0.000696,0.005809,0.016058,0.041074,0.084879,0.034762,0.027511,0.045921,0.039646,0.020777,0.025682,0.013556,0.006258,0.002911,0.001018,0.000389,0.000407,0.000231,0.000117,0.000145
  giBassEh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.021924,0.063232,0.468119,0.077063,0.024778,0.005806,0.000302,0.000961,0.000747,0.004455,0.014122,0.052633,0.036527,0.011211,0.001822,0.007148,0.023183,0.078131,0.056059,0.032330,0.053450,0.040659,0.024048,0.024961,0.009037,0.003707,0.001049,0.000481,0.000402,0.000146,0.000173,0.000107
  giBassEh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.031418,0.143370,0.161725,0.038046,0.007682,0.000705,0.000709,0.002656,0.011998,0.051134,0.039384,0.009717,0.002474,0.018934,0.072998,0.064883,0.039139,0.080623,0.030556,0.035586,0.010987,0.003730,0.000725,0.000582,0.000232,0.000203,0.000114
  giBassEh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.047801,0.499973,0.083544,0.014840,0.000643,0.000586,0.005528,0.028009,0.081360,0.013462,0.004094,0.031800,0.150499,0.047062,0.090670,0.035868,0.033668,0.008649,0.001814,0.000708,0.000286,0.000243,0.000113
  giBassEh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.079790,0.470826,0.039758,0.000985,0.000829,0.008383,0.060699,0.034340,0.001390,0.032127,0.169760,0.055035,0.079288,0.051382,0.012516,0.002036,0.000814,0.000234,0.000188,0.000140
  giBassEh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.126470,0.154125,0.011612,0.001926,0.008899,0.105301,0.022420,0.014314,0.156341,0.064675,0.081320,0.049928,0.007414,0.000961,0.000290,0.000213,0.000158
  giBassEh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.286799,0.076099,0.001412,0.005314,0.102313,0.019436,0.037889,0.129794,0.161320,0.071197,0.007462,0.001164,0.000406,0.000186,0.000100
  giBassEh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.999999,0.029677,0.001175,0.056021,0.026921,0.063607,0.094120,0.071732,0.017296,0.001415,0.000487,0.000226,0.000113
  giBassEh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.941545,0.001970,0.016767,0.068666,0.064251,0.110055,0.102753,0.004071,0.000467,0.000280,0.000117
  giBassEh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.308230,0.003850,0.210611,0.028628,0.129335,0.099831,0.001921,0.000427,0.000186
  giBassEh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.152203,0.010631,0.038873,0.259601,0.142407,0.002328,0.000372,0.000102
  giBassEh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.059354,0.112042,0.127210,0.143460,0.002830,0.000452,0.000116
  giBassEh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.003941,0.137332,0.220127,0.008140,0.000560
  giBassEh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.007699,0.057256,0.199662,0.000854,0.000109
 elseif iVoice2==3 then
  giBassEee24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.010612,0.014495,0.019809,0.028477,0.041398,0.063942,0.114518,0.124942,0.078855,0.046228,0.033959,0.023394,0.018143,0.013375,0.010099,0.007577,0.005431,0.003901,0.002611,0.001632,0.000932,0.000386,0.000171,0.000289,0.000422,0.000452,0.000449,0.000398,0.000315,0.000237,0.000144,0.000104,0.000105,0.000153,0.000215,0.000290,0.000380,0.000487,0.000633,0.000816,0.001099,0.001507,0.002203,0.003126,0.003165,0.002277,0.001567,0.001153,0.000855,0.000658,0.000500,0.000378,0.000277,0.000186,0.000102,0.000154,0.000300,0.000490,0.000733,0.001033,0.001413,0.001885,0.002475,0.003256,0.004250,0.005714,0.007781,0.011083,0.015021,0.015148,0.011378,0.008165,0.006212,0.004895,0.004095,0.003568,0.003306,0.003268,0.003450,0.003938,0.004750,0.006020,0.007017,0.006500,0.005013,0.003837,0.003082,0.002641,0.002479,0.002562
  giBassEee27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.013361,0.019632,0.028170,0.046001,0.073446,0.142620,0.135320,0.067419,0.045253,0.028671,0.021074,0.014727,0.010611,0.007252,0.004920,0.003071,0.001747,0.000807,0.000249,0.000303,0.000500,0.000540,0.000520,0.000422,0.000309,0.000180,0.000119,0.000127,0.000103,0.000168,0.000256,0.000366,0.000497,0.000673,0.000915,0.001288,0.001877,0.003013,0.003942,0.003063,0.001933,0.001339,0.000949,0.000692,0.000498,0.000349,0.000219,0.000102,0.000105,0.000283,0.000531,0.000867,0.001300,0.001875,0.002609,0.003613,0.004968,0.007064,0.010239,0.015759,0.018744,0.014260,0.009604,0.006959,0.005391,0.004464,0.003989,0.003908,0.004207,0.005028,0.006490,0.008241,0.007854,0.005766,0.004257,0.003368,0.002997,0.003039,0.003534,0.003719,0.002952,0.002107,0.001565,0.001195,0.000929,0.000719,0.000551,0.000410,0.000295,0.000202,0.000133
  giBassEee30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.016942,0.026728,0.044993,0.073582,0.166083,0.150187,0.067471,0.042023,0.027730,0.018157,0.012270,0.007756,0.004771,0.002544,0.001124,0.000288,0.000418,0.000628,0.000639,0.000532,0.000370,0.000189,0.000133,0.000151,0.000118,0.000167,0.000288,0.000442,0.000634,0.000915,0.001325,0.002073,0.003575,0.004636,0.003020,0.001869,0.001215,0.000836,0.000565,0.000361,0.000186,0.000308,0.000658,0.001154,0.001829,0.002760,0.004060,0.005937,0.009069,0.014428,0.021876,0.018429,0.011479,0.007850,0.005939,0.004916,0.004612,0.004928,0.006092,0.008369,0.009952,0.007626,0.005224,0.003945,0.003510,0.003799,0.004447,0.003710,0.002491,0.001752,0.001285,0.000950,0.000696,0.000493,0.000332,0.000209,0.000125
  giBassEee33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.021903,0.037076,0.070524,0.170727,0.184856,0.075853,0.042006,0.026533,0.016384,0.009694,0.005423,0.002512,0.000745,0.000367,0.000734,0.000760,0.000589,0.000358,0.000113,0.000157,0.000177,0.000123,0.000164,0.000331,0.000549,0.000839,0.001298,0.002144,0.003934,0.005481,0.003133,0.001834,0.001150,0.000725,0.000430,0.000185,0.000178,0.000585,0.001216,0.002131,0.003468,0.005492,0.008727,0.014780,0.025548,0.021159,0.012320,0.008120,0.006189,0.005514,0.005907,0.007722,0.011241,0.010580,0.006710,0.004738,0.004184,0.004875,0.005074,0.003298,0.002150,0.001485,0.001035,0.000705,0.000453,0.000265,0.000143
  giBassEee36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.028999,0.056971,0.127925,0.249973,0.092491,0.046808,0.026763,0.015161,0.007806,0.003265,0.000772,0.000577,0.000905,0.000797,0.000474,0.000128,0.000118,0.000208,0.000175,0.000190,0.000430,0.000762,0.001269,0.002203,0.004418,0.006347,0.003143,0.001714,0.001003,0.000555,0.000205,0.000308,0.000984,0.002075,0.003788,0.006543,0.011486,0.022285,0.030464,0.016426,0.009849,0.007181,0.006579,0.007931,0.012129,0.013098,0.007735,0.005325,0.005167,0.006343,0.004251,0.002537,0.001641,0.001065,0.000654,0.000360,0.000176,0.000109,0.000108
  giBassEee39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.039208,0.091871,0.284850,0.134665,0.057264,0.029417,0.014487,0.006136,0.001614,0.000605,0.001080,0.000843,0.000360,0.000237,0.000205,0.000101,0.000336,0.000730,0.001345,0.002573,0.006021,0.006121,0.002677,0.001384,0.000697,0.000205,0.000565,0.001734,0.003749,0.007227,0.014133,0.031533,0.028537,0.013927,0.008934,0.007821,0.010066,0.016499,0.011547,0.006746,0.006088,0.007451,0.004223,0.002395,0.001442,0.000822,0.000405,0.000174,0.000127,0.000124
  giBassEee42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.053470,0.147208,0.300415,0.084059,0.036324,0.015517,0.005091,0.000576,0.001256,0.001064,0.000377,0.000160,0.000303,0.000150,0.000341,0.000891,0.001840,0.004161,0.009292,0.003746,0.001676,0.000724,0.000174,0.001325,0.003681,0.008171,0.018259,0.044043,0.023100,0.011960,0.009296,0.012291,0.020077,0.010540,0.007087,0.008984,0.005033,0.002599,0.001411,0.000675,0.000255,0.000150,0.000143
  giBassEee45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.074148,0.341465,0.151691,0.053062,0.019387,0.005023,0.000735,0.001519,0.000715,0.000141,0.000355,0.000141,0.000664,0.001681,0.004296,0.010976,0.003672,0.001452,0.000370,0.001174,0.004275,0.011021,0.029674,0.042441,0.016298,0.011074,0.015522,0.021253,0.009522,0.009803,0.006632,0.002990,0.001422,0.000537,0.000189,0.000178,0.000110
  giBassEee48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.113952,0.499961,0.093618,0.030324,0.006532,0.001153,0.001593,0.000254,0.000416,0.000182,0.000869,0.002549,0.008846,0.006282,0.002003,0.000401,0.001977,0.007592,0.023020,0.061020,0.019732,0.013187,0.024324,0.015505,0.010364,0.008518,0.003290,0.001312,0.000353,0.000214,0.000143
  giBassEee51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.183849,0.269450,0.058864,0.012277,0.001210,0.001686,0.000115,0.000411,0.000673,0.002694,0.012061,0.005358,0.001397,0.001131,0.007512,0.028323,0.057149,0.017903,0.020182,0.023136,0.012207,0.008465,0.002892,0.000813,0.000255,0.000186
  giBassEee54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.294425,0.168105,0.031031,0.001152,0.002129,0.000319,0.000295,0.001773,0.008324,0.007496,0.001451,0.002648,0.016348,0.088119,0.023921,0.024596,0.021078,0.017984,0.005195,0.001344,0.000303,0.000185,0.000105
  giBassEee57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.682961,0.106120,0.010046,0.003039,0.000281,0.000282,0.003363,0.021952,0.002902,0.002350,0.022051,0.084871,0.022153,0.042507,0.019617,0.005980,0.001075,0.000355,0.000117
  giBassEee60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.999999,0.060648,0.002312,0.000510,0.000354,0.005083,0.012579,0.000821,0.015185,0.122052,0.026381,0.031005,0.017044,0.002622,0.000436,0.000138
  giBassEee63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.538975,0.024555,0.003372,0.000823,0.005390,0.010718,0.002266,0.056680,0.035812,0.046282,0.016933,0.001625,0.000371,0.000151
  giBassEee66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.336196,0.002303,0.000636,0.003546,0.014990,0.005298,0.176248,0.049197,0.035968,0.002688,0.000369,0.000111
  giBassEee69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.212220,0.006076,0.000565,0.043897,0.004701,0.169692,0.084989,0.011954,0.000713,0.000170
  giBassEee72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.121291,0.001021,0.010166,0.001642,0.244060,0.061995,0.005244,0.000275,0.000113
  giBassEee75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.049112,0.001647,0.021440,0.113388,0.092568,0.003249,0.000304,0.000100
  giBassEee78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.004609,0.007093,0.010596,0.098393,0.005375,0.000222
  giBassEee81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.012152,0.087798,0.339376,0.023913,0.000340
  giBassEee84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.002039,0.003283,0.123990,0.000551,0.000112
 elseif iVoice2==4 then
  giBassOh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.002656,0.003252,0.004105,0.005622,0.007160,0.009459,0.013108,0.015856,0.025894,0.031448,0.069195,0.117293,0.097983,0.042979,0.032065,0.021929,0.018078,0.015475,0.014066,0.014550,0.016289,0.021526,0.023582,0.017583,0.010850,0.007974,0.005821,0.004584,0.003615,0.002841,0.002234,0.001707,0.001268,0.000905,0.000600,0.000361,0.000186,0.000124,0.000163,0.000187,0.000186,0.000166,0.000139,0.000101,0.000100,0.000137,0.000189,0.000263,0.000361,0.000489,0.000654,0.000858,0.001117,0.001451,0.001869,0.002471,0.003247,0.004440,0.005393,0.004629,0.002917,0.002066,0.002532,0.003852,0.005391,0.005792,0.004814,0.003675,0.002842,0.002214,0.001763,0.001401,0.001091,0.000746,0.000345,0.000127,0.000165,0.000194,0.000194,0.000185,0.000171,0.000154,0.000137,0.000119,0.000101
  giBassOh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.003258,0.004239,0.005824,0.008031,0.011084,0.016213,0.021193,0.035813,0.061897,0.136532,0.111390,0.042393,0.033786,0.022028,0.018961,0.016732,0.017591,0.021496,0.028305,0.022987,0.013061,0.009039,0.006327,0.004802,0.003617,0.002714,0.001972,0.001369,0.000897,0.000513,0.000245,0.000108,0.000159,0.000210,0.000226,0.000203,0.000166,0.000112,0.000138,0.000201,0.000297,0.000435,0.000622,0.000874,0.001199,0.001639,0.002216,0.003089,0.004305,0.006101,0.005978,0.003630,0.002459,0.003410,0.005563,0.006988,0.005953,0.004329,0.003190,0.002395,0.001829,0.001373,0.000891,0.000334,0.000160,0.000222,0.000234,0.000222,0.000202,0.000178,0.000153,0.000127,0.000102
  giBassOh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.004023,0.005567,0.008482,0.012147,0.018935,0.027436,0.043784,0.124923,0.156060,0.054359,0.038530,0.025628,0.020731,0.020302,0.024164,0.033681,0.024398,0.013158,0.008555,0.006066,0.004329,0.003073,0.002076,0.001307,0.000716,0.000311,0.000128,0.000209,0.000264,0.000252,0.000205,0.000130,0.000103,0.000161,0.000252,0.000400,0.000621,0.000939,0.001374,0.001992,0.002862,0.004244,0.006522,0.007371,0.004231,0.002979,0.004965,0.007920,0.007484,0.005169,0.003596,0.002580,0.001862,0.001227,0.000441,0.000198,0.000273,0.000271,0.000248,0.000214,0.000179,0.000142,0.000109
  giBassOh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.005059,0.007690,0.012187,0.020150,0.031282,0.054249,0.186526,0.120147,0.052781,0.030786,0.024379,0.024804,0.034918,0.035401,0.016807,0.010063,0.006707,0.004475,0.002912,0.001727,0.000868,0.000313,0.000184,0.000294,0.000312,0.000251,0.000145,0.000162,0.000275,0.000476,0.000802,0.001297,0.002024,0.003113,0.004960,0.008196,0.007816,0.003687,0.004946,0.009187,0.008701,0.005588,0.003662,0.002498,0.001599,0.000503,0.000264,0.000332,0.000310,0.000265,0.000215,0.000163,0.000117
  giBassOh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.006506,0.011247,0.018924,0.031724,0.062920,0.234685,0.085997,0.043880,0.030968,0.029117,0.043082,0.035194,0.015962,0.009176,0.005688,0.003419,0.001811,0.000723,0.000184,0.000326,0.000373,0.000279,0.000129,0.000146,0.000275,0.000527,0.000981,0.001724,0.002915,0.004964,0.008921,0.009303,0.004152,0.007745,0.011649,0.007393,0.004456,0.002821,0.001501,0.000256,0.000391,0.000373,0.000311,0.000239,0.000169,0.000110
  giBassOh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.008468,0.016044,0.032394,0.071542,0.272804,0.084708,0.044003,0.033428,0.042935,0.045900,0.018050,0.009589,0.005421,0.002735,0.001024,0.000216,0.000420,0.000405,0.000225,0.000125,0.000277,0.000594,0.001244,0.002397,0.004430,0.008609,0.011959,0.004919,0.011123,0.011908,0.006381,0.003660,0.001783,0.000320,0.000469,0.000405,0.000305,0.000205,0.000123
  giBassOh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.011135,0.024299,0.054889,0.249961,0.108722,0.051276,0.040626,0.067415,0.026335,0.012142,0.006153,0.002616,0.000622,0.000415,0.000502,0.000257,0.000119,0.000122,0.000324,0.000801,0.001884,0.004002,0.008536,0.014820,0.005995,0.015949,0.010404,0.005196,0.002469,0.000400,0.000547,0.000433,0.000288,0.000164
  giBassOh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.015381,0.040301,0.108505,0.240245,0.061575,0.049617,0.070803,0.020128,0.008953,0.003454,0.000627,0.000589,0.000502,0.000134,0.000183,0.000551,0.001609,0.004060,0.009951,0.015670,0.009932,0.017456,0.007348,0.003209,0.000530,0.000621,0.000431,0.000236,0.000113
  giBassOh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.022496,0.063453,0.469427,0.087754,0.058244,0.070388,0.018356,0.006838,0.001446,0.000651,0.000556,0.000143,0.000122,0.000550,0.001961,0.005833,0.017875,0.008317,0.023338,0.008927,0.003005,0.000784,0.000625,0.000341,0.000143
  giBassOh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.032102,0.143154,0.169468,0.066893,0.091855,0.019191,0.005474,0.000433,0.000811,0.000162,0.000249,0.001190,0.004802,0.017251,0.009855,0.023855,0.007334,0.000641,0.000812,0.000411,0.000150,0.000101
  giBassOh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.048599,0.499974,0.102552,0.134829,0.024280,0.005230,0.000837,0.000520,0.000237,0.000241,0.001607,0.008013,0.029637,0.031910,0.010392,0.000800,0.000865,0.000325,0.000132
  giBassOh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.080605,0.480428,0.099238,0.040254,0.006909,0.001178,0.000182,0.000102,0.001103,0.008123,0.031336,0.034912,0.006417,0.001243,0.000473,0.000155
  giBassOh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.126917,0.175507,0.140772,0.013677,0.001307,0.000241,0.003934,0.035768,0.046682,0.006011,0.001247,0.000283,0.000138
  giBassOh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.286365,0.133810,0.038389,0.000867,0.000107,0.000499,0.009611,0.019717,0.014670,0.001624,0.000299,0.000129
  giBassOh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000002,0.269661,0.010462,0.001040,0.000483,0.016028,0.063825,0.001601,0.000650,0.000177
  giBassOh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.960747,0.080495,0.002355,0.000205,0.016245,0.069803,0.002487,0.000311,0.000112
  giBassOh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.350990,0.027349,0.007868,0.093348,0.002493,0.000277
  giBassOh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.267641,0.001732,0.001001,0.039440,0.003247,0.000257
  giBassOh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.539313,0.002079,0.032056,0.003202,0.000354
  giBassOh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.160992,0.000409,0.139609,0.000620,0.000116
  giBassOh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.054697,0.015735,0.004986,0.000134
 elseif iVoice2==5 then
  giBassOoh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.004170,0.004767,0.005809,0.007125,0.009423,0.012338,0.016616,0.025281,0.036773,0.084223,0.097509,0.055644,0.026776,0.021850,0.013883,0.011537,0.008646,0.005282,0.002155,0.001654,0.001414,0.001239,0.001040,0.000878,0.000785,0.000712,0.000629,0.000575,0.000483,0.000406,0.000327,0.000237,0.000173,0.000104,0.000121,0.000160,0.000207,0.000263,0.000334,0.000421,0.000551,0.000726,0.001018,0.001329,0.001313,0.001049,0.000891,0.000885,0.000983,0.001199,0.001540,0.001977,0.002121,0.001784,0.001340,0.001034,0.000826,0.000705,0.000652,0.000680,0.000715,0.000646,0.000503,0.000390,0.000309,0.000249,0.000204,0.000167,0.000137,0.000111
  giBassOoh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.005030,0.006191,0.007651,0.010372,0.014571,0.021434,0.030781,0.071907,0.119545,0.072776,0.031574,0.022556,0.015399,0.011394,0.007041,0.002518,0.001911,0.001634,0.001295,0.001096,0.000937,0.000834,0.000721,0.000630,0.000508,0.000397,0.000272,0.000181,0.000107,0.000153,0.000213,0.000285,0.000379,0.000500,0.000688,0.000967,0.001445,0.001618,0.001277,0.001057,0.001077,0.001285,0.001687,0.002301,0.002526,0.001994,0.001425,0.001068,0.000863,0.000780,0.000826,0.000841,0.000676,0.000495,0.000375,0.000290,0.000229,0.000180,0.000141,0.000108
  giBassOoh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.006083,0.008003,0.010711,0.016374,0.024519,0.037449,0.112833,0.124998,0.041417,0.028771,0.018019,0.012343,0.005402,0.002386,0.001972,0.001531,0.001231,0.001066,0.000905,0.000784,0.000607,0.000449,0.000285,0.000153,0.000101,0.000109,0.000153,0.000230,0.000327,0.000460,0.000642,0.000947,0.001505,0.001937,0.001502,0.001242,0.001345,0.001774,0.002567,0.002998,0.002232,0.001518,0.001113,0.000935,0.000970,0.000984,0.000727,0.000511,0.000372,0.000279,0.000210,0.000157,0.000113
  giBassOoh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.007502,0.010380,0.016096,0.024230,0.043922,0.147876,0.113247,0.043397,0.022792,0.015355,0.005760,0.002715,0.002152,0.001629,0.001298,0.001071,0.000882,0.000647,0.000405,0.000215,0.000125,0.000127,0.000114,0.000198,0.000315,0.000473,0.000703,0.001106,0.001912,0.002216,0.001551,0.001530,0.002044,0.003150,0.003426,0.002208,0.001462,0.001135,0.001161,0.001127,0.000745,0.000502,0.000355,0.000253,0.000178,0.000120
  giBassOoh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.009537,0.014254,0.024685,0.050579,0.168510,0.111335,0.043720,0.023086,0.010571,0.003311,0.002480,0.001757,0.001426,0.001152,0.000813,0.000474,0.000209,0.000141,0.000155,0.000117,0.000124,0.000242,0.000415,0.000671,0.001107,0.002045,0.002638,0.001790,0.001976,0.003097,0.004267,0.002696,0.001662,0.001314,0.001439,0.001013,0.000623,0.000411,0.000275,0.000178,0.000106
  giBassOoh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.012372,0.020730,0.042850,0.143721,0.145460,0.045094,0.022771,0.005016,0.003262,0.002189,0.001666,0.001260,0.000793,0.000362,0.000117,0.000168,0.000179,0.000112,0.000138,0.000307,0.000569,0.001000,0.001934,0.003236,0.002114,0.002570,0.004603,0.003988,0.002137,0.001561,0.001683,0.000990,0.000581,0.000361,0.000217,0.000117
  giBassOoh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.016008,0.032755,0.074919,0.250025,0.057545,0.024692,0.004776,0.003064,0.002133,0.001569,0.000898,0.000306,0.000146,0.000215,0.000146,0.000188,0.000459,0.000921,0.001903,0.003897,0.002498,0.003572,0.006036,0.003057,0.001884,0.001984,0.001031,0.000565,0.000317,0.000160
  giBassOoh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.020759,0.048464,0.295785,0.086792,0.030710,0.005430,0.003258,0.002143,0.001295,0.000430,0.000194,0.000255,0.000111,0.000112,0.000397,0.000949,0.002219,0.004444,0.003070,0.006325,0.004431,0.002278,0.002265,0.001008,0.000508,0.000241
  giBassOoh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.028510,0.101166,0.222637,0.046174,0.006623,0.003513,0.002303,0.000949,0.000184,0.000305,0.000110,0.000102,0.000478,0.001339,0.004097,0.003587,0.006207,0.005401,0.002632,0.002031,0.000827,0.000359,0.000115
  giBassOoh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.041474,0.287568,0.090200,0.010044,0.004379,0.002520,0.000724,0.000336,0.000224,0.000275,0.001139,0.003874,0.004235,0.009226,0.004283,0.003373,0.001165,0.000434,0.000107
  giBassOoh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.065511,0.500015,0.049381,0.006128,0.003138,0.000611,0.000437,0.000383,0.001850,0.007794,0.007149,0.006113,0.003967,0.001128,0.000317
  giBassOoh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.096931,0.173581,0.010859,0.004286,0.000861,0.000510,0.000225,0.001899,0.008888,0.012656,0.004558,0.002017,0.000482
  giBassOoh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.202350,0.092351,0.007027,0.001897,0.000620,0.000129,0.000970,0.008202,0.012422,0.005266,0.001649,0.000227
  giBassOoh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.575236,0.020098,0.005041,0.000672,0.000138,0.002281,0.008473,0.008567,0.002331,0.000214
  giBassOoh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000000,0.012255,0.001224,0.000167,0.000119,0.003699,0.014299,0.007933,0.000634,0.000110
  giBassOoh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.347133,0.008571,0.001019,0.003798,0.025311,0.004032,0.000169
  giBassOoh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.184689,0.003794,0.000258,0.001940,0.024843,0.003298,0.000182
  giBassOoh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.040199,0.001344,0.000201,0.016948,0.004662,0.000147
  giBassOoh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.024509,0.000334,0.007398,0.015866,0.000219
  giBassOoh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.017142,0.050630,0.000336
  giBassOoh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.007586,0.003880,0.006596
 ; TENOR
 elseif iVoice2==6 then
  giTenorAh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.000583,0.000538,0.000460,0.000380,0.000388,0.000576,0.000951,0.001458,0.002147,0.003005,0.004083,0.005479,0.007166,0.009451,0.012430,0.016493,0.023155,0.032764,0.051969,0.061659,0.047828,0.030764,0.022969,0.017260,0.014303,0.012261,0.011262,0.011171,0.011848,0.014033,0.017624,0.024861,0.029338,0.024087,0.015948,0.011498,0.008368,0.006382,0.004890,0.003733,0.002846,0.002112,0.001533,0.001072,0.000708,0.000443,0.000260,0.000174,0.000165,0.000177,0.000179,0.000163,0.000135,0.000113,0.000143,0.000167,0.000180,0.000184,0.000185,0.000201,0.000268,0.000401,0.000606,0.000882,0.001240,0.001688,0.002239,0.002922,0.003753,0.004799,0.006128,0.007860,0.010282,0.013541,0.017840,0.020070,0.017502,0.013105,0.010276,0.009233,0.009930,0.012379,0.015621,0.016680,0.014400,0.011294,0.008942,0.007204,0.005991,0.005111,0.004508,0.004114,0.003872,0.003547,0.002939
  giTenorAh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.000687,0.000612,0.000487,0.000443,0.000667,0.001204,0.001979,0.003057,0.004439,0.006332,0.008723,0.012150,0.016682,0.024614,0.036410,0.062524,0.071897,0.047198,0.030625,0.021709,0.017111,0.014322,0.013279,0.013564,0.015875,0.020510,0.030613,0.034231,0.023560,0.015114,0.010394,0.007498,0.005453,0.003961,0.002827,0.001944,0.001277,0.000775,0.000433,0.000237,0.000194,0.000210,0.000212,0.000185,0.000139,0.000103,0.000152,0.000190,0.000213,0.000219,0.000222,0.000264,0.000405,0.000664,0.001044,0.001561,0.002232,0.003095,0.004195,0.005621,0.007518,0.010165,0.014016,0.019655,0.023962,0.020629,0.014615,0.011499,0.011311,0.014111,0.018689,0.019687,0.015745,0.011810,0.009057,0.007235,0.006000,0.005218,0.004763,0.004396,0.003608,0.002646,0.002014,0.001621,0.001344,0.001117,0.000916,0.000734,0.000568,0.000422,0.000298,0.000203,0.000142,0.000117,0.000116,0.000118,0.000115
  giTenorAh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.000802,0.000674,0.000523,0.000689,0.001389,0.002504,0.004118,0.006355,0.009422,0.013917,0.020353,0.032454,0.055699,0.087276,0.061357,0.036058,0.024181,0.018765,0.016031,0.015921,0.018738,0.025814,0.040021,0.034794,0.019857,0.012723,0.008611,0.005883,0.004012,0.002612,0.001611,0.000893,0.000439,0.000242,0.000241,0.000254,0.000220,0.000152,0.000106,0.000178,0.000230,0.000257,0.000260,0.000288,0.000455,0.000818,0.001385,0.002191,0.003277,0.004737,0.006712,0.009481,0.013715,0.020280,0.027934,0.024626,0.016448,0.013135,0.014739,0.020750,0.023477,0.018047,0.012882,0.009552,0.007507,0.006271,0.005611,0.005025,0.003750,0.002614,0.001991,0.001589,0.001273,0.000999,0.000753,0.000538,0.000359,0.000228,0.000153,0.000136,0.000139,0.000137,0.000122
  giTenorAh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.000932,0.000718,0.000659,0.001407,0.002892,0.005202,0.008652,0.013651,0.021662,0.036789,0.070120,0.102139,0.054521,0.032839,0.023181,0.019038,0.019227,0.024920,0.041242,0.045371,0.023935,0.014094,0.008980,0.005704,0.003505,0.002002,0.000991,0.000417,0.000274,0.000304,0.000270,0.000176,0.000182,0.000265,0.000306,0.000311,0.000394,0.000759,0.001464,0.002554,0.004117,0.006330,0.009569,0.014606,0.023150,0.033635,0.026442,0.017079,0.016240,0.023483,0.027945,0.019964,0.013524,0.009824,0.007738,0.006717,0.005807,0.003917,0.002676,0.002012,0.001549,0.001160,0.000819,0.000533,0.000316,0.000191,0.000162,0.000167,0.000158,0.000129
  giTenorAh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.001076,0.000760,0.001152,0.002918,0.006013,0.010963,0.018910,0.033003,0.065564,0.123397,0.061570,0.034546,0.024544,0.022364,0.028096,0.049779,0.048235,0.023028,0.012783,0.007477,0.004231,0.002147,0.000887,0.000349,0.000355,0.000327,0.000195,0.000152,0.000288,0.000361,0.000371,0.000539,0.001217,0.002491,0.004498,0.007542,0.012318,0.020674,0.035880,0.035208,0.020677,0.019986,0.031450,0.029001,0.018014,0.012074,0.009087,0.007807,0.005927,0.003587,0.002503,0.001834,0.001291,0.000830,0.000470,0.000249,0.000194,0.000200,0.000178,0.000130
  giTenorAh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.001221,0.000884,0.002403,0.006103,0.012641,0.024258,0.049147,0.124843,0.094249,0.043351,0.028599,0.027085,0.040952,0.068353,0.030182,0.014975,0.007910,0.003883,0.001548,0.000473,0.000421,0.000370,0.000161,0.000304,0.000426,0.000443,0.000809,0.002086,0.004462,0.008387,0.015033,0.028032,0.047931,0.029239,0.022629,0.037393,0.031509,0.018127,0.012012,0.009537,0.007224,0.004034,0.002693,0.001836,0.001138,0.000598,0.000284,0.000232,0.000231,0.000177,0.000105
  giTenorAh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.001347,0.001378,0.005009,0.012714,0.027848,0.064949,0.174667,0.072154,0.037557,0.031873,0.051698,0.069662,0.025483,0.011788,0.005236,0.001791,0.000485,0.000510,0.000306,0.000357,0.000517,0.000580,0.001645,0.004407,0.009531,0.019083,0.040846,0.049561,0.026461,0.041849,0.036370,0.019264,0.012655,0.010146,0.005280,0.003211,0.002020,0.001088,0.000460,0.000274,0.000277,0.000199,0.000106,0.000102
  giTenorAh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.001435,0.002814,0.010404,0.027304,0.073592,0.204283,0.065680,0.038086,0.049867,0.090768,0.028199,0.011415,0.004006,0.000835,0.000609,0.000353,0.000145,0.000530,0.000623,0.001521,0.005124,0.012704,0.029322,0.067515,0.034280,0.047183,0.040088,0.019737,0.013502,0.007873,0.004048,0.002335,0.001073,0.000384,0.000337,0.000259,0.000126,0.000121,0.000110
  giTenorAh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.001521,0.005836,0.021928,0.066019,0.246823,0.069099,0.044735,0.099607,0.046071,0.014964,0.004298,0.000698,0.000655,0.000576,0.000744,0.002438,0.009011,0.024682,0.071912,0.041426,0.063055,0.036096,0.018215,0.011882,0.005020,0.002589,0.000943,0.000389,0.000358,0.000166,0.000147,0.000118
  giTenorAh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.001771,0.012217,0.048556,0.249892,0.086762,0.054219,0.136820,0.029974,0.007772,0.000947,0.000741,0.000186,0.000854,0.001622,0.008944,0.030132,0.096048,0.045359,0.063137,0.024077,0.014480,0.005399,0.002282,0.000570,0.000463,0.000211,0.000176,0.000111
  giTenorAh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.002757,0.025433,0.129910,0.144292,0.063753,0.139297,0.023565,0.003578,0.001018,0.000106,0.001032,0.003292,0.019074,0.081740,0.052932,0.072742,0.025316,0.010562,0.004040,0.000921,0.000554,0.000212,0.000203
  giTenorAh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.005629,0.054612,0.408549,0.076173,0.181522,0.022828,0.001670,0.000706,0.001061,0.003044,0.025417,0.135064,0.094412,0.039477,0.015746,0.004670,0.000769,0.000519,0.000243,0.000120,0.000116
  giTenorAh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.011675,0.132064,0.138210,0.199271,0.029918,0.001395,0.000113,0.001485,0.018035,0.143887,0.126163,0.036439,0.010040,0.001886,0.000716,0.000291,0.000124
  giTenorAh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.024440,0.499943,0.108469,0.059961,0.001897,0.000373,0.003245,0.060299,0.090767,0.048171,0.010803,0.001140,0.000421,0.000222,0.000136
  giTenorAh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.050867,0.288566,0.278569,0.007157,0.000212,0.006586,0.163506,0.145461,0.021119,0.001842,0.000424,0.000177,0.000102
  giTenorAh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.109214,0.152326,0.045647,0.001411,0.006088,0.270092,0.078932,0.009337,0.001038,0.000241,0.000131
  giTenorAh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.264112,0.398522,0.002789,0.002968,0.287762,0.072864,0.003770,0.000582,0.000183,0.000108
  giTenorAh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,1.000000,0.119936,0.000746,0.120626,0.096351,0.002278,0.000445,0.000181
  giTenorAh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.577120,0.014313,0.013172,0.290914,0.003683,0.000354,0.000139
  giTenorAh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.304668,0.002822,0.540234,0.018674,0.000482,0.000178
  giTenorAh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.797043,0.005936,0.145727,0.001164,0.000216
 elseif iVoice2==7 then
  giTenorEh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.003711,0.004448,0.005620,0.007420,0.009664,0.012761,0.017128,0.022353,0.032172,0.044202,0.073680,0.104950,0.091828,0.054791,0.037776,0.026500,0.019680,0.014875,0.011123,0.008435,0.006205,0.004488,0.003152,0.002073,0.001292,0.000697,0.000324,0.000231,0.000316,0.000377,0.000378,0.000345,0.000277,0.000200,0.000121,0.000127,0.000231,0.000372,0.000572,0.000833,0.001169,0.001615,0.002156,0.002881,0.003837,0.005088,0.007184,0.010048,0.015987,0.019744,0.015794,0.009935,0.007135,0.005064,0.003843,0.002896,0.002183,0.001646,0.001193,0.000842,0.000546,0.000290,0.000179,0.000420,0.000705,0.001053,0.001472,0.002002,0.002658,0.003482,0.004575,0.005975,0.008042,0.010971,0.015644,0.021201,0.021315,0.015876,0.011198,0.008260,0.006172,0.004757,0.003667,0.002902,0.002421,0.002278,0.002500,0.003021,0.003820,0.004887,0.006330,0.008360,0.011171,0.014732,0.016196,0.013872,0.010516
  giTenorEh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.004552,0.005760,0.007814,0.010792,0.014955,0.021165,0.029612,0.045235,0.074156,0.122974,0.105870,0.057214,0.038741,0.025625,0.018589,0.013142,0.009439,0.006465,0.004364,0.002718,0.001559,0.000735,0.000308,0.000321,0.000440,0.000451,0.000405,0.000302,0.000194,0.000158,0.000311,0.000537,0.000866,0.001309,0.001925,0.002713,0.003839,0.005325,0.007903,0.011718,0.020082,0.022933,0.014817,0.009328,0.006254,0.004484,0.003195,0.002291,0.001599,0.001065,0.000642,0.000286,0.000381,0.000762,0.001246,0.001850,0.002651,0.003674,0.005077,0.006985,0.009944,0.014441,0.022249,0.026413,0.019930,0.013155,0.009153,0.006590,0.004826,0.003613,0.002894,0.002744,0.003188,0.004118,0.005508,0.007468,0.010395,0.014756,0.019114,0.017765,0.012963,0.009621,0.007541,0.006420,0.005922,0.006058,0.006871,0.008554,0.009629,0.007928,0.005529,0.003971,0.002926,0.002205,0.001657,0.001233,0.000892
  giTenorEh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.005607,0.007614,0.011207,0.016365,0.024712,0.037348,0.060375,0.121842,0.141500,0.072108,0.044425,0.028047,0.018770,0.012615,0.008179,0.005124,0.002879,0.001420,0.000505,0.000350,0.000521,0.000532,0.000443,0.000289,0.000131,0.000204,0.000437,0.000812,0.001364,0.002176,0.003280,0.004954,0.007398,0.011851,0.021792,0.027213,0.015773,0.009598,0.006088,0.004119,0.002767,0.001798,0.001092,0.000534,0.000408,0.000945,0.001653,0.002596,0.003890,0.005707,0.008347,0.012773,0.020361,0.030867,0.025823,0.015735,0.010227,0.007000,0.004859,0.003600,0.003233,0.003864,0.005306,0.007526,0.011040,0.016627,0.022728,0.019669,0.013379,0.009687,0.007778,0.007000,0.007281,0.008747,0.011171,0.010086,0.006655,0.004490,0.003153,0.002248,0.001593,0.001092,0.000713,0.000433,0.000247,0.000157,0.000144,0.000149,0.000141,0.000119
  giTenorEh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.007001,0.010427,0.016456,0.026630,0.043168,0.076989,0.169617,0.125049,0.061026,0.034673,0.021596,0.013484,0.007893,0.004258,0.001928,0.000584,0.000462,0.000640,0.000585,0.000382,0.000160,0.000329,0.000737,0.001424,0.002500,0.004083,0.006621,0.011222,0.020906,0.033101,0.018125,0.010139,0.006169,0.003823,0.002339,0.001300,0.000525,0.000152,0.000829,0.001747,0.003025,0.004885,0.007717,0.012276,0.020847,0.036056,0.029616,0.016722,0.010179,0.006565,0.004476,0.003867,0.004909,0.007299,0.011244,0.018113,0.027071,0.022114,0.014163,0.010128,0.008501,0.008626,0.010768,0.013620,0.009945,0.006082,0.003944,0.002638,0.001744,0.001093,0.000630,0.000328,0.000188,0.000175,0.000178,0.000156,0.000116
  giTenorEh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.008898,0.014845,0.025531,0.044722,0.088437,0.209987,0.109632,0.053027,0.029768,0.016881,0.008983,0.004150,0.001395,0.000463,0.000755,0.000691,0.000400,0.000255,0.000745,0.001668,0.003236,0.005773,0.010198,0.020143,0.039588,0.019923,0.010157,0.005810,0.003302,0.001689,0.000582,0.000359,0.001415,0.002957,0.005341,0.009195,0.016165,0.031454,0.042867,0.022527,0.012419,0.007380,0.004874,0.005036,0.007696,0.012757,0.022519,0.032658,0.021211,0.013339,0.010317,0.010331,0.013816,0.015314,0.008765,0.005134,0.003184,0.001938,0.001090,0.000535,0.000250,0.000207,0.000210,0.000173,0.000113
  giTenorEh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.011502,0.021549,0.042263,0.090322,0.245564,0.114253,0.051172,0.026245,0.012912,0.005428,0.001469,0.000641,0.000901,0.000604,0.000162,0.000315,0.001073,0.002614,0.005421,0.010640,0.023418,0.045836,0.018645,0.008963,0.004582,0.002130,0.000572,0.000762,0.002491,0.005301,0.010156,0.019894,0.044518,0.039884,0.018319,0.009661,0.005793,0.006382,0.011028,0.020816,0.038283,0.025968,0.015109,0.011870,0.013774,0.019307,0.011088,0.005870,0.003324,0.001790,0.000830,0.000329,0.000246,0.000244,0.000178
  giTenorEh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.015233,0.032741,0.074724,0.243797,0.144246,0.056112,0.025240,0.010252,0.002839,0.000700,0.001067,0.000580,0.000104,0.000883,0.002741,0.006585,0.014845,0.043733,0.031620,0.012214,0.005557,0.002198,0.000195,0.001893,0.005217,0.011479,0.025712,0.062141,0.031662,0.014093,0.007252,0.007795,0.015192,0.033589,0.039695,0.019565,0.014153,0.017708,0.020404,0.009087,0.004554,0.002215,0.000878,0.000317,0.000301,0.000241,0.000125
  giTenorEh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.020852,0.053258,0.153984,0.250056,0.069342,0.026968,0.008516,0.001168,0.001282,0.000765,0.000162,0.001475,0.005006,0.013261,0.041892,0.036286,0.012356,0.004686,0.001052,0.001663,0.006070,0.015487,0.041856,0.059403,0.020427,0.008986,0.009865,0.022609,0.054432,0.028468,0.017103,0.021683,0.020009,0.007941,0.003514,0.001272,0.000379,0.000357,0.000233,0.000104,0.000116
  giTenorEh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.029695,0.089457,0.420024,0.106052,0.033762,0.008298,0.000925,0.001385,0.000186,0.001501,0.006486,0.020422,0.079237,0.020329,0.006617,0.001174,0.002828,0.010697,0.032394,0.085861,0.024877,0.009765,0.015434,0.045175,0.042511,0.020691,0.027733,0.017578,0.006386,0.002185,0.000498,0.000422,0.000230,0.000123,0.000124
  giTenorEh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.043127,0.180772,0.228631,0.052523,0.010863,0.001282,0.001207,0.000630,0.005235,0.021308,0.091756,0.017942,0.004263,0.001528,0.010624,0.039871,0.079869,0.019355,0.012797,0.041749,0.052038,0.023798,0.038715,0.011768,0.003590,0.000660,0.000489,0.000194,0.000164
  giTenorEh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.065477,0.487624,0.112219,0.020505,0.001401,0.001155,0.001754,0.013158,0.087497,0.024420,0.004380,0.003803,0.022978,0.124331,0.028184,0.015603,0.067238,0.039134,0.035441,0.018175,0.004427,0.000636,0.000482,0.000172,0.000145
  giTenorEh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.106518,0.500069,0.053933,0.002334,0.001530,0.002950,0.026528,0.072557,0.009371,0.003329,0.030986,0.118789,0.017972,0.045243,0.056938,0.043394,0.015883,0.002543,0.000714,0.000208,0.000179,0.000107
  giTenorEh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.178931,0.212116,0.016600,0.002767,0.002985,0.040842,0.040651,0.002326,0.021412,0.171741,0.019535,0.090410,0.041394,0.035157,0.004372,0.000844,0.000247,0.000120
  giTenorEh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.361640,0.105064,0.002567,0.001261,0.042631,0.035890,0.003057,0.079786,0.038715,0.083552,0.047617,0.023541,0.001321,0.000388,0.000190,0.000108
  giTenorEh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.975277,0.041007,0.002314,0.026319,0.048835,0.007606,0.248676,0.031209,0.078257,0.036342,0.001274,0.000344,0.000170
  giTenorEh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.999999,0.004669,0.005900,0.145077,0.006659,0.237507,0.090489,0.086786,0.005082,0.000415,0.000215,0.000103
  giTenorEh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.424187,0.005531,0.081687,0.004650,0.343420,0.180822,0.070293,0.001686,0.000240,0.000139
  giTenorEh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.210142,0.002525,0.071786,0.159607,0.167145,0.047085,0.000779,0.000220
  giTenorEh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.082013,0.052640,0.015212,0.062417,0.072683,0.000688,0.000200
  giTenorEh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.009338,0.290149,0.475002,0.173594,0.000829,0.000206
  giTenorEh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.011062,0.009300,0.361645,0.003372,0.000278
 elseif iVoice2==8 then
  giTenorEee24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.007836,0.009017,0.011311,0.014842,0.018645,0.029937,0.038490,0.092836,0.120812,0.077127,0.033458,0.027877,0.016830,0.013887,0.009932,0.007423,0.005705,0.003965,0.002903,0.001947,0.001187,0.000715,0.000276,0.000168,0.000280,0.000294,0.000295,0.000264,0.000200,0.000154,0.000111,0.000202,0.000314,0.000470,0.000672,0.000922,0.001256,0.001662,0.002194,0.002908,0.003823,0.005290,0.007297,0.010980,0.014156,0.012522,0.008422,0.005960,0.004302,0.003220,0.002441,0.001839,0.001385,0.001017,0.000728,0.000498,0.000315,0.000172,0.000153,0.000269,0.000415,0.000594,0.000822,0.001105,0.001457,0.001919,0.002506,0.003348,0.004555,0.006428,0.008898,0.009355,0.007247,0.005218,0.004033,0.003245,0.002812,0.002568,0.002513,0.002633,0.002927,0.003483,0.004336,0.005623,0.006787,0.006501
  giTenorEee27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.009549,0.011595,0.015950,0.020100,0.035178,0.051554,0.130198,0.125062,0.047558,0.034534,0.019595,0.015158,0.010326,0.007489,0.004997,0.003422,0.002102,0.001160,0.000517,0.000131,0.000196,0.000347,0.000349,0.000335,0.000261,0.000188,0.000102,0.000117,0.000241,0.000405,0.000645,0.000960,0.001392,0.001953,0.002719,0.003785,0.005367,0.007876,0.012538,0.016966,0.013606,0.008442,0.005765,0.004012,0.002889,0.002064,0.001464,0.000997,0.000649,0.000381,0.000181,0.000125,0.000280,0.000481,0.000736,0.001075,0.001512,0.002098,0.002890,0.004070,0.005876,0.009027,0.011425,0.009230,0.006235,0.004602,0.003652,0.003170,0.003002,0.003138,0.003573,0.004460,0.005915,0.007838,0.007914,0.005903,0.004220,0.003145,0.002512,0.002171,0.002123,0.002075,0.001670,0.001214,0.000931,0.000732,0.000589,0.000472,0.000374,0.000289,0.000216
  giTenorEee30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.011630,0.015290,0.021759,0.037725,0.057788,0.163156,0.117975,0.044909,0.026599,0.018812,0.011933,0.008144,0.004975,0.003001,0.001508,0.000594,0.000130,0.000311,0.000418,0.000409,0.000314,0.000210,0.000107,0.000263,0.000494,0.000849,0.001339,0.002047,0.003027,0.004491,0.006866,0.010927,0.018703,0.017814,0.010239,0.006481,0.004290,0.002881,0.001930,0.001234,0.000740,0.000381,0.000122,0.000299,0.000577,0.000950,0.001469,0.002176,0.003199,0.004773,0.007413,0.012158,0.012680,0.008124,0.005577,0.004243,0.003652,0.003588,0.004011,0.005100,0.007116,0.009602,0.008448,0.005627,0.003915,0.002969,0.002529,0.002523,0.002225,0.001542,0.001115,0.000840,0.000647,0.000496,0.000369,0.000263,0.000176,0.000109
  giTenorEee33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.014340,0.021213,0.032104,0.055421,0.189690,0.120221,0.050550,0.026047,0.016045,0.010125,0.005723,0.003021,0.001308,0.000246,0.000324,0.000496,0.000471,0.000309,0.000160,0.000115,0.000295,0.000626,0.001166,0.001976,0.003173,0.005084,0.008393,0.014976,0.024026,0.015041,0.008440,0.005111,0.003182,0.001956,0.001114,0.000544,0.000160,0.000136,0.000453,0.000897,0.001549,0.002490,0.003935,0.006420,0.011227,0.016103,0.010451,0.006593,0.004861,0.004256,0.004516,0.005730,0.008419,0.011629,0.008791,0.005493,0.003779,0.003042,0.003014,0.002416,0.001563,0.001093,0.000802,0.000584,0.000408,0.000266,0.000158
  giTenorEee36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.018039,0.029693,0.059894,0.185737,0.154314,0.055778,0.027787,0.014854,0.007935,0.003897,0.001432,0.000187,0.000561,0.000590,0.000401,0.000185,0.000117,0.000134,0.000109,0.000405,0.000942,0.001848,0.003332,0.005831,0.010609,0.022023,0.025121,0.011959,0.006463,0.003691,0.002042,0.001000,0.000346,0.000540,0.001193,0.002221,0.003859,0.006735,0.012934,0.018831,0.010505,0.006536,0.005174,0.005305,0.007021,0.011338,0.013114,0.007641,0.004738,0.003619,0.003554,0.002391,0.001498,0.001024,0.000703,0.000458,0.000268,0.000136
  giTenorEee39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.023171,0.040165,0.103046,0.249899,0.069009,0.030296,0.014964,0.006837,0.002317,0.000262,0.000693,0.000670,0.000375,0.000134,0.000157,0.000234,0.000809,0.001919,0.003902,0.007563,0.015739,0.033909,0.016874,0.008020,0.004127,0.001993,0.000762,0.000560,0.001472,0.003025,0.005784,0.011760,0.022871,0.012484,0.007312,0.006012,0.007155,0.011847,0.015854,0.008455,0.005033,0.004257,0.003348,0.001866,0.001182,0.000751,0.000433,0.000208
  giTenorEee42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.030585,0.075466,0.326397,0.089842,0.037631,0.016293,0.006004,0.001189,0.000623,0.000819,0.000421,0.000187,0.000122,0.000220,0.000997,0.002690,0.006079,0.013783,0.037543,0.020537,0.008608,0.003873,0.001483,0.000243,0.000604,0.001913,0.004382,0.009617,0.024507,0.016361,0.008552,0.007239,0.010299,0.019396,0.011363,0.006002,0.005105,0.003117,0.001699,0.001004,0.000533,0.000219,0.000103,0.000108
  giTenorEee45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.042426,0.110844,0.240398,0.052093,0.020251,0.006042,0.000491,0.000993,0.000619,0.000230,0.000590,0.002336,0.006356,0.016818,0.048133,0.016906,0.006376,0.002233,0.000321,0.000910,0.003109,0.007901,0.022561,0.020975,0.009763,0.009076,0.016934,0.017670,0.007601,0.006066,0.003146,0.001615,0.000823,0.000318,0.000124,0.000127
  giTenorEee48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.059390,0.371533,0.111553,0.029708,0.007793,0.000375,0.001180,0.000370,0.000232,0.000146,0.000819,0.003706,0.011680,0.044106,0.023926,0.007382,0.001997,0.000198,0.002392,0.007734,0.025933,0.021044,0.010370,0.014081,0.026288,0.009501,0.007130,0.003002,0.001408,0.000534,0.000152,0.000151
  giTenorEee51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.080361,0.499941,0.060609,0.013679,0.000525,0.001340,0.000163,0.000315,0.000469,0.003843,0.015148,0.067904,0.016057,0.003990,0.002951,0.011593,0.045827,0.014651,0.014346,0.031778,0.010090,0.006709,0.002369,0.000868,0.000190,0.000173
  giTenorEee54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.150937,0.179680,0.032584,0.002376,0.001637,0.000158,0.000252,0.001981,0.012152,0.075113,0.017221,0.002972,0.001205,0.008768,0.049043,0.017107,0.020616,0.022725,0.010214,0.003400,0.001068,0.000208,0.000166
  giTenorEee57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.221694,0.104185,0.012082,0.001986,0.000188,0.000185,0.004673,0.033646,0.033807,0.004465,0.001822,0.015810,0.041944,0.018160,0.035337,0.012137,0.003231,0.000635,0.000256
  giTenorEee60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.743154,0.059418,0.000747,0.000739,0.000301,0.007400,0.088245,0.014776,0.000388,0.015476,0.042088,0.028181,0.019002,0.006011,0.001075,0.000301,0.000101
  giTenorEee63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.999999,0.027360,0.002680,0.000630,0.007690,0.135857,0.007982,0.005905,0.091694,0.028709,0.020187,0.004740,0.000381,0.000133
  giTenorEee66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.359360,0.004751,0.000315,0.003964,0.150241,0.005941,0.017540,0.034212,0.045442,0.006799,0.000416,0.000140
  giTenorEee69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.208351,0.003970,0.000371,0.067291,0.008928,0.031620,0.036318,0.024271,0.001270,0.000155
  giTenorEee72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.118830,0.001480,0.014801,0.029546,0.030951,0.056360,0.012020,0.000602
  giTenorEee75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.054723,0.001258,0.271741,0.011812,0.057430,0.009479,0.000266
  giTenorEee78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.009503,0.007927,0.011882,0.068422,0.013599,0.000280
  giTenorEee81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.007941,0.134597,0.063248,0.048544,0.000311
  giTenorEee84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.002957,0.059093,0.112719,0.001203,0.000131
 elseif iVoice2==9 then
  giTenorOh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.002609,0.003212,0.004071,0.005593,0.007140,0.009447,0.013108,0.015876,0.025921,0.031520,0.069218,0.117217,0.097674,0.042519,0.031624,0.021238,0.017323,0.014353,0.012627,0.012174,0.012732,0.014614,0.019478,0.026308,0.024844,0.016400,0.010959,0.008002,0.005958,0.004623,0.003544,0.002734,0.002055,0.001508,0.001067,0.000705,0.000436,0.000235,0.000143,0.000144,0.000177,0.000192,0.000182,0.000161,0.000124,0.000104,0.000128,0.000145,0.000153,0.000156,0.000157,0.000181,0.000242,0.000356,0.000521,0.000741,0.001027,0.001380,0.001829,0.002381,0.003069,0.003965,0.005082,0.006691,0.008822,0.012074,0.015298,0.014009,0.009283,0.006092,0.006121,0.008720,0.012324,0.013604,0.011387,0.008379,0.005974,0.003793,0.001511,0.000709,0.001826,0.001937,0.001735,0.001456,0.001193,0.000965,0.000774,0.000620
  giTenorOh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.003203,0.004193,0.005787,0.008004,0.011069,0.016214,0.021222,0.035866,0.061952,0.136461,0.110996,0.041891,0.033083,0.021225,0.017671,0.014999,0.014528,0.015884,0.020089,0.030066,0.029796,0.018039,0.011760,0.008010,0.005921,0.004311,0.003164,0.002231,0.001534,0.000966,0.000557,0.000270,0.000164,0.000189,0.000228,0.000219,0.000192,0.000139,0.000133,0.000165,0.000180,0.000186,0.000188,0.000227,0.000336,0.000535,0.000821,0.001216,0.001726,0.002403,0.003262,0.004422,0.005950,0.008247,0.011486,0.016679,0.017937,0.011848,0.007184,0.007840,0.012354,0.016239,0.014304,0.010016,0.006636,0.003533,0.000306,0.002001,0.002325,0.002034,0.001635,0.001282,0.000989,0.000760,0.000584,0.000453,0.000355,0.000283,0.000227,0.000183,0.000149,0.000123,0.000103
  giTenorOh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.003960,0.005516,0.008444,0.012125,0.018936,0.027471,0.043879,0.124923,0.155727,0.053732,0.037654,0.024575,0.019046,0.017222,0.018408,0.024108,0.037434,0.030007,0.016142,0.010296,0.007092,0.004871,0.003350,0.002173,0.001311,0.000693,0.000293,0.000193,0.000251,0.000266,0.000236,0.000161,0.000147,0.000194,0.000215,0.000221,0.000238,0.000349,0.000604,0.001006,0.001590,0.002388,0.003477,0.004987,0.007107,0.010506,0.015883,0.021912,0.015844,0.008572,0.009907,0.016773,0.018799,0.013123,0.008157,0.003769,0.000813,0.002743,0.002542,0.001996,0.001496,0.001097,0.000802,0.000589,0.000439,0.000334,0.000257,0.000200,0.000157,0.000127,0.000107
  giTenorOh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.004986,0.007636,0.012153,0.020144,0.031323,0.054374,0.186452,0.119440,0.051970,0.029606,0.022330,0.020530,0.024026,0.039836,0.039006,0.018975,0.011161,0.007228,0.004658,0.002839,0.001584,0.000735,0.000266,0.000262,0.000323,0.000286,0.000181,0.000138,0.000212,0.000253,0.000263,0.000299,0.000510,0.000968,0.001707,0.002787,0.004353,0.006688,0.010300,0.016543,0.025924,0.017916,0.009680,0.015265,0.023074,0.016822,0.009587,0.003290,0.002408,0.003229,0.002528,0.001800,0.001245,0.000857,0.000598,0.000428,0.000313,0.000233,0.000176,0.000139,0.000117,0.000102
  giTenorOh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.006425,0.011190,0.018900,0.031763,0.063063,0.234533,0.085077,0.042498,0.028723,0.024364,0.029248,0.052657,0.032828,0.016019,0.009256,0.005473,0.003020,0.001411,0.000472,0.000289,0.000385,0.000324,0.000169,0.000209,0.000292,0.000313,0.000363,0.000715,0.001488,0.002773,0.004783,0.007968,0.013450,0.024276,0.028174,0.012256,0.017548,0.027382,0.016871,0.007639,0.001429,0.003903,0.002935,0.001946,0.001251,0.000808,0.000539,0.000371,0.000260,0.000189,0.000148,0.000125,0.000105
  giTenorOh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.008376,0.015990,0.032397,0.071647,0.272662,0.083705,0.042399,0.029965,0.031725,0.060042,0.036024,0.015995,0.008610,0.004457,0.001931,0.000539,0.000377,0.000438,0.000278,0.000188,0.000329,0.000372,0.000455,0.001070,0.002432,0.004805,0.008844,0.016499,0.033376,0.023715,0.015683,0.032489,0.020044,0.007073,0.004006,0.004073,0.002568,0.001523,0.000908,0.000566,0.000367,0.000246,0.000180,0.000147,0.000118
  giTenorOh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.011034,0.024255,0.054958,0.249960,0.107468,0.049168,0.034461,0.048258,0.060049,0.020609,0.009752,0.004351,0.001387,0.000382,0.000531,0.000321,0.000134,0.000183,0.000390,0.000442,0.000696,0.002019,0.004800,0.010033,0.021149,0.044106,0.017250,0.033814,0.026431,0.007582,0.005537,0.004027,0.002216,0.001191,0.000676,0.000405,0.000256,0.000192,0.000151,0.000111
  giTenorOh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.015273,0.040290,0.108755,0.238831,0.059213,0.041067,0.079700,0.037954,0.014459,0.005681,0.001470,0.000525,0.000572,0.000143,0.000149,0.000276,0.000507,0.000599,0.001942,0.005592,0.013421,0.033213,0.035928,0.030674,0.033762,0.006591,0.006489,0.003618,0.001724,0.000862,0.000468,0.000280,0.000206,0.000147,0.000101
  giTenorOh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.022381,0.063531,0.469122,0.084990,0.048733,0.105348,0.032042,0.010949,0.002821,0.000574,0.000647,0.000166,0.000185,0.000584,0.000720,0.002976,0.009583,0.026955,0.056424,0.035183,0.033796,0.002879,0.005885,0.002510,0.001081,0.000521,0.000299,0.000213,0.000130
  giTenorOh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.031994,0.143365,0.167462,0.059961,0.120183,0.032013,0.008919,0.001080,0.000876,0.000112,0.000190,0.000377,0.000745,0.002145,0.009630,0.033066,0.047478,0.065117,0.014161,0.008162,0.003051,0.001136,0.000494,0.000295,0.000181,0.000121
  giTenorOh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.048511,0.499973,0.098337,0.096526,0.041209,0.008700,0.000772,0.000645,0.000267,0.000365,0.000886,0.004049,0.020077,0.088236,0.067669,0.015154,0.008053,0.002378,0.000809,0.000383,0.000218,0.000140
  giTenorOh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.080583,0.477600,0.082135,0.075901,0.011361,0.001051,0.000287,0.001014,0.003886,0.026853,0.071839,0.067519,0.012982,0.003449,0.000937,0.000412,0.000202,0.000131
  giTenorOh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.127073,0.169977,0.210725,0.021898,0.001155,0.000367,0.001455,0.019176,0.112848,0.067589,0.011765,0.002161,0.000595,0.000260,0.000153
  giTenorOh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.286787,0.119943,0.064037,0.002159,0.000224,0.000754,0.004292,0.066166,0.130304,0.016330,0.002272,0.000591,0.000243,0.000124
  giTenorOh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000000,0.193060,0.017402,0.001290,0.000730,0.008099,0.176475,0.030291,0.004757,0.000766,0.000282,0.000138
  giTenorOh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.955092,0.151776,0.002102,0.000192,0.007772,0.143628,0.025957,0.001874,0.000402,0.000171
  giTenorOh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.339931,0.043789,0.000154,0.002909,0.225646,0.023525,0.001190,0.000305,0.000120
  giTenorOh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.239903,0.004318,0.001511,0.132362,0.032662,0.001182,0.000249
  giTenorOh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.386114,0.002580,0.016199,0.060581,0.001531,0.000277
  giTenorOh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.303561,0.000384,0.287242,0.003746,0.000342
  giTenorOh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.087577,0.005818,0.047050,0.000610,0.000103
 elseif iVoice2==10 then
  giTenorOoh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.004160,0.004756,0.005796,0.007110,0.009404,0.012319,0.016588,0.025253,0.036732,0.084172,0.097499,0.055655,0.026840,0.021900,0.014064,0.011724,0.009450,0.006251,0.002854,0.002079,0.001522,0.001320,0.001012,0.000844,0.000726,0.000657,0.000578,0.000532,0.000449,0.000378,0.000306,0.000221,0.000161,0.000112,0.000145,0.000197,0.000271,0.000371,0.000502,0.000666,0.000873,0.001133,0.001454,0.001875,0.002392,0.003125,0.004070,0.005451,0.006740,0.005949,0.003815,0.003032,0.004216,0.006465,0.009166,0.010218,0.008738,0.006687,0.005210,0.004127,0.003395,0.002882,0.002550,0.002381,0.002374,0.002539,0.002842
  giTenorOoh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.005018,0.006177,0.007634,0.010351,0.014548,0.021399,0.030750,0.071843,0.119517,0.072786,0.031652,0.022635,0.015612,0.012023,0.008250,0.003348,0.002272,0.001786,0.001280,0.001064,0.000866,0.000770,0.000661,0.000586,0.000473,0.000372,0.000254,0.000169,0.000102,0.000123,0.000163,0.000233,0.000341,0.000493,0.000700,0.000970,0.001326,0.001786,0.002410,0.003241,0.004476,0.006249,0.008081,0.006509,0.003821,0.004319,0.007145,0.010967,0.012075,0.009457,0.006928,0.005206,0.004101,0.003382,0.002964,0.002823,0.002955,0.003370,0.003360,0.002535,0.001765,0.001296,0.000988,0.000767,0.000592,0.000451,0.000332,0.000236,0.000160,0.000105
  giTenorOoh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.006069,0.007985,0.010689,0.016347,0.024478,0.037411,0.112759,0.124998,0.041485,0.028849,0.018264,0.013449,0.006554,0.003139,0.002141,0.001509,0.001181,0.000981,0.000833,0.000728,0.000565,0.000421,0.000265,0.000141,0.000101,0.000117,0.000142,0.000198,0.000307,0.000479,0.000736,0.001090,0.001584,0.002260,0.003208,0.004668,0.006836,0.009554,0.007100,0.004250,0.006687,0.011831,0.014395,0.010804,0.007515,0.005449,0.004235,0.003554,0.003339,0.003592,0.004112,0.003422,0.002223,0.001532,0.001113,0.000822,0.000601,0.000424,0.000284,0.000179,0.000109
  giTenorOoh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.007484,0.010358,0.016064,0.024189,0.043878,0.147795,0.113259,0.043500,0.023110,0.016473,0.007061,0.003259,0.002323,0.001591,0.001196,0.000986,0.000820,0.000603,0.000378,0.000199,0.000110,0.000113,0.000104,0.000123,0.000148,0.000202,0.000327,0.000558,0.000928,0.001473,0.002265,0.003459,0.005340,0.008451,0.011341,0.006178,0.006471,0.013053,0.017137,0.011865,0.007810,0.005560,0.004372,0.003989,0.004406,0.004836,0.003238,0.002015,0.001363,0.000950,0.000651,0.000420,0.000250,0.000139
  giTenorOoh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.009514,0.014225,0.024646,0.050523,0.168409,0.111356,0.043820,0.023460,0.012509,0.004160,0.002642,0.001689,0.001315,0.001065,0.000757,0.000443,0.000192,0.000123,0.000137,0.000101,0.000120,0.000148,0.000186,0.000291,0.000545,0.001008,0.001755,0.002924,0.004809,0.008185,0.013559,0.007678,0.008486,0.018454,0.017597,0.010496,0.006842,0.005140,0.004787,0.005732,0.004640,0.002578,0.001610,0.001047,0.000655,0.000372,0.000189,0.000109,0.000102
  giTenorOoh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.012343,0.020687,0.042780,0.143593,0.145481,0.045253,0.024030,0.006681,0.003567,0.002127,0.001539,0.001172,0.000743,0.000337,0.000147,0.000158,0.000139,0.000180,0.000245,0.000466,0.000987,0.001939,0.003573,0.006484,0.012507,0.013031,0.008641,0.021946,0.018929,0.010423,0.006772,0.005654,0.006751,0.005079,0.002597,0.001537,0.000903,0.000473,0.000212,0.000124,0.000121
  giTenorOoh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.015973,0.032700,0.074843,0.250024,0.057702,0.026906,0.006281,0.003020,0.001964,0.001456,0.000842,0.000281,0.000120,0.000190,0.000125,0.000117,0.000172,0.000230,0.000397,0.000965,0.002195,0.004549,0.009401,0.019240,0.008562,0.023865,0.021772,0.010988,0.007172,0.007255,0.006908,0.003095,0.001662,0.000857,0.000363,0.000158,0.000146,0.000111
  giTenorOoh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.020715,0.048381,0.295623,0.086997,0.032947,0.006518,0.003184,0.001972,0.001207,0.000399,0.000165,0.000225,0.000166,0.000247,0.000405,0.001120,0.002956,0.006943,0.016973,0.012394,0.026234,0.023823,0.011169,0.008018,0.009723,0.004053,0.001912,0.000846,0.000280,0.000172,0.000144
  giTenorOoh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.028452,0.101054,0.222679,0.046921,0.008320,0.003378,0.002130,0.000886,0.000146,0.000269,0.000181,0.000290,0.000582,0.002022,0.005860,0.016406,0.015371,0.037003,0.021032,0.010304,0.011498,0.005167,0.002100,0.000748,0.000221,0.000191
  giTenorOoh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.041389,0.287313,0.090518,0.013371,0.004256,0.002343,0.000674,0.000295,0.000189,0.000130,0.000278,0.000491,0.001979,0.007162,0.025068,0.017327,0.037924,0.013572,0.013536,0.005205,0.001811,0.000424,0.000243,0.000123
  giTenorOoh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.065400,0.500015,0.053810,0.006040,0.002913,0.000562,0.000387,0.000141,0.000350,0.000798,0.004390,0.018813,0.017129,0.043544,0.014348,0.013820,0.003325,0.000724,0.000290,0.000124
  giTenorOoh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.096766,0.173992,0.013034,0.003944,0.000798,0.000451,0.000332,0.000810,0.005915,0.033962,0.052501,0.022340,0.019449,0.003824,0.000560,0.000287,0.000114
  giTenorOoh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.202127,0.093846,0.006755,0.001771,0.000549,0.000186,0.000594,0.004041,0.032826,0.074042,0.020610,0.010337,0.001493,0.000384,0.000136
  giTenorOoh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.574725,0.026750,0.004688,0.000591,0.000260,0.000983,0.014330,0.034680,0.027154,0.010414,0.000848,0.000246,0.000117
  giTenorOoh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.999999,0.012080,0.001126,0.000101,0.000282,0.001597,0.037631,0.087072,0.027634,0.001448,0.000249
  giTenorOoh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.347954,0.007887,0.000902,0.000132,0.001620,0.067927,0.044669,0.007647,0.000575,0.000129
  giTenorOoh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.187680,0.003542,0.000182,0.001188,0.065653,0.041213,0.002984,0.000272
  giTenorOoh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.053502,0.001181,0.000524,0.028666,0.054314,0.001696,0.000234
  giTenorOoh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.024159,0.000201,0.003194,0.174139,0.002895,0.000179
  giTenorOoh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.015774,0.000264,0.135872,0.015294,0.000258
  giTenorOoh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.007082,0.002375,0.082425,0.000545
 ; COUNTERTENOR
 elseif iVoice2==11 then
  giCountertenorAh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.000622,0.000579,0.000505,0.000416,0.000404,0.000563,0.000935,0.001450,0.002158,0.003063,0.004178,0.005657,0.007437,0.009784,0.012967,0.016948,0.023788,0.032799,0.051846,0.069434,0.060781,0.039066,0.027913,0.020569,0.016254,0.013456,0.011669,0.010899,0.010953,0.011875,0.014299,0.018107,0.025757,0.032911,0.029411,0.019830,0.013946,0.010125,0.007600,0.005820,0.004423,0.003377,0.002523,0.001848,0.001319,0.000899,0.000595,0.000380,0.000265,0.000228,0.000227,0.000228,0.000214,0.000192,0.000160,0.000127,0.000135,0.000199,0.000281,0.000383,0.000508,0.000661,0.000851,0.001091,0.001400,0.001829,0.002402,0.003168,0.003603,0.003179,0.002384,0.001859,0.001660,0.001771,0.002195,0.002778,0.002998,0.002611,0.002049,0.001621,0.001304,0.001084,0.000925,0.000816,0.000745
  giCountertenorAh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.000735,0.000662,0.000536,0.000470,0.000654,0.001187,0.001971,0.003095,0.004543,0.006527,0.009051,0.012579,0.017418,0.025067,0.036897,0.062520,0.083382,0.061395,0.037338,0.026171,0.019458,0.015588,0.013449,0.012898,0.013720,0.016734,0.022266,0.033995,0.038925,0.027154,0.017317,0.011854,0.008501,0.006153,0.004459,0.003184,0.002202,0.001467,0.000918,0.000551,0.000338,0.000273,0.000271,0.000267,0.000243,0.000203,0.000155,0.000111,0.000121,0.000200,0.000308,0.000447,0.000626,0.000852,0.001149,0.001540,0.002109,0.002917,0.003991,0.004205,0.003195,0.002308,0.001985,0.002187,0.002895,0.003567,0.003282,0.002484,0.001880,0.001465,0.001192,0.001010,0.000899,0.000836,0.000741,0.000567,0.000414,0.000324,0.000266,0.000223,0.000186,0.000152,0.000121
  giCountertenorAh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.000859,0.000737,0.000570,0.000685,0.001368,0.002498,0.004192,0.006507,0.009781,0.014407,0.021215,0.033314,0.054502,0.095648,0.079270,0.043918,0.028744,0.021058,0.016749,0.015325,0.016241,0.020739,0.030451,0.046307,0.036821,0.021184,0.013513,0.009191,0.006237,0.004238,0.002750,0.001701,0.000967,0.000518,0.000340,0.000320,0.000317,0.000281,0.000219,0.000154,0.000105,0.000201,0.000341,0.000534,0.000794,0.001139,0.001619,0.002329,0.003422,0.004882,0.004650,0.003126,0.002393,0.002545,0.003536,0.004253,0.003418,0.002429,0.001787,0.001395,0.001156,0.001026,0.000931,0.000712,0.000489,0.000368,0.000293,0.000237,0.000189,0.000145,0.000106
  giCountertenorAh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.001001,0.000790,0.000685,0.001374,0.002883,0.005307,0.008904,0.014153,0.022631,0.037604,0.068470,0.117881,0.069973,0.039907,0.026187,0.019843,0.018237,0.020805,0.030099,0.052526,0.044464,0.023490,0.013922,0.008867,0.005619,0.003393,0.001911,0.000964,0.000476,0.000379,0.000381,0.000331,0.000241,0.000152,0.000120,0.000258,0.000474,0.000785,0.001223,0.001861,0.002867,0.004562,0.006080,0.004315,0.002924,0.003059,0.004530,0.004842,0.003317,0.002269,0.001676,0.001348,0.001192,0.000989,0.000643,0.000449,0.000343,0.000266,0.000200,0.000142
  giCountertenorAh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.001159,0.000833,0.001127,0.002901,0.006128,0.011319,0.019577,0.033913,0.065634,0.138955,0.078186,0.041169,0.026935,0.021819,0.023774,0.036256,0.065905,0.039715,0.020280,0.011658,0.006765,0.003703,0.001801,0.000761,0.000458,0.000456,0.000385,0.000254,0.000139,0.000103,0.000271,0.000566,0.001022,0.001711,0.002815,0.004833,0.007250,0.004799,0.003342,0.004420,0.006040,0.004129,0.002629,0.001866,0.001503,0.001298,0.000824,0.000531,0.000388,0.000284,0.000196,0.000121
  giCountertenorAh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.001323,0.000939,0.002370,0.006179,0.013031,0.025116,0.050054,0.124839,0.122598,0.052264,0.031128,0.025756,0.033416,0.067885,0.054229,0.023674,0.012290,0.006360,0.002931,0.001101,0.000546,0.000534,0.000406,0.000222,0.000118,0.000103,0.000242,0.000615,0.001251,0.002297,0.004218,0.007984,0.006394,0.003973,0.005794,0.006570,0.003764,0.002387,0.001801,0.001484,0.000829,0.000532,0.000372,0.000243,0.000138
  giCountertenorAh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.001473,0.001369,0.004998,0.013019,0.028826,0.066670,0.191435,0.087882,0.042145,0.030678,0.041530,0.092746,0.042423,0.018412,0.008493,0.003408,0.001039,0.000642,0.000564,0.000309,0.000143,0.000118,0.000210,0.000683,0.001594,0.003257,0.006894,0.009365,0.004822,0.007137,0.006892,0.003605,0.002334,0.001880,0.000988,0.000593,0.000383,0.000215
  giCountertenorAh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.001580,0.002749,0.010614,0.028309,0.075224,0.235778,0.079815,0.039694,0.041631,0.105124,0.046998,0.017743,0.006790,0.001929,0.000758,0.000664,0.000304,0.000154,0.000113,0.000242,0.000953,0.002455,0.005759,0.012206,0.005872,0.009102,0.006662,0.003369,0.002397,0.001292,0.000690,0.000403,0.000185
  giCountertenorAh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.001665,0.005802,0.022639,0.067838,0.277959,0.082343,0.043643,0.072542,0.079439,0.023326,0.007411,0.001524,0.000914,0.000509,0.000186,0.000132,0.000538,0.002041,0.005636,0.014527,0.006699,0.012109,0.005270,0.003013,0.001652,0.000778,0.000392,0.000133
  giCountertenorAh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.001879,0.012367,0.050272,0.249889,0.104596,0.051557,0.135917,0.047385,0.012730,0.002203,0.001070,0.000444,0.000205,0.000486,0.002508,0.008455,0.012808,0.011617,0.007544,0.003612,0.001661,0.000745,0.000277
  giCountertenorAh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.002739,0.026042,0.133355,0.175744,0.061363,0.185512,0.036813,0.006813,0.001283,0.000619,0.000238,0.000104,0.001374,0.006525,0.018731,0.014280,0.007212,0.003761,0.001187,0.000430,0.000106
  giCountertenorAh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.005498,0.056620,0.471547,0.079386,0.210272,0.035484,0.003858,0.001328,0.000308,0.000117,0.001906,0.011522,0.011745,0.013323,0.004796,0.001380,0.000371,0.000133
  giCountertenorAh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.011607,0.135706,0.164696,0.145131,0.046646,0.003046,0.001018,0.000264,0.001087,0.011288,0.013399,0.010542,0.003304,0.000785,0.000154
  giCountertenorAh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.024741,0.499940,0.103142,0.094789,0.004409,0.000888,0.000177,0.005018,0.025621,0.015091,0.003323,0.000554,0.000160
  giCountertenorAh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.052086,0.351464,0.371032,0.013625,0.001238,0.000209,0.013050,0.028566,0.007522,0.000861,0.000171
  giCountertenorAh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.113230,0.158750,0.070954,0.002655,0.000236,0.023043,0.026638,0.002759,0.000266
  giCountertenorAh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.271393,0.290247,0.006089,0.000528,0.022576,0.021078,0.001570,0.000118
  giCountertenorAh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,1.000001,0.189594,0.001776,0.010036,0.030183,0.001107,0.000150
  giCountertenorAh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.702916,0.027248,0.000418,0.057129,0.001722,0.000133
  giCountertenorAh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.317514,0.005311,0.046093,0.005518,0.000190
  giCountertenorAh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.580494,0.001056,0.042157,0.000235
 elseif iVoice2==12 then
  giCountertenorEh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.001407,0.001909,0.002654,0.003601,0.004888,0.006467,0.008499,0.011368,0.014805,0.020766,0.028754,0.045419,0.070436,0.068915,0.043266,0.028132,0.020165,0.014552,0.011119,0.008287,0.006270,0.004661,0.003363,0.002384,0.001583,0.000990,0.000554,0.000266,0.000187,0.000237,0.000286,0.000284,0.000259,0.000207,0.000143,0.000115,0.000212,0.000361,0.000554,0.000797,0.001116,0.001495,0.001997,0.002660,0.003508,0.004943,0.006866,0.010908,0.013782,0.011286,0.007103,0.005073,0.003604,0.002726,0.002060,0.001553,0.001173,0.000855,0.000607,0.000405,0.000237,0.000102,0.000142,0.000257,0.000388,0.000538,0.000722,0.000944,0.001216,0.001571,0.002018,0.002669,0.003593,0.005060,0.006932,0.007192,0.005615,0.004200,0.003465,0.003069,0.003039,0.003286,0.003878,0.004703,0.004898,0.004167,0.003337,0.002887,0.002710,0.002818,0.003204,0.003908,0.004910
  giCountertenorEh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.001760,0.002580,0.003766,0.005400,0.007590,0.010513,0.014826,0.020586,0.031392,0.050841,0.085318,0.074894,0.040539,0.027387,0.018076,0.013133,0.009279,0.006665,0.004564,0.003073,0.001908,0.001086,0.000511,0.000236,0.000261,0.000339,0.000336,0.000293,0.000209,0.000123,0.000106,0.000226,0.000430,0.000709,0.001084,0.001575,0.002218,0.003126,0.004372,0.006522,0.010363,0.016022,0.014016,0.008187,0.005534,0.003746,0.002718,0.001937,0.001388,0.000948,0.000622,0.000358,0.000154,0.000191,0.000360,0.000557,0.000798,0.001104,0.001493,0.002025,0.002744,0.003904,0.005749,0.008373,0.008259,0.005844,0.004423,0.003722,0.003634,0.004027,0.005025,0.005913,0.005254,0.004033,0.003408,0.003246,0.003572,0.004377,0.005760,0.006334,0.005015,0.003516,0.002533,0.001874,0.001408,0.001059,0.000788,0.000573,0.000405,0.000272,0.000175,0.000109
  giCountertenorEh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.002230,0.003552,0.005479,0.008344,0.012266,0.018381,0.028255,0.045291,0.091892,0.093320,0.046217,0.028853,0.018680,0.012419,0.008371,0.005365,0.003320,0.001818,0.000856,0.000315,0.000307,0.000407,0.000387,0.000299,0.000173,0.000214,0.000472,0.000862,0.001419,0.002166,0.003268,0.004826,0.007739,0.013719,0.019452,0.012299,0.007218,0.004524,0.003078,0.002064,0.001362,0.000843,0.000456,0.000166,0.000314,0.000574,0.000894,0.001319,0.001890,0.002704,0.004006,0.006196,0.009856,0.009370,0.006160,0.004704,0.004267,0.004706,0.006105,0.006965,0.005453,0.004205,0.003832,0.004261,0.005524,0.007380,0.006685,0.004432,0.002994,0.002099,0.001494,0.001061,0.000733,0.000488,0.000304,0.000177,0.000104
  giCountertenorEh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.002887,0.005002,0.008336,0.013250,0.021432,0.036173,0.067184,0.125012,0.067454,0.036388,0.021564,0.013251,0.008195,0.004726,0.002405,0.000993,0.000329,0.000429,0.000479,0.000385,0.000209,0.000200,0.000531,0.001087,0.001932,0.003147,0.005052,0.008695,0.016946,0.022167,0.011132,0.006361,0.003959,0.002459,0.001486,0.000816,0.000333,0.000360,0.000731,0.001217,0.001892,0.002907,0.004565,0.007708,0.012368,0.009215,0.006087,0.005095,0.005647,0.007722,0.007793,0.005459,0.004595,0.005065,0.006975,0.009015,0.006440,0.003971,0.002578,0.001720,0.001140,0.000724,0.000427,0.000229,0.000121,0.000102
  giCountertenorEh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.003820,0.007205,0.012939,0.022744,0.041548,0.090875,0.137894,0.056293,0.029121,0.016585,0.009329,0.004772,0.001982,0.000532,0.000474,0.000568,0.000415,0.000167,0.000107,0.000231,0.000723,0.001597,0.002997,0.005333,0.009912,0.021876,0.022637,0.010177,0.005470,0.003116,0.001715,0.000813,0.000206,0.000285,0.000781,0.001450,0.002444,0.004058,0.007225,0.013945,0.011299,0.006974,0.006118,0.007810,0.009866,0.006724,0.005463,0.006461,0.009905,0.009117,0.005103,0.003040,0.001880,0.001137,0.000638,0.000315,0.000147,0.000119,0.000122,0.000104
  giCountertenorEh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.005152,0.010784,0.020993,0.041111,0.101534,0.149567,0.054696,0.026229,0.013311,0.006139,0.002170,0.000472,0.000676,0.000586,0.000246,0.000127,0.000452,0.001417,0.003147,0.006248,0.013035,0.032027,0.016366,0.007490,0.003873,0.001896,0.000717,0.000720,0.001596,0.002988,0.005491,0.011506,0.016531,0.008854,0.007274,0.010059,0.010519,0.006824,0.007155,0.011539,0.010048,0.005075,0.002823,0.001580,0.000811,0.000351,0.000153,0.000145,0.000134
  giCountertenorEh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.007108,0.016693,0.036775,0.090623,0.186687,0.057721,0.024848,0.010735,0.003637,0.000631,0.000815,0.000597,0.000102,0.000150,0.000433,0.001732,0.004346,0.009685,0.027540,0.024661,0.009079,0.004147,0.001697,0.000337,0.000629,0.001796,0.003803,0.008068,0.019855,0.012401,0.008599,0.012317,0.010998,0.007737,0.011164,0.013499,0.006048,0.003019,0.001484,0.000615,0.000211,0.000173,0.000156
  giCountertenorEh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.010002,0.026500,0.072350,0.250024,0.072769,0.026502,0.009452,0.001985,0.000858,0.000771,0.000180,0.001063,0.003870,0.010120,0.033965,0.022292,0.007930,0.002977,0.000666,0.000722,0.002442,0.005835,0.015483,0.018490,0.010231,0.015520,0.010969,0.010186,0.018128,0.007983,0.003461,0.001456,0.000460,0.000199,0.000194,0.000107
  giCountertenorEh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.014412,0.045494,0.181791,0.112588,0.033171,0.009544,0.001064,0.001135,0.000329,0.000220,0.001449,0.006000,0.019846,0.045292,0.010951,0.003439,0.000417,0.001563,0.004898,0.014483,0.022627,0.012262,0.019775,0.010955,0.019872,0.010229,0.003771,0.001279,0.000296,0.000245,0.000142
  giCountertenorEh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.021582,0.082278,0.299289,0.052488,0.012284,0.000944,0.001172,0.000108,0.000906,0.006303,0.026106,0.032757,0.007753,0.001434,0.001444,0.005988,0.023064,0.017739,0.020167,0.013678,0.023142,0.010173,0.003168,0.000704,0.000290,0.000184
  giCountertenorEh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.033385,0.181253,0.115431,0.021468,0.001260,0.001196,0.000294,0.003460,0.019374,0.049307,0.008286,0.000663,0.003599,0.016148,0.024802,0.024651,0.015477,0.027005,0.006039,0.001230,0.000344,0.000180,0.000106
  giCountertenorEh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.053000,0.500048,0.052999,0.003969,0.001543,0.000360,0.007742,0.067956,0.015859,0.001332,0.004887,0.030983,0.020466,0.021939,0.036267,0.006921,0.000920,0.000388,0.000110
  giCountertenorEh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.090997,0.225190,0.019088,0.002273,0.000427,0.012001,0.090580,0.006866,0.003130,0.028980,0.024535,0.021910,0.020460,0.002558,0.000490,0.000131
  giCountertenorEh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.164597,0.104995,0.001890,0.000217,0.012610,0.065521,0.002869,0.011981,0.035487,0.027366,0.020351,0.001408,0.000368,0.000158
  giCountertenorEh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.362512,0.042934,0.002393,0.006921,0.098595,0.001326,0.032300,0.049308,0.054000,0.002460,0.000362,0.000116
  giCountertenorEh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.999999,0.007938,0.000720,0.135918,0.002663,0.061969,0.043865,0.013840,0.000774,0.000180
  giCountertenorEh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.450330,0.004546,0.024003,0.013728,0.057960,0.043814,0.005114,0.000263,0.000126
  giCountertenorEh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.210003,0.000432,0.131045,0.023965,0.054738,0.002817,0.000316
  giCountertenorEh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.085867,0.013843,0.002652,0.098614,0.004920,0.000232
  giCountertenorEh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.015875,0.271870,0.123957,0.027679,0.000360
  giCountertenorEh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.009092,0.027457,0.087628,0.000525
 elseif iVoice2==13 then
  giCountertenorEee24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.008519,0.010447,0.014054,0.016869,0.027416,0.033625,0.072566,0.124934,0.105409,0.045534,0.032582,0.021052,0.015918,0.011883,0.008760,0.006708,0.004852,0.003497,0.002443,0.001561,0.000949,0.000462,0.000135,0.000131,0.000257,0.000327,0.000331,0.000315,0.000264,0.000201,0.000142,0.000104,0.000153,0.000219,0.000301,0.000411,0.000551,0.000728,0.000969,0.001276,0.001735,0.002395,0.003484,0.004988,0.005127,0.003704,0.002506,0.001822,0.001332,0.001016,0.000765,0.000579,0.000431,0.000311,0.000217,0.000141,0.000129,0.000204,0.000297,0.000417,0.000566,0.000751,0.000993,0.001302,0.001740,0.002374,0.003337,0.004655,0.004963,0.003852,0.002706,0.002017,0.001532,0.001224,0.000999,0.000848,0.000756,0.000711,0.000725,0.000791
  giCountertenorEee27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.010469,0.013785,0.018825,0.028603,0.039687,0.096960,0.152046,0.086764,0.039736,0.026663,0.018685,0.012926,0.009150,0.006583,0.004367,0.002875,0.001660,0.000901,0.000280,0.000124,0.000303,0.000390,0.000394,0.000348,0.000255,0.000176,0.000123,0.000196,0.000297,0.000428,0.000613,0.000855,0.001201,0.001667,0.002454,0.003667,0.005797,0.006023,0.003933,0.002587,0.001760,0.001267,0.000906,0.000649,0.000452,0.000302,0.000186,0.000179,0.000295,0.000448,0.000651,0.000911,0.001270,0.001753,0.002529,0.003719,0.005568,0.005777,0.004005,0.002734,0.001948,0.001480,0.001166,0.000973,0.000869,0.000856,0.000935,0.001116,0.001223,0.000990,0.000701,0.000607,0.000678,0.000910,0.001178,0.001113,0.000812,0.000589,0.000435,0.000331,0.000252,0.000192,0.000143
  giCountertenorEee30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.012961,0.018883,0.025737,0.046165,0.109238,0.177786,0.070937,0.042673,0.023396,0.016018,0.010600,0.006942,0.004369,0.002442,0.001200,0.000319,0.000192,0.000435,0.000467,0.000430,0.000306,0.000189,0.000101,0.000137,0.000238,0.000385,0.000597,0.000894,0.001336,0.001977,0.003125,0.005266,0.007527,0.005281,0.003131,0.001989,0.001349,0.000906,0.000600,0.000374,0.000211,0.000133,0.000263,0.000450,0.000714,0.001069,0.001585,0.002351,0.003649,0.006036,0.006943,0.004523,0.002921,0.002003,0.001483,0.001174,0.001024,0.001026,0.001183,0.001440,0.001243,0.000829,0.000719,0.000881,0.001279,0.001377,0.000977,0.000667,0.000469,0.000340,0.000246,0.000175,0.000119
  giCountertenorEee33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.016304,0.025781,0.047131,0.093856,0.213577,0.070728,0.040472,0.024108,0.014169,0.008749,0.004988,0.002391,0.000889,0.000136,0.000481,0.000556,0.000488,0.000316,0.000133,0.000122,0.000156,0.000299,0.000521,0.000870,0.001395,0.002231,0.003813,0.007216,0.008330,0.004423,0.002570,0.001607,0.001002,0.000602,0.000328,0.000143,0.000109,0.000260,0.000510,0.000882,0.001420,0.002252,0.003738,0.006675,0.008239,0.004852,0.002954,0.001992,0.001464,0.001227,0.001245,0.001533,0.001684,0.001095,0.000857,0.001094,0.001647,0.001436,0.000900,0.000587,0.000399,0.000272,0.000179,0.000111
  giCountertenorEee36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.020901,0.033747,0.067272,0.249956,0.091103,0.042121,0.023777,0.013422,0.006998,0.003123,0.000925,0.000261,0.000655,0.000630,0.000402,0.000153,0.000132,0.000136,0.000207,0.000439,0.000824,0.001459,0.002557,0.004804,0.010004,0.007430,0.003657,0.002038,0.001162,0.000624,0.000283,0.000156,0.000410,0.000838,0.001510,0.002619,0.004778,0.009373,0.007757,0.004064,0.002466,0.001710,0.001434,0.001595,0.002067,0.001417,0.001024,0.001442,0.002020,0.001289,0.000763,0.000481,0.000304,0.000181
  giCountertenorEee39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.027550,0.057179,0.193782,0.173420,0.053305,0.025836,0.013158,0.005747,0.001800,0.000247,0.000779,0.000697,0.000352,0.000147,0.000159,0.000139,0.000391,0.000854,0.001707,0.003330,0.007329,0.012038,0.005171,0.002532,0.001299,0.000603,0.000200,0.000196,0.000590,0.001302,0.002541,0.005062,0.011149,0.008021,0.003901,0.002335,0.001741,0.001874,0.002450,0.001406,0.001360,0.002362,0.001629,0.000873,0.000506,0.000288,0.000146
  giCountertenorEee42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.037773,0.092350,0.355634,0.085357,0.032041,0.013887,0.004884,0.000639,0.000872,0.000861,0.000379,0.000202,0.000130,0.000134,0.000483,0.001202,0.002686,0.006275,0.015101,0.006277,0.002705,0.001203,0.000420,0.000268,0.000907,0.002154,0.004740,0.012176,0.009112,0.004038,0.002369,0.002070,0.002909,0.001676,0.001786,0.002785,0.001348,0.000686,0.000353,0.000154
  giCountertenorEee45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.051559,0.187745,0.141432,0.048215,0.017498,0.004781,0.000273,0.001114,0.000633,0.000244,0.000108,0.000313,0.001044,0.002794,0.007641,0.016682,0.005148,0.002007,0.000658,0.000106,0.000523,0.001771,0.004523,0.013415,0.009741,0.003999,0.002464,0.003082,0.002203,0.002205,0.002887,0.001181,0.000547,0.000222
  giCountertenorEee48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.067500,0.499957,0.084236,0.026845,0.006246,0.000524,0.001260,0.000307,0.000263,0.000154,0.000423,0.001657,0.005126,0.020030,0.007311,0.002320,0.000560,0.000313,0.001681,0.005251,0.018793,0.008143,0.003428,0.003199,0.002838,0.002897,0.002582,0.000962,0.000359
  giCountertenorEee51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.114392,0.346891,0.051686,0.011497,0.000495,0.001393,0.000319,0.000278,0.001711,0.006671,0.024100,0.005070,0.001207,0.000395,0.002611,0.010148,0.016063,0.004678,0.003757,0.002818,0.004738,0.001749,0.000577,0.000121
  giCountertenorEee54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.184699,0.170704,0.027773,0.001277,0.001722,0.000135,0.000260,0.000954,0.005364,0.030213,0.005416,0.000846,0.000161,0.001816,0.009486,0.018223,0.004741,0.005826,0.003571,0.002699,0.000709,0.000111
  giCountertenorEee57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.375525,0.096426,0.009561,0.002227,0.000108,0.000215,0.002089,0.015287,0.010294,0.001315,0.000212,0.003544,0.026847,0.007999,0.006168,0.004412,0.002362,0.000444,0.000109
  giCountertenorEee60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,1.000001,0.053693,0.001047,0.000612,0.000311,0.003299,0.040074,0.004652,0.000183,0.003363,0.037603,0.006857,0.005682,0.005170,0.000725,0.000129
  giCountertenorEee63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.693800,0.022994,0.002786,0.000638,0.003424,0.048210,0.002415,0.000789,0.020308,0.009360,0.005636,0.003499,0.000242
  giCountertenorEee66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.341398,0.002553,0.000269,0.001909,0.060427,0.001690,0.003632,0.036438,0.011652,0.005397,0.000221
  giCountertenorEee69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.192837,0.004454,0.000431,0.030573,0.002629,0.007088,0.015996,0.008825,0.000889
  giCountertenorEee72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.107381,0.001225,0.006599,0.009301,0.006726,0.013712,0.010336,0.000259
  giCountertenorEee75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.045989,0.001278,0.096423,0.001577,0.018720,0.006999,0.000144
  giCountertenorEee78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.005108,0.003817,0.003379,0.072873,0.010794
  giCountertenorEee81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.008908,0.061152,0.014177,0.017652,0.000134
  giCountertenorEee84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.002448,0.018603,0.027424,0.000518
 elseif iVoice2==14 then
  giCountertenorOh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.000976,0.001418,0.001996,0.002649,0.003721,0.004755,0.006289,0.008700,0.010470,0.017279,0.020680,0.047585,0.075931,0.059717,0.025499,0.020090,0.013249,0.011161,0.009282,0.008320,0.008180,0.008813,0.010336,0.014497,0.017321,0.013886,0.008690,0.006221,0.004488,0.003467,0.002671,0.002061,0.001592,0.001183,0.000868,0.000603,0.000392,0.000239,0.000128,0.000106,0.000126,0.000132,0.000124,0.000110,0.000135,0.000180,0.000235,0.000301,0.000381,0.000484,0.000612,0.000797,0.001059,0.001479,0.002029,0.002140,0.001753,0.001427,0.001336,0.001376,0.001574,0.001924,0.002473,0.003150,0.003321,0.002752,0.002057,0.001579,0.001239,0.001017,0.000865,0.000776,0.000736
  giCountertenorOh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.001238,0.001930,0.002779,0.004109,0.005581,0.007795,0.011138,0.015868,0.023498,0.051584,0.090982,0.058872,0.024979,0.018745,0.013492,0.010997,0.009771,0.009964,0.011317,0.016000,0.020636,0.015422,0.009037,0.006223,0.004400,0.003252,0.002389,0.001745,0.001211,0.000817,0.000496,0.000277,0.000133,0.000115,0.000142,0.000158,0.000148,0.000126,0.000131,0.000189,0.000261,0.000353,0.000467,0.000621,0.000824,0.001156,0.001681,0.002451,0.002478,0.001885,0.001618,0.001623,0.001883,0.002404,0.003279,0.004007,0.003505,0.002498,0.001824,0.001384,0.001115,0.000952,0.000886,0.000783,0.000591,0.000435,0.000341,0.000275,0.000225,0.000183,0.000147,0.000116
  giCountertenorOh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.001604,0.002674,0.004072,0.006176,0.009086,0.013563,0.023369,0.035194,0.101582,0.077033,0.029449,0.018986,0.014865,0.011952,0.011694,0.013514,0.020789,0.023285,0.013030,0.008051,0.005265,0.003665,0.002550,0.001705,0.001091,0.000616,0.000306,0.000141,0.000150,0.000186,0.000177,0.000148,0.000124,0.000198,0.000294,0.000421,0.000588,0.000823,0.001193,0.001814,0.002885,0.002835,0.002063,0.001887,0.002113,0.002782,0.004010,0.004725,0.003536,0.002382,0.001692,0.001306,0.001100,0.001011,0.000790,0.000541,0.000403,0.000313,0.000246,0.000192,0.000145,0.000106
  giCountertenorOh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.002118,0.003720,0.006338,0.009787,0.015959,0.030101,0.065229,0.125015,0.039115,0.024298,0.017276,0.013843,0.014630,0.020706,0.028742,0.015233,0.008650,0.005463,0.003511,0.002232,0.001316,0.000660,0.000266,0.000162,0.000213,0.000212,0.000170,0.000102,0.000144,0.000251,0.000396,0.000597,0.000891,0.001362,0.002256,0.003634,0.002895,0.002272,0.002441,0.003352,0.005140,0.005242,0.003308,0.002161,0.001568,0.001294,0.001139,0.000764,0.000520,0.000381,0.000286,0.000212,0.000149
  giCountertenorOh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.002837,0.005300,0.009514,0.017406,0.034571,0.095209,0.119489,0.040201,0.022335,0.016651,0.017637,0.029015,0.027795,0.012453,0.006942,0.004127,0.002369,0.001209,0.000479,0.000192,0.000252,0.000249,0.000173,0.000136,0.000271,0.000472,0.000765,0.001230,0.002129,0.004081,0.003528,0.002689,0.003169,0.004981,0.006691,0.004146,0.002498,0.001745,0.001484,0.001074,0.000655,0.000451,0.000321,0.000221,0.000141
  giCountertenorOh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.003856,0.008213,0.015579,0.031723,0.103107,0.117667,0.037472,0.021976,0.019904,0.031955,0.030799,0.012429,0.006496,0.003487,0.001632,0.000553,0.000231,0.000316,0.000252,0.000119,0.000102,0.000262,0.000522,0.000934,0.001650,0.003365,0.004960,0.003239,0.003768,0.006564,0.007017,0.003653,0.002234,0.001775,0.001184,0.000682,0.000451,0.000296,0.000176
  giCountertenorOh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.005349,0.012355,0.027133,0.070422,0.154071,0.037978,0.023916,0.027050,0.046604,0.016115,0.007338,0.003414,0.001231,0.000279,0.000372,0.000296,0.000104,0.000102,0.000249,0.000591,0.001183,0.002404,0.005812,0.004154,0.004259,0.008091,0.007129,0.003415,0.002221,0.001594,0.000813,0.000497,0.000294,0.000145
  giCountertenorOh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.007441,0.019575,0.060208,0.250025,0.048593,0.027690,0.041429,0.030466,0.010928,0.004465,0.001320,0.000325,0.000425,0.000205,0.000122,0.000289,0.000795,0.001788,0.004530,0.005810,0.004904,0.010331,0.006646,0.003152,0.002288,0.001045,0.000576,0.000300,0.000117
  giCountertenorOh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.010601,0.034815,0.190470,0.080405,0.033305,0.058053,0.024910,0.008255,0.002415,0.000380,0.000499,0.000157,0.000134,0.000544,0.001534,0.004269,0.007067,0.006352,0.013410,0.005008,0.002977,0.001313,0.000644,0.000282
  giCountertenorOh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.016431,0.063467,0.235366,0.043971,0.063965,0.024873,0.006978,0.001106,0.000633,0.000237,0.000165,0.000526,0.001871,0.006744,0.006491,0.013161,0.007320,0.003558,0.001368,0.000593,0.000178
  giCountertenorOh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.024710,0.140863,0.075949,0.054103,0.032225,0.006828,0.000565,0.000591,0.000130,0.000122,0.000130,0.001184,0.004810,0.008308,0.016195,0.006829,0.003189,0.000995,0.000290
  giCountertenorOh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.039150,0.500037,0.055380,0.060924,0.008930,0.000650,0.000410,0.000244,0.001590,0.009065,0.009812,0.013291,0.004577,0.001153,0.000235,0.000102
  giCountertenorOh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.069637,0.160819,0.116129,0.016511,0.000768,0.000310,0.000199,0.000121,0.001088,0.008540,0.012711,0.010013,0.002627,0.000564,0.000125
  giCountertenorOh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.126948,0.087953,0.049754,0.002213,0.000474,0.000165,0.003744,0.012986,0.014644,0.002737,0.000355,0.000112
  giCountertenorOh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.281744,0.108208,0.013656,0.001182,0.000243,0.000261,0.009621,0.032394,0.006377,0.000580,0.000114
  giCountertenorOh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,1.000000,0.121825,0.001300,0.000487,0.000199,0.018133,0.026571,0.002305,0.000202
  giCountertenorOh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.321614,0.033016,0.000620,0.000243,0.017081,0.020022,0.001128
  giCountertenorOh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.175915,0.004425,0.000329,0.007489,0.029288,0.000710,0.000103
  giCountertenorOh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.216413,0.002364,0.000521,0.064787,0.001159
  giCountertenorOh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.243654,0.000974,0.036270,0.004610,0.000126
  giCountertenorOh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.066032,0.000487,0.040044,0.000145
 elseif iVoice2==15 then
  giCountertenorOoh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.002930,0.003338,0.004209,0.005237,0.006779,0.009344,0.011428,0.018295,0.022905,0.047515,0.085264,0.075133,0.033639,0.022964,0.015677,0.011837,0.009420,0.008016,0.005598,0.002825,0.001635,0.001188,0.001026,0.000841,0.000709,0.000648,0.000588,0.000526,0.000477,0.000401,0.000336,0.000264,0.000191,0.000133,0.000108,0.000159,0.000223,0.000304,0.000406,0.000530,0.000690,0.000897,0.001168,0.001579,0.002149,0.003081,0.003723,0.003238,0.002269,0.001653,0.001261,0.001042,0.001008,0.001130,0.001209,0.001084,0.000893,0.000767,0.000687,0.000655,0.000659,0.000701,0.000788
  giCountertenorOoh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.003586,0.004256,0.005702,0.007556,0.010921,0.014024,0.024945,0.035150,0.092038,0.093788,0.037116,0.026081,0.015318,0.012347,0.009771,0.006560,0.002960,0.001559,0.001368,0.001026,0.000848,0.000758,0.000673,0.000602,0.000498,0.000407,0.000303,0.000209,0.000120,0.000160,0.000244,0.000355,0.000499,0.000686,0.000937,0.001278,0.001812,0.002618,0.003961,0.004359,0.003122,0.002085,0.001507,0.001221,0.001248,0.001432,0.001351,0.001078,0.000901,0.000805,0.000781,0.000820,0.000929,0.001142,0.001493,0.001896,0.001791,0.001284,0.000903,0.000650,0.000483,0.000359,0.000267,0.000194,0.000137
  giCountertenorOoh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.004387,0.005685,0.007842,0.011742,0.016404,0.030852,0.061026,0.124965,0.062106,0.031873,0.017790,0.013428,0.010387,0.004571,0.002046,0.001614,0.001183,0.000962,0.000842,0.000729,0.000595,0.000464,0.000313,0.000193,0.000161,0.000272,0.000424,0.000634,0.000922,0.001335,0.001984,0.003066,0.004916,0.004770,0.002924,0.001939,0.001459,0.001499,0.001713,0.001437,0.001123,0.000961,0.000924,0.000992,0.001191,0.001581,0.002177,0.002117,0.001415,0.000940,0.000647,0.000456,0.000320,0.000219,0.000143
  giCountertenorOoh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.005388,0.007726,0.011546,0.018473,0.036095,0.086804,0.138411,0.041289,0.024565,0.016650,0.011902,0.004266,0.001997,0.001497,0.001165,0.000998,0.000847,0.000647,0.000450,0.000263,0.000103,0.000118,0.000101,0.000207,0.000376,0.000627,0.000985,0.001532,0.002453,0.004202,0.006295,0.004173,0.002462,0.001735,0.001864,0.001976,0.001463,0.001175,0.001103,0.001211,0.001558,0.002280,0.002680,0.001752,0.001076,0.000697,0.000459,0.000297,0.000181,0.000101
  giCountertenorOoh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.006678,0.010477,0.018695,0.036602,0.095066,0.150330,0.045948,0.023685,0.016041,0.005653,0.002378,0.001682,0.001298,0.001053,0.000802,0.000528,0.000265,0.000105,0.000138,0.000118,0.000217,0.000449,0.000815,0.001387,0.002350,0.004323,0.007492,0.004566,0.002539,0.002030,0.002436,0.001799,0.001386,0.001329,0.001590,0.002352,0.003257,0.002059,0.001161,0.000702,0.000425,0.000243,0.000123
  giCountertenorOoh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.008507,0.015099,0.028021,0.070256,0.187408,0.052114,0.024673,0.013102,0.003116,0.002049,0.001515,0.001204,0.000813,0.000417,0.000115,0.000127,0.000167,0.000128,0.000197,0.000489,0.000998,0.001875,0.003627,0.007929,0.006251,0.003018,0.002497,0.002704,0.001804,0.001564,0.001861,0.002991,0.003590,0.001811,0.000967,0.000535,0.000275,0.000118
  giCountertenorOoh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.011372,0.023489,0.061720,0.249997,0.063763,0.026863,0.009143,0.003230,0.001926,0.001459,0.000929,0.000387,0.000186,0.000157,0.000177,0.000548,0.001276,0.002689,0.006178,0.009604,0.003905,0.003024,0.002899,0.001942,0.002006,0.003198,0.004277,0.001900,0.000922,0.000444,0.000179
  giCountertenorOoh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.015452,0.036944,0.173650,0.082571,0.033299,0.008530,0.002994,0.001997,0.001295,0.000526,0.000105,0.000236,0.000137,0.000202,0.000755,0.001978,0.004926,0.012637,0.004941,0.003746,0.002939,0.002218,0.003135,0.005390,0.002165,0.000924,0.000366,0.000102
  giCountertenorOoh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.020956,0.073214,0.300650,0.047374,0.011304,0.003365,0.002105,0.001056,0.000160,0.000271,0.000143,0.000105,0.000437,0.001635,0.004710,0.015016,0.005087,0.004883,0.002780,0.003191,0.006532,0.002329,0.000854,0.000249
  giCountertenorOoh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.030210,0.140577,0.104265,0.026214,0.004100,0.002408,0.000835,0.000254,0.000256,0.000123,0.000395,0.002001,0.007270,0.012518,0.005007,0.003616,0.003732,0.007195,0.001939,0.000551
  giCountertenorOoh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.046979,0.499997,0.053723,0.006460,0.002919,0.000773,0.000378,0.000118,0.000106,0.000152,0.001094,0.005381,0.019208,0.006051,0.003882,0.006400,0.003801,0.000886,0.000115
  giCountertenorOoh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.073889,0.165133,0.017058,0.003994,0.001052,0.000471,0.000177,0.001512,0.009857,0.009881,0.005879,0.006275,0.004329,0.000733,0.000125
  giCountertenorOoh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.146440,0.094756,0.006731,0.002111,0.000551,0.000141,0.000870,0.009425,0.010175,0.005557,0.013068,0.001707,0.000146
  giCountertenorOoh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.281199,0.052436,0.004817,0.000509,0.000247,0.004004,0.025038,0.007235,0.014393,0.001102,0.000174
  giCountertenorOoh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000000,0.012920,0.001547,0.000236,0.000305,0.010763,0.012104,0.012802,0.001771,0.000185
  giCountertenorOoh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.330234,0.007988,0.000942,0.000128,0.000355,0.019716,0.011755,0.008656,0.000251
  giCountertenorOoh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.189497,0.004222,0.000170,0.000283,0.018851,0.011112,0.003413,0.000105
  giCountertenorOoh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.104873,0.001016,0.008010,0.014471,0.002205,0.000124
  giCountertenorOoh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.025840,0.000472,0.000610,0.024208,0.003543,0.000116
  giCountertenorOoh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.015975,0.000256,0.039437,0.017313,0.000164
  giCountertenorOoh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.008443,0.000565,0.022224,0.000210
 ; ALTO
 elseif iVoice2==16 then
  giAltoAh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.000189,0.000296,0.000384,0.000470,0.000519,0.000516,0.000488,0.000371,0.000272,0.000362,0.000762,0.001312,0.002068,0.003050,0.004182,0.005758,0.007569,0.009859,0.013271,0.016801,0.024312,0.033001,0.056588,0.102975,0.102169,0.056712,0.036664,0.028737,0.023278,0.022005,0.021266,0.024719,0.028942,0.046575,0.063428,0.051044,0.027265,0.019362,0.013395,0.010268,0.007894,0.006039,0.004698,0.003520,0.002602,0.001851,0.001231,0.000766,0.000406,0.000199,0.000194,0.000263,0.000310,0.000314,0.000289,0.000244,0.000184,0.000124,0.000126,0.000177,0.000255,0.000364,0.000514,0.000708,0.000954,0.001263,0.001646,0.002126,0.002727,0.003491,0.004448,0.005556,0.006485,0.006605,0.005809,0.004697,0.003698,0.002896,0.002268,0.001771,0.001374,0.001053,0.000794,0.000590,0.000437,0.000344,0.000317,0.000349
  giAltoAh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.000244,0.000397,0.000518,0.000607,0.000616,0.000568,0.000396,0.000335,0.000726,0.001487,0.002548,0.003987,0.005730,0.008260,0.011227,0.015932,0.020978,0.034291,0.049287,0.113108,0.122662,0.059916,0.041411,0.029183,0.026718,0.025244,0.029994,0.038221,0.068475,0.069216,0.035306,0.022725,0.014644,0.011021,0.007946,0.005866,0.004193,0.002921,0.001910,0.001127,0.000578,0.000244,0.000241,0.000340,0.000374,0.000356,0.000298,0.000215,0.000128,0.000113,0.000169,0.000257,0.000396,0.000598,0.000875,0.001242,0.001721,0.002342,0.003155,0.004233,0.005634,0.007200,0.007995,0.007183,0.005618,0.004224,0.003160,0.002359,0.001750,0.001279,0.000914,0.000639,0.000456,0.000380,0.000412,0.000508,0.000638,0.000799,0.000989,0.001180,0.001248,0.001102,0.000867,0.000662,0.000507,0.000390,0.000299,0.000227,0.000169,0.000122
  giAltoAh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.000322,0.000523,0.000687,0.000739,0.000683,0.000446,0.000481,0.001318,0.002593,0.004536,0.007027,0.010628,0.015538,0.022888,0.037867,0.059838,0.147441,0.117395,0.052829,0.036972,0.031809,0.030303,0.037765,0.063847,0.088422,0.044709,0.025986,0.015831,0.011132,0.007664,0.005234,0.003397,0.002015,0.001038,0.000395,0.000267,0.000404,0.000445,0.000401,0.000296,0.000178,0.000161,0.000262,0.000438,0.000716,0.001120,0.001683,0.002450,0.003502,0.004969,0.006966,0.009036,0.009149,0.007148,0.005115,0.003622,0.002556,0.001780,0.001208,0.000794,0.000531,0.000449,0.000528,0.000692,0.000905,0.001168,0.001426,0.001435,0.001144,0.000835,0.000607,0.000444,0.000323,0.000231,0.000159,0.000104
  giAltoAh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.000435,0.000700,0.000875,0.000843,0.000544,0.000647,0.001988,0.004188,0.007324,0.012179,0.019122,0.029406,0.054206,0.141337,0.157466,0.062492,0.040883,0.036334,0.041267,0.064121,0.105743,0.045932,0.025684,0.015869,0.010059,0.006455,0.003844,0.001991,0.000808,0.000291,0.000460,0.000530,0.000454,0.000294,0.000125,0.000105,0.000104,0.000188,0.000335,0.000614,0.001080,0.001786,0.002812,0.004302,0.006516,0.009550,0.011271,0.008968,0.006048,0.004011,0.002642,0.001695,0.001040,0.000640,0.000541,0.000692,0.000956,0.001306,0.001686,0.001690,0.001250,0.000853,0.000587,0.000404,0.000271,0.000172,0.000103,0.000102,0.000112
  giAltoAh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.000591,0.000940,0.001033,0.000742,0.000724,0.002625,0.006104,0.011523,0.019729,0.033624,0.066046,0.206109,0.113522,0.057528,0.044055,0.049494,0.093266,0.102229,0.038781,0.020569,0.012100,0.007054,0.003709,0.001535,0.000398,0.000528,0.000629,0.000488,0.000249,0.000106,0.000124,0.000180,0.000356,0.000732,0.001422,0.002539,0.004275,0.007024,0.011180,0.013295,0.009456,0.005833,0.003569,0.002122,0.001189,0.000693,0.000704,0.001020,0.001484,0.002021,0.001923,0.001279,0.000814,0.000523,0.000327,0.000190,0.000101,0.000126,0.000129
  giAltoAh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.000792,0.001213,0.001135,0.000668,0.002969,0.007965,0.016502,0.031835,0.068519,0.226014,0.119754,0.058321,0.050446,0.076384,0.138316,0.045413,0.022028,0.011725,0.005840,0.002252,0.000488,0.000680,0.000712,0.000429,0.000104,0.000120,0.000144,0.000149,0.000337,0.000791,0.001747,0.003438,0.006303,0.011258,0.015979,0.011229,0.006318,0.003500,0.001828,0.000913,0.000824,0.001276,0.001980,0.002497,0.001737,0.001015,0.000599,0.000339,0.000171,0.000148,0.000149
  giAltoAh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.001046,0.001479,0.000892,0.002638,0.009076,0.021269,0.045807,0.119803,0.234872,0.073996,0.060678,0.127922,0.089484,0.031708,0.015355,0.006806,0.002080,0.000536,0.000893,0.000593,0.000132,0.000160,0.000143,0.000109,0.000328,0.000883,0.002256,0.004934,0.010012,0.018210,0.014399,0.007297,0.003587,0.001599,0.000905,0.001398,0.002364,0.002903,0.001690,0.000899,0.000467,0.000211,0.000109,0.000178,0.000165
  giAltoAh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.001400,0.001687,0.001294,0.008377,0.024362,0.058827,0.282834,0.125013,0.072697,0.128373,0.091880,0.031757,0.012918,0.003985,0.000583,0.001062,0.000588,0.000210,0.000100,0.000377,0.001233,0.003586,0.008637,0.019180,0.018001,0.008054,0.003404,0.001286,0.001393,0.002629,0.003400,0.001717,0.000813,0.000347,0.000124,0.000156,0.000228,0.000128
  giAltoAh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.001880,0.001482,0.005251,0.023053,0.067263,0.412366,0.115064,0.099033,0.204459,0.041147,0.014111,0.003072,0.001057,0.000977,0.000247,0.000164,0.000717,0.002848,0.008565,0.022410,0.018941,0.007145,0.002378,0.001413,0.002978,0.003857,0.001634,0.000655,0.000202,0.000113,0.000183,0.000258,0.000115
  giAltoAh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.002427,0.001338,0.015940,0.063709,0.452338,0.116697,0.152906,0.090878,0.023464,0.004506,0.001360,0.000857,0.000242,0.000142,0.000677,0.003504,0.012640,0.032038,0.012664,0.003664,0.001654,0.003974,0.003484,0.001202,0.000342,0.000133,0.000199,0.000300,0.000106
  giAltoAh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.002958,0.005276,0.042538,0.239659,0.147975,0.255925,0.063414,0.013610,0.001072,0.001187,0.000319,0.000216,0.001764,0.009875,0.036436,0.014599,0.003203,0.002798,0.005807,0.001797,0.000423,0.000157,0.000220,0.000330
  giAltoAh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.003374,0.016755,0.117663,0.250019,0.256814,0.063510,0.007969,0.002123,0.000140,0.000201,0.002468,0.017282,0.036001,0.006807,0.002788,0.006801,0.001627,0.000248,0.000137,0.000312,0.000255
  giAltoAh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.002967,0.046107,0.824869,0.198093,0.082301,0.006142,0.001955,0.000496,0.001429,0.017146,0.037895,0.004766,0.005960,0.003267,0.000407,0.000157,0.000155,0.000519
  giAltoAh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.002677,0.127443,0.233422,0.181784,0.009013,0.001717,0.000284,0.007015,0.064115,0.007331,0.007952,0.002405,0.000267,0.000125,0.000600
  giAltoAh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.010553,0.479368,0.511917,0.027219,0.002374,0.000432,0.019753,0.029194,0.005598,0.003593,0.000314,0.000115,0.000440,0.000188
  giAltoAh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.033509,0.499985,0.127000,0.004247,0.000403,0.034562,0.013610,0.013599,0.000498,0.000136,0.000625,0.000114
  giAltoAh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.092211,0.396173,0.012280,0.000993,0.034289,0.009528,0.006532,0.000315,0.000310,0.000185
  giAltoAh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.254908,0.363579,0.003434,0.014036,0.014660,0.004810,0.000173,0.001201
  giAltoAh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.958726,0.054439,0.000865,0.058385,0.007186,0.000230,0.000375
  giAltoAh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,1.000000,0.008493,0.069131,0.027200,0.000272,0.000228
  giAltoAh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.792347,0.001986,0.019056,0.000630,0.000371
 elseif iVoice2==17 then
  giAltoEh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.003362,0.004063,0.005154,0.006879,0.008943,0.011836,0.016028,0.020650,0.030582,0.041256,0.073678,0.110417,0.094909,0.051940,0.035835,0.024628,0.018327,0.013819,0.010292,0.007833,0.005745,0.004159,0.002926,0.001916,0.001193,0.000629,0.000269,0.000196,0.000305,0.000382,0.000393,0.000375,0.000314,0.000239,0.000159,0.000177,0.000278,0.000395,0.000541,0.000706,0.000932,0.001227,0.001618,0.002273,0.003186,0.005042,0.006061,0.004724,0.002963,0.002132,0.001509,0.001144,0.000862,0.000649,0.000491,0.000359,0.000260,0.000181,0.000118,0.000128,0.000186,0.000258,0.000348,0.000460,0.000604,0.000788,0.001043,0.001394,0.001891,0.002418,0.002462,0.001970,0.001456,0.001095,0.000833,0.000649,0.000509,0.000407,0.000337,0.000299,0.000298,0.000331,0.000397,0.000491,0.000619,0.000790,0.001006,0.001234
  giAltoEh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.004127,0.005275,0.007209,0.009991,0.013867,0.019808,0.027445,0.042904,0.071987,0.129126,0.108909,0.053400,0.036759,0.023688,0.017299,0.012158,0.008774,0.005971,0.004053,0.002515,0.001440,0.000656,0.000247,0.000296,0.000443,0.000468,0.000441,0.000347,0.000241,0.000122,0.000217,0.000362,0.000543,0.000763,0.001053,0.001462,0.002056,0.003040,0.004969,0.007193,0.005760,0.003352,0.002274,0.001551,0.001119,0.000797,0.000571,0.000393,0.000264,0.000165,0.000115,0.000188,0.000283,0.000406,0.000567,0.000781,0.001077,0.001517,0.002169,0.002914,0.002838,0.002063,0.001457,0.001046,0.000776,0.000583,0.000451,0.000372,0.000353,0.000394,0.000488,0.000633,0.000840,0.001123,0.001453,0.001578,0.001331,0.000989,0.000724,0.000534,0.000397,0.000294,0.000216,0.000156,0.000110
  giAltoEh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.005092,0.006977,0.010396,0.015163,0.023130,0.034809,0.056474,0.124888,0.147945,0.067675,0.042041,0.026103,0.017387,0.011699,0.007558,0.004760,0.002659,0.001309,0.000425,0.000311,0.000523,0.000556,0.000492,0.000344,0.000181,0.000108,0.000276,0.000487,0.000768,0.001122,0.001665,0.002495,0.003962,0.007356,0.007971,0.004348,0.002698,0.001734,0.001166,0.000785,0.000511,0.000320,0.000182,0.000190,0.000315,0.000486,0.000720,0.001051,0.001562,0.002371,0.003434,0.003234,0.002153,0.001437,0.000994,0.000705,0.000521,0.000426,0.000432,0.000535,0.000722,0.001009,0.001423,0.001836,0.001720,0.001239,0.000855,0.000596,0.000418,0.000292,0.000200,0.000132
  giAltoEh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.006374,0.009593,0.015226,0.024859,0.040118,0.071823,0.177585,0.125051,0.058039,0.032167,0.019996,0.012533,0.007307,0.003947,0.001782,0.000489,0.000427,0.000651,0.000634,0.000446,0.000221,0.000149,0.000413,0.000750,0.001205,0.001923,0.003136,0.005583,0.010175,0.006418,0.003433,0.002024,0.001259,0.000784,0.000459,0.000244,0.000111,0.000114,0.000244,0.000436,0.000711,0.001122,0.001783,0.002933,0.004251,0.003241,0.001974,0.001256,0.000835,0.000588,0.000498,0.000577,0.000806,0.001196,0.001794,0.002240,0.001746,0.001130,0.000733,0.000481,0.000313,0.000196,0.000116,0.000107,0.000123,0.000104
  giAltoEh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.008128,0.013762,0.023679,0.041314,0.082543,0.220926,0.103928,0.049281,0.027654,0.015677,0.008325,0.003834,0.001260,0.000392,0.000764,0.000751,0.000478,0.000163,0.000354,0.000791,0.001415,0.002458,0.004555,0.010107,0.009469,0.004274,0.002294,0.001301,0.000721,0.000362,0.000148,0.000256,0.000519,0.000925,0.001584,0.002804,0.004864,0.003965,0.002203,0.001307,0.000820,0.000602,0.000668,0.000990,0.001593,0.002490,0.002373,0.001446,0.000864,0.000523,0.000310,0.000173,0.000135,0.000143,0.000103
  giAltoEh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.010535,0.019953,0.039560,0.085683,0.257901,0.106658,0.047311,0.024286,0.011928,0.005025,0.001312,0.000591,0.000935,0.000693,0.000244,0.000433,0.001084,0.002103,0.004108,0.009930,0.011512,0.004545,0.002237,0.001142,0.000527,0.000191,0.000230,0.000566,0.001133,0.002154,0.004340,0.005678,0.002915,0.001553,0.000903,0.000707,0.000977,0.001682,0.002909,0.002665,0.001449,0.000795,0.000433,0.000220,0.000158,0.000169,0.000109
  giAltoEh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.013958,0.030334,0.069643,0.249891,0.135371,0.052225,0.023407,0.009523,0.002620,0.000623,0.001115,0.000690,0.000220,0.000978,0.002252,0.005006,0.014753,0.008712,0.003477,0.001575,0.000644,0.000191,0.000198,0.000633,0.001450,0.003146,0.006918,0.004334,0.002002,0.001048,0.000872,0.001457,0.002874,0.003471,0.001726,0.000845,0.000403,0.000170,0.000107,0.000188,0.000185,0.000102
  giAltoEh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.019186,0.049717,0.143654,0.250057,0.064333,0.025066,0.007894,0.000978,0.001303,0.000893,0.000298,0.001501,0.003852,0.011185,0.012845,0.004053,0.001570,0.000488,0.000229,0.000874,0.002252,0.005891,0.006503,0.002521,0.001180,0.001158,0.002404,0.004502,0.002271,0.000968,0.000395,0.000131,0.000158,0.000250,0.000148
  giAltoEh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.027527,0.082639,0.441905,0.098560,0.031354,0.007669,0.000785,0.001502,0.000330,0.001589,0.004927,0.020236,0.008549,0.002604,0.000730,0.000119,0.000515,0.001854,0.005620,0.007941,0.002618,0.001208,0.001989,0.004995,0.002897,0.001049,0.000348,0.000188,0.000287,0.000130
  giAltoEh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.039930,0.171475,0.213415,0.048597,0.010054,0.001181,0.001385,0.000867,0.004211,0.019887,0.009096,0.002286,0.000383,0.000109,0.000463,0.002271,0.008699,0.005840,0.001810,0.001961,0.005836,0.002906,0.000868,0.000198,0.000198,0.000336,0.000125
  giAltoEh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.060669,0.499820,0.104443,0.019045,0.001245,0.001376,0.001951,0.010010,0.017427,0.003148,0.000373,0.001269,0.006296,0.008668,0.002099,0.002917,0.006949,0.001692,0.000339,0.000217,0.000372,0.000107
  giAltoEh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.099436,0.500065,0.050129,0.001955,0.001786,0.003004,0.022378,0.008105,0.000976,0.000133,0.001750,0.011786,0.005041,0.002318,0.009006,0.001935,0.000261,0.000101,0.000315,0.000297
  giAltoEh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.165294,0.197124,0.015337,0.003004,0.003167,0.040484,0.005206,0.000220,0.001029,0.011247,0.005236,0.003975,0.005800,0.000693,0.000124,0.000151,0.000577,0.000103
  giAltoEh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.343032,0.097209,0.002364,0.001736,0.039793,0.004573,0.000222,0.004544,0.011682,0.003924,0.005813,0.000396,0.000102,0.000125,0.000672
  giAltoEh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.999678,0.038087,0.002755,0.020021,0.006295,0.000187,0.012594,0.004197,0.013896,0.000677,0.000104,0.000433,0.000213
  giAltoEh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,1.000001,0.003910,0.006008,0.016207,0.000265,0.023574,0.004637,0.003870,0.000201,0.000631,0.000123
  giAltoEh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.394210,0.006006,0.080968,0.000440,0.022493,0.007950,0.001386,0.000302,0.000206
  giAltoEh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.194429,0.003472,0.009145,0.009090,0.007850,0.000792,0.001344
  giAltoEh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.076172,0.040042,0.000373,0.008393,0.001355,0.000427
  giAltoEh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.007820,0.032414,0.047156,0.007739,0.000246
  giAltoEh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.012011,0.000881,0.015900,0.000124,0.000412
 elseif iVoice2==18 then
  giAltoEee24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.004734,0.005426,0.006616,0.008187,0.010807,0.014143,0.019214,0.028383,0.042150,0.084741,0.096689,0.058804,0.031217,0.023777,0.015637,0.012369,0.009000,0.006764,0.005109,0.003621,0.002624,0.001760,0.001102,0.000655,0.000294,0.000144,0.000174,0.000252,0.000263,0.000258,0.000224,0.000168,0.000120,0.000134,0.000222,0.000342,0.000495,0.000695,0.000938,0.001253,0.001659,0.002197,0.003006,0.004145,0.005948,0.006987,0.005870,0.004071,0.002952,0.002161,0.001639,0.001249,0.000949,0.000719,0.000532,0.000385,0.000268,0.000175,0.000107,0.000136,0.000197,0.000274,0.000368,0.000490,0.000641,0.000851,0.001137,0.001539,0.001960,0.001987,0.001583,0.001163,0.000868,0.000655,0.000503,0.000386,0.000296,0.000224,0.000165,0.000118
  giAltoEee27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.005733,0.007002,0.008794,0.011941,0.016656,0.024458,0.035967,0.075403,0.118292,0.075882,0.036455,0.024866,0.016911,0.011927,0.008415,0.006026,0.004028,0.002674,0.001591,0.000891,0.000358,0.000167,0.000242,0.000314,0.000312,0.000271,0.000193,0.000123,0.000109,0.000199,0.000350,0.000552,0.000829,0.001181,0.001665,0.002311,0.003324,0.004850,0.007381,0.008158,0.005824,0.003857,0.002663,0.001912,0.001382,0.000998,0.000708,0.000485,0.000316,0.000189,0.000101,0.000135,0.000213,0.000320,0.000454,0.000635,0.000878,0.001238,0.001765,0.002361,0.002288,0.001653,0.001159,0.000824,0.000602,0.000439,0.000319,0.000226,0.000154,0.000147,0.000209,0.000290,0.000395,0.000538,0.000733,0.000978,0.001145,0.001038,0.000788,0.000577,0.000424,0.000314
  giAltoEee30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.006959,0.009081,0.012315,0.018565,0.028153,0.044588,0.114187,0.124993,0.048859,0.031482,0.019683,0.012844,0.008644,0.005471,0.003421,0.001862,0.000902,0.000286,0.000233,0.000370,0.000371,0.000309,0.000202,0.000154,0.000316,0.000568,0.000934,0.001428,0.002143,0.003188,0.004956,0.008182,0.009679,0.006341,0.003982,0.002603,0.001775,0.001204,0.000799,0.000504,0.000291,0.000147,0.000134,0.000236,0.000382,0.000579,0.000856,0.001275,0.001931,0.002784,0.002605,0.001721,0.001138,0.000774,0.000533,0.000364,0.000239,0.000148,0.000108,0.000174,0.000263,0.000386,0.000556,0.000803,0.001139,0.001354,0.001132,0.000793,0.000548,0.000383,0.000267,0.000183,0.000121
  giAltoEee33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.008585,0.011910,0.018442,0.028601,0.051881,0.148152,0.116598,0.047937,0.025511,0.015999,0.009973,0.005757,0.003142,0.001432,0.000410,0.000282,0.000444,0.000423,0.000287,0.000140,0.000115,0.000271,0.000586,0.001072,0.001777,0.002864,0.004718,0.008269,0.011724,0.007336,0.004255,0.002631,0.001661,0.001032,0.000600,0.000308,0.000128,0.000173,0.000332,0.000566,0.000913,0.001456,0.002388,0.003440,0.002602,0.001568,0.000982,0.000630,0.000399,0.000238,0.000128,0.000101,0.000183,0.000304,0.000480,0.000742,0.001146,0.001590,0.001412,0.000931,0.000601,0.000393,0.000254,0.000158
  giAltoEee36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.010854,0.016378,0.028295,0.056786,0.169548,0.117657,0.047576,0.024750,0.013535,0.007247,0.003523,0.001311,0.000289,0.000504,0.000516,0.000336,0.000126,0.000268,0.000684,0.001392,0.002510,0.004405,0.008309,0.014010,0.008164,0.004335,0.002505,0.001443,0.000773,0.000352,0.000118,0.000180,0.000395,0.000740,0.001290,0.002287,0.003943,0.003185,0.001748,0.001013,0.000595,0.000332,0.000163,0.000200,0.000368,0.000630,0.001059,0.001722,0.001799,0.001123,0.000669,0.000403,0.000236,0.000128,0.000109,0.000115
  giAltoEee39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.013990,0.023858,0.048875,0.150665,0.151627,0.049694,0.023833,0.012042,0.005344,0.001779,0.000333,0.000629,0.000543,0.000246,0.000119,0.000397,0.001102,0.002359,0.004615,0.009688,0.016299,0.007706,0.003821,0.001994,0.000969,0.000378,0.000160,0.000427,0.000907,0.001756,0.003531,0.004578,0.002318,0.001204,0.000638,0.000309,0.000117,0.000128,0.000294,0.000580,0.001078,0.001961,0.002080,0.001158,0.000629,0.000339,0.000169,0.000128,0.000135
  giAltoEee42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.018166,0.037139,0.089205,0.250023,0.062972,0.025695,0.010945,0.003725,0.000572,0.000740,0.000619,0.000188,0.000311,0.001143,0.002867,0.006399,0.016417,0.012713,0.005223,0.002421,0.001017,0.000300,0.000144,0.000475,0.001166,0.002569,0.005607,0.003464,0.001560,0.000733,0.000297,0.000129,0.000351,0.000781,0.001627,0.002743,0.001605,0.000774,0.000369,0.000155,0.000152,0.000148
  giAltoEee45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.023819,0.057203,0.296329,0.095870,0.031997,0.011515,0.002863,0.000564,0.000846,0.000280,0.000125,0.000542,0.002146,0.005737,0.016566,0.014688,0.005270,0.002068,0.000618,0.000104,0.000168,0.000665,0.001832,0.004795,0.005221,0.001970,0.000800,0.000255,0.000106,0.000370,0.000967,0.002308,0.002840,0.001209,0.000509,0.000186,0.000129,0.000201,0.000118
  giAltoEee48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.032760,0.113585,0.235289,0.049500,0.014494,0.002623,0.001009,0.000673,0.000541,0.002795,0.008824,0.028041,0.008677,0.002894,0.000711,0.000113,0.000362,0.001483,0.004584,0.006379,0.002029,0.000664,0.000124,0.000401,0.001264,0.003457,0.002250,0.000805,0.000255,0.000153,0.000229,0.000104
  giAltoEee51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.047737,0.301484,0.099415,0.024093,0.003561,0.001256,0.000492,0.000239,0.002207,0.009245,0.032634,0.007649,0.001940,0.000195,0.000322,0.001819,0.007079,0.004645,0.001280,0.000235,0.000103,0.000590,0.002164,0.004171,0.001262,0.000339,0.000163,0.000268
  giAltoEee54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.074277,0.500015,0.051387,0.007447,0.001481,0.000373,0.000619,0.005727,0.032843,0.010441,0.002023,0.000168,0.000950,0.005141,0.006928,0.001467,0.000161,0.000256,0.001563,0.005490,0.001551,0.000312,0.000179,0.000297
  giAltoEee57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.114410,0.191733,0.023028,0.001128,0.000560,0.001084,0.011476,0.029371,0.004136,0.000208,0.001331,0.009595,0.003939,0.000510,0.000212,0.001935,0.005681,0.001019,0.000104,0.000257,0.000236
  giAltoEee60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.227192,0.099004,0.005244,0.001344,0.001073,0.017641,0.017350,0.001409,0.000721,0.009172,0.004060,0.000250,0.000806,0.006918,0.001617,0.000117,0.000126,0.000462
  giAltoEee63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.603091,0.048190,0.002514,0.000479,0.018496,0.015302,0.000391,0.003639,0.009291,0.000469,0.001181,0.008347,0.000679,0.000111,0.000537
  giAltoEee66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000000,0.014894,0.000748,0.011456,0.020880,0.000335,0.010283,0.002934,0.000513,0.010980,0.000625,0.000358,0.000173
  giAltoEee69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.383427,0.002256,0.002169,0.058726,0.000416,0.019191,0.001023,0.003871,0.002038,0.000111,0.000515,0.000103
  giAltoEee72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.197992,0.002686,0.035282,0.002817,0.018344,0.000500,0.013835,0.000234,0.000252,0.000171
  giAltoEee75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.096384,0.000955,0.030609,0.007281,0.000937,0.016694,0.000198,0.001073
  giAltoEee78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.029788,0.022912,0.000669,0.005868,0.021959,0.000130,0.000347
  giAltoEee81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.004512,0.117453,0.038387,0.007742,0.000221,0.000205
  giAltoEee84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.005371,0.005634,0.001000,0.000468,0.000341
 elseif iVoice2==19 then
  giAltoOh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.001309,0.001824,0.002584,0.003538,0.004745,0.006302,0.008180,0.010795,0.014134,0.018933,0.026852,0.038995,0.065576,0.075348,0.053728,0.033361,0.025424,0.019181,0.016443,0.014528,0.014284,0.015282,0.019092,0.024569,0.022662,0.014749,0.009877,0.007312,0.005506,0.004369,0.003422,0.002692,0.002087,0.001569,0.001146,0.000793,0.000510,0.000298,0.000163,0.000137,0.000169,0.000197,0.000203,0.000190,0.000165,0.000129,0.000119,0.000199,0.000310,0.000458,0.000645,0.000885,0.001184,0.001561,0.002059,0.002697,0.003630,0.004955,0.007043,0.009589,0.009724,0.007266,0.005090,0.003734,0.002767,0.002112,0.001597,0.001205,0.000890,0.000637,0.000446,0.000328,0.000322,0.000409
  giAltoOh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.001648,0.002485,0.003697,0.005250,0.007398,0.010097,0.014102,0.019191,0.029184,0.043902,0.080410,0.085492,0.050018,0.033402,0.023668,0.019505,0.017033,0.017382,0.020190,0.028304,0.027191,0.016200,0.010650,0.007363,0.005554,0.004157,0.003124,0.002287,0.001614,0.001073,0.000647,0.000344,0.000174,0.000178,0.000225,0.000243,0.000227,0.000190,0.000138,0.000111,0.000208,0.000358,0.000570,0.000849,0.001223,0.001706,0.002371,0.003273,0.004688,0.006827,0.010463,0.011909,0.008681,0.005736,0.003964,0.002840,0.002045,0.001463,0.001019,0.000679,0.000447,0.000373,0.000475,0.000673,0.000930,0.001272,0.001741,0.002371,0.002729,0.002294,0.001637,0.001172,0.000856,0.000635,0.000471,0.000346,0.000248,0.000173,0.000114
  giAltoOh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.002103,0.003449,0.005368,0.008129,0.011793,0.017521,0.025816,0.041586,0.082368,0.104390,0.056599,0.036283,0.025378,0.020950,0.020421,0.024179,0.034910,0.027177,0.014524,0.009440,0.006646,0.004712,0.003347,0.002258,0.001434,0.000804,0.000377,0.000190,0.000239,0.000286,0.000274,0.000222,0.000148,0.000107,0.000224,0.000430,0.000739,0.001171,0.001769,0.002615,0.003842,0.005897,0.009422,0.014095,0.011544,0.006995,0.004512,0.003054,0.002058,0.001364,0.000851,0.000518,0.000455,0.000652,0.000977,0.001416,0.002057,0.002938,0.003127,0.002245,0.001501,0.001027,0.000720,0.000504,0.000346,0.000228,0.000141
  giAltoOh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.002740,0.004898,0.008092,0.012802,0.020408,0.032961,0.058600,0.125001,0.084118,0.045111,0.029687,0.024133,0.025379,0.037896,0.035451,0.017081,0.010266,0.006879,0.004582,0.002935,0.001729,0.000859,0.000329,0.000247,0.000333,0.000330,0.000255,0.000149,0.000102,0.000140,0.000335,0.000687,0.001224,0.002024,0.003222,0.005108,0.008605,0.015395,0.014809,0.008247,0.004922,0.003108,0.001934,0.001129,0.000626,0.000566,0.000903,0.001426,0.002224,0.003432,0.003660,0.002373,0.001484,0.000966,0.000632,0.000404,0.000244,0.000132,0.000134,0.000170,0.000126
  giAltoOh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.003650,0.007078,0.012608,0.021597,0.037879,0.078022,0.150765,0.066755,0.038384,0.029074,0.030586,0.049175,0.029523,0.014638,0.008747,0.005389,0.003142,0.001589,0.000598,0.000274,0.000395,0.000382,0.000259,0.000113,0.000103,0.000103,0.000112,0.000118,0.000144,0.000399,0.000921,0.001779,0.003139,0.005426,0.009970,0.019301,0.014629,0.007520,0.004255,0.002429,0.001283,0.000662,0.000825,0.001460,0.002472,0.004136,0.004073,0.002334,0.001367,0.000828,0.000489,0.000266,0.000125,0.000105,0.000173,0.000193,0.000114
  giAltoOh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.004962,0.010483,0.020163,0.038322,0.087672,0.170724,0.066701,0.038951,0.034708,0.056519,0.032353,0.014706,0.008302,0.004569,0.002143,0.000687,0.000355,0.000485,0.000379,0.000169,0.000126,0.000109,0.000144,0.000141,0.000416,0.001139,0.002447,0.004744,0.009383,0.020945,0.017383,0.007938,0.004095,0.002042,0.000896,0.000951,0.001864,0.003489,0.005470,0.003280,0.001715,0.000944,0.000498,0.000228,0.000109,0.000199,0.000222,0.000115
  giAltoOh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.006900,0.016262,0.035055,0.083207,0.208846,0.072590,0.041922,0.048400,0.054385,0.018894,0.009431,0.004519,0.001606,0.000377,0.000573,0.000445,0.000153,0.000115,0.000150,0.000151,0.000158,0.000452,0.001488,0.003562,0.007739,0.018995,0.023248,0.009091,0.004149,0.001716,0.000922,0.001977,0.004164,0.006325,0.003036,0.001458,0.000702,0.000287,0.000117,0.000240,0.000232,0.000105
  giAltoOh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.009796,0.025603,0.065926,0.250014,0.090217,0.048269,0.075817,0.034166,0.013762,0.005873,0.001719,0.000494,0.000660,0.000298,0.000114,0.000177,0.000166,0.000192,0.000672,0.002456,0.006469,0.017288,0.029726,0.009883,0.003885,0.001258,0.001818,0.004479,0.007363,0.002986,0.001274,0.000492,0.000134,0.000177,0.000345,0.000157
  giAltoOh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.014156,0.043198,0.156079,0.133513,0.058152,0.098380,0.029278,0.010778,0.003174,0.000545,0.000767,0.000229,0.000206,0.000126,0.000217,0.000289,0.001848,0.006291,0.019985,0.029302,0.008528,0.002573,0.001656,0.004961,0.008168,0.002742,0.000983,0.000252,0.000109,0.000211,0.000387,0.000132
  giAltoOh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.020980,0.076697,0.341641,0.077953,0.113133,0.029431,0.009144,0.001375,0.000970,0.000338,0.000253,0.000198,0.000284,0.002284,0.009509,0.041987,0.015901,0.004091,0.001908,0.007000,0.006575,0.001893,0.000457,0.000130,0.000219,0.000443,0.000123
  giAltoOh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.032525,0.166419,0.145169,0.096808,0.037784,0.009041,0.000762,0.000889,0.000227,0.000137,0.000311,0.000902,0.007130,0.038020,0.018180,0.003431,0.003954,0.012653,0.002915,0.000570,0.000161,0.000236,0.000466,0.000102
  giAltoOh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.051206,0.500042,0.096536,0.068326,0.011745,0.000989,0.000597,0.000354,0.000333,0.001345,0.012944,0.059447,0.007770,0.003639,0.014729,0.002548,0.000268,0.000152,0.000355,0.000315
  giAltoOh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.086407,0.267040,0.196788,0.021563,0.001095,0.000450,0.000254,0.000578,0.012590,0.058595,0.005142,0.009927,0.005483,0.000502,0.000175,0.000149,0.000775
  giAltoOh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.153433,0.155941,0.058873,0.002749,0.000677,0.000168,0.000395,0.004570,0.084026,0.008181,0.014008,0.003788,0.000261,0.000109,0.000887
  giAltoOh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.332841,0.193621,0.018082,0.001777,0.000274,0.000623,0.014262,0.036353,0.007910,0.005829,0.000322,0.000104,0.000472,0.000204
  giAltoOh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,1.000000,0.136629,0.001978,0.000708,0.000666,0.025888,0.015536,0.029450,0.000535,0.000119,0.000710,0.000111
  giAltoOh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.534019,0.043120,0.000901,0.000124,0.025180,0.010281,0.010963,0.000350,0.000297,0.000193
  giAltoOh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.311907,0.005497,0.000334,0.009143,0.016361,0.007576,0.000139,0.001773
  giAltoOh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.387236,0.003555,0.001246,0.072704,0.011658,0.000208,0.000409
  giAltoOh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.273266,0.001417,0.051783,0.058903,0.000238,0.000221
  giAltoOh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.086239,0.000247,0.020561,0.000699,0.000385
 elseif iVoice2==20 then
  giAltoOoh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.006790,0.007793,0.009214,0.011657,0.015318,0.019466,0.030008,0.039968,0.083795,0.111525,0.078922,0.039204,0.030853,0.020835,0.017482,0.014376,0.012615,0.012668,0.013183,0.017897,0.025293,0.022448,0.012428,0.008118,0.005897,0.004389,0.003515,0.002702,0.002132,0.001638,0.001205,0.000879,0.000585,0.000364,0.000202,0.000102,0.000109,0.000138,0.000164,0.000161,0.000150,0.000126,0.000128,0.000187,0.000263,0.000356,0.000471,0.000613,0.000790,0.001010,0.001285,0.001628,0.002033,0.002437,0.002674,0.002567,0.002185,0.001749,0.001375,0.001076,0.000841,0.000653,0.000504,0.000384,0.000287,0.000212,0.000152,0.000108
  giAltoOoh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.008303,0.009791,0.012449,0.017203,0.022681,0.037000,0.060114,0.125058,0.111065,0.049238,0.035746,0.023284,0.018963,0.015616,0.014922,0.015724,0.023589,0.030602,0.019515,0.010173,0.007095,0.005069,0.003809,0.002822,0.002111,0.001470,0.001008,0.000607,0.000336,0.000144,0.000127,0.000167,0.000197,0.000186,0.000164,0.000119,0.000110,0.000180,0.000275,0.000399,0.000559,0.000764,0.001030,0.001376,0.001826,0.002391,0.002958,0.003205,0.002870,0.002255,0.001699,0.001269,0.000946,0.000699,0.000510,0.000363,0.000253,0.000171,0.000113
  giAltoOoh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.010169,0.012589,0.017650,0.024669,0.043280,0.081468,0.157756,0.089091,0.046827,0.028403,0.022088,0.017894,0.018301,0.023035,0.036512,0.022702,0.011378,0.007064,0.005144,0.003564,0.002530,0.001649,0.001018,0.000530,0.000218,0.000148,0.000204,0.000233,0.000212,0.000159,0.000100,0.000185,0.000312,0.000486,0.000722,0.001039,0.001472,0.002066,0.002849,0.003616,0.003709,0.002969,0.002140,0.001514,0.001066,0.000741,0.000504,0.000331,0.000209,0.000128
  giAltoOoh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.012545,0.016792,0.026024,0.045873,0.091044,0.186641,0.074850,0.043441,0.028672,0.021632,0.021853,0.032802,0.039616,0.015496,0.009086,0.006060,0.003944,0.002536,0.001505,0.000730,0.000267,0.000184,0.000267,0.000264,0.000210,0.000125,0.000135,0.000272,0.000482,0.000785,0.001216,0.001840,0.002746,0.003908,0.004522,0.003656,0.002483,0.001644,0.001079,0.000691,0.000424,0.000246,0.000138
  giAltoOoh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.015591,0.023322,0.038944,0.079963,0.223136,0.078440,0.041689,0.028767,0.025351,0.035817,0.044928,0.016249,0.008786,0.005409,0.003279,0.001761,0.000729,0.000205,0.000277,0.000322,0.000253,0.000133,0.000168,0.000376,0.000714,0.001232,0.002029,0.003271,0.004898,0.005161,0.003518,0.002165,0.001315,0.000773,0.000427,0.000218,0.000115
  giAltoOoh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.019557,0.034366,0.073907,0.249826,0.098371,0.046511,0.031196,0.031410,0.061129,0.020319,0.010124,0.005639,0.002938,0.001212,0.000288,0.000334,0.000372,0.000238,0.000103,0.000220,0.000550,0.001116,0.002056,0.003647,0.005908,0.005733,0.003394,0.001890,0.001019,0.000506,0.000227,0.000115
  giAltoOoh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.025184,0.049350,0.163002,0.178182,0.056821,0.035802,0.046106,0.045414,0.014134,0.007132,0.003299,0.001061,0.000296,0.000463,0.000314,0.000107,0.000120,0.000103,0.000103,0.000367,0.000973,0.002088,0.004156,0.007278,0.005975,0.003047,0.001492,0.000668,0.000258,0.000125,0.000108,0.000107
  giAltoOoh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.033583,0.091750,0.373265,0.086876,0.043265,0.065631,0.030991,0.012123,0.005072,0.001461,0.000369,0.000528,0.000250,0.000145,0.000109,0.000271,0.000967,0.002440,0.005510,0.009071,0.004981,0.002164,0.000851,0.000277,0.000138,0.000103,0.000143
  giAltoOoh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.046646,0.159951,0.156876,0.057537,0.071660,0.032501,0.010818,0.003521,0.000411,0.000638,0.000258,0.000149,0.000132,0.000138,0.000328,0.001425,0.004063,0.009814,0.007048,0.002634,0.000855,0.000228,0.000144,0.000107,0.000164
  giAltoOoh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.068767,0.499897,0.093067,0.062857,0.040662,0.011282,0.002425,0.000668,0.000477,0.000140,0.000137,0.000132,0.000162,0.001101,0.004121,0.011845,0.006803,0.002043,0.000456,0.000183,0.000114,0.000193
  giAltoOoh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.098701,0.356319,0.071604,0.090809,0.014265,0.002121,0.000932,0.000159,0.000235,0.000129,0.000214,0.001955,0.008318,0.011948,0.002984,0.000517,0.000201,0.000125,0.000215
  giAltoOoh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.183506,0.173742,0.131280,0.024244,0.002922,0.001057,0.000152,0.000219,0.001935,0.011024,0.009962,0.001703,0.000276,0.000113,0.000181,0.000173
  giAltoOoh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.319941,0.115082,0.065007,0.007044,0.001288,0.000283,0.000127,0.000673,0.008134,0.014095,0.001709,0.000289,0.000332
  giAltoOoh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,1.000001,0.125741,0.022569,0.001337,0.000279,0.000263,0.002205,0.023707,0.004085,0.000367,0.000387
  giAltoOoh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.712592,0.181596,0.004243,0.000318,0.000259,0.003910,0.023893,0.001033,0.000156,0.000248,0.000124
  giAltoOoh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.347452,0.048482,0.002113,0.000185,0.003869,0.019916,0.000554,0.000125,0.000363
  giAltoOoh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.230150,0.014084,0.000566,0.001346,0.028182,0.000579,0.000168,0.000119
  giAltoOoh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.251504,0.002674,0.000531,0.047427,0.000734,0.000774
  giAltoOoh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.363188,0.000636,0.007820,0.002067,0.000109,0.000249
  giAltoOoh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.096965,0.000370,0.039833,0.000247,0.000141
  giAltoOoh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.028166,0.002692,0.001157,0.000239
 ; SOPRANO
 elseif iVoice2==21 then
  giSopranoAh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.000243,0.000331,0.000420,0.000500,0.000551,0.000561,0.000535,0.000457,0.000407,0.000521,0.000895,0.001468,0.002246,0.003261,0.004498,0.006101,0.008059,0.010521,0.013877,0.018007,0.024724,0.033985,0.051610,0.076040,0.075738,0.052053,0.036243,0.028064,0.022858,0.020510,0.019512,0.020774,0.023901,0.031564,0.037490,0.031794,0.021090,0.015085,0.011109,0.008541,0.006680,0.005201,0.004060,0.003097,0.002311,0.001666,0.001140,0.000733,0.000435,0.000268,0.000244,0.000282,0.000311,0.000312,0.000289,0.000247,0.000195,0.000140,0.000112,0.000165,0.000229,0.000305,0.000401,0.000521,0.000683,0.000907,0.001221,0.001582,0.001659,0.001357,0.001002,0.000751,0.000566,0.000433,0.000328,0.000245,0.000178,0.000123
  giSopranoAh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.000306,0.000437,0.000557,0.000646,0.000669,0.000627,0.000515,0.000518,0.000901,0.001666,0.002762,0.004254,0.006165,0.008737,0.011991,0.016649,0.022748,0.033722,0.050609,0.085620,0.090728,0.057631,0.039541,0.029400,0.025061,0.023243,0.025034,0.030554,0.042366,0.041517,0.026592,0.017689,0.012328,0.009171,0.006810,0.005073,0.003690,0.002597,0.001732,0.001065,0.000592,0.000326,0.000296,0.000351,0.000375,0.000355,0.000301,0.000227,0.000150,0.000102,0.000108,0.000106,0.000176,0.000263,0.000371,0.000511,0.000698,0.000974,0.001379,0.001893,0.001941,0.001448,0.001016,0.000723,0.000525,0.000378,0.000266,0.000177,0.000111,0.000134,0.000234,0.000381,0.000580,0.000844,0.001189,0.001641,0.002250,0.003093,0.004320,0.006062,0.007661
  giSopranoAh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.000390,0.000574,0.000730,0.000796,0.000751,0.000597,0.000704,0.001497,0.002843,0.004841,0.007543,0.011311,0.016537,0.024179,0.037585,0.060986,0.108486,0.093219,0.053086,0.036955,0.029857,0.027640,0.031245,0.043881,0.052110,0.032914,0.020217,0.013324,0.009422,0.006627,0.004584,0.003021,0.001841,0.000997,0.000471,0.000343,0.000417,0.000445,0.000401,0.000308,0.000200,0.000106,0.000107,0.000127,0.000122,0.000173,0.000287,0.000435,0.000637,0.000926,0.001386,0.002094,0.002326,0.001646,0.001087,0.000731,0.000499,0.000330,0.000205,0.000116,0.000177,0.000335,0.000575,0.000913,0.001379,0.002023,0.002940,0.004333,0.006473,0.008929,0.008243,0.005616,0.003765,0.002575,0.001770,0.001205,0.000794,0.000500,0.000293,0.000161
  giSopranoAh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.000506,0.000759,0.000930,0.000920,0.000718,0.000914,0.002228,0.004508,0.007891,0.012923,0.020316,0.031841,0.054837,0.111349,0.120063,0.062505,0.041150,0.033769,0.034811,0.047594,0.062278,0.035528,0.020648,0.013186,0.008671,0.005635,0.003425,0.001850,0.000831,0.000409,0.000484,0.000529,0.000455,0.000312,0.000163,0.000128,0.000152,0.000137,0.000193,0.000356,0.000577,0.000900,0.001436,0.002358,0.002764,0.001789,0.001100,0.000697,0.000432,0.000248,0.000125,0.000200,0.000427,0.000795,0.001343,0.002141,0.003346,0.005301,0.008544,0.010941,0.007840,0.004856,0.003081,0.001971,0.001236,0.000735,0.000401,0.000199,0.000112,0.000101,0.000129,0.000193,0.000281,0.000339,0.000248,0.000156,0.000103
  giSopranoAh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.000663,0.001001,0.001123,0.000915,0.001042,0.002938,0.006525,0.012209,0.021053,0.036037,0.068016,0.152197,0.104196,0.056180,0.041062,0.041595,0.063208,0.063675,0.030215,0.017110,0.010421,0.006205,0.003340,0.001470,0.000537,0.000565,0.000626,0.000496,0.000281,0.000117,0.000145,0.000181,0.000152,0.000138,0.000333,0.000614,0.001047,0.001825,0.003186,0.002732,0.001512,0.000873,0.000495,0.000248,0.000109,0.000106,0.000245,0.000590,0.001185,0.002126,0.003633,0.006253,0.010976,0.012260,0.007210,0.004145,0.002442,0.001401,0.000743,0.000345,0.000154,0.000120,0.000112,0.000142,0.000230,0.000359,0.000380,0.000227,0.000136
  giSopranoAh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.000871,0.001289,0.001252,0.001035,0.003325,0.008494,0.017445,0.033243,0.067334,0.170973,0.115091,0.058712,0.046417,0.061016,0.082910,0.035326,0.018317,0.010133,0.005189,0.002129,0.000651,0.000702,0.000709,0.000454,0.000173,0.000166,0.000215,0.000159,0.000213,0.000526,0.001023,0.001949,0.003788,0.002898,0.001448,0.000757,0.000355,0.000135,0.000106,0.000155,0.000469,0.001164,0.002384,0.004512,0.008666,0.015371,0.010478,0.005371,0.002862,0.001463,0.000658,0.000247,0.000145,0.000134,0.000130,0.000242,0.000414,0.000435,0.000230,0.000129
  giSopranoAh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.001148,0.001591,0.001193,0.002995,0.009688,0.022636,0.048394,0.122093,0.186546,0.073972,0.055348,0.087905,0.065898,0.026687,0.013278,0.006054,0.001998,0.000689,0.000892,0.000619,0.000212,0.000215,0.000244,0.000119,0.000343,0.000873,0.001866,0.004223,0.003315,0.001471,0.000664,0.000232,0.000124,0.000185,0.000679,0.001851,0.004101,0.008791,0.018115,0.011387,0.005227,0.002448,0.001017,0.000329,0.000170,0.000149,0.000119,0.000263,0.000501,0.000460,0.000214,0.000113
  giSopranoAh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.001518,0.001840,0.001829,0.009018,0.025849,0.063699,0.222787,0.125025,0.067561,0.095260,0.071075,0.026387,0.011277,0.003702,0.000818,0.001059,0.000625,0.000181,0.000305,0.000184,0.000153,0.000714,0.001808,0.004736,0.003592,0.001400,0.000498,0.000149,0.000193,0.000861,0.002706,0.006746,0.017233,0.015795,0.006213,0.002495,0.000811,0.000226,0.000191,0.000161,0.000390,0.000688,0.000316,0.000144
  giSopranoAh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.002001,0.001828,0.005878,0.024425,0.072091,0.304494,0.112379,0.083227,0.127363,0.034228,0.012415,0.002941,0.001132,0.000993,0.000234,0.000361,0.000158,0.000116,0.000662,0.002099,0.006388,0.003027,0.000988,0.000219,0.000213,0.001186,0.004269,0.012560,0.024604,0.008321,0.002813,0.000693,0.000241,0.000162,0.000153,0.000462,0.000765,0.000273,0.000112
  giSopranoAh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.002580,0.002072,0.017002,0.066543,0.342244,0.117509,0.122162,0.070710,0.020284,0.004261,0.001405,0.000907,0.000333,0.000319,0.000105,0.001054,0.003908,0.005806,0.001517,0.000271,0.000310,0.002335,0.009056,0.030843,0.010772,0.002935,0.000495,0.000270,0.000120,0.000486,0.000873,0.000259
  giSopranoAh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.003183,0.005991,0.045273,0.244218,0.147927,0.175844,0.053372,0.012105,0.001376,0.001237,0.000431,0.000243,0.000165,0.000696,0.003738,0.006635,0.001332,0.000247,0.001360,0.008210,0.036260,0.010453,0.002030,0.000343,0.000153,0.000529,0.000919,0.000227
  giSopranoAh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.003681,0.018037,0.127409,0.250032,0.190549,0.052769,0.007402,0.002118,0.000363,0.000368,0.000180,0.001429,0.009476,0.002801,0.000297,0.001724,0.013504,0.031591,0.004990,0.000452,0.000189,0.000780,0.000631,0.000132
  giSopranoAh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.003660,0.048855,0.609099,0.166474,0.068463,0.005879,0.001985,0.000726,0.000219,0.001335,0.012780,0.001982,0.000427,0.008543,0.049218,0.005623,0.000484,0.000309,0.001530,0.000224
  giSopranoAh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.004145,0.133126,0.235070,0.141452,0.008522,0.001818,0.000636,0.000100,0.007822,0.003035,0.000622,0.018126,0.021549,0.000989,0.000240,0.001747,0.000173
  giSopranoAh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.011982,0.488465,0.351711,0.024209,0.002473,0.000485,0.001393,0.013267,0.000494,0.016424,0.020902,0.000686,0.001057,0.000454
  giSopranoAh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.036071,0.499985,0.105518,0.004235,0.000736,0.002858,0.005599,0.003447,0.063157,0.000904,0.001561,0.000264
  giSopranoAh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.097704,0.332922,0.011754,0.001452,0.002670,0.003963,0.017086,0.011243,0.000618,0.000448
  giSopranoAh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.266284,0.282922,0.003635,0.000202,0.006066,0.036263,0.001979,0.003494
  giSopranoAh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.976912,0.048417,0.000970,0.026534,0.032847,0.001372,0.000907
  giSopranoAh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,1.000001,0.008470,0.005716,0.006895,0.001809,0.000528
  giSopranoAh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.665845,0.002904,0.007927,0.022486,0.000897
 elseif iVoice2==22 then
  giSopranoEh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.005375,0.006159,0.007497,0.009319,0.012241,0.016007,0.021786,0.031498,0.046972,0.085088,0.095763,0.062094,0.035620,0.026336,0.017966,0.013939,0.010264,0.007721,0.005804,0.004160,0.003000,0.002025,0.001285,0.000766,0.000371,0.000201,0.000222,0.000300,0.000319,0.000316,0.000283,0.000227,0.000174,0.000113,0.000102,0.000163,0.000248,0.000363,0.000507,0.000697,0.000930,0.001233,0.001635,0.002152,0.002976,0.004104,0.006179,0.007906,0.006942,0.004677,0.003337,0.002428,0.001839,0.001407,0.001067,0.000799,0.000561,0.000352,0.000151,0.000282,0.000528,0.000809,0.001145,0.001552,0.002050,0.002702,0.003535,0.004730,0.006447,0.009098,0.012551,0.013076,0.009939,0.006919,0.005067,0.003738,0.002853,0.002164,0.001644,0.001235,0.000905,0.000647,0.000439,0.000277,0.000158
  giSopranoEh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.006525,0.007911,0.010016,0.013570,0.018820,0.027493,0.040827,0.078633,0.116838,0.079159,0.041386,0.027974,0.019129,0.013581,0.009612,0.006849,0.004621,0.003064,0.001847,0.001036,0.000450,0.000231,0.000298,0.000377,0.000380,0.000341,0.000263,0.000186,0.000103,0.000174,0.000289,0.000453,0.000672,0.000967,0.001352,0.001893,0.002630,0.003869,0.005875,0.008999,0.008671,0.005518,0.003725,0.002567,0.001876,0.001352,0.000961,0.000627,0.000335,0.000265,0.000606,0.001006,0.001504,0.002127,0.002953,0.004076,0.005751,0.008319,0.012764,0.016038,0.012717,0.008268,0.005702,0.004030,0.002916,0.002101,0.001501,0.001036,0.000687,0.000418,0.000226,0.000113,0.000126,0.000192,0.000258,0.000326,0.000405,0.000509,0.000644,0.000773,0.000757,0.000593,0.000433,0.000318,0.000237,0.000178,0.000133
  giSopranoEh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.007934,0.010268,0.014014,0.020866,0.031775,0.051069,0.115300,0.124988,0.056004,0.035128,0.022210,0.014648,0.009836,0.006282,0.003915,0.002162,0.001056,0.000377,0.000301,0.000440,0.000452,0.000392,0.000282,0.000165,0.000151,0.000284,0.000497,0.000798,0.001223,0.001822,0.002695,0.004186,0.006781,0.010969,0.009099,0.005272,0.003380,0.002303,0.001560,0.001027,0.000581,0.000179,0.000255,0.000730,0.001308,0.002061,0.003062,0.004509,0.006743,0.010496,0.017164,0.017655,0.010910,0.006960,0.004610,0.003129,0.002119,0.001387,0.000863,0.000480,0.000225,0.000124,0.000196,0.000291,0.000386,0.000499,0.000657,0.000858,0.000920,0.000712,0.000490,0.000340,0.000241,0.000172,0.000121
  giSopranoEh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.009785,0.013551,0.020874,0.032859,0.059208,0.148132,0.120179,0.053098,0.029187,0.018238,0.011338,0.006607,0.003610,0.001666,0.000527,0.000362,0.000533,0.000521,0.000386,0.000227,0.000106,0.000248,0.000500,0.000893,0.001489,0.002395,0.003832,0.006593,0.012423,0.010837,0.005786,0.003451,0.002202,0.001367,0.000718,0.000151,0.000475,0.001206,0.002167,0.003503,0.005548,0.009076,0.015888,0.022570,0.014146,0.008203,0.005068,0.003190,0.001979,0.001150,0.000585,0.000233,0.000166,0.000296,0.000428,0.000584,0.000810,0.001083,0.001004,0.000662,0.000427,0.000283,0.000188,0.000123,0.000110,0.000151,0.000172,0.000144,0.000101
  giSopranoEh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.012322,0.018644,0.032024,0.063018,0.170241,0.124239,0.052696,0.027893,0.015451,0.008324,0.004052,0.001533,0.000402,0.000600,0.000632,0.000454,0.000226,0.000100,0.000205,0.000497,0.001017,0.001865,0.003279,0.005970,0.012399,0.013933,0.006699,0.003693,0.002143,0.001128,0.000303,0.000567,0.001626,0.003121,0.005435,0.009514,0.018307,0.026321,0.013931,0.007528,0.004359,0.002490,0.001304,0.000559,0.000184,0.000297,0.000486,0.000704,0.001040,0.001332,0.000946,0.000559,0.000342,0.000211,0.000126,0.000132,0.000189,0.000199,0.000143
  giSopranoEh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.015802,0.027106,0.054923,0.157076,0.158131,0.055887,0.027131,0.013682,0.006122,0.002071,0.000461,0.000753,0.000682,0.000372,0.000101,0.000111,0.000194,0.000578,0.001344,0.002701,0.005258,0.011744,0.017335,0.007449,0.003752,0.001922,0.000671,0.000529,0.002012,0.004255,0.008157,0.016650,0.032107,0.016554,0.008071,0.004209,0.002076,0.000838,0.000227,0.000384,0.000654,0.001019,0.001550,0.001189,0.000637,0.000357,0.000199,0.000105,0.000142,0.000222,0.000233,0.000149
  giSopranoEh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.020540,0.041742,0.102173,0.250022,0.070268,0.029304,0.012569,0.004326,0.000755,0.000881,0.000785,0.000331,0.000140,0.000302,0.000996,0.002454,0.005411,0.013623,0.018252,0.006779,0.003128,0.001162,0.000516,0.002634,0.006167,0.013587,0.034596,0.021969,0.009288,0.004271,0.001737,0.000452,0.000399,0.000782,0.001331,0.001862,0.000991,0.000488,0.000244,0.000113,0.000153,0.000264,0.000258,0.000143
  giSopranoEh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.027101,0.065718,0.296280,0.106187,0.036474,0.013213,0.003330,0.000725,0.001042,0.000455,0.000159,0.000213,0.001002,0.002983,0.007680,0.024903,0.011593,0.004412,0.001440,0.000954,0.004351,0.011141,0.031928,0.028388,0.010175,0.003972,0.001172,0.000336,0.000864,0.001632,0.002019,0.000860,0.000380,0.000158,0.000105,0.000224,0.000349,0.000206
  giSopranoEh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.037289,0.126049,0.248462,0.055787,0.016649,0.003067,0.001199,0.000908,0.000105,0.000195,0.000408,0.002033,0.006568,0.024837,0.013399,0.004282,0.000601,0.003260,0.010894,0.036706,0.027905,0.008733,0.002613,0.000372,0.000976,0.002088,0.001895,0.000683,0.000249,0.000103,0.000265,0.000399,0.000182
  giSopranoEh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.054241,0.314339,0.111819,0.027377,0.004143,0.001507,0.000744,0.000202,0.000389,0.002690,0.010531,0.034705,0.007511,0.001342,0.004034,0.016350,0.064332,0.016169,0.004160,0.000455,0.001312,0.003109,0.001277,0.000398,0.000117,0.000282,0.000466,0.000173
  giSopranoEh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.083485,0.500013,0.058605,0.008650,0.001763,0.000659,0.000290,0.000127,0.001996,0.010824,0.036509,0.006267,0.001028,0.012340,0.069232,0.018578,0.003478,0.000794,0.002663,0.001984,0.000490,0.000121,0.000310,0.000517,0.000148
  giSopranoEh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.131440,0.212366,0.026425,0.001451,0.000910,0.000318,0.000426,0.005968,0.049821,0.008823,0.001910,0.022293,0.056767,0.007944,0.000675,0.003266,0.001719,0.000315,0.000447,0.000411
  giSopranoEh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.252130,0.111578,0.006131,0.001816,0.000401,0.000821,0.013137,0.026816,0.001212,0.021797,0.055807,0.005226,0.001951,0.003795,0.000506,0.000104,0.000211,0.000803,0.000143
  giSopranoEh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.628823,0.054762,0.003015,0.000404,0.000779,0.021073,0.015027,0.008072,0.128717,0.008320,0.002625,0.002555,0.000234,0.000174,0.000934,0.000112
  giSopranoEh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000000,0.017299,0.001321,0.000255,0.021650,0.012533,0.024684,0.037149,0.001588,0.003967,0.000242,0.000619,0.000296
  giSopranoEh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.424682,0.002901,0.000637,0.011935,0.017641,0.044586,0.015886,0.006533,0.000631,0.000895,0.000169
  giSopranoEh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.223136,0.003631,0.001641,0.053621,0.043595,0.010450,0.007588,0.000208,0.000421,0.000286
  giSopranoEh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.109531,0.000806,0.042151,0.016149,0.016640,0.005110,0.000133,0.001866
  giSopranoEh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.034598,0.000509,0.025064,0.074296,0.007935,0.000130,0.000593
  giSopranoEh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.005801,0.023872,0.089183,0.013066,0.000176,0.000337
  giSopranoEh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.007260,0.107242,0.020899,0.000417,0.000572
 elseif iVoice2==23 then
  giSopranoEee24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.010638,0.013301,0.017800,0.022937,0.034070,0.046177,0.082117,0.124931,0.109118,0.060110,0.041165,0.028412,0.021053,0.015886,0.011808,0.008953,0.006545,0.004708,0.003285,0.002126,0.001294,0.000655,0.000252,0.000225,0.000362,0.000444,0.000451,0.000423,0.000354,0.000271,0.000187,0.000100,0.000102,0.000106,0.000127,0.000140,0.000138,0.000125,0.000158,0.000300,0.000501,0.000769,0.001120,0.001555,0.002122,0.002825,0.003721,0.004928,0.006460,0.008829,0.012145,0.017913,0.024865,0.024284,0.017013,0.011692,0.008487,0.006258,0.004772,0.003600,0.002730,0.002031,0.001468,0.001022,0.000652,0.000355,0.000117,0.000141,0.000322,0.000509,0.000701,0.000919,0.001190,0.001528,0.002038,0.002743,0.003904,0.004865,0.004376,0.003081,0.002205,0.001615,0.001216,0.000928,0.000704,0.000533,0.000394,0.000284
  giSopranoEee27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.013143,0.017627,0.024484,0.036616,0.054010,0.106371,0.151537,0.098520,0.052871,0.035393,0.024586,0.017401,0.012353,0.008770,0.005907,0.003868,0.002280,0.001215,0.000447,0.000241,0.000427,0.000533,0.000532,0.000465,0.000348,0.000231,0.000110,0.000104,0.000121,0.000112,0.000126,0.000155,0.000168,0.000158,0.000125,0.000106,0.000207,0.000430,0.000749,0.001201,0.001793,0.002594,0.003630,0.005050,0.007012,0.009981,0.014625,0.023409,0.030837,0.024037,0.014956,0.010218,0.007150,0.005156,0.003700,0.002635,0.001801,0.001168,0.000663,0.000277,0.000105,0.000356,0.000621,0.000897,0.001231,0.001659,0.002305,0.003282,0.004967,0.005817,0.004341,0.002841,0.001963,0.001402,0.001015,0.000731,0.000517,0.000352,0.000228,0.000134,0.000126,0.000225,0.000358,0.000534,0.000764,0.001064,0.001470,0.002034,0.002872,0.004116,0.005339
  giSopranoEee30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.016328,0.023966,0.035128,0.059257,0.121512,0.177707,0.090670,0.053040,0.031736,0.021506,0.014296,0.009353,0.005850,0.003306,0.001623,0.000515,0.000325,0.000586,0.000639,0.000576,0.000417,0.000250,0.000114,0.000143,0.000131,0.000145,0.000186,0.000199,0.000175,0.000123,0.000211,0.000510,0.000977,0.001671,0.002629,0.003977,0.005882,0.008680,0.013457,0.021730,0.035492,0.029923,0.017181,0.010895,0.007320,0.004922,0.003309,0.002111,0.001243,0.000586,0.000119,0.000355,0.000727,0.001119,0.001622,0.002326,0.003519,0.005702,0.006806,0.004491,0.002815,0.001839,0.001255,0.000850,0.000559,0.000348,0.000196,0.000189,0.000349,0.000574,0.000885,0.001314,0.001924,0.002866,0.004356,0.006214,0.005671,0.003728,0.002462,0.001676,0.001149,0.000782,0.000516,0.000324,0.000187
  giSopranoEee33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.020626,0.032969,0.058370,0.115099,0.213096,0.095777,0.052675,0.031539,0.019117,0.011719,0.006681,0.003278,0.001235,0.000319,0.000658,0.000760,0.000651,0.000420,0.000175,0.000135,0.000172,0.000141,0.000168,0.000224,0.000231,0.000176,0.000187,0.000547,0.001202,0.002233,0.003774,0.006037,0.009634,0.016040,0.029029,0.043390,0.025586,0.014544,0.008907,0.005565,0.003428,0.001951,0.000904,0.000184,0.000459,0.000990,0.001589,0.002432,0.003929,0.006894,0.007727,0.004364,0.002585,0.001631,0.001026,0.000621,0.000346,0.000162,0.000243,0.000488,0.000854,0.001385,0.002187,0.003507,0.005798,0.007630,0.005258,0.003185,0.002005,0.001280,0.000803,0.000478,0.000258,0.000123,0.000170,0.000276,0.000437,0.000708,0.000961,0.000685,0.000413,0.000261,0.000170,0.000110
  giSopranoEee36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.026610,0.045888,0.092383,0.249950,0.120266,0.056848,0.031787,0.017916,0.009422,0.004255,0.001310,0.000450,0.000889,0.000846,0.000542,0.000201,0.000187,0.000191,0.000120,0.000112,0.000212,0.000280,0.000251,0.000180,0.000602,0.001541,0.003119,0.005667,0.009889,0.017719,0.035957,0.048756,0.023480,0.012570,0.007234,0.004081,0.002055,0.000713,0.000283,0.001024,0.001849,0.003076,0.005523,0.009798,0.006208,0.003254,0.001871,0.001075,0.000573,0.000258,0.000108,0.000344,0.000740,0.001368,0.002373,0.004137,0.007509,0.008478,0.004765,0.002699,0.001585,0.000909,0.000481,0.000219,0.000165,0.000304,0.000524,0.000930,0.001082,0.000610,0.000347,0.000208,0.000124
  giSopranoEee39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.035209,0.073147,0.212478,0.196804,0.070708,0.034761,0.017520,0.007728,0.002428,0.000481,0.001065,0.000929,0.000462,0.000208,0.000225,0.000124,0.000181,0.000310,0.000315,0.000211,0.000859,0.002401,0.005185,0.010096,0.019955,0.046808,0.048070,0.020438,0.010314,0.005271,0.002337,0.000554,0.000712,0.001795,0.003321,0.006570,0.011649,0.005690,0.002809,0.001465,0.000706,0.000269,0.000254,0.000717,0.001531,0.002949,0.005763,0.010715,0.006995,0.003499,0.001858,0.000950,0.000424,0.000150,0.000118,0.000278,0.000538,0.001060,0.001234,0.000610,0.000321,0.000174
  giSopranoEee42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.047942,0.118546,0.355496,0.106101,0.043023,0.018713,0.006614,0.001030,0.001174,0.001153,0.000500,0.000284,0.000191,0.000193,0.000380,0.000359,0.000423,0.001960,0.005278,0.011817,0.027041,0.071322,0.034504,0.014706,0.006650,0.002498,0.000238,0.001464,0.003270,0.007097,0.013721,0.005674,0.002532,0.001128,0.000397,0.000382,0.001165,0.002666,0.005818,0.012613,0.007557,0.003399,0.001587,0.000656,0.000201,0.000104,0.000255,0.000576,0.001288,0.001283,0.000555,0.000270,0.000125
  giSopranoEee45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.065934,0.230214,0.191525,0.063074,0.023438,0.006556,0.000638,0.001521,0.000841,0.000343,0.000179,0.000337,0.000464,0.000376,0.002409,0.007565,0.019312,0.058226,0.051263,0.017853,0.006873,0.001812,0.000923,0.003191,0.007896,0.015518,0.005189,0.002060,0.000693,0.000490,0.001722,0.004412,0.011699,0.010593,0.004042,0.001620,0.000520,0.000134,0.000111,0.000344,0.000887,0.001947,0.000836,0.000344,0.000137
  giSopranoEee48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.091783,0.499950,0.113693,0.035833,0.008510,0.000901,0.001693,0.000403,0.000370,0.000237,0.000436,0.000513,0.001200,0.006244,0.019806,0.072046,0.047005,0.014484,0.004113,0.000568,0.003707,0.011075,0.012437,0.003752,0.001151,0.000146,0.000689,0.002748,0.008313,0.017019,0.005417,0.001825,0.000438,0.000148,0.000332,0.001055,0.002175,0.000698,0.000249
  giSopranoEee51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.146370,0.393762,0.069555,0.015463,0.000962,0.001858,0.000450,0.000362,0.000631,0.001719,0.010386,0.039977,0.096231,0.020650,0.004678,0.001429,0.006658,0.023341,0.005628,0.001415,0.001439,0.005919,0.021497,0.007017,0.001904,0.000301,0.000156,0.000237,0.001080,0.002476,0.000648,0.000179
  giSopranoEee54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.237094,0.212185,0.037422,0.002059,0.002305,0.000169,0.000384,0.000744,0.000848,0.010559,0.054094,0.068997,0.013299,0.000476,0.006544,0.027445,0.005064,0.000793,0.000765,0.005332,0.025238,0.006800,0.001313,0.000208,0.000144,0.001157,0.002565,0.000537
  giSopranoEee57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.460456,0.126143,0.013110,0.003041,0.000110,0.000358,0.000928,0.004819,0.038635,0.102507,0.013746,0.001848,0.015801,0.010379,0.001387,0.000981,0.008831,0.021187,0.003241,0.000268,0.000155,0.001775,0.001673,0.000275
  giSopranoEee60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.999999,0.071669,0.001801,0.000804,0.000480,0.001006,0.012498,0.144146,0.028964,0.001138,0.022162,0.007499,0.000284,0.005499,0.034034,0.003651,0.000296,0.000666,0.004356,0.000498
  giSopranoEee63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.787626,0.030929,0.003718,0.000901,0.001263,0.020781,0.192483,0.009358,0.013320,0.011259,0.000185,0.011846,0.014037,0.000603,0.000473,0.004954,0.000357
  giSopranoEee66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.424357,0.004117,0.000335,0.001490,0.021121,0.137972,0.000951,0.054887,0.001585,0.010667,0.013598,0.000416,0.002313,0.001073
  giSopranoEee69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.252260,0.006081,0.000717,0.009639,0.204955,0.003697,0.020753,0.001961,0.042356,0.000535,0.003552,0.000551
  giSopranoEee72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.143330,0.001607,0.002011,0.288307,0.002276,0.014994,0.010998,0.007299,0.001333,0.000996
  giSopranoEee75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.061863,0.001803,0.041574,0.018718,0.022519,0.023699,0.001207,0.009907,0.000135
  giSopranoEee78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.008237,0.002981,0.275939,0.109772,0.021334,0.000832,0.002147
  giSopranoEee81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.012164,0.019280,0.007395,0.003926,0.001069,0.001101
  giSopranoEee84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.003212,0.576616,0.029989,0.014597,0.001991
 elseif iVoice2==24 then
  giSopranoOh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.001294,0.001804,0.002559,0.003508,0.004711,0.006268,0.008148,0.010769,0.014119,0.018933,0.026874,0.039039,0.065610,0.075277,0.053501,0.033025,0.024955,0.018554,0.015599,0.013411,0.012721,0.013105,0.015740,0.019623,0.017772,0.011470,0.007689,0.005735,0.004357,0.003493,0.002766,0.002194,0.001717,0.001300,0.000955,0.000665,0.000428,0.000251,0.000137,0.000118,0.000148,0.000173,0.000179,0.000168,0.000147,0.000115,0.000151,0.000226,0.000321,0.000442,0.000592,0.000782,0.001031,0.001351,0.001818,0.002481,0.003526,0.004801,0.004868,0.003637,0.002548,0.001868,0.001383,0.001054,0.000797,0.000601,0.000445,0.000319,0.000220,0.000141
  giSopranoOh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.001630,0.002458,0.003663,0.005211,0.007357,0.010060,0.014075,0.019183,0.029201,0.043953,0.080445,0.085357,0.049683,0.032906,0.022953,0.018492,0.015615,0.015240,0.016916,0.022734,0.021331,0.012592,0.008310,0.005802,0.004428,0.003356,0.002549,0.001886,0.001341,0.000898,0.000544,0.000289,0.000147,0.000155,0.000197,0.000214,0.000201,0.000168,0.000123,0.000175,0.000282,0.000423,0.000611,0.000854,0.001187,0.001639,0.002348,0.003418,0.005238,0.005962,0.004346,0.002871,0.001982,0.001418,0.001020,0.000730,0.000509,0.000340,0.000212,0.000117,0.000123,0.000210,0.000333,0.000498,0.000717,0.001000,0.001380,0.001892,0.002632,0.003685,0.005013,0.005372,0.004237,0.003015
  giSopranoOh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.002079,0.003414,0.005324,0.008081,0.011748,0.017492,0.025815,0.041628,0.082430,0.104259,0.056186,0.035627,0.024387,0.019456,0.018039,0.020232,0.027855,0.021210,0.011299,0.007422,0.005297,0.003812,0.002741,0.001871,0.001197,0.000675,0.000317,0.000162,0.000209,0.000251,0.000242,0.000197,0.000132,0.000107,0.000210,0.000366,0.000584,0.000885,0.001310,0.001924,0.002953,0.004717,0.007057,0.005779,0.003501,0.002256,0.001525,0.001026,0.000681,0.000426,0.000243,0.000117,0.000130,0.000248,0.000426,0.000678,0.001026,0.001509,0.002196,0.003251,0.004858,0.006394,0.005489,0.003690,0.002478,0.001703,0.001172,0.000797,0.000524,0.000327,0.000190,0.000103
  giSopranoOh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.002709,0.004852,0.008035,0.012747,0.020371,0.032964,0.058669,0.125001,0.083706,0.044369,0.028467,0.022150,0.021874,0.030653,0.027728,0.013291,0.008093,0.005517,0.003734,0.002427,0.001444,0.000721,0.000277,0.000215,0.000292,0.000291,0.000227,0.000134,0.000161,0.000338,0.000610,0.001012,0.001614,0.002559,0.004309,0.007707,0.007414,0.004128,0.002461,0.001551,0.000965,0.000565,0.000295,0.000120,0.000108,0.000252,0.000490,0.000850,0.001378,0.002169,0.003420,0.005508,0.007680,0.005998,0.003714,0.002344,0.001506,0.000954,0.000577,0.000322,0.000161,0.000107,0.000158,0.000239,0.000301,0.000222,0.000137
  giSopranoOh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.003609,0.007017,0.012541,0.021545,0.037880,0.078111,0.150624,0.066083,0.037128,0.026838,0.026228,0.039276,0.022960,0.011480,0.006993,0.004394,0.002604,0.001332,0.000502,0.000236,0.000346,0.000337,0.000231,0.000102,0.000101,0.000192,0.000455,0.000888,0.001572,0.002718,0.004992,0.009663,0.007323,0.003762,0.002124,0.001211,0.000642,0.000285,0.000266,0.000594,0.001132,0.001982,0.003379,0.005927,0.009151,0.006623,0.003751,0.002204,0.001292,0.000718,0.000358,0.000154,0.000119,0.000190,0.000308,0.000340,0.000202,0.000117
  giSopranoOh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.004909,0.010405,0.020089,0.038305,0.087774,0.170455,0.065711,0.036928,0.030431,0.045396,0.025146,0.011587,0.006704,0.003768,0.001793,0.000577,0.000309,0.000427,0.000337,0.000152,0.000119,0.000109,0.000199,0.000563,0.001223,0.002376,0.004699,0.010486,0.008702,0.003969,0.002043,0.001020,0.000424,0.000103,0.000125,0.000420,0.000997,0.002005,0.003793,0.007390,0.010776,0.006049,0.003151,0.001678,0.000839,0.000359,0.000132,0.000102,0.000113,0.000198,0.000355,0.000391,0.000203,0.000109
  giSopranoOh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.006830,0.016166,0.034995,0.083292,0.208584,0.071275,0.038932,0.040498,0.042444,0.014854,0.007630,0.003744,0.001348,0.000320,0.000504,0.000396,0.000138,0.000106,0.000146,0.000215,0.000737,0.001782,0.003877,0.009510,0.011639,0.004546,0.002069,0.000859,0.000236,0.000109,0.000501,0.001372,0.003057,0.006592,0.012961,0.007475,0.003451,0.001615,0.000663,0.000208,0.000120,0.000112,0.000108,0.000215,0.000428,0.000411,0.000185
  giSopranoOh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.009704,0.025493,0.065932,0.250014,0.088732,0.044302,0.061324,0.026584,0.011036,0.004855,0.001443,0.000431,0.000582,0.000267,0.000105,0.000172,0.000323,0.001223,0.003240,0.008657,0.014882,0.004940,0.001937,0.000593,0.000507,0.001712,0.004370,0.011103,0.012077,0.004722,0.001922,0.000649,0.000167,0.000146,0.000143,0.000321,0.000611,0.000278,0.000117
  giSopranoOh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.014035,0.043095,0.156256,0.132167,0.053681,0.078574,0.022962,0.008787,0.002659,0.000470,0.000678,0.000207,0.000194,0.000136,0.000137,0.000913,0.003151,0.010006,0.014670,0.004259,0.001289,0.000160,0.000534,0.002274,0.006787,0.018371,0.007525,0.002591,0.000717,0.000172,0.000153,0.000144,0.000382,0.000684,0.000234
  giSopranoOh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.020824,0.076663,0.341102,0.073904,0.090867,0.023190,0.007540,0.001155,0.000855,0.000304,0.000238,0.000105,0.000141,0.001129,0.004763,0.021021,0.007952,0.002044,0.000206,0.000843,0.004024,0.014832,0.012128,0.003366,0.000720,0.000204,0.000117,0.000109,0.000398,0.000782,0.000218
  giSopranoOh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.032333,0.166589,0.142539,0.081002,0.029705,0.007490,0.000649,0.000790,0.000209,0.000157,0.000135,0.000430,0.003567,0.019036,0.009090,0.001717,0.000218,0.002747,0.013196,0.014956,0.003234,0.000418,0.000224,0.000433,0.000823,0.000182
  giSopranoOh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.050988,0.500042,0.088602,0.053164,0.009710,0.000862,0.000535,0.000344,0.000132,0.000646,0.006483,0.029761,0.003875,0.000144,0.003426,0.022224,0.009445,0.001297,0.000291,0.000642,0.000556
  giSopranoOh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.086202,0.264349,0.157171,0.017580,0.000946,0.000407,0.000273,0.000276,0.006305,0.029333,0.002574,0.001071,0.013581,0.015055,0.001438,0.000307,0.000288,0.001369,0.000173
  giSopranoOh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.153365,0.147839,0.046389,0.002309,0.000608,0.000209,0.000135,0.002259,0.042068,0.004087,0.001687,0.029687,0.006733,0.000409,0.000218,0.001565,0.000131
  giSopranoOh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.333182,0.162007,0.014981,0.001580,0.000313,0.000270,0.007135,0.018178,0.000437,0.026398,0.006466,0.000447,0.000865,0.000364
  giSopranoOh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.999999,0.106309,0.001723,0.000687,0.000264,0.012966,0.007747,0.006853,0.018884,0.000582,0.001285,0.000199
  giSopranoOh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.528638,0.035156,0.000814,0.000113,0.012612,0.005146,0.027163,0.002875,0.000576,0.000345
  giSopranoOh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.295700,0.004617,0.000416,0.004518,0.008174,0.059391,0.000819,0.003130
  giSopranoOh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.324009,0.003159,0.000539,0.036354,0.052795,0.000895,0.000729
  giSopranoOh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.212624,0.001375,0.025936,0.013708,0.001164,0.000397
  giSopranoOh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.070310,0.000226,0.010293,0.005749,0.000690
 elseif iVoice2==25 then
  giSopranoOoh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.006736,0.007744,0.009173,0.011625,0.015301,0.019467,0.030031,0.040022,0.083847,0.111438,0.078627,0.038797,0.030293,0.020098,0.016517,0.013075,0.010944,0.010263,0.009904,0.012389,0.016364,0.013953,0.007558,0.004976,0.003673,0.002791,0.002294,0.001798,0.001446,0.001130,0.000839,0.000619,0.000412,0.000255,0.000142,0.000113,0.000132,0.000128,0.000119,0.000126,0.000173,0.000228,0.000299,0.000385,0.000495,0.000633,0.000806,0.001004,0.001164,0.001174,0.001025,0.000827,0.000650,0.000509,0.000398,0.000310,0.000240,0.000183,0.000137
  giSopranoOoh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.008240,0.009736,0.012406,0.017177,0.022681,0.037033,0.060187,0.125058,0.110778,0.048765,0.035061,0.022345,0.017647,0.013782,0.012248,0.011790,0.016043,0.019429,0.011945,0.006214,0.004417,0.003235,0.002505,0.001898,0.001450,0.001023,0.000710,0.000427,0.000235,0.000103,0.000106,0.000137,0.000159,0.000148,0.000130,0.000131,0.000191,0.000267,0.000368,0.000496,0.000668,0.000893,0.001169,0.001399,0.001375,0.001123,0.000853,0.000637,0.000476,0.000354,0.000259,0.000186,0.000129,0.000104,0.000147,0.000202,0.000272,0.000362,0.000479,0.000623,0.000759,0.000789,0.000681,0.000530
  giSopranoOoh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.010095,0.012529,0.017608,0.024663,0.043315,0.081558,0.157661,0.088611,0.046106,0.027324,0.020478,0.015548,0.014372,0.016229,0.023376,0.013886,0.006978,0.004439,0.003346,0.002384,0.001737,0.001149,0.000718,0.000372,0.000154,0.000123,0.000167,0.000187,0.000168,0.000125,0.000135,0.000212,0.000319,0.000466,0.000663,0.000940,0.001310,0.001651,0.001590,0.001211,0.000862,0.000610,0.000429,0.000296,0.000198,0.000125,0.000140,0.000209,0.000299,0.000421,0.000586,0.000792,0.000937,0.000852,0.000643,0.000461,0.000327,0.000231,0.000160,0.000107
  giSopranoOoh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.012459,0.016728,0.025997,0.045900,0.091150,0.186461,0.074200,0.042360,0.026982,0.018939,0.017010,0.022371,0.024731,0.009432,0.005684,0.003945,0.002654,0.001756,0.001058,0.000513,0.000188,0.000154,0.000216,0.000210,0.000166,0.000171,0.000292,0.000464,0.000712,0.001081,0.001602,0.002011,0.001703,0.001162,0.000770,0.000507,0.000326,0.000198,0.000111,0.000102,0.000176,0.000278,0.000423,0.000631,0.000914,0.001124,0.000963,0.000669,0.000448,0.000297,0.000192,0.000119,0.000112
  giSopranoOoh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.015493,0.023256,0.038946,0.080072,0.222961,0.077625,0.040215,0.026163,0.020537,0.024794,0.027925,0.009959,0.005586,0.003599,0.002262,0.001239,0.000511,0.000154,0.000227,0.000257,0.000200,0.000105,0.000183,0.000348,0.000602,0.000995,0.001621,0.002342,0.002063,0.001309,0.000802,0.000483,0.000275,0.000140,0.000124,0.000233,0.000396,0.000642,0.001014,0.001340,0.001090,0.000692,0.000426,0.000257,0.000148,0.000123,0.000130
  giSopranoOoh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.019447,0.034315,0.073971,0.249826,0.097427,0.044636,0.027534,0.023553,0.038811,0.012411,0.006462,0.003791,0.002045,0.000853,0.000206,0.000273,0.000295,0.000187,0.000173,0.000381,0.000736,0.001334,0.002335,0.002747,0.001704,0.000952,0.000519,0.000258,0.000107,0.000208,0.000405,0.000725,0.001247,0.001582,0.001061,0.000602,0.000333,0.000173,0.000145,0.000152
  giSopranoOoh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.025063,0.049337,0.163181,0.177222,0.054661,0.031107,0.032482,0.027778,0.008882,0.004771,0.002299,0.000744,0.000246,0.000371,0.000246,0.000109,0.000162,0.000428,0.000938,0.001893,0.003326,0.002439,0.001229,0.000596,0.000251,0.000103,0.000284,0.000607,0.001190,0.001900,0.001302,0.000662,0.000323,0.000141,0.000101,0.000172,0.000168
  giSopranoOoh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.033454,0.091804,0.372905,0.084714,0.037879,0.044758,0.018864,0.007892,0.003513,0.001026,0.000308,0.000420,0.000196,0.000129,0.000187,0.000586,0.001429,0.003217,0.003417,0.001546,0.000653,0.000221,0.000103,0.000354,0.000853,0.001843,0.001938,0.000899,0.000386,0.000141,0.000147,0.000226,0.000135
  giSopranoOoh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.046516,0.160169,0.155245,0.052329,0.049605,0.019920,0.007198,0.002478,0.000308,0.000509,0.000201,0.000135,0.000127,0.000370,0.001207,0.003248,0.004134,0.001606,0.000550,0.000125,0.000106,0.000468,0.001290,0.002692,0.001390,0.000516,0.000158,0.000174,0.000259,0.000118
  giSopranoOoh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.068664,0.499897,0.089313,0.047133,0.024836,0.007586,0.001706,0.000547,0.000375,0.000132,0.000136,0.000347,0.001476,0.004681,0.003416,0.001040,0.000214,0.000172,0.000813,0.002504,0.002129,0.000667,0.000162,0.000186,0.000304,0.000115
  giSopranoOoh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.098674,0.354398,0.062213,0.055545,0.009543,0.001487,0.000749,0.000134,0.000212,0.000123,0.000856,0.003788,0.004876,0.001194,0.000166,0.000569,0.002379,0.002608,0.000649,0.000109,0.000205,0.000338
  giSopranoOoh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.183614,0.169417,0.089526,0.015785,0.002053,0.000839,0.000154,0.000122,0.000129,0.001172,0.006436,0.003091,0.000442,0.000113,0.000708,0.003688,0.001800,0.000283,0.000294,0.000271
  giSopranoOoh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.320378,0.104663,0.039843,0.004956,0.001029,0.000257,0.000735,0.006501,0.003212,0.000252,0.000215,0.002579,0.002779,0.000315,0.000143,0.000522
  giSopranoOoh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,1.000001,0.094284,0.015175,0.001094,0.000262,0.000162,0.002953,0.006834,0.000428,0.000345,0.005011,0.001335,0.000107,0.000114,0.000609
  giSopranoOoh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.708750,0.111076,0.002977,0.000267,0.000246,0.007577,0.002388,0.000170,0.004759,0.001298,0.000408,0.000196
  giSopranoOoh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.338803,0.031564,0.001678,0.000244,0.000258,0.012872,0.000885,0.001416,0.003598,0.000134,0.000588,0.000112
  giSopranoOoh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.209313,0.009909,0.000515,0.000104,0.013000,0.000503,0.005157,0.000630,0.000287,0.000189
  giSopranoOoh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.188584,0.002188,0.000154,0.005909,0.000855,0.010025,0.000214,0.001217
  giSopranoOoh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.222149,0.000533,0.000493,0.004776,0.009517,0.000194,0.000393
  giSopranoOoh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.063129,0.000488,0.025748,0.002834,0.000268,0.000223
  giSopranoOoh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.019817,0.000207,0.001007,0.001261,0.000379
 elseif iVoice2==26 then
  giMixedAh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24), i(gkBW1_V2)  , i(gkPS1)  ,1,1,1    ,0.000412,0.000370,0.000337,0.000371,0.000555,0.000859,0.001305,0.001896,0.002601,0.003585,0.004729,0.006226,0.008387,0.010759,0.015604,0.021153,0.036276,0.057553,0.053066,0.030189,0.020360,0.014722,0.011349,0.009414,0.008036,0.007520,0.007518,0.008204,0.010188,0.013445,0.021468,0.024378,0.017218,0.010300,0.007574,0.005320,0.004125,0.003105,0.002351,0.001777,0.001280,0.000908,0.000603,0.000362,0.000194,0.000113,0.000107,0.000165,0.000269,0.000410,0.000591,0.000829,0.001124,0.001497,0.001964,0.002540,0.003281,0.004214,0.005465,0.007172,0.009487,0.012221,0.012290,0.008928,0.005554,0.004575,0.006297,0.009642,0.012002,0.011210,0.008864,0.007021,0.005649,0.004715,0.004092,0.003711,0.003540,0.003269,0.002720,0.002116,0.001696,0.001392,0.001163,0.000978,0.000817,0.000679,0.000554,0.000445,0.000349,0.000267,0.000201,0.000148,0.000112
  giMixedAh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27), i(gkBW2_V2)  , i(gkPS2)  ,1,1,1    ,0.000484,0.000423,0.000410,0.000587,0.001000,0.001640,0.002515,0.003647,0.005207,0.007188,0.010221,0.013861,0.021835,0.034169,0.065821,0.062536,0.031595,0.021762,0.014578,0.011706,0.009599,0.008934,0.009048,0.011008,0.014402,0.024811,0.028584,0.017346,0.010562,0.007037,0.005089,0.003642,0.002621,0.001845,0.001237,0.000781,0.000431,0.000198,0.000132,0.000128,0.000107,0.000154,0.000279,0.000469,0.000726,0.001077,0.001533,0.002136,0.002909,0.003944,0.005309,0.007297,0.010064,0.014010,0.014733,0.009883,0.005835,0.006234,0.010391,0.014244,0.013109,0.009848,0.007528,0.005944,0.004974,0.004420,0.004181,0.003659,0.002784,0.002107,0.001664,0.001346,0.001093,0.000881,0.000697,0.000538,0.000403,0.000292,0.000206,0.000145,0.000112
  giMixedAh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30), i(gkBW3_V2)  , i(gkPS3)  ,1,1,1    ,0.000560,0.000482,0.000562,0.001054,0.001899,0.003144,0.004913,0.007274,0.010934,0.015824,0.026652,0.050310,0.084386,0.048694,0.027304,0.017198,0.013209,0.010896,0.010619,0.012323,0.016925,0.031534,0.030303,0.014903,0.009410,0.006192,0.004171,0.002825,0.001804,0.001089,0.000559,0.000223,0.000135,0.000159,0.000132,0.000111,0.000205,0.000408,0.000717,0.001165,0.001784,0.002644,0.003807,0.005452,0.007846,0.011507,0.016933,0.016329,0.008970,0.006585,0.011345,0.016921,0.014673,0.010437,0.007690,0.006096,0.005243,0.004888,0.003982,0.002810,0.002096,0.001620,0.001264,0.000975,0.000730,0.000526,0.000361,0.000238,0.000159,0.000123,0.000113,0.000108
  giMixedAh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33), i(gkBW4_V2)  , i(gkPS4)  ,1,1,1    ,0.000645,0.000569,0.000955,0.001964,0.003652,0.006202,0.009824,0.015928,0.027700,0.054004,0.099838,0.045530,0.026122,0.017357,0.013212,0.012640,0.015734,0.025801,0.041244,0.020359,0.011360,0.006947,0.004321,0.002669,0.001503,0.000708,0.000231,0.000110,0.000186,0.000170,0.000130,0.000254,0.000564,0.001062,0.001820,0.002933,0.004538,0.006945,0.010838,0.017334,0.020883,0.011091,0.008199,0.016262,0.019876,0.013707,0.009453,0.007159,0.006126,0.005456,0.003786,0.002617,0.001920,0.001428,0.001039,0.000721,0.000469,0.000288,0.000180,0.000140,0.000132,0.000122,0.000102
  giMixedAh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36), i(gkBW5_V2)  , i(gkPS5)  ,1,1,1    ,0.001076,0.000760,0.001152,0.002918,0.006013,0.010963,0.018910,0.033003,0.065564,0.123397,0.061570,0.034546,0.024544,0.022364,0.028096,0.049779,0.048235,0.023028,0.012783,0.007477,0.004231,0.002147,0.000887,0.000349,0.000355,0.000327,0.000195,0.000152,0.000288,0.000361,0.000371,0.000539,0.001217,0.002491,0.004498,0.007542,0.012318,0.020674,0.035880,0.035208,0.020677,0.019986,0.031450,0.029001,0.018014,0.012074,0.009087,0.007807,0.005927,0.003587,0.002503,0.001834,0.001291,0.000830,0.000470,0.000249,0.000194,0.000200,0.000178,0.000130
  giMixedAh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39), i(gkBW6_V2)  , i(gkPS6)  ,1,1,1    ,0.001221,0.000884,0.002403,0.006103,0.012641,0.024258,0.049147,0.124843,0.094249,0.043351,0.028599,0.027085,0.040952,0.068353,0.030182,0.014975,0.007910,0.003883,0.001548,0.000473,0.000421,0.000370,0.000161,0.000304,0.000426,0.000443,0.000809,0.002086,0.004462,0.008387,0.015033,0.028032,0.047931,0.029239,0.022629,0.037393,0.031509,0.018127,0.012012,0.009537,0.007224,0.004034,0.002693,0.001836,0.001138,0.000598,0.000284,0.000232,0.000231,0.000177,0.000105
  giMixedAh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42), i(gkBW7_V2)  , i(gkPS7)  ,1,1,1    ,0.001347,0.001378,0.005009,0.012714,0.027848,0.064949,0.174667,0.072154,0.037557,0.031873,0.051698,0.069662,0.025483,0.011788,0.005236,0.001791,0.000485,0.000510,0.000306,0.000357,0.000517,0.000580,0.001645,0.004407,0.009531,0.019083,0.040846,0.049561,0.026461,0.041849,0.036370,0.019264,0.012655,0.010146,0.005280,0.003211,0.002020,0.001088,0.000460,0.000274,0.000277,0.000199,0.000106,0.000102
  giMixedAh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45), i(gkBW8_V2)  , i(gkPS8)  ,1,1,1    ,0.001435,0.002814,0.010404,0.027304,0.073592,0.204283,0.065680,0.038086,0.049867,0.090768,0.028199,0.011415,0.004006,0.000835,0.000609,0.000353,0.000145,0.000530,0.000623,0.001521,0.005124,0.012704,0.029322,0.067515,0.034280,0.047183,0.040088,0.019737,0.013502,0.007873,0.004048,0.002335,0.001073,0.000384,0.000337,0.000259,0.000126,0.000121,0.000110
  giMixedAh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48), i(gkBW9_V2)  , i(gkPS9)  ,1,1,1    ,0.001665,0.005802,0.022639,0.067838,0.277959,0.082343,0.043643,0.072542,0.079439,0.023326,0.007411,0.001524,0.000914,0.000509,0.000186,0.000132,0.000538,0.002041,0.005636,0.014527,0.006699,0.012109,0.005270,0.003013,0.001652,0.000778,0.000392,0.000133
  giMixedAh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51), i(gkBW10_V2) , i(gkPS10) ,1,1,1    ,0.001879,0.012367,0.050272,0.249889,0.104596,0.051557,0.135917,0.047385,0.012730,0.002203,0.001070,0.000444,0.000205,0.000486,0.002508,0.008455,0.012808,0.011617,0.007544,0.003612,0.001661,0.000745,0.000277
  giMixedAh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54), i(gkBW11_V2) , i(gkPS11) ,1,1,1    ,0.002739,0.026042,0.133355,0.175744,0.061363,0.185512,0.036813,0.006813,0.001283,0.000619,0.000238,0.000104,0.001374,0.006525,0.018731,0.014280,0.007212,0.003761,0.001187,0.000430,0.000106
  giMixedAh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57), i(gkBW12_V2) , i(gkPS12) ,1,1,1    ,0.005498,0.056620,0.471547,0.079386,0.210272,0.035484,0.003858,0.001328,0.000308,0.000117,0.001906,0.011522,0.011745,0.013323,0.004796,0.001380,0.000371,0.000133
  giMixedAh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60), i(gkBW13_V2) , i(gkPS13) ,1,1,1    ,0.002967,0.046107,0.824869,0.198093,0.082301,0.006142,0.001955,0.000496,0.001429,0.017146,0.037895,0.004766,0.005960,0.003267,0.000407,0.000157,0.000155,0.000519
  giMixedAh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63), i(gkBW14_V2) , i(gkPS14) ,1,1,1    ,0.002677,0.127443,0.233422,0.181784,0.009013,0.001717,0.000284,0.007015,0.064115,0.007331,0.007952,0.002405,0.000267,0.000125,0.000600
  giMixedAh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66), i(gkBW15_V2) , i(gkPS15) ,1,1,1    ,0.010553,0.479368,0.511917,0.027219,0.002374,0.000432,0.019753,0.029194,0.005598,0.003593,0.000314,0.000115,0.000440,0.000188
  giMixedAh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69), i(gkBW16_V2) , i(gkPS16) ,1,1,1    ,0.033509,0.499985,0.127000,0.004247,0.000403,0.034562,0.013610,0.013599,0.000498,0.000136,0.000625,0.000114
  giMixedAh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72), i(gkBW17_V2) , i(gkPS17) ,1,1,1    ,0.097704,0.332922,0.011754,0.001452,0.002670,0.003963,0.017086,0.011243,0.000618,0.000448
  giMixedAh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75), i(gkBW18_V2) , i(gkPS18) ,1,1,1    ,0.266284,0.282922,0.003635,0.000202,0.006066,0.036263,0.001979,0.003494
  giMixedAh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78), i(gkBW19_V2) , i(gkPS19) ,1,1,1    ,0.976912,0.048417,0.000970,0.026534,0.032847,0.001372,0.000907
  giMixedAh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81), i(gkBW20_V2) , i(gkPS20) ,1,1,1    ,1.000001,0.008470,0.005716,0.006895,0.001809,0.000528
  giMixedAh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84), i(gkBW21_V2) , i(gkPS21) ,1,1,1    ,0.665845,0.002904,0.007927,0.022486,0.000897
 elseif iVoice2==27 then
  giMixedEh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.002549,0.003134,0.003974,0.005478,0.007017,0.009323,0.012997,0.015800,0.025880,0.031561,0.069262,0.116966,0.096839,0.041388,0.030133,0.019258,0.014690,0.010922,0.008058,0.006191,0.004473,0.003237,0.002267,0.001451,0.000894,0.000442,0.000126,0.000210,0.000263,0.000265,0.000240,0.000174,0.000100,0.000185,0.000338,0.000545,0.000796,0.001110,0.001516,0.002015,0.002652,0.003519,0.004603,0.006308,0.008742,0.013114,0.019649,0.020048,0.013747,0.009105,0.006633,0.004842,0.003722,0.002793,0.002088,0.001484,0.000943,0.000453,0.000147,0.000600,0.001155,0.001779,0.002508,0.003367,0.004414,0.005764,0.007472,0.010017,0.013566,0.019406,0.025684,0.025076,0.018739,0.013927,0.011049,0.009282,0.008375,0.008024,0.008229,0.009065,0.010646,0.013251,0.016228,0.016401,0.013297,0.010080,0.008006,0.006682,0.006049,0.005955,0.006476,0.007437,0.007523,0.006180,0.004613,0.003551,0.002779,0.002235
  giMixedEh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.003129,0.004092,0.005657,0.007860,0.010921,0.016087,0.021139,0.035868,0.062023,0.136227,0.109956,0.040456,0.031049,0.018374,0.013762,0.009515,0.006960,0.004597,0.003159,0.001921,0.001079,0.000444,0.000177,0.000312,0.000316,0.000280,0.000177,0.000171,0.000376,0.000664,0.001031,0.001514,0.002167,0.002999,0.004195,0.005777,0.008581,0.012777,0.021830,0.024145,0.015340,0.009845,0.006641,0.004815,0.003434,0.002428,0.001585,0.000853,0.000201,0.000618,0.001398,0.002299,0.003391,0.004727,0.006500,0.008859,0.012573,0.018214,0.027784,0.031145,0.022997,0.016195,0.012414,0.010478,0.009647,0.009763,0.010916,0.013349,0.017534,0.020106,0.016727,0.012049,0.009207,0.007617,0.007098,0.007517,0.008848,0.008867,0.006730,0.004810,0.003566,0.002737,0.002122,0.001644,0.001252,0.000929,0.000662,0.000447,0.000285,0.000180,0.000139,0.000140,0.000147,0.000144,0.000130,0.000107
  giMixedEh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.003867,0.005384,0.008278,0.011948,0.018781,0.027378,0.043927,0.124923,0.154756,0.052079,0.035175,0.020879,0.013635,0.009198,0.005837,0.003709,0.002010,0.000982,0.000220,0.000161,0.000370,0.000376,0.000299,0.000147,0.000155,0.000424,0.000832,0.001379,0.002152,0.003202,0.004758,0.006982,0.011177,0.019325,0.029701,0.020304,0.011682,0.007361,0.005041,0.003356,0.002114,0.001056,0.000187,0.001017,0.002186,0.003609,0.005412,0.007905,0.011471,0.017562,0.028740,0.037414,0.026954,0.018054,0.013544,0.011691,0.011440,0.012869,0.016457,0.022485,0.022347,0.015528,0.011112,0.008894,0.008412,0.009625,0.010828,0.008346,0.005596,0.003939,0.002900,0.002141,0.001562,0.001098,0.000729,0.000449,0.000257,0.000169,0.000165,0.000175,0.000167,0.000140,0.000106
  giMixedEh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.004867,0.007460,0.011945,0.019946,0.031209,0.054453,0.186212,0.117747,0.049349,0.025385,0.015675,0.009940,0.005668,0.003035,0.001342,0.000247,0.000261,0.000440,0.000408,0.000208,0.000179,0.000569,0.001196,0.002093,0.003439,0.005483,0.008701,0.015148,0.030996,0.029556,0.014724,0.008576,0.005437,0.003329,0.001707,0.000348,0.001213,0.002897,0.005055,0.008006,0.012446,0.020472,0.036633,0.042329,0.025547,0.017331,0.014092,0.013706,0.016241,0.022858,0.028248,0.019753,0.013157,0.010345,0.010349,0.012777,0.010845,0.006746,0.004465,0.003112,0.002165,0.001447,0.000902,0.000506,0.000263,0.000193,0.000207,0.000202,0.000165,0.000114
  giMixedEh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.008898,0.014845,0.025531,0.044722,0.088437,0.209987,0.109632,0.053027,0.029768,0.016881,0.008983,0.004150,0.001395,0.000463,0.000755,0.000691,0.000400,0.000255,0.000745,0.001668,0.003236,0.005773,0.010198,0.020143,0.039588,0.019923,0.010157,0.005810,0.003302,0.001689,0.000582,0.000359,0.001415,0.002957,0.005341,0.009195,0.016165,0.031454,0.042867,0.022527,0.012419,0.007380,0.004874,0.005036,0.007696,0.012757,0.022519,0.032658,0.021211,0.013339,0.010317,0.010331,0.013816,0.015314,0.008765,0.005134,0.003184,0.001938,0.001090,0.000535,0.000250,0.000207,0.000210,0.000173,0.000113
  giMixedEh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.011502,0.021549,0.042263,0.090322,0.245564,0.114253,0.051172,0.026245,0.012912,0.005428,0.001469,0.000641,0.000901,0.000604,0.000162,0.000315,0.001073,0.002614,0.005421,0.010640,0.023418,0.045836,0.018645,0.008963,0.004582,0.002130,0.000572,0.000762,0.002491,0.005301,0.010156,0.019894,0.044518,0.039884,0.018319,0.009661,0.005793,0.006382,0.011028,0.020816,0.038283,0.025968,0.015109,0.011870,0.013774,0.019307,0.011088,0.005870,0.003324,0.001790,0.000830,0.000329,0.000246,0.000244,0.000178
  giMixedEh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.015233,0.032741,0.074724,0.243797,0.144246,0.056112,0.025240,0.010252,0.002839,0.000700,0.001067,0.000580,0.000104,0.000883,0.002741,0.006585,0.014845,0.043733,0.031620,0.012214,0.005557,0.002198,0.000195,0.001893,0.005217,0.011479,0.025712,0.062141,0.031662,0.014093,0.007252,0.007795,0.015192,0.033589,0.039695,0.019565,0.014153,0.017708,0.020404,0.009087,0.004554,0.002215,0.000878,0.000317,0.000301,0.000241,0.000125
  giMixedEh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.020852,0.053258,0.153984,0.250056,0.069342,0.026968,0.008516,0.001168,0.001282,0.000765,0.000162,0.001475,0.005006,0.013261,0.041892,0.036286,0.012356,0.004686,0.001052,0.001663,0.006070,0.015487,0.041856,0.059403,0.020427,0.008986,0.009865,0.022609,0.054432,0.028468,0.017103,0.021683,0.020009,0.007941,0.003514,0.001272,0.000379,0.000357,0.000233,0.000104,0.000116
  giMixedEh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.014412,0.045494,0.181791,0.112588,0.033171,0.009544,0.001064,0.001135,0.000329,0.000220,0.001449,0.006000,0.019846,0.045292,0.010951,0.003439,0.000417,0.001563,0.004898,0.014483,0.022627,0.012262,0.019775,0.010955,0.019872,0.010229,0.003771,0.001279,0.000296,0.000245,0.000142
  giMixedEh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.021582,0.082278,0.299289,0.052488,0.012284,0.000944,0.001172,0.000108,0.000906,0.006303,0.026106,0.032757,0.007753,0.001434,0.001444,0.005988,0.023064,0.017739,0.020167,0.013678,0.023142,0.010173,0.003168,0.000704,0.000290,0.000184
  giMixedEh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.033385,0.181253,0.115431,0.021468,0.001260,0.001196,0.000294,0.003460,0.019374,0.049307,0.008286,0.000663,0.003599,0.016148,0.024802,0.024651,0.015477,0.027005,0.006039,0.001230,0.000344,0.000180,0.000106
  giMixedEh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.053000,0.500048,0.052999,0.003969,0.001543,0.000360,0.007742,0.067956,0.015859,0.001332,0.004887,0.030983,0.020466,0.021939,0.036267,0.006921,0.000920,0.000388,0.000110
  giMixedEh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.165294,0.197124,0.015337,0.003004,0.003167,0.040484,0.005206,0.000220,0.001029,0.011247,0.005236,0.003975,0.005800,0.000693,0.000124,0.000151,0.000577,0.000103
  giMixedEh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.343032,0.097209,0.002364,0.001736,0.039793,0.004573,0.000222,0.004544,0.011682,0.003924,0.005813,0.000396,0.000102,0.000125,0.000672
  giMixedEh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.999678,0.038087,0.002755,0.020021,0.006295,0.000187,0.012594,0.004197,0.013896,0.000677,0.000104,0.000433,0.000213
  giMixedEh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,1.000001,0.003910,0.006008,0.016207,0.000265,0.023574,0.004637,0.003870,0.000201,0.000631,0.000123
  giMixedEh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.223136,0.003631,0.001641,0.053621,0.043595,0.010450,0.007588,0.000208,0.000421,0.000286
  giMixedEh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.109531,0.000806,0.042151,0.016149,0.016640,0.005110,0.000133,0.001866
  giMixedEh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.034598,0.000509,0.025064,0.074296,0.007935,0.000130,0.000593
  giMixedEh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.005801,0.023872,0.089183,0.013066,0.000176,0.000337
  giMixedEh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.007260,0.107242,0.020899,0.000417,0.000572
 elseif iVoice2==28 then
  giMixedEee24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.010612,0.014495,0.019809,0.028477,0.041398,0.063942,0.114518,0.124942,0.078855,0.046228,0.033959,0.023394,0.018143,0.013375,0.010099,0.007577,0.005431,0.003901,0.002611,0.001632,0.000932,0.000386,0.000171,0.000289,0.000422,0.000452,0.000449,0.000398,0.000315,0.000237,0.000144,0.000104,0.000105,0.000153,0.000215,0.000290,0.000380,0.000487,0.000633,0.000816,0.001099,0.001507,0.002203,0.003126,0.003165,0.002277,0.001567,0.001153,0.000855,0.000658,0.000500,0.000378,0.000277,0.000186,0.000102,0.000154,0.000300,0.000490,0.000733,0.001033,0.001413,0.001885,0.002475,0.003256,0.004250,0.005714,0.007781,0.011083,0.015021,0.015148,0.011378,0.008165,0.006212,0.004895,0.004095,0.003568,0.003306,0.003268,0.003450,0.003938,0.004750,0.006020,0.007017,0.006500,0.005013,0.003837,0.003082,0.002641,0.002479,0.002562
  giMixedEee27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.013361,0.019632,0.028170,0.046001,0.073446,0.142620,0.135320,0.067419,0.045253,0.028671,0.021074,0.014727,0.010611,0.007252,0.004920,0.003071,0.001747,0.000807,0.000249,0.000303,0.000500,0.000540,0.000520,0.000422,0.000309,0.000180,0.000119,0.000127,0.000103,0.000168,0.000256,0.000366,0.000497,0.000673,0.000915,0.001288,0.001877,0.003013,0.003942,0.003063,0.001933,0.001339,0.000949,0.000692,0.000498,0.000349,0.000219,0.000102,0.000105,0.000283,0.000531,0.000867,0.001300,0.001875,0.002609,0.003613,0.004968,0.007064,0.010239,0.015759,0.018744,0.014260,0.009604,0.006959,0.005391,0.004464,0.003989,0.003908,0.004207,0.005028,0.006490,0.008241,0.007854,0.005766,0.004257,0.003368,0.002997,0.003039,0.003534,0.003719,0.002952,0.002107,0.001565,0.001195,0.000929,0.000719,0.000551,0.000410,0.000295,0.000202,0.000133
  giMixedEee30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.016942,0.026728,0.044993,0.073582,0.166083,0.150187,0.067471,0.042023,0.027730,0.018157,0.012270,0.007756,0.004771,0.002544,0.001124,0.000288,0.000418,0.000628,0.000639,0.000532,0.000370,0.000189,0.000133,0.000151,0.000118,0.000167,0.000288,0.000442,0.000634,0.000915,0.001325,0.002073,0.003575,0.004636,0.003020,0.001869,0.001215,0.000836,0.000565,0.000361,0.000186,0.000308,0.000658,0.001154,0.001829,0.002760,0.004060,0.005937,0.009069,0.014428,0.021876,0.018429,0.011479,0.007850,0.005939,0.004916,0.004612,0.004928,0.006092,0.008369,0.009952,0.007626,0.005224,0.003945,0.003510,0.003799,0.004447,0.003710,0.002491,0.001752,0.001285,0.000950,0.000696,0.000493,0.000332,0.000209,0.000125
  giMixedEee33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.021903,0.037076,0.070524,0.170727,0.184856,0.075853,0.042006,0.026533,0.016384,0.009694,0.005423,0.002512,0.000745,0.000367,0.000734,0.000760,0.000589,0.000358,0.000113,0.000157,0.000177,0.000123,0.000164,0.000331,0.000549,0.000839,0.001298,0.002144,0.003934,0.005481,0.003133,0.001834,0.001150,0.000725,0.000430,0.000185,0.000178,0.000585,0.001216,0.002131,0.003468,0.005492,0.008727,0.014780,0.025548,0.021159,0.012320,0.008120,0.006189,0.005514,0.005907,0.007722,0.011241,0.010580,0.006710,0.004738,0.004184,0.004875,0.005074,0.003298,0.002150,0.001485,0.001035,0.000705,0.000453,0.000265,0.000143
  giMixedEee36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.018039,0.029693,0.059894,0.185737,0.154314,0.055778,0.027787,0.014854,0.007935,0.003897,0.001432,0.000187,0.000561,0.000590,0.000401,0.000185,0.000117,0.000134,0.000109,0.000405,0.000942,0.001848,0.003332,0.005831,0.010609,0.022023,0.025121,0.011959,0.006463,0.003691,0.002042,0.001000,0.000346,0.000540,0.001193,0.002221,0.003859,0.006735,0.012934,0.018831,0.010505,0.006536,0.005174,0.005305,0.007021,0.011338,0.013114,0.007641,0.004738,0.003619,0.003554,0.002391,0.001498,0.001024,0.000703,0.000458,0.000268,0.000136
  giMixedEee39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.023171,0.040165,0.103046,0.249899,0.069009,0.030296,0.014964,0.006837,0.002317,0.000262,0.000693,0.000670,0.000375,0.000134,0.000157,0.000234,0.000809,0.001919,0.003902,0.007563,0.015739,0.033909,0.016874,0.008020,0.004127,0.001993,0.000762,0.000560,0.001472,0.003025,0.005784,0.011760,0.022871,0.012484,0.007312,0.006012,0.007155,0.011847,0.015854,0.008455,0.005033,0.004257,0.003348,0.001866,0.001182,0.000751,0.000433,0.000208
  giMixedEee42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.030585,0.075466,0.326397,0.089842,0.037631,0.016293,0.006004,0.001189,0.000623,0.000819,0.000421,0.000187,0.000122,0.000220,0.000997,0.002690,0.006079,0.013783,0.037543,0.020537,0.008608,0.003873,0.001483,0.000243,0.000604,0.001913,0.004382,0.009617,0.024507,0.016361,0.008552,0.007239,0.010299,0.019396,0.011363,0.006002,0.005105,0.003117,0.001699,0.001004,0.000533,0.000219,0.000103,0.000108
  giMixedEee45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.042426,0.110844,0.240398,0.052093,0.020251,0.006042,0.000491,0.000993,0.000619,0.000230,0.000590,0.002336,0.006356,0.016818,0.048133,0.016906,0.006376,0.002233,0.000321,0.000910,0.003109,0.007901,0.022561,0.020975,0.009763,0.009076,0.016934,0.017670,0.007601,0.006066,0.003146,0.001615,0.000823,0.000318,0.000124,0.000127
  giMixedEee48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.067500,0.499957,0.084236,0.026845,0.006246,0.000524,0.001260,0.000307,0.000263,0.000154,0.000423,0.001657,0.005126,0.020030,0.007311,0.002320,0.000560,0.000313,0.001681,0.005251,0.018793,0.008143,0.003428,0.003199,0.002838,0.002897,0.002582,0.000962,0.000359
  giMixedEee51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.114392,0.346891,0.051686,0.011497,0.000495,0.001393,0.000319,0.000278,0.001711,0.006671,0.024100,0.005070,0.001207,0.000395,0.002611,0.010148,0.016063,0.004678,0.003757,0.002818,0.004738,0.001749,0.000577,0.000121
  giMixedEee54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.184699,0.170704,0.027773,0.001277,0.001722,0.000135,0.000260,0.000954,0.005364,0.030213,0.005416,0.000846,0.000161,0.001816,0.009486,0.018223,0.004741,0.005826,0.003571,0.002699,0.000709,0.000111
  giMixedEee57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.375525,0.096426,0.009561,0.002227,0.000108,0.000215,0.002089,0.015287,0.010294,0.001315,0.000212,0.003544,0.026847,0.007999,0.006168,0.004412,0.002362,0.000444,0.000109
  giMixedEee60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.227192,0.099004,0.005244,0.001344,0.001073,0.017641,0.017350,0.001409,0.000721,0.009172,0.004060,0.000250,0.000806,0.006918,0.001617,0.000117,0.000126,0.000462
  giMixedEee63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.603091,0.048190,0.002514,0.000479,0.018496,0.015302,0.000391,0.003639,0.009291,0.000469,0.001181,0.008347,0.000679,0.000111,0.000537
  giMixedEee66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,1.000000,0.014894,0.000748,0.011456,0.020880,0.000335,0.010283,0.002934,0.000513,0.010980,0.000625,0.000358,0.000173
  giMixedEee69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.383427,0.002256,0.002169,0.058726,0.000416,0.019191,0.001023,0.003871,0.002038,0.000111,0.000515,0.000103
  giMixedEee72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.143330,0.001607,0.002011,0.288307,0.002276,0.014994,0.010998,0.007299,0.001333,0.000996
  giMixedEee75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.061863,0.001803,0.041574,0.018718,0.022519,0.023699,0.001207,0.009907,0.000135
  giMixedEee78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.008237,0.002981,0.275939,0.109772,0.021334,0.000832,0.002147
  giMixedEee81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.012164,0.019280,0.007395,0.003926,0.001069,0.001101
  giMixedEee84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.003212,0.576616,0.029989,0.014597,0.001991
 elseif iVoice2==29 then
  giMixedOh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.002656,0.003252,0.004105,0.005622,0.007160,0.009459,0.013108,0.015856,0.025894,0.031448,0.069195,0.117293,0.097983,0.042979,0.032065,0.021929,0.018078,0.015475,0.014066,0.014550,0.016289,0.021526,0.023582,0.017583,0.010850,0.007974,0.005821,0.004584,0.003615,0.002841,0.002234,0.001707,0.001268,0.000905,0.000600,0.000361,0.000186,0.000124,0.000163,0.000187,0.000186,0.000166,0.000139,0.000101,0.000100,0.000137,0.000189,0.000263,0.000361,0.000489,0.000654,0.000858,0.001117,0.001451,0.001869,0.002471,0.003247,0.004440,0.005393,0.004629,0.002917,0.002066,0.002532,0.003852,0.005391,0.005792,0.004814,0.003675,0.002842,0.002214,0.001763,0.001401,0.001091,0.000746,0.000345,0.000127,0.000165,0.000194,0.000194,0.000185,0.000171,0.000154,0.000137,0.000119,0.000101
  giMixedOh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.003258,0.004239,0.005824,0.008031,0.011084,0.016213,0.021193,0.035813,0.061897,0.136532,0.111390,0.042393,0.033786,0.022028,0.018961,0.016732,0.017591,0.021496,0.028305,0.022987,0.013061,0.009039,0.006327,0.004802,0.003617,0.002714,0.001972,0.001369,0.000897,0.000513,0.000245,0.000108,0.000159,0.000210,0.000226,0.000203,0.000166,0.000112,0.000138,0.000201,0.000297,0.000435,0.000622,0.000874,0.001199,0.001639,0.002216,0.003089,0.004305,0.006101,0.005978,0.003630,0.002459,0.003410,0.005563,0.006988,0.005953,0.004329,0.003190,0.002395,0.001829,0.001373,0.000891,0.000334,0.000160,0.000222,0.000234,0.000222,0.000202,0.000178,0.000153,0.000127,0.000102
  giMixedOh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.004023,0.005567,0.008482,0.012147,0.018935,0.027436,0.043784,0.124923,0.156060,0.054359,0.038530,0.025628,0.020731,0.020302,0.024164,0.033681,0.024398,0.013158,0.008555,0.006066,0.004329,0.003073,0.002076,0.001307,0.000716,0.000311,0.000128,0.000209,0.000264,0.000252,0.000205,0.000130,0.000103,0.000161,0.000252,0.000400,0.000621,0.000939,0.001374,0.001992,0.002862,0.004244,0.006522,0.007371,0.004231,0.002979,0.004965,0.007920,0.007484,0.005169,0.003596,0.002580,0.001862,0.001227,0.000441,0.000198,0.000273,0.000271,0.000248,0.000214,0.000179,0.000142,0.000109
  giMixedOh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.005059,0.007690,0.012187,0.020150,0.031282,0.054249,0.186526,0.120147,0.052781,0.030786,0.024379,0.024804,0.034918,0.035401,0.016807,0.010063,0.006707,0.004475,0.002912,0.001727,0.000868,0.000313,0.000184,0.000294,0.000312,0.000251,0.000145,0.000162,0.000275,0.000476,0.000802,0.001297,0.002024,0.003113,0.004960,0.008196,0.007816,0.003687,0.004946,0.009187,0.008701,0.005588,0.003662,0.002498,0.001599,0.000503,0.000264,0.000332,0.000310,0.000265,0.000215,0.000163,0.000117
  giMixedOh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.006425,0.011190,0.018900,0.031763,0.063063,0.234533,0.085077,0.042498,0.028723,0.024364,0.029248,0.052657,0.032828,0.016019,0.009256,0.005473,0.003020,0.001411,0.000472,0.000289,0.000385,0.000324,0.000169,0.000209,0.000292,0.000313,0.000363,0.000715,0.001488,0.002773,0.004783,0.007968,0.013450,0.024276,0.028174,0.012256,0.017548,0.027382,0.016871,0.007639,0.001429,0.003903,0.002935,0.001946,0.001251,0.000808,0.000539,0.000371,0.000260,0.000189,0.000148,0.000125,0.000105
  giMixedOh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.008376,0.015990,0.032397,0.071647,0.272662,0.083705,0.042399,0.029965,0.031725,0.060042,0.036024,0.015995,0.008610,0.004457,0.001931,0.000539,0.000377,0.000438,0.000278,0.000188,0.000329,0.000372,0.000455,0.001070,0.002432,0.004805,0.008844,0.016499,0.033376,0.023715,0.015683,0.032489,0.020044,0.007073,0.004006,0.004073,0.002568,0.001523,0.000908,0.000566,0.000367,0.000246,0.000180,0.000147,0.000118
  giMixedOh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.011034,0.024255,0.054958,0.249960,0.107468,0.049168,0.034461,0.048258,0.060049,0.020609,0.009752,0.004351,0.001387,0.000382,0.000531,0.000321,0.000134,0.000183,0.000390,0.000442,0.000696,0.002019,0.004800,0.010033,0.021149,0.044106,0.017250,0.033814,0.026431,0.007582,0.005537,0.004027,0.002216,0.001191,0.000676,0.000405,0.000256,0.000192,0.000151,0.000111
  giMixedOh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.015273,0.040290,0.108755,0.238831,0.059213,0.041067,0.079700,0.037954,0.014459,0.005681,0.001470,0.000525,0.000572,0.000143,0.000149,0.000276,0.000507,0.000599,0.001942,0.005592,0.013421,0.033213,0.035928,0.030674,0.033762,0.006591,0.006489,0.003618,0.001724,0.000862,0.000468,0.000280,0.000206,0.000147,0.000101
  giMixedOh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.010601,0.034815,0.190470,0.080405,0.033305,0.058053,0.024910,0.008255,0.002415,0.000380,0.000499,0.000157,0.000134,0.000544,0.001534,0.004269,0.007067,0.006352,0.013410,0.005008,0.002977,0.001313,0.000644,0.000282
  giMixedOh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.016431,0.063467,0.235366,0.043971,0.063965,0.024873,0.006978,0.001106,0.000633,0.000237,0.000165,0.000526,0.001871,0.006744,0.006491,0.013161,0.007320,0.003558,0.001368,0.000593,0.000178
  giMixedOh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.024710,0.140863,0.075949,0.054103,0.032225,0.006828,0.000565,0.000591,0.000130,0.000122,0.000130,0.001184,0.004810,0.008308,0.016195,0.006829,0.003189,0.000995,0.000290
  giMixedOh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.039150,0.500037,0.055380,0.060924,0.008930,0.000650,0.000410,0.000244,0.001590,0.009065,0.009812,0.013291,0.004577,0.001153,0.000235,0.000102
  giMixedOh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.086407,0.267040,0.196788,0.021563,0.001095,0.000450,0.000254,0.000578,0.012590,0.058595,0.005142,0.009927,0.005483,0.000502,0.000175,0.000149,0.000775
  giMixedOh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,0.153433,0.155941,0.058873,0.002749,0.000677,0.000168,0.000395,0.004570,0.084026,0.008181,0.014008,0.003788,0.000261,0.000109,0.000887
  giMixedOh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.332841,0.193621,0.018082,0.001777,0.000274,0.000623,0.014262,0.036353,0.007910,0.005829,0.000322,0.000104,0.000472,0.000204
  giMixedOh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,1.000000,0.136629,0.001978,0.000708,0.000666,0.025888,0.015536,0.029450,0.000535,0.000119,0.000710,0.000111
  giMixedOh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.528638,0.035156,0.000814,0.000113,0.012612,0.005146,0.027163,0.002875,0.000576,0.000345
  giMixedOh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.295700,0.004617,0.000416,0.004518,0.008174,0.059391,0.000819,0.003130
  giMixedOh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.324009,0.003159,0.000539,0.036354,0.052795,0.000895,0.000729
  giMixedOh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.212624,0.001375,0.025936,0.013708,0.001164,0.000397
  giMixedOh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.070310,0.000226,0.010293,0.005749,0.000690
 elseif iVoice2==30 then
  giMixedOoh24 ftgen 100+1,0,giTabLen,"padsynth",cpsmidinn(24),i(gkBW1_V2) , i(gkPS1)  ,1,1,1    ,0.004170,0.004767,0.005809,0.007125,0.009423,0.012338,0.016616,0.025281,0.036773,0.084223,0.097509,0.055644,0.026776,0.021850,0.013883,0.011537,0.008646,0.005282,0.002155,0.001654,0.001414,0.001239,0.001040,0.000878,0.000785,0.000712,0.000629,0.000575,0.000483,0.000406,0.000327,0.000237,0.000173,0.000104,0.000121,0.000160,0.000207,0.000263,0.000334,0.000421,0.000551,0.000726,0.001018,0.001329,0.001313,0.001049,0.000891,0.000885,0.000983,0.001199,0.001540,0.001977,0.002121,0.001784,0.001340,0.001034,0.000826,0.000705,0.000652,0.000680,0.000715,0.000646,0.000503,0.000390,0.000309,0.000249,0.000204,0.000167,0.000137,0.000111
  giMixedOoh27 ftgen 100+2,0,giTabLen,"padsynth",cpsmidinn(27),i(gkBW2_V2) , i(gkPS2)  ,1,1,1    ,0.005030,0.006191,0.007651,0.010372,0.014571,0.021434,0.030781,0.071907,0.119545,0.072776,0.031574,0.022556,0.015399,0.011394,0.007041,0.002518,0.001911,0.001634,0.001295,0.001096,0.000937,0.000834,0.000721,0.000630,0.000508,0.000397,0.000272,0.000181,0.000107,0.000153,0.000213,0.000285,0.000379,0.000500,0.000688,0.000967,0.001445,0.001618,0.001277,0.001057,0.001077,0.001285,0.001687,0.002301,0.002526,0.001994,0.001425,0.001068,0.000863,0.000780,0.000826,0.000841,0.000676,0.000495,0.000375,0.000290,0.000229,0.000180,0.000141,0.000108
  giMixedOoh30 ftgen 100+3,0,giTabLen,"padsynth",cpsmidinn(30),i(gkBW3_V2) , i(gkPS3)  ,1,1,1    ,0.006083,0.008003,0.010711,0.016374,0.024519,0.037449,0.112833,0.124998,0.041417,0.028771,0.018019,0.012343,0.005402,0.002386,0.001972,0.001531,0.001231,0.001066,0.000905,0.000784,0.000607,0.000449,0.000285,0.000153,0.000101,0.000109,0.000153,0.000230,0.000327,0.000460,0.000642,0.000947,0.001505,0.001937,0.001502,0.001242,0.001345,0.001774,0.002567,0.002998,0.002232,0.001518,0.001113,0.000935,0.000970,0.000984,0.000727,0.000511,0.000372,0.000279,0.000210,0.000157,0.000113
  giMixedOoh33 ftgen 100+4,0,giTabLen,"padsynth",cpsmidinn(33),i(gkBW4_V2) , i(gkPS4)  ,1,1,1    ,0.007502,0.010380,0.016096,0.024230,0.043922,0.147876,0.113247,0.043397,0.022792,0.015355,0.005760,0.002715,0.002152,0.001629,0.001298,0.001071,0.000882,0.000647,0.000405,0.000215,0.000125,0.000127,0.000114,0.000198,0.000315,0.000473,0.000703,0.001106,0.001912,0.002216,0.001551,0.001530,0.002044,0.003150,0.003426,0.002208,0.001462,0.001135,0.001161,0.001127,0.000745,0.000502,0.000355,0.000253,0.000178,0.000120
  giMixedOoh36 ftgen 100+5,0,giTabLen,"padsynth",cpsmidinn(36),i(gkBW5_V2) , i(gkPS5)  ,1,1,1    ,0.009514,0.014225,0.024646,0.050523,0.168409,0.111356,0.043820,0.023460,0.012509,0.004160,0.002642,0.001689,0.001315,0.001065,0.000757,0.000443,0.000192,0.000123,0.000137,0.000101,0.000120,0.000148,0.000186,0.000291,0.000545,0.001008,0.001755,0.002924,0.004809,0.008185,0.013559,0.007678,0.008486,0.018454,0.017597,0.010496,0.006842,0.005140,0.004787,0.005732,0.004640,0.002578,0.001610,0.001047,0.000655,0.000372,0.000189,0.000109,0.000102
  giMixedOoh39 ftgen 100+6,0,giTabLen,"padsynth",cpsmidinn(39),i(gkBW6_V2) , i(gkPS6)  ,1,1,1    ,0.012343,0.020687,0.042780,0.143593,0.145481,0.045253,0.024030,0.006681,0.003567,0.002127,0.001539,0.001172,0.000743,0.000337,0.000147,0.000158,0.000139,0.000180,0.000245,0.000466,0.000987,0.001939,0.003573,0.006484,0.012507,0.013031,0.008641,0.021946,0.018929,0.010423,0.006772,0.005654,0.006751,0.005079,0.002597,0.001537,0.000903,0.000473,0.000212,0.000124,0.000121
  giMixedOoh42 ftgen 100+7,0,giTabLen,"padsynth",cpsmidinn(42),i(gkBW7_V2) , i(gkPS7)  ,1,1,1    ,0.015973,0.032700,0.074843,0.250024,0.057702,0.026906,0.006281,0.003020,0.001964,0.001456,0.000842,0.000281,0.000120,0.000190,0.000125,0.000117,0.000172,0.000230,0.000397,0.000965,0.002195,0.004549,0.009401,0.019240,0.008562,0.023865,0.021772,0.010988,0.007172,0.007255,0.006908,0.003095,0.001662,0.000857,0.000363,0.000158,0.000146,0.000111
  giMixedOoh45 ftgen 100+8,0,giTabLen,"padsynth",cpsmidinn(45),i(gkBW8_V2) , i(gkPS8)  ,1,1,1    ,0.020715,0.048381,0.295623,0.086997,0.032947,0.006518,0.003184,0.001972,0.001207,0.000399,0.000165,0.000225,0.000166,0.000247,0.000405,0.001120,0.002956,0.006943,0.016973,0.012394,0.026234,0.023823,0.011169,0.008018,0.009723,0.004053,0.001912,0.000846,0.000280,0.000172,0.000144
  giMixedOoh48 ftgen 100+9,0,giTabLen,"padsynth",cpsmidinn(48),i(gkBW9_V2) , i(gkPS9)  ,1,1,1    ,0.020956,0.073214,0.300650,0.047374,0.011304,0.003365,0.002105,0.001056,0.000160,0.000271,0.000143,0.000105,0.000437,0.001635,0.004710,0.015016,0.005087,0.004883,0.002780,0.003191,0.006532,0.002329,0.000854,0.000249
  giMixedOoh51 ftgen 100+10,0,giTabLen,"padsynth",cpsmidinn(51),i(gkBW10_V2), i(gkPS10) ,1,1,1    ,0.030210,0.140577,0.104265,0.026214,0.004100,0.002408,0.000835,0.000254,0.000256,0.000123,0.000395,0.002001,0.007270,0.012518,0.005007,0.003616,0.003732,0.007195,0.001939,0.000551
  giMixedOoh54 ftgen 100+11,0,giTabLen,"padsynth",cpsmidinn(54),i(gkBW11_V2), i(gkPS11) ,1,1,1    ,0.046979,0.499997,0.053723,0.006460,0.002919,0.000773,0.000378,0.000118,0.000106,0.000152,0.001094,0.005381,0.019208,0.006051,0.003882,0.006400,0.003801,0.000886,0.000115
  giMixedOoh57 ftgen 100+12,0,giTabLen,"padsynth",cpsmidinn(57),i(gkBW12_V2), i(gkPS12) ,1,1,1    ,0.073889,0.165133,0.017058,0.003994,0.001052,0.000471,0.000177,0.001512,0.009857,0.009881,0.005879,0.006275,0.004329,0.000733,0.000125
  giMixedOoh60 ftgen 100+13,0,giTabLen,"padsynth",cpsmidinn(60),i(gkBW13_V2), i(gkPS13) ,1,1,1    ,0.319941,0.115082,0.065007,0.007044,0.001288,0.000283,0.000127,0.000673,0.008134,0.014095,0.001709,0.000289,0.000332
  giMixedOoh63 ftgen 100+14,0,giTabLen,"padsynth",cpsmidinn(63),i(gkBW14_V2), i(gkPS14) ,1,1,1    ,1.000001,0.125741,0.022569,0.001337,0.000279,0.000263,0.002205,0.023707,0.004085,0.000367,0.000387
  giMixedOoh66 ftgen 100+15,0,giTabLen,"padsynth",cpsmidinn(66),i(gkBW15_V2), i(gkPS15) ,1,1,1    ,0.712592,0.181596,0.004243,0.000318,0.000259,0.003910,0.023893,0.001033,0.000156,0.000248,0.000124
  giMixedOoh69 ftgen 100+16,0,giTabLen,"padsynth",cpsmidinn(69),i(gkBW16_V2), i(gkPS16) ,1,1,1    ,0.347452,0.048482,0.002113,0.000185,0.003869,0.019916,0.000554,0.000125,0.000363
  giMixedOoh72 ftgen 100+17,0,giTabLen,"padsynth",cpsmidinn(72),i(gkBW17_V2), i(gkPS17) ,1,1,1    ,0.209313,0.009909,0.000515,0.000104,0.013000,0.000503,0.005157,0.000630,0.000287,0.000189
  giMixedOoh75 ftgen 100+18,0,giTabLen,"padsynth",cpsmidinn(75),i(gkBW18_V2), i(gkPS18) ,1,1,1    ,0.188584,0.002188,0.000154,0.005909,0.000855,0.010025,0.000214,0.001217
  giMixedOoh78 ftgen 100+19,0,giTabLen,"padsynth",cpsmidinn(78),i(gkBW19_V2), i(gkPS19) ,1,1,1    ,0.222149,0.000533,0.000493,0.004776,0.009517,0.000194,0.000393
  giMixedOoh81 ftgen 100+20,0,giTabLen,"padsynth",cpsmidinn(81),i(gkBW20_V2), i(gkPS20) ,1,1,1    ,0.063129,0.000488,0.025748,0.002834,0.000268,0.000223
  giMixedOoh84 ftgen 100+21,0,giTabLen,"padsynth",cpsmidinn(84),i(gkBW21_V2), i(gkPS21) ,1,1,1    ,0.019817,0.000207,0.001007,0.001261,0.000379
 endif

 prints    "Ready!"

endin

i_ ftgen 999,0,4096,10,1



instr 3 ; triggered by MIDI
 icps cpsmidi
 ;icps   cpstmid  i(gkTuning) + 200

 gkcps = icps
 iAmp ampmidi 1
 if i(gkLegatoOnOff)==0 then ;polyphonic
   iNum    notnum
   event_i "i",4 + iNum*0.001,0,3600*24*365,icps,iAmp
   if release:k()==1 then
    turnoff2 4 + iNum*0.001,4,1
   endif
 elseif i(gkLegatoOnOff)==1 && active:i(4,0,1)==0 then
  event_i "i", 4, 0, -1, icps, iAmp
 endif
endin


instr    4    ; SOUNDING INSTRUMENT. TRIGGERED BY MIDI NOTES.
 kRelease  release
 if i(gkLegatoOnOff)==0 then ; poly mode
  kcps = p4
  inum ftom p4
 else                     ; mono mode
  if kRelease==0 then
   if i(gkLegType)==1 then
    kRamp     linseg  0,0.001,1
    inum      ftom   i(gkcps)
    kcps      lineto gkcps, gkLegTime*kRamp
   else
    kcps      portk gkcps, gkLegTime, i(gkcps)   
   endif
  endif
  if active:k(3)==0 then
   turnoff
  endif
 endif

 ivel    =        p5
; aAmpEnv    cossegr        0, i(gkAAtt),1, i(gkADec), i(gkASus), 36000, i(gkASus),i(gkARel), 0    ; SUSTAIN SEGMENT (36000) NECESSARY DUE TO cosseg BUG
 ; aAmpEnv     linsegr        0, i(gkAAtt),1, i(gkADec), i(gkASus),                  i(gkARel), 0    
 kAtt    cosseg   0, i(gkAAtt),  1,i(gkADec),   i(gkASus)
 kRel    transegr 0, i(gkAAtt),0,1,i(gkADec),0, i(gkASus), i(gkARel), -8, 0
 aAmpEnv =       a(kAtt * kRel)
 
    
 cngoto changed(gkVoice,gkVoice2)==1, NEW_VOICE
 reinit NEW_VOICE
 NEW_VOICE:
 iVoice     init    i(gkVoice)
 iVoice2    init    i(gkVoice2)

 iKGSize    =    3                     ; KEYGROUP SIZE
 iMinNum    =    24                    ; MINIMUM NOTE NUMBER FOR WHICH A WAVEFORM HAS BEEN CREATED
 iMaxNum    =    84                    ; MAXIMUM NOTE NUMBER FOR WHICH A WAVEFORM HAS BEEN CREATED
 iFirstFT   =    1                     ; FIRST FUNCTION TABLE IN THE SEQUENCE

 iBase1    table    inum+iKGSize, giNoteMap                                                                ; BASE NUMBER FOR LAYER 1 KEYGROUP
 ifn1      =        iFirstFT+((iBase1-iMinNum)/iKGSize)                                                    ; FUNCTION TABLE FOR LAYER 1 KEYGROUP
 iFade1    table    (inum+iKGSize)%(iKGSize*2), giFadeMap                                                  ; FADE VALUE FOR LAYER 1
 iBase2    table    inum, giNoteMap                                                                        ; BASE NUMBER FOR LAYER 2 KEYGROUP
 iBase2    limit    iBase2+iKGSize,iMinNum,iMaxNum                                                         ; BASE NUMBER FOR LAYER 2 OFFSET
 ifn2      limit    iFirstFT+((iBase2-iMinNum)/iKGSize),iFirstFT, iFirstFT+((iMaxNum-iMinNum)/iKGSize)     ; FUNCTION TABLE FOR LAYER 2 KEYGROUP
 iFade2    table    inum%(iKGSize*2), giFadeMap                                                            ; FADE VALUE FOR LAYER 2
 
 kporttime linseg   0, 0.001,0.1
 kBend     pchbend  0, 12
 kBend     portk    kBend, kporttime
 
 /* LAYER 1 */
 aL        poscil3  ivel*aAmpEnv*gkAGain*iFade1, (sr*kcps*semitone(kBend))/(giTabLen*cpsmidinn(iBase1)), ifn1
 aR        poscil3  ivel*aAmpEnv*gkAGain*iFade1, (sr*kcps*semitone(kBend))/(giTabLen*cpsmidinn(iBase1)), ifn1, i(gkMonoStereo)/2        ; OFFSET RIGHT CHANNEL PHASE TO CREATE A BROAD STEREO EFFECT


 /* LAYER 2 */
 aL2       poscil3  ivel*aAmpEnv*gkAGain*iFade2, (sr*kcps*semitone(kBend))/(giTabLen*cpsmidinn(iBase2)), ifn2
 aR2       poscil3  ivel*aAmpEnv*gkAGain*iFade2, (sr*kcps*semitone(kBend))/(giTabLen*cpsmidinn(iBase2)), ifn2, i(gkMonoStereo)/2        ; OFFSET RIGHT CHANNEL PHASE TO CREATE A BROAD STEREO EFFECT 

 aL        +=       aL2                                            ; MIX IN LAYER 2
 aR        +=       aR2

 ; VOICE 2
 ifn1b     =        101+((iBase1-iMinNum)/iKGSize)                                         ; FUNCTION TABLE FOR LAYER 1 KEYGROUP
 ifn2b     limit    101+((iBase2-iMinNum)/iKGSize),101, 101+((iMaxNum-iMinNum)/iKGSize)    ; FUNCTION TABLE FOR LAYER 2 KEYGROUP
 
 ; LAYER 1
 aLb       poscil3  ivel*aAmpEnv*gkAGain*iFade1, (sr*kcps)/(giTabLen*cpsmidinn(iBase1)), ifn1b
 aRb       poscil3  ivel*aAmpEnv*gkAGain*iFade1, (sr*kcps)/(giTabLen*cpsmidinn(iBase1)), ifn1b, i(gkMonoStereo)/2        ; OFFSET RIGHT CHANNEL PHASE TO CREATE A BROAD STEREO EFFECT
 
 ; LAYER 2
 aL2b      poscil3  ivel*aAmpEnv*gkAGain*iFade2, (sr*kcps)/(giTabLen*cpsmidinn(iBase2)), ifn2b
 aR2b      poscil3  ivel*aAmpEnv*gkAGain*iFade2, (sr*kcps)/(giTabLen*cpsmidinn(iBase2)), ifn2b, i(gkMonoStereo)/2        ; OFFSET RIGHT CHANNEL PHASE TO CREATE A BROAD STEREO EFFECT 

 aLb       +=       aL2b                                                ; MIX IN LAYER 2
 aRb       +=       aR2b
 
 aL        ntrpol   aL, aLb, gkVoiceMix                                 ; MIX IN LAYER 2
 aR        ntrpol   aR, aRb, gkVoiceMix

 /* FILTER */
 if i(gkFOnOff)==1 then                                                 ; IF FILTER SWITCH IS 'ON'
  if gkFiltLowCut>20 then
   aL      buthp    aL,gkFiltLowCut
   aR      buthp    aR,gkFiltLowCut
  endif
  aFiltL   resonr   aL,gkFiltCF,gkFiltCF*gkFiltBW,1
  aFiltR   resonr   aR,gkFiltCF,gkFiltCF*gkFiltBW,1
  aL       sum      aL,aFiltL*ampdb(gkFiltGain)
  aR       sum      aR,aFiltR*ampdb(gkFiltGain)
  if gkFiltHighCut<20000 then
   aL      butlp    aL,gkFiltHighCut
   aR      butlp    aR,gkFiltHighCut
  endif
 endif
 
 
 /* EXCITER */
 if gkExciterAmount>0 then
  aEL     exciter   aL, gkExciterFreq, 20000, 1, 10
  aER     exciter   aR, gkExciterFreq, 20000, 1, 10
  aL      +=        aEL*gkExciterAmount
  aR      +=        aER*gkExciterAmount
 endif
          outs      aL,aR

 /* SEND SOME SIGNAL TO REVERB */
 gaRvbL    +=    aL*gkRSend
 gaRvbR    +=    aR*gkRSend
 
endin



instr    99    ; REVERB (ALWAYS ON)
 aL,aR    reverbsc    gaRvbL,gaRvbR,gkRSize,gkR__CF
          outs        aL,aR
          clear       gaRvbL,gaRvbR
endin



instr 2000 ; meter and spectrogram

a1,a2 monitor

; meter
if   metro:k(10)==1 then
reinit REFRESH_METER
endif
REFRESH_METER:
kres init    0
kres limit   kres-0.001,0,1 
kres peak    a1                            
rireturn
chnset kres, "vMeter1"              
if release:k()==1 then
 chnset k(0), "vMeter1"              
endif

endin


</CsInstruments>

<CsScore>
i 1  0    [3600*24*7]    ; READ IN WIDGETS
i 2  0.01 [3600*24*7]    ; REINITIALISES THE "PADSYNTH" GEN ROUTINE WHEN CHANGES MADE TO WIDGETS PERTAINING TO ITS CREATION ARE MADE.
i 99 0    [3600*24*7]    ; REVERB
i 2000 0 [3600*24*7]  ; METER
</CsScore>

</CsoundSynthesizer>