
/* Attribution-NonCommercial-ShareAlike 4.0 International
Attribution - You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.
NonCommercial - You may not use the material for commercial purposes.
ShareAlike - If you remix, transform, or build upon the material, you must distribute your contributions under the same license as the original.
https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode */

; oscbnk_synth.csd
; Written by Iain McCurdy, 2012

; A synthesiser utilising the oscbnk opcode
; 
; CONTROLS
; --------
; 
; -Oscillators-
; Amp        --    output amplitude of the oscillators sections
; N.Oscs    --    number of oscillators
; Spread    --    pitch spread
; SPeed        --    speed of pitch modulations
; Waveform    --    (dropdown menu) wavefrom used by the oscillator. Trombone 2 is a multi-waveform wavetable instrument.
; mono        --    activate monophonic mode
; Time        --    portamento time (monophonic mode)
; T.Shift    --    wavetable shift (will alter the tone if a multi-waveform wavetable instrument is chosen)
; P.Bend    --    pitch bend. The is a GUI alternative to use if you don't have hardware pitch bend.
; Bend Rng.    --    pitch bend range
; 
; -Filter Envelope-
; Filter    --    filter cutoff global control
; Env.        --    amount of influnce of the envelope
; Att.        --    envelope attack time
; Dec.        --    envelope decay time
; Sus.        --    envelope sustain level
; Rel.        --    envelope release time
; 
; -Amplitude Envelope-
; Att        --    envelope attack time  
; Dec        --    envelope decay time   
; Sus        --    envelope sustain level
; Rel        --    envelope release time 
; 
; -Random Seed-
; Mode for setting random seed for random number generators in oscbnk:
; Clock M.    --    seeded from the system clock therefore each note (even if the same note is played) 
;            will produced the same 'random' values. Left and right channels use the same value.
; Clock St.    --    same as the above except that left and right channels will use different values.
; Note M.    --    seeded by the note played therefore repeating a note will produce the same 'random' values.
;            (Left and right channels use the same seed.) 
; Note St.    --    Same as the above except that left and right channels will use different values.
; Fixed    M.    --    All notes use a fixed seed all of the time. Left and right channels use different fixed values.
; Fixed    St.    --    Same as the above except that left and right channels will use different values.
;  N.B. You can most clearly hear the effects of different settings by setting 'Spread' to a large value and 
;    by setting speed to zero.
;
; -Filter-
; Layers        --    number of interations of tonex filter. Increasing this number will sharpen cutoff.
; 
; -Filter LFO-
; shape        --    (drop down menu) LFO shape
; Rate         --    LFO rate
; Depth        --    LFO depth / amplitude
; Delay        --    delay time before LFO has any effect
; Rise        --    time it take for LFO amplitude to rise from zero to 'Depth'
; 
; -Reverb-
; Mix        --    dry/wet mix
; Size        --    room size


<Cabbage>
form caption("Oscillator Bank Synth") size(595, 335), pluginId("oscb")

;OSCILLATOR
groupbox bounds(  0,  0,595, 85), text("Oscillators"), fontColour("white") plant("oscillators"){
rslider  bounds(  5, 25, 55, 55), text("Amp."),    channel("amp"),   range(0, 1.00, 0.4),                 trackerColour("white") colour(LightBlue)
rslider  bounds( 55, 25, 55, 55), text("N.Oscs."), channel("NOscs"), range(1, 100, 10, 1, 1),             trackerColour("white") colour(LightBlue)
rslider  bounds(105, 25, 55, 55), text("Spread"),  channel("fmd"),   range(0,10.00, 0.005,0.25,0.000001), trackerColour("white") colour(LightBlue)
rslider  bounds(155, 25, 55, 55), text("Speed"),   channel("mvt"),   range(0,1000.000, 1, 0.25,0.0001),   trackerColour("white") colour(LightBlue)
rslider  bounds(205, 25, 55, 55), text("Width"),   channel("width"), range(0, 1.000, 1, 1,0.001),         trackerColour("white") colour(LightBlue)
combobox bounds(270, 25, 80, 20), channel("waveform"), value(1), text("saw", "square", "organ", "Clarinet","Bass Clarinet", "C.Bass Clarinet","Oboe","Bassoon","C.Bassoon","Violin","Cello","Piccolo","Flute","Alto Flute","Bass Flute", "Ahh", "Ooh", "Eee", "Horn P", "Horn F", "B.Tbn.Harmon", "B.Tbn.Straight", "B.Tbn.Open")
checkbox bounds(270, 50, 85, 13), text("Legato"), colour("yellow"), channel("legato"),  value(0)
checkbox bounds(270, 65, 85, 13), text("Mono"), colour("yellow"), channel("mono"),  value(0)
rslider  bounds(322, 44, 38, 38), text("Time"),    channel("LegTim"),      range(0, 4.00, 0.09, 0.5, 0.01), trackerColour("white") colour(LightBlue)
rslider  bounds(355, 25, 55, 55), text("T.Shift"), channel("WTableShift"), range(-36, 36, 0, 1,0.001),      trackerColour("white") colour(LightBlue)
;PITCH BEND
rslider  bounds(405, 25, 55, 55), text("P.Bend"),    channel("PBend"),    range(-1,1, 0), trackerColour("white") colour(LightBlue)
rslider  bounds(455, 25, 55, 55), text("Bend Rng."), channel("BendRange"),   range(1, 24, 12, 1,1),   trackerColour("white") colour(LightBlue)
label    bounds(505, 31, 80, 12), text("Random Seed")
combobox bounds(505, 45, 80, 20), text("Clock M.","Clock St.","Note M.","Note St.","Fixed M","Fixed St"), value(2), channel("seed")
}

;FILTER ENVELOPE
groupbox bounds(  0, 85,380, 85), text("Filter"), fontColour("white"), plant("FilterEnvelope") {
rslider  bounds(  5, 25, 55, 55), text("Manual"), channel("cf"), range(0, 10.00, 8),     trackerColour("white") colour(LightBlue)
rslider  bounds( 55, 25, 55, 55), text("Env."), channel("FEnvAmt"), range(0, 10.00, 1.16), trackerColour("white") colour(LightBlue)
rslider  bounds(105, 25, 55, 55), text("Att."), channel("FAtt"), range(0, 8.00, 0.1,0.5),  trackerColour("white") colour(LightBlue)
rslider  bounds(155, 25, 55, 55), text("Dec."), channel("FDec"), range(0, 8.00, 1,0.5),    trackerColour("white") colour(LightBlue)
rslider  bounds(205, 25, 55, 55), text("Sus."), channel("FSus"), range(0, 1.00, 4, 0.5),   trackerColour("white") colour(LightBlue)
rslider  bounds(255, 25, 55, 55), text("Rel."), channel("FRel"), range(0.01, 8, 1.3, 0.5), trackerColour("white") colour(LightBlue)
line     bounds(315, 30,  1, 45)
rslider  bounds(320, 25, 55, 55), text("Layers"), channel("FiltLayers"), range(1, 10, 8,1,1), trackerColour("white") colour(LightBlue)
}

;AMPLITUDE ENVELOPE
groupbox bounds(380,85, 215, 85), text("Amplitude Envelope"), plant("AmpEnv"), fontColour("white"){
rslider bounds(  5, 25, 55, 55), text("Att."), channel("AAtt"), range(0, 8.00, 0.13,0.5),  trackerColour("white") colour(LightBlue)
rslider bounds( 55, 25, 55, 55), text("Dec."), channel("ADec"), range(0, 8.00, 0.01,0.5),  trackerColour("white") colour(LightBlue)
rslider bounds(105, 25, 55, 55), text("Sus."), channel("ASus"), range(0, 1.00, 1.00, 0.5), trackerColour("white") colour(LightBlue)
rslider bounds(155, 25, 55, 55), text("Rel."), channel("ARel"), range(0.01, 8, 1.67, 0.5), trackerColour("white") colour(LightBlue)
}

;FILTER LFO
groupbox bounds(  0,170,310, 85), text("Filter LFO"), plant("FilterLFO"), fontColour("white"){
label    bounds( 15, 31, 80, 13), text("Shape")
combobox bounds( 15, 45, 80, 20), channel("FlfoType"), value(2), text("sine", "splines", "S+H", "square")
rslider  bounds(100, 25, 55, 55), text("Rate"),  channel("FRte"), range(0, 16.00, 4),       trackerColour("white") colour(LightBlue)
rslider  bounds(150, 25, 55, 55), text("Depth"), channel("FDep"), range(0, 4.00, 0),        trackerColour("white") colour(LightBlue)
rslider  bounds(200, 25, 55, 55), text("Delay"), channel("FDel"), range(0, 2.00, 0, 0.5),   trackerColour("white") colour(LightBlue)
rslider  bounds(250, 25, 55, 55), text("Rise"),  channel("FRis"), range(0, 2.00, 0.1, 0.5), trackerColour("white") colour(LightBlue)
}

;REVERB
groupbox bounds(310,170,165, 85), text("Reverb"), plant("Reverb"), fontColour("white"){
rslider  bounds(  5, 25, 55, 55), fontColour("white"), text("Send"),   channel("RvbMix"),  range(0, 1.00, 0.5),  trackerColour("white") colour(LightBlue)
rslider  bounds( 55, 25, 55, 55), fontColour("white"), text("Size"),   channel("RvbSize"), range(0, 1.00, 0.77), trackerColour("white") colour(LightBlue)
rslider  bounds(105, 25, 55, 55), fontColour("white"), text("Cutoff"), channel("RvbCF"),   range(50, 20000, 7000,0.5,1), trackerColour("white") colour(LightBlue)
}

;EXCITER
groupbox bounds(475,170,120, 85), text("Exciter"), plant("Exciter"), fontColour("white") , identChannel("ExciterID"){
rslider  bounds(  5, 25, 55, 55), text("Amount"),    channel("ExciterAmount"),   range(0, 100, 0,1,0.1),  trackerColour("white") colour(LightBlue)
rslider  bounds( 55, 25, 55, 55), text("Freq."),    channel("ExciterFreq"),   range(1000, 10000, 3000,1,1),  trackerColour("white") colour(LightBlue)
}

keyboard bounds(0, 255, 595, 80)

</Cabbage>

<CsoundSynthesizer>

<CsOptions>
-dm0 -n -+rtmidi=NULL -M0
</CsOptions>

<CsInstruments>

; sr set by host
ksmps    =    64
nchnls    =    2
0dbfs    =    1
massign    0,1

;Author: Iain McCurdy (2012)

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;  CLARINET
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;FOUR GEN10 FUNCTION TABLES WHICH CORRESPOND TO THE SPECTRAL ENVELOPES OF FOUR DIFFERENT NOTES PLAYED ON THE CLARINET
;THE FOUR NOTES CHOSEN COVER THE SOUNDING RANGE OF THE INSTRUMENT
;NOTE THAT ALL FUNCTION TABLES USED BY ftmorph IN THE MORPHING PROCESS, INCLUDING THE ONE INTO WHICH THE MORPHED TABLE IS WRITTEN (SEE INSTR 1), SHOULD BE THE SAME SIZE
;FOUR GEN10 FUNCTION TABLES WHICH CORRESPOND TO THE SPECTRAL ENVELOPES OF FOUR DIFFERENT NOTES PLAYED ON THE CLARINET
;THE FOUR NOTES CHOSEN COVER THE SOUNDING RANGE OF THE INSTRUMENT
;NOTE THAT ALL FUNCTION TABLES USED BY ftmorph IN THE MORPHING PROCESS, INCLUDING THE ONE INTO WHICH THE MORPHED TABLE IS WRITTEN (SEE INSTR 1), SHOULD BE THE SAME SIZE
giclar50 ftgen 0,0,4096,10,0.273597,0.027434,0.737705,0.049480,0.448437,0.272536,0.131175,0.129945,0.283082,0.130714,0.026719,0.037582,0.018953,0.010729,0.067580,0.024573,0.016586,0.049988,0.033294,0.017090,0.008591,0.021128,0.007237,0.016060,0.016060
giclar62 ftgen 0,0,4096,10,0.753504,0.011446,0.501720,0.055791,0.241768,0.037889,0.027203,0.062793,0.102494,0.030506,0.042453,0.050202,0.031077,0.027208,0.020897,0.022205,0.012154,0.008160,0.004886,0.005076,0.003805,0.001322,0.001149,0.001450,0.001450
giclar74 ftgen 0,0,4096,10,0.519536,0.062430,0.745521,0.055790,0.103856,0.097554,0.090420,0.077181,0.017130,0.013446,0.005674,0.003296,0.003438,0.003395,0.002506,0.001169,0.002525,0.001237,0.002624,0.001440,0.000880,0.001061,0.002672,0.002090,0.002090
giclar86 ftgen 0,0,4096,10,0.823209,0.201690,0.171118,0.161374,0.041951,0.004950,0.010889,0.001107,0.001365,0.003340,0.002259,0.001175,0.000442,0.001544,0.001259,0.000549,0.000455,0.000352,0.000345,0.000396,0.000477,0.000848,0.001199,0.001081,0.001081
giwavemapClar    ftgen    0,0,128,-27, 0,0, 50,0,  62,1,  74,2,  86,3,   127,3
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfClar    ftgen    0,0,4,-2,giclar50,giclar62,giclar74,giclar86

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;  BASS CLARINET
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giBClar35 ftgen 0,0,4096,10,0.384232,0.038085,0.578537,0.029558,0.143002,0.119033,0.397678,0.113380,0.577246,0.158831,0.577514,0.094994,0.440674,0.109136,0.500666,0.132354,0.360370,0.104810,0.231403,0.089565,0.207353,0.099773,0.209066,0.123801,0.158769,0.079383,0.036078,0.019583,0.010310,0.017060,0.029465,0.045821,0.031622,0.038326,0.052222,0.058647,0.083956,0.079748,0.081955,0.097274,0.069934,0.075100,0.049259,0.058121,0.068078,0.065276,0.070165,0.065898,0.072432,0.055423,0.052283,0.036547,0.034082,0.035287,0.044801,0.053917,0.050263,0.036979,0.034264,0.035892,0.035011,0.037199,0.041542,0.043201,0.039923,0.035164,0.035828,0.036193,0.037155,0.035493,0.034546,0.035091,0.029891,0.027394,0.026174,0.023757,0.021365,0.019468,0.016295,0.015301,0.015263,0.014310,0.013239,0.011972,0.011445,0.011727,0.012391,0.013892,0.015395,0.015147,0.015137,0.014816,0.013898,0.012682,0.011462,0.009883,0.008579,0.007797,0.007749
giBClar41 ftgen 0,0,4096,10,0.525063,0.002869,0.469239,0.024384,0.895713,0.021156,0.199553,0.014871,0.215693,0.036697,0.212856,0.060220,0.224360,0.026202,0.021469,0.019475,0.067282,0.034970,0.025525,0.028958,0.038550,0.041724,0.035988,0.016617,0.026564,0.031021,0.037801,0.030070,0.055636,0.040575,0.029792,0.031356,0.031871,0.058063,0.027829,0.023825,0.015824,0.011714,0.011110,0.015367,0.011850,0.008957,0.011489,0.007029,0.004640,0.004862,0.006536,0.006771,0.006683,0.005231,0.005323,0.006834,0.005342,0.005818,0.004902,0.005468,0.005431,0.006151,0.004672,0.003041,0.002322,0.002274,0.002661,0.003122,0.002931,0.002587,0.002527,0.002499,0.003095,0.003755,0.003651,0.003768,0.003972,0.003962,0.003999,0.003352,0.002866,0.002449,0.002120,0.001972,0.002040,0.002221,0.002063,0.002057,0.002415,0.002849,0.003276,0.003776,0.003994,0.003829,0.003419,0.003328,0.003575,0.003829,0.003703,0.003454,0.003149,0.002825,0.002697,0.002765
giBClar47 ftgen 0,0,4096,10,0.447815,0.011780,0.911495,0.027076,0.846723,0.050517,0.224579,0.039801,0.048967,0.024162,0.071418,0.087315,0.031778,0.048063,0.058394,0.078770,0.021135,0.041585,0.092206,0.058125,0.063991,0.033673,0.025813,0.045142,0.037450,0.016766,0.014278,0.016668,0.017869,0.025925,0.010244,0.010062,0.010634,0.008986,0.006992,0.005186,0.004618,0.003658,0.002913,0.003004,0.004250,0.003142,0.002387,0.002692,0.003592,0.004496,0.002831,0.001838,0.001680,0.002144,0.002567,0.003185,0.002727,0.002419,0.001998,0.001883,0.002286,0.002626,0.003202,0.003257,0.003056,0.003241,0.002838,0.002521,0.002294,0.002409,0.002746,0.002848,0.003387,0.004173,0.004189,0.004587,0.005402,0.004977,0.004132,0.003842,0.003974,0.003650,0.003605,0.003810,0.003745,0.003770,0.003520,0.003363,0.003391,0.003302,0.003124,0.003422,0.004022,0.004064,0.003584,0.002939,0.002329,0.001970,0.001947,0.002034,0.002069,0.002043,0.001854,0.001658
giBClar53 ftgen 0,0,4096,10,0.819856,0.006052,0.508263,0.007439,0.271031,0.012967,0.059374,0.017922,0.016997,0.004014,0.027132,0.014921,0.020230,0.012123,0.021066,0.004875,0.016894,0.004428,0.004157,0.007603,0.004211,0.011103,0.005054,0.007329,0.004043,0.002169,0.003183,0.001822,0.001158,0.001758,0.002350,0.001442,0.002218,0.002815,0.001503,0.001764,0.001171,0.001508,0.000934,0.000714,0.000902,0.000884,0.000943,0.000825,0.000579,0.000572,0.000654,0.000752,0.000902,0.000932,0.000868,0.000909,0.000903,0.001065,0.001472,0.001483,0.001803,0.001941,0.002100,0.002756,0.002543,0.001984,0.001589,0.001342,0.001414,0.001219,0.001004,0.000908,0.000887,0.001102,0.001342,0.001312,0.001089,0.000869,0.000733,0.000620,0.000574,0.000518,0.000451,0.000422,0.000446,0.000473,0.000423,0.000354,0.000335,0.000370,0.000410,0.000418,0.000435,0.000434,0.000417,0.000417,0.000388,0.000342,0.000316,0.000303,0.000306,0.000309,0.000292,0.000267
giBClar59 ftgen 0,0,4096,10,0.866645,0.024133,0.420624,0.171074,0.353043,0.050546,0.198619,0.047824,0.110458,0.081558,0.064116,0.041109,0.026267,0.036570,0.013031,0.023724,0.007023,0.010067,0.004550,0.011618,0.009601,0.005731,0.005607,0.005036,0.002347,0.002907,0.001472,0.001519,0.001304,0.002584,0.001383,0.001362,0.001591,0.002655,0.004442,0.004905,0.003148,0.004830,0.002947,0.002926,0.003063,0.002394,0.002276,0.002019,0.001689,0.002028,0.002368,0.003510,0.004637,0.004228,0.003257,0.002700,0.002231,0.001681,0.001293,0.001062,0.001121,0.000894,0.000797,0.000865,0.000752,0.000724,0.000719,0.000787,0.000896,0.000801,0.000686,0.000551,0.000468,0.000466,0.000515,0.000460,0.000382,0.000358,0.000366,0.000400,0.000405,0.000355,0.000333,0.000349,0.000352,0.000346,0.000335,0.000317,0.000303,0.000297,0.000289,0.000276,0.000290,0.000292,0.000287,0.000307,0.000318,0.000332,0.000353,0.000374,0.000388,0.000398,0.000399,0.000407
giBClar65 ftgen 0,0,4096,10,0.934141,0.012361,0.023358,0.045609,0.025913,0.017772,0.009955,0.010837,0.007253,0.001002,0.004215,0.001570,0.004226,0.001334,0.000580,0.001740,0.000839,0.001364,0.001337,0.000523,0.000377,0.000535,0.000290,0.000461,0.000486,0.000333,0.000462,0.000685,0.001136,0.000794,0.000625,0.000436,0.000303,0.000270,0.000259,0.000289,0.000208,0.000224,0.000249,0.000271,0.000200,0.000240,0.000298,0.000226,0.000198,0.000188,0.000200,0.000259,0.000198,0.000169,0.000155,0.000144,0.000123,0.000120,0.000118,0.000112,0.000108,0.000105,0.000096,0.000091,0.000090,0.000093,0.000089,0.000093,0.000094,0.000095,0.000101,0.000112,0.000120,0.000126,0.000134,0.000142,0.000149,0.000170,0.000190,0.000212,0.000246,0.000289,0.000286,0.000269,0.000279,0.000310,0.000352,0.000355,0.000341,0.000361,0.000376,0.000373,0.000385,0.000427,0.000475,0.000539,0.000655,0.000838,0.001061,0.001284,0.001429,0.001364,0.001171,0.001020
giBClar71 ftgen 0,0,4096,10,0.951349,0.052635,0.166289,0.036346,0.080225,0.105328,0.019032,0.014538,0.003453,0.003349,0.001065,0.003529,0.004876,0.003334,0.003537,0.009903,0.004407,0.002506,0.002863,0.005317,0.004116,0.003426,0.001538,0.001829,0.001351,0.001942,0.001728,0.001103,0.000733,0.000413,0.000643,0.000468,0.000336,0.000631,0.000868,0.000470,0.000257,0.000240,0.000210,0.000190,0.000182,0.000170,0.000178,0.000178
giBClar77 ftgen 0,0,4096,10,0.679870,0.126072,0.031529,0.103885,0.017231,0.001781,0.005132,0.000787,0.003992,0.001819,0.002500,0.002660,0.001298,0.001834,0.001238,0.004273,0.001518,0.000670,0.000588,0.000395,0.000786,0.000400,0.000399,0.000471,0.000235,0.000212,0.000208,0.000169,0.000163,0.000146,0.000134
giwavemapBClar    ftgen    0,0,128,-27, 0,0, 35,0, 41,1, 47,2, 53,3,  59,4,  65,5,  71,6,  77,7, 127,7
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfBClar    ftgen    0,0,8,-2,giBClar35,giBClar41,giBClar47,giBClar53,giBClar59,giBClar65,giBClar71,giBClar77

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;  CONTRA-BASS CLARINET
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giCBClar26 ftgen 0,0,4096,10,0.100160,0.005519,0.631940,0.013046,0.749042,0.308868,0.475605,0.152792,0.074315,0.238927,0.152260,0.251283,0.030787,0.052465,0.032473,0.121972,0.064172,0.090564,0.043994,0.091868,0.039563,0.058622,0.024531,0.023127,0.026665,0.067522,0.081377,0.057914,0.066176,0.036134,0.026135,0.021056,0.038011,0.036534,0.058393,0.040915,0.050051,0.038446,0.034166,0.021341,0.014481,0.015708,0.025527,0.026622,0.033577,0.027355,0.034434,0.022920,0.016354,0.010905,0.011160,0.015075,0.019871,0.017505,0.013189,0.011442,0.008511,0.007974,0.006368,0.005988,0.005976,0.005922,0.006590,0.008199,0.006566,0.005254,0.004955,0.005576,0.005463,0.005101,0.003955,0.003622,0.004027,0.003772,0.003504,0.002848,0.002183,0.002075,0.002143,0.002014,0.001907,0.001850,0.001736,0.001543,0.001318,0.001180,0.001107,0.001066,0.001169,0.001372,0.001533,0.001667,0.001738,0.001655,0.001604,0.001603,0.001571,0.001575,0.001638,0.001696
giCBClar32 ftgen 0,0,4096,10,0.227476,0.006718,0.707706,0.018101,0.539432,0.018848,0.590242,0.029143,0.512926,0.035712,0.324304,0.035306,0.035964,0.050992,0.096916,0.025035,0.098203,0.072427,0.068155,0.060324,0.042818,0.074538,0.049430,0.100999,0.052956,0.109337,0.053778,0.108791,0.040535,0.041998,0.050107,0.102568,0.075823,0.121836,0.062713,0.047451,0.042146,0.061836,0.063525,0.079775,0.056177,0.039271,0.021422,0.017934,0.022472,0.016758,0.023407,0.025034,0.018765,0.012566,0.009566,0.008232,0.011234,0.015948,0.018924,0.020397,0.016720,0.016224,0.010210,0.006760,0.006525,0.006464,0.006410,0.007832,0.010929,0.013172,0.010412,0.008153,0.005887,0.004927,0.005543,0.006358,0.006415,0.006246,0.004977,0.004037,0.003084,0.002646,0.002504,0.002716,0.003253,0.003484,0.003416,0.003566,0.003392,0.002924,0.002750,0.002979,0.003398,0.003635,0.003855,0.004032,0.004285,0.004278,0.003688,0.003182,0.002942,0.003043,0.003240,0.003317
giCBClar38 ftgen 0,0,4096,10,0.805943,0.003127,0.607438,0.006915,0.078379,0.025071,0.319458,0.040967,0.418440,0.096874,0.067519,0.066830,0.181701,0.148461,0.024736,0.047734,0.027229,0.065092,0.020252,0.015890,0.047795,0.096969,0.049469,0.024097,0.032227,0.032247,0.057763,0.064995,0.031760,0.023574,0.021449,0.017946,0.015989,0.010329,0.017096,0.025053,0.020021,0.014542,0.009659,0.010282,0.011754,0.016404,0.013970,0.019292,0.012226,0.007260,0.007191,0.010064,0.007246,0.007400,0.007562,0.005179,0.005520,0.005811,0.004710,0.005570,0.007523,0.007671,0.005694,0.004881,0.005241,0.004991,0.005944,0.006659,0.006213,0.004986,0.004006,0.003617,0.003290,0.003479,0.004050,0.003911,0.003373,0.003294,0.003284,0.002858,0.002674,0.003290,0.003986,0.003982,0.003503,0.003424,0.003640,0.003903,0.004267,0.004774,0.005048,0.005020,0.005284,0.005837,0.006116,0.006124,0.006474,0.007329,0.008198,0.008721,0.009014,0.008920,0.008202,0.007419
giCBClar44 ftgen 0,0,4096,10,0.715187,0.010393,0.137973,0.010188,0.404486,0.073659,0.420943,0.145565,0.080881,0.104506,0.227785,0.021310,0.035316,0.052724,0.023335,0.014334,0.087911,0.036461,0.047658,0.016909,0.047453,0.017513,0.031591,0.010028,0.009677,0.014384,0.012660,0.031094,0.010732,0.009677,0.005632,0.007197,0.005202,0.005881,0.004438,0.005851,0.003283,0.002870,0.001732,0.001824,0.003100,0.002099,0.002084,0.002567,0.003519,0.003856,0.003017,0.003288,0.003518,0.003481,0.002717,0.002212,0.002009,0.002374,0.002508,0.003004,0.002356,0.001681,0.001376,0.001435,0.001604,0.001881,0.001924,0.001783,0.002097,0.002582,0.002971,0.002922,0.002735,0.002884,0.003498,0.003726,0.003454,0.003486,0.003620,0.003356,0.003457,0.003775,0.003730,0.003481,0.003126,0.002660,0.002244,0.001883,0.001625,0.001518,0.001533,0.001594,0.001565,0.001529,0.001485,0.001598,0.001965,0.002295,0.002394,0.002424,0.002401,0.002320,0.002329,0.002436
giCBClar50 ftgen 0,0,4096,10,0.880714,0.104962,0.497883,0.045337,0.172861,0.109234,0.166844,0.037601,0.179775,0.048578,0.136813,0.079191,0.068937,0.019908,0.087293,0.031265,0.010402,0.048393,0.020759,0.005769,0.015444,0.004339,0.006113,0.010598,0.009591,0.007441,0.002840,0.003349,0.003193,0.003806,0.004954,0.003108,0.003744,0.003254,0.003682,0.002736,0.002830,0.004080,0.005132,0.004023,0.004801,0.004386,0.003962,0.002657,0.003137,0.003478,0.004208,0.004600,0.004272,0.004501,0.004804,0.004328,0.003739,0.003495,0.003378,0.002513,0.002268,0.002182,0.002140,0.001833,0.001900,0.001666,0.001525,0.001441,0.001541,0.001518,0.001571,0.001664,0.001746,0.001600,0.001474,0.001527,0.001644,0.001577,0.001450,0.001382,0.001479,0.001646,0.001713,0.001652,0.001720,0.001859,0.001891,0.001654,0.001420,0.001306,0.001228,0.001173,0.001188,0.001247,0.001299,0.001317,0.001264,0.001186,0.001091,0.000947,0.000797,0.000685,0.000610,0.000547
giCBClar54 ftgen 0,0,4096,10,0.889990,0.001348,0.087695,0.023926,0.066262,0.023447,0.075142,0.026596,0.020409,0.052699,0.027246,0.016811,0.017828,0.004566,0.005199,0.010830,0.003227,0.005838,0.001883,0.001580,0.004878,0.001870,0.000924,0.001648,0.002681,0.001616,0.003610,0.002587,0.002221,0.002332,0.001110,0.001616,0.002062,0.001241,0.000899,0.001174,0.001196,0.001105,0.001254,0.001253,0.001237,0.001093,0.001281,0.001260,0.001266,0.001255,0.001194,0.000995,0.001213,0.000908,0.000857,0.000902,0.000863,0.001171,0.001146,0.001074,0.000908,0.000908,0.000752,0.000618,0.000456,0.000395,0.000445,0.000503,0.000434,0.000353,0.000347,0.000373,0.000416,0.000392,0.000334,0.000271,0.000275,0.000314,0.000314,0.000317,0.000280,0.000275,0.000289,0.000253,0.000221,0.000203,0.000194,0.000194,0.000208,0.000221,0.000212,0.000185,0.000172,0.000176,0.000173,0.000178,0.000175,0.000159,0.000152,0.000150,0.000135,0.000120,0.000115,0.000119
giwavemapCBClar    ftgen    0,0,128,-27, 0,0, 26,0, 32,1, 38,2, 44,3,  50,4,  54,5,  127,5
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfCBClar    ftgen    0,0,8,-2,giCBClar26,giCBClar32,giCBClar38,giCBClar44,giCBClar50,giCBClar54,giCBClar54,giCBClar54

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;  OBOE
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
gioboe59 ftgen 0,0,4096,10,0.239013,0.078927,0.241030,0.206347,0.832266,0.054540,0.013821,0.007450,0.022905,0.021737,0.018123,0.013105,0.002361,0.001433,0.003509,0.002589,0.001326,0.000743,0.000990,0.000868,0.000863,0.000994,0.000406,0.000288,0.000288
gioboe65 ftgen 0,0,4096,10,0.199317,0.508088,0.859044,0.346880,0.121336,0.005269,0.071954,0.035188,0.012195,0.010362,0.001516,0.002397,0.004325,0.002943,0.008168,0.003519,0.002074,0.001366,0.001019,0.000715,0.000788,0.000757,0.001403,0.001108,0.001108
gioboe73 ftgen 0,0,4096,10,0.297865,0.916220,0.218345,0.030903,0.016039,0.045984,0.003571,0.006255,0.007310,0.003701,0.001306,0.001355,0.000361,0.001108,0.001552,0.000991,0.000588,0.000780,0.000678,0.001103,0.000615,0.000264,0.000310,0.000464,0.000464
gioboe81 ftgen 0,0,4096,10,0.875554,0.822621,0.094665,0.308960,0.046707,0.031827,0.002702,0.002222,0.009030,0.009257,0.007814,0.000781,0.002099,0.001154,0.002638,0.000756,0.000750,0.001290,0.000476,0.001285,0.000627,0.000379,0.000594,0.000379,0.000379
gioboe89 ftgen 0,0,4096,10,0.965505,0.344315,0.033044,0.068398,0.013147,0.008580,0.003609,0.001848,0.001060,0.001429,0.001832,0.001218,0.000703,0.000642,0.000646,0.000867,0.000550,0.000567,0.001386,0.001239,0.001416,0.001354,0.001710,0.003111,0.003111
giwavemapOboe    ftgen    0,0,128,-27, 0,0, 59,0,  65,1,  73,2,  81,3,  89,4,  127,4
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfOboe    ftgen    0,0,8,-2,gioboe59,gioboe65,gioboe73,gioboe81,gioboe89,gioboe89,gioboe89,gioboe89

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; BASSOON
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
gibassoon34 ftgen 0,0,4096,10,0.213868,0.268493,0.244166,0.230801,0.105833,0.308087,0.556920,0.478956,0.523357,0.900665,0.213470,0.229596,0.031221,0.040117,0.067113,0.060158,0.030778,0.061028,0.143814,0.063668,0.126426,0.055009,0.049138,0.085854,0.113027,0.111464,0.088765,0.037342,0.050990,0.035579,0.040460,0.032219,0.027305,0.034141,0.019655,0.009315,0.008270,0.006748,0.006472,0.007088,0.008133,0.007046,0.007850,0.005791,0.006273,0.006847,0.007249,0.009398,0.010309,0.010418,0.010247,0.010333,0.009562,0.008180,0.009576,0.009469,0.008529,0.008844,0.008053,0.007565,0.008026,0.007284,0.007299,0.007423,0.008280,0.008374,0.008239,0.008512,0.009431,0.010246,0.010350,0.009381,0.008652,0.008150,0.007888,0.007951,0.008166,0.008210,0.007921,0.007548,0.007147,0.006991,0.006978,0.006527,0.005617,0.004781,0.004549,0.004707,0.004803,0.004640,0.004303,0.003866,0.003524,0.003348,0.003108,0.002766,0.002439,0.002278,0.002406,0.002733
gibassoon38 ftgen 0,0,4096,10,0.242897,0.242266,0.380380,0.667003,0.720921,0.402509,0.422424,0.475496,0.823674,0.135651,0.169435,0.062761,0.017494,0.023826,0.049210,0.074371,0.035978,0.086397,0.160820,0.122072,0.061609,0.020374,0.023031,0.018479,0.013838,0.015511,0.012693,0.005626,0.007015,0.007590,0.008854,0.004833,0.006678,0.008155,0.006694,0.007708,0.007823,0.004336,0.003679,0.003698,0.005616,0.004049,0.003555,0.002815,0.002166,0.001778,0.002195,0.002446,0.003427,0.003600,0.005046,0.005043,0.003980,0.002591,0.002168,0.002644,0.003363,0.004603,0.004571,0.005024,0.004653,0.004659,0.004447,0.004128,0.004325,0.004464,0.004393,0.004144,0.004285,0.004442,0.004203,0.003640,0.003229,0.003115,0.003017,0.002902,0.002799,0.002839,0.003045,0.003262,0.003391,0.003250,0.003008,0.002726,0.002506,0.002370,0.002215,0.002130,0.002058,0.001832,0.001574,0.001339,0.001133,0.001005,0.000924,0.000860,0.000849,0.000896,0.000994,0.001115
gibassoon43 ftgen 0,0,4096,10,0.101647,0.135289,0.245027,0.142720,0.728360,0.361154,0.030730,0.079698,0.011579,0.074893,0.094091,0.028910,0.084641,0.058767,0.035341,0.016688,0.006781,0.020365,0.020545,0.008172,0.007968,0.017127,0.016825,0.009248,0.003609,0.002361,0.002361,0.001409,0.001208,0.001278,0.001752,0.002711,0.002221,0.001898,0.001255,0.001150,0.001935,0.001759,0.001661,0.001219,0.001111,0.001527,0.001167,0.001290,0.001366,0.001058,0.000935,0.000898,0.000937,0.001254,0.001464,0.001695,0.002012,0.002054,0.001872,0.001370,0.001074,0.000988,0.001009,0.000959,0.000905,0.000957,0.000885,0.000825,0.000811,0.000776,0.000727,0.000696,0.000758,0.000768,0.000667,0.000604,0.000548,0.000525,0.000508,0.000485,0.000445,0.000380,0.000330,0.000307,0.000329,0.000366,0.000367,0.000334,0.000291,0.000259,0.000247,0.000265,0.000309,0.000378,0.000436,0.000425,0.000390,0.000375,0.000353,0.000328,0.000312,0.000292,0.000273,0.000269
gibassoon50 ftgen 0,0,4096,10,0.147763,0.340255,0.879202,0.404023,0.259274,0.130887,0.095096,0.039056,0.091976,0.009828,0.004879,0.013408,0.016090,0.015931,0.012402,0.003430,0.006421,0.004323,0.002018,0.001465,0.003012,0.001397,0.002446,0.002683,0.002425,0.004828,0.002422,0.002479,0.002086,0.002351,0.002269,0.002034,0.001708,0.001324,0.001927,0.002764,0.003997,0.003514,0.002736,0.002327,0.002054,0.001842,0.001577,0.002009,0.002098,0.001411,0.001019,0.000816,0.000899,0.000893,0.001005,0.001296,0.000987,0.000657,0.000539,0.000400,0.000313,0.000284,0.000331,0.000417,0.000479,0.000633,0.000798,0.000893,0.001067,0.001242,0.001260,0.001282,0.001218,0.001023,0.000887,0.000830,0.000746,0.000637,0.000553,0.000534,0.000537,0.000506,0.000482,0.000477,0.000490,0.000515,0.000532,0.000508,0.000473,0.000416,0.000336,0.000271,0.000229,0.000193,0.000170,0.000169,0.000185,0.000197,0.000205,0.000223,0.000247,0.000255,0.000248,0.000249
gibassoon56 ftgen 0,0,4096,10,0.258143,0.954570,0.553228,0.188964,0.031203,0.252375,0.179421,0.053961,0.022121,0.016058,0.006372,0.002854,0.002507,0.006348,0.006626,0.008348,0.005125,0.001342,0.001956,0.002664,0.005721,0.003529,0.004933,0.002543,0.003026,0.003332,0.001828,0.003024,0.002223,0.001464,0.001354,0.001310,0.000827,0.000966,0.001111,0.001685,0.001048,0.000806,0.000660,0.000445,0.000368,0.000436,0.000578,0.000518,0.000372,0.000346,0.000342,0.000340,0.000259,0.000272,0.000318,0.000395,0.000481,0.000454,0.000428,0.000391,0.000351,0.000302,0.000261,0.000245,0.000253,0.000249,0.000223,0.000228,0.000244,0.000217,0.000203,0.000222,0.000250,0.000271,0.000235,0.000202,0.000179,0.000160,0.000151,0.000151,0.000144,0.000126,0.000116,0.000123,0.000134,0.000140,0.000142,0.000142,0.000130,0.000117,0.000109,0.000109,0.000110,0.000111,0.000122,0.000129,0.000124,0.000116,0.000115,0.000113,0.000104,0.000092,0.000080,0.000071
gibassoon62 ftgen 0,0,4096,10,0.802206,0.854305,0.242064,0.037491,0.024258,0.012991,0.003920,0.008373,0.001787,0.001801,0.003935,0.008957,0.009944,0.004249,0.003799,0.003374,0.001734,0.004753,0.008252,0.004132,0.002330,0.001700,0.001925,0.001098,0.001154,0.001943,0.000954,0.000487,0.000432,0.000605,0.000812,0.000806,0.001001,0.001178,0.001065,0.001137,0.000877,0.000630,0.000538,0.000499,0.000605,0.000532,0.000532,0.000387,0.000288,0.000253,0.000313,0.000347,0.000292,0.000225,0.000167,0.000144,0.000121,0.000116,0.000110,0.000110,0.000119,0.000111,0.000106,0.000111,0.000115,0.000114,0.000101,0.000094,0.000100,0.000102,0.000097,0.000088,0.000079,0.000070,0.000064,0.000059,0.000054,0.000053,0.000053
gibassoon67 ftgen 0,0,4096,10,0.262539,0.973041,0.138354,0.113396,0.058172,0.030635,0.005565,0.001686,0.003483,0.001977,0.003774,0.002477,0.002439,0.001226,0.002781,0.002527,0.001020,0.000611,0.000658,0.000715,0.000728,0.000531,0.000442,0.000238,0.000414,0.000325,0.000560,0.000689,0.000445,0.000306,0.000301,0.000305,0.000296,0.000325,0.000252,0.000218,0.000248,0.000193,0.000174,0.000179,0.000143,0.000140,0.000140,0.000104,0.000096,0.000116,0.000128,0.000137,0.000144,0.000113,0.000112,0.000119,0.000140,0.000116,0.000107,0.000102
giwavemapBassoon    ftgen    0,0,128,-27, 0,0, 34,0,  38,1,  43,2,  50,3,   56,4,  62,5,  67,6, 127,6
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfBassoon    ftgen    0,0,8,-2,gibassoon34,gibassoon38,gibassoon43,gibassoon50,gibassoon56,gibassoon62,gibassoon67,gibassoon67

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; CONTRA-BASSOON
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giCBassoon22 ftgen 0,0,4096,10,0.497216,0.592986,0.588338,0.378063,0.343716,0.540022,0.620921,0.613540,0.848864,0.535558,0.632297,0.584663,0.443376,0.419439,0.291262,0.249631,0.256447,0.182209,0.062888,0.160047,0.156792,0.115884,0.107766,0.130415,0.183253,0.145108,0.156478,0.170472,0.185752,0.181506,0.155042,0.154256,0.167703,0.145571,0.172382,0.169936,0.191248,0.207520,0.161789,0.114105,0.096757,0.068310,0.056357,0.068193,0.048257,0.046235,0.041903,0.032361,0.023090,0.012851,0.008788,0.010598,0.010151,0.010905,0.009426,0.009439,0.012674,0.016976,0.019287,0.017791,0.014995,0.015280,0.014007,0.012293,0.013920,0.015130,0.017138,0.020001,0.026003,0.029085,0.026391,0.021776,0.018388,0.016337,0.014700,0.015521,0.019019,0.022414,0.024168,0.024006,0.023403,0.022377,0.020534,0.018428,0.016165,0.013675,0.011929,0.011056,0.010352,0.010070,0.009561,0.009378,0.008855,0.008393,0.008147,0.008048,0.008099,0.007480,0.006551,0.005481
giCBassoon26 ftgen 0,0,4096,10,0.076357,0.137793,0.174617,0.207802,0.302523,0.705801,0.162577,0.073808,0.224559,0.050465,0.066110,0.181682,0.062409,0.052317,0.037698,0.029936,0.042341,0.053436,0.135414,0.059248,0.025689,0.033567,0.051201,0.029910,0.028697,0.052991,0.059280,0.029742,0.027742,0.023801,0.013878,0.006197,0.004814,0.005139,0.007873,0.007104,0.009903,0.007952,0.008234,0.005783,0.006682,0.007272,0.006943,0.007674,0.007728,0.006326,0.005179,0.004259,0.004142,0.003705,0.002547,0.001785,0.002073,0.002887,0.004184,0.004075,0.004174,0.005101,0.004993,0.004189,0.003170,0.002599,0.002866,0.003689,0.005272,0.006515,0.007262,0.006420,0.005939,0.005247,0.004710,0.004619,0.004018,0.003371,0.002611,0.001906,0.001540,0.001341,0.001128,0.001025,0.001114,0.001151,0.001224,0.001403,0.001564,0.001709,0.001914,0.002097,0.002148,0.002190,0.002192,0.002124,0.002190,0.002485,0.002819,0.002892,0.002769,0.002559,0.002374,0.002161
giCBassoon38 ftgen 0,0,4096,10,0.240531,0.304670,0.289169,0.727482,0.578083,0.169149,0.057305,0.193624,0.167977,0.206006,0.182632,0.057408,0.103574,0.044785,0.084239,0.068303,0.030771,0.133375,0.096231,0.037016,0.046566,0.020770,0.020264,0.015211,0.029647,0.018603,0.027940,0.062861,0.033828,0.015594,0.016305,0.025478,0.034356,0.038642,0.028451,0.026747,0.014914,0.016727,0.015524,0.013236,0.014264,0.013651,0.014838,0.016490,0.021208,0.017660,0.013009,0.014867,0.013013,0.010426,0.009144,0.009462,0.009367,0.009624,0.008079,0.007399,0.009012,0.009384,0.008661,0.009051,0.009394,0.010578,0.011610,0.012184,0.010442,0.009130,0.008795,0.008468,0.010039,0.011205,0.011438,0.011489,0.010526,0.008902,0.007391,0.006198,0.005970,0.005264,0.004331,0.003874,0.003514,0.003418,0.003518,0.003409,0.003150,0.003023,0.003047,0.003199,0.003624,0.003999,0.003839,0.003629,0.003712,0.003872,0.003974,0.003836,0.003596,0.003353,0.003177,0.003070
giCBassoon50 ftgen 0,0,4096,10,0.200451,0.511764,0.153876,0.491651,0.722151,0.174751,0.087283,0.068341,0.008971,0.040014,0.041083,0.015335,0.047809,0.037606,0.038259,0.020124,0.022425,0.024747,0.016480,0.008528,0.006892,0.004131,0.005947,0.005585,0.006842,0.008448,0.015885,0.005820,0.002834,0.004626,0.003886,0.003621,0.004837,0.005276,0.002715,0.002900,0.003817,0.005081,0.005598,0.006033,0.006164,0.004260,0.003944,0.004776,0.004223,0.004351,0.004769,0.004836,0.004959,0.005380,0.004538,0.003969,0.003860,0.003536,0.003521,0.003397,0.002823,0.002162,0.002001,0.001663,0.001430,0.001505,0.001566,0.001468,0.001406,0.001439,0.001225,0.001095,0.001134,0.001316,0.001630,0.001839,0.001964,0.001940,0.001824,0.001694,0.001617,0.001526,0.001348,0.001116,0.000972,0.000973,0.001011,0.001015,0.000944,0.000785,0.000650,0.000609,0.000675,0.000782,0.000839,0.000818,0.000755,0.000661,0.000566,0.000498,0.000471,0.000478,0.000486,0.000479
giwavemapCBassoon    ftgen    0,0,128,-27, 0,0, 22,0,  26,1,  38,2,  50,3,  127,3
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfCBassoon    ftgen    0,0,4,-2,giCBassoon22,giCBassoon26,giCBassoon38,giCBassoon50


;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; VIOLIN
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giviolin55 ftgen 0,0,4096,10,0.081672,0.785854,0.808302,0.221447,0.136644,0.103608,0.051154,0.040070,0.071067,0.358903,0.053101,0.090036,0.033729,0.068969,0.060398,0.147336,0.060769,0.055245,0.044695,0.022179,0.058528,0.083706,0.029581,0.013173;,0.013173
giviolin67 ftgen 0,0,4096,10,0.344116,0.095361,0.038223,0.090317,0.145798,0.049369,0.076997,0.057555,0.016248,0.036189,0.014161,0.008738,0.010238,0.009625,0.005588,0.004762,0.002635,0.002162,0.002822,0.002144,0.001687,0.001955,0.001870,0.001011;,0.001011
giviolin79 ftgen 0,0,4096,10,0.403041,0.281156,0.259204,0.392900,0.236559,0.108769,0.117288,0.063588,0.039483,0.018245,0.012717,0.020183,0.008506,0.017915,0.008925,0.008575,0.006947,0.008142,0.005487,0.005970,0.004402,0.003161,0.002361,0.001942;,0.001942
giviolin91 ftgen 0,0,4096,10,0.231708,0.906742,0.215622,0.092709,0.057338,0.038101,0.030758,0.023439,0.009905,0.001298,0.002108,0.000516,0.000499,0.000123,0.000642,0.000732,0.004757,0.004547,0.017582,0.033035,0.098559,0.013616,0.017674,0.019601;,0.019601
giwavemapViolin    ftgen    0,0,128,-27, 0,0, 55,0,  67,1,  79,2,  91,3,  127,3
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfViolin    ftgen    0,0,4,-2,giviolin55,giviolin67,giviolin79,giviolin91    ;

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; CELLO
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
gicello36 ftgen 0,0,4096,10,0.022943,0.412962,0.548431,0.578468,0.092241,0.116378,0.052974,0.029585,0.093977,0.119660,0.064074,0.051807,0.015094,0.094184,0.015488,0.006640,0.007091,0.011557,0.043467,0.023330,0.018118,0.026921,0.010285,0.007653,0.015306,0.023175,0.013051,0.004840,0.005658,0.010814,0.014921,0.026066,0.028334,0.015227,0.010740,0.008966,0.009389,0.012547,0.011544,0.009900,0.012403,0.013093,0.008259,0.007148,0.006129,0.004932,0.004332,0.003397,0.002497,0.002241,0.002187,0.002227,0.001634,0.001186,0.001119,0.001131,0.001276,0.000970,0.001101,0.001647,0.001936,0.001632,0.001190,0.000891,0.000883,0.000906,0.000894,0.000855,0.000768,0.000721,0.000750,0.000900,0.000965,0.000844,0.000765,0.000643,0.000535,0.000464,0.000409,0.000403,0.000489,0.000642,0.000773,0.000819,0.000871,0.000930,0.000979,0.000950,0.000781,0.000649,0.000620,0.000653,0.000661,0.000706,0.000805,0.000856,0.000832,0.000793,0.000750,0.000702
gicello48 ftgen 0,0,4096,10,0.171752,0.537455,0.217980,0.172188,0.552902,0.583964,0.452897,0.095360,0.059208,0.033024,0.126570,0.028481,0.067671,0.029869,0.055112,0.056284,0.025151,0.038689,0.027323,0.027935,0.070762,0.023514,0.016984,0.013118,0.008692,0.004591,0.002235,0.003399,0.002568,0.002224,0.002366,0.002892,0.004660,0.003562,0.002923,0.002328,0.002327,0.001705,0.001566,0.002356,0.002199,0.003381,0.006360,0.007320,0.004930,0.002809,0.002686,0.002156,0.001566,0.001273,0.001192,0.001380,0.001683,0.002133,0.003160,0.002820,0.002065,0.002097,0.002013,0.001649,0.001366,0.001053,0.000819,0.000730,0.000775,0.000937,0.001097,0.001063,0.000917,0.000815,0.000792,0.000857,0.001010,0.001203,0.001372,0.001408,0.001295,0.001173,0.001062,0.000942,0.000825,0.000761,0.000774,0.000854,0.000932,0.000968,0.000987,0.001022,0.001018,0.000944,0.000882,0.000829,0.000779,0.000734,0.000693,0.000655,0.000602,0.000547,0.000529,0.000548
gicello60 ftgen 0,0,4096,10,0.762584,0.124687,0.088950,0.091324,0.058688,0.034069,0.038828,0.065420,0.062254,0.058503,0.033332,0.014105,0.011585,0.004135,0.005176,0.004496,0.006698,0.008491,0.007044,0.003450,0.004771,0.016317,0.009897,0.007180,0.004105,0.002477,0.003995,0.002992,0.002346,0.002233,0.002558,0.002135,0.002301,0.001713,0.001191,0.001850,0.004203,0.001813,0.000969,0.001090,0.000636,0.001237,0.000844,0.001107,0.000880,0.000791,0.000640,0.000894,0.000554,0.000727,0.001041,0.001046,0.001079,0.000456,0.000410,0.000442,0.000431,0.000437,0.000427,0.000389,0.000350,0.000284,0.000269,0.000260,0.000210,0.000253,0.000328,0.000296,0.000290,0.000419,0.000452,0.000328,0.000215,0.000229,0.000333,0.000275,0.000208,0.000204,0.000163,0.000283,0.000253,0.000264,0.000178,0.000208,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
gicello72 ftgen 0,0,4096,10,0.601382,0.263076,0.197963,0.244245,0.217992,0.114034,0.025768,0.041518,0.011954,0.006391,0.012239,0.017818,0.016819,0.012798,0.007576,0.007376,0.002891,0.021377,0.005261,0.004421,0.009226,0.004333,0.001808,0.003989,0.002078,0.001344,0.001560,0.001240,0.001586,0.001307,0.000519,0.000761,0.001009,0.000632,0.000412,0.000414,0.000361,0.000406,0.000544,0.000741,0.000281,0.000164,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giwavemapCello    ftgen    0,0,128,-27, 0,0, 36,0,  48,1,  60,2,  72,3,  127,3
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfCello    ftgen    0,0,4,-2,gicello36,gicello48,gicello60,gicello72    ;

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; PICCOLO
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
gipiccolo74 ftgen 0,0,4096,10,0.898113,0.247693,0.060222,0.127742,0.034019,0.044738,0.029980,0.018041,0.008219,0.009819,0.015719,0.009423,0.020117,0.011802,0.007226,0.003198,0.001597,0.002062,0.001590,0.001420,0.001087,0.000581,0.000370,0.000393,0.000393
gipiccolo80 ftgen 0,0,4096,10,0.837581,0.024769,0.144022,0.109176,0.031512,0.026122,0.004565,0.008379,0.004995,0.002685,0.003489,0.002674,0.002743,0.001129,0.000485,0.000367,0.000701,0.001563,0.001166,0.000647,0.000808,0.000622,0.000358,0.000227,0.000227
gipiccolo86 ftgen 0,0,4096,10,0.730803,0.323120,0.277720,0.044844,0.009400,0.022785,0.004602,0.005762,0.006400,0.002483,0.001490,0.002354,0.004728,0.002669,0.003175,0.001177,0.000761,0.000509,0.000449,0.000703,0.001078,0.002471,0.003905,0.004114,0.004114
gipiccolo92 ftgen 0,0,4096,10,0.883567,0.117309,0.144273,0.012664,0.016747,0.002921,0.002043,0.001429,0.002107,0.008510,0.001611,0.002330,0.000075,0.000381,0.000943,0.002268,0.001748,0.001626,0.000882,0.002765,0.003187,0.004911,0.008342,0.017209,0.017209
gipiccolo98 ftgen 0,0,4096,10,0.922186,0.144948,0.042529,0.011837,0.006901,0.000900,0.001625,0.000686,0.000507 ; ,0.000305,0.000305,0.000730,0.000922,0.001408,0.001499,0.005116,0.021056,0.151393,0.034986,0.124701,0.024182,0.008037,0.002664,0.001934,0.001934
gipiccolo106 ftgen 0,0,4096,10,0.501042,0.019695,0.008976,0.003473,0.006493,0.001700,0.002765,0.003224,0.006276 ; ,0.025062,0.036907,0.005386,0.046443,0.013720,0.005391,0.007819,0.019832,0.008441,0.006927,0.007528,0.018483,0.042516,0.057060,0.083779,0.083779
giwavemapPiccolo    ftgen    0,0,128,-27, 0,0, 74,0,  80,1,  86,2,  92,3,  98,4, 106,5, 127,5
gitabs4morfPiccolo    ftgen    0,0,8,-2,gipiccolo74,gipiccolo80,gipiccolo86,gipiccolo92,gipiccolo98,gipiccolo106,gipiccolo106,gipiccolo106    ;

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; FLUTE
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giflute59 ftgen 0,0,4096,10,0.876155,0.188136,0.766537,0.278649,0.134428,0.143342,0.201465,0.068043,0.011284,0.021093,0.021494,0.010577,0.016431,0.007352,0.006864,0.007433,0.006500,0.004716,0.004282,0.002545,0.002718,0.002045,0.001768,0.002409,0.002409
giflute65 ftgen 0,0,4096,10,0.799971,0.564039,0.327027,0.076562,0.324116,0.049951,0.035116,0.048856,0.017988,0.004316,0.007909,0.004093,0.003013,0.003329,0.001707,0.001995,0.001913,0.000937,0.001543,0.001868,0.001289,0.001463,0.001865,0.001076,0.001076
giflute71 ftgen 0,0,4096,10,0.902388,0.107786,0.423495,0.078015,0.066093,0.032990,0.012352,0.008687,0.005253,0.004091,0.001790,0.002202,0.000853,0.001228,0.000810,0.000828,0.000601,0.000593,0.000647,0.000906,0.000925,0.001111,0.001302,0.000899,0.000899
giflute77 ftgen 0,0,4096,10,0.925667,0.059839,0.214284,0.013980,0.026778,0.008733,0.019436,0.002640,0.004217,0.002126,0.003513,0.002455,0.001336,0.001508,0.001589,0.002155,0.002549,0.001951,0.001614,0.001076,0.001253,0.001138,0.001073,0.000990,0.000990
giflute83 ftgen 0,0,4096,10,0.865610,0.041090,0.054192,0.043068,0.022098,0.002450,0.005687,0.002093,0.005076,0.001357,0.003848,0.006820,0.002879,0.005038,0.002273,0.002877,0.001043,0.001188,0.000857,0.000726,0.000537,0.000320,0.000390,0.000666,0.000666
giflute89 ftgen 0,0,4096,10,0.907108,0.032221,0.007003,0.010577,0.004166,0.002274,0.001089,0.002739,0.002459,0.002194,0.001351,0.000570,0.000469,0.000388,0.000228,0.000180,0.000377,0.000470,0.000724,0.001478,0.002258,0.002625,0.006255,0.003183,0.003183
giflute95 ftgen 0,0,4096,10,0.927168,0.094811,0.014160,0.010743,0.004640,0.002540,0.000535,0.000356,0.000205,0.000147,0.000042,0.000032,0.000090,0.000201,0.000171,0.000595,0.000136,0.000338,0.000284,0.000244,0.001813,0.001310,0.002113,0.000510,0.000510
giflute98 ftgen 0,0,4096,10,0.793870,0.077609,0.047389,0.012277,0.005127,0.002760,0.002619,0.000390,0.000109,0.000105,0.000147,0.000425,0.000266,0.000201,0.000312,0.000210,0.001391,0.003570,0.001720,0.002746,0.001399,0.000749,0.000411,0.001156,0.001156
giwavemapFlute    ftgen    0,0,128,-27, 0,0, 59,0, 65,1, 71,2, 77,3, 83,4, 89,5, 95,6, 98,7, 127,7
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfFlute    ftgen    0,0,8,-2,giflute59,giflute65,giflute71,giflute77,giflute83,giflute89,giflute95,giflute98

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; ALTO FLUTE
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giAFlute55 ftgen 0,0,4096,10,0.821986,0.323726,0.027144,0.049562,0.045630,0.004837,0.008584,0.002449,0.002921,0.004780,0.006174,0.006496,0.009180,0.010020,0.008884,0.006397,0.007728,0.007543,0.005540,0.004643,0.004213,0.003464,0.002247,0.001529,0.001529
giAFlute61 ftgen 0,0,4096,10,0.629423,0.050378,0.078267,0.098147,0.002593,0.003473,0.002978,0.007847,0.002718,0.004264,0.004001,0.003129,0.002278,0.001767,0.001591,0.001198,0.000790,0.000910,0.000907,0.000909,0.000690,0.001006,0.000870,0.000801,0.000801
giAFlute67 ftgen 0,0,4096,10,0.734185,0.213440,0.086548,0.055143,0.004785,0.001811,0.011006,0.004212,0.001873,0.002130,0.001001,0.000376,0.000632,0.000823,0.000595,0.000764,0.000514,0.000403,0.000326,0.000317,0.000537,0.000523,0.000835,0.000743,0.000743
giAFlute76 ftgen 0,0,4096,10,0.868899,0.390848,0.052590,0.023839,0.033699,0.031143,0.013923,0.008881,0.003821,0.001797,0.003048,0.002506,0.002295,0.001878,0.001118,0.001109,0.001133,0.001732,0.002085,0.002020,0.001240,0.000963,0.000929,0.000456,0.000456
giwavemapAFlute    ftgen    0,0,128,-27, 0,0, 55,0, 61,1, 67,2, 76,3, 127,3
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfAFlute    ftgen    0,0,4,-2,giAFlute55,giAFlute61,giAFlute67,giAFlute76

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; BASS FLUTE
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giBFlute48 ftgen 0,0,4096,10,0.691967,0.034269,0.271945,0.033667,0.017446,0.011342,0.007238,0.006576,0.003027,0.003556,0.004425,0.004988,0.003990,0.005253,0.006140,0.003051,0.004437,0.003567,0.002838,0.001703,0.001528,0.001368,0.001703,0.001153,0.000827,0.001023,0.000899,0.000612,0.000827,0.000823,0.000948,0.001184,0.001201,0.001272,0.001032,0.001141,0.001128,0.000887,0.000790,0.000612,0.000565,0.000573,0.000586,0.000554,0.000483,0.000450,0.000404,0.000416,0.000392,0.000391,0.000327,0.000336,0.000370,0.000424,0.000460,0.000459,0.000456,0.000395,0.000319,0.000306,0.000283,0.000306,0.000338,0.000305,0.000288,0.000263,0.000247,0.000239,0.000245,0.000233,0.000190,0.000166,0.000161,0.000160,0.000150,0.000136,0.000130,0.000130,0.000126,0.000126,0.000128,0.000131,0.000131,0.000136,0.000138,0.000132,0.000122,0.000116,0.000119,0.000126,0.000122,0.000112,0.000109,0.000113,0.000118,0.000118,0.000118,0.000117,0.000116,0.000118
giBFlute54 ftgen 0,0,4096,10,0.633439,0.578363,0.200810,0.008032,0.068714,0.005615,0.005448,0.012421,0.009987,0.006289,0.004137,0.002738,0.001944,0.002935,0.003674,0.002567,0.002545,0.004116,0.003915,0.002935,0.002322,0.002647,0.003186,0.003370,0.003213,0.002193,0.001902,0.001629,0.001652,0.001274,0.001102,0.001205,0.001277,0.001134,0.001117,0.001286,0.001477,0.001607,0.002607,0.003270,0.002939,0.002558,0.001826,0.001324,0.001228,0.001070,0.001110,0.001120,0.001042,0.000924,0.000909,0.000880,0.000765,0.000674,0.000600,0.000491,0.000437,0.000441,0.000443,0.000452,0.000467,0.000527,0.000606,0.000571,0.000554,0.000518,0.000420,0.000363,0.000315,0.000275,0.000245,0.000221,0.000200,0.000202,0.000219,0.000211,0.000194,0.000187,0.000197,0.000197,0.000193,0.000201,0.000217,0.000223,0.000219,0.000213,0.000200,0.000175,0.000149,0.000131,0.000124,0.000120,0.000115,0.000111,0.000113,0.000117,0.000115,0.000107,0.000100,0.000098
giBFlute60 ftgen 0,0,4096,10,0.801592,0.068816,0.032303,0.012614,0.002660,0.014692,0.002493,0.006175,0.001826,0.001416,0.002607,0.001063,0.001542,0.001053,0.001293,0.001226,0.000781,0.001142,0.000975,0.000922,0.000693,0.000570,0.000538,0.000429,0.000587,0.000775,0.000855,0.001295,0.001190,0.000941,0.000912,0.000796,0.000662,0.000789,0.000828,0.000784,0.000614,0.000480,0.000455,0.000378,0.000334,0.000309,0.000351,0.000375,0.000427,0.000378,0.000293,0.000232,0.000224,0.000199,0.000150,0.000119,0.000100,0.000084,0.000084,0.000088,0.000085,0.000081,0.000076,0.000081,0.000080,0.000075,0.000071,0.000068,0.000063,0.000060,0.000064,0.000067,0.000065,0.000058,0.000055,0.000054,0.000054,0.000053,0.000052,0.000053,0.000053,0.000053,0.000052,0.000050,0.000049,0.000049,0.000049,0.000049,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giBFlute66 ftgen 0,0,4096,10,0.743019,0.068238,0.021487,0.010397,0.001744,0.013212,0.001676,0.005245,0.002049,0.001401,0.001686,0.001346,0.001288,0.000879,0.001087,0.000922,0.000878,0.000678,0.000904,0.000729,0.000481,0.000542,0.000525,0.000561,0.000481,0.000552,0.000745,0.001153,0.001211,0.001000,0.000706,0.000621,0.000613,0.000779,0.000764,0.000717,0.000619,0.000486,0.000448,0.000429,0.000341,0.000327,0.000331,0.000340,0.000340,0.000335,0.000288,0.000255,0.000233,0.000195,0.000173,0.000150,0.000127,0.000123,0.000119,0.000113,0.000111,0.000115,0.000116,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giBFlute72 ftgen 0,0,4096,10,0.741444,0.066744,0.018973,0.009713,0.002046,0.010742,0.001911,0.004984,0.002277,0.001308,0.001841,0.000932,0.001065,0.000889,0.001029,0.000970,0.000767,0.000708,0.000743,0.000629,0.000541,0.000457,0.000410,0.000534,0.000487,0.000489,0.000743,0.001263,0.001164,0.001062,0.000757,0.000529,0.000538,0.000606,0.000594,0.000583,0.000497,0.000463,0.000498,0.000433,0.000369,0.000363,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giBFlute76 ftgen 0,0,4096,10,0.745977,0.066489,0.014197,0.007352,0.002175,0.009766,0.002280,0.004362,0.001870,0.001375,0.001383,0.001328,0.001381,0.001194,0.001429,0.000949,0.000917,0.000802,0.000818,0.000932,0.000636,0.000646,0.000571,0.000575,0.000613,0.000679,0.000959,0.001343,0.001448,0.001365,0.001241,0.001091,0.001093,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giwavemapBFlute    ftgen    0,0,128,-27, 0,0, 48,0, 54,1, 60,2, 66,3, 72,4, 76,5, 127,5
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfBFlute    ftgen    0,0,8,-2,giBFlute48,giBFlute54,giBFlute60,giBFlute66,giBFlute72,giBFlute76,giBFlute76,giBFlute76

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; AHH
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
/*
giAhh24 ftgen 0,0,4096,10, 0.000412,0.000370,0.000337,0.000371,0.000555,0.000859,0.001305,0.001896,0.002601,0.003585,0.004729,0.006226,0.008387,0.010759,0.015604,0.021153,0.036276,0.057553,0.053066,0.030189,0.020360,0.014722,0.011349,0.009414,0.008036,0.007520,0.007518,0.008204,0.010188,0.013445,0.021468,0.024378,0.017218,0.010300,0.007574,0.005320,0.004125,0.003105,0.002351,0.001777,0.001280,0.000908,0.000603,0.000362,0.000194,0.000113,0.000107,0.000165,0.000269,0.000410,0.000591,0.000829,0.001124,0.001497,0.001964,0.002540,0.003281,0.004214,0.005465,0.007172,0.009487,0.012221,0.012290,0.008928,0.005554,0.004575,0.006297,0.009642,0.012002,0.011210,0.008864,0.007021,0.005649,0.004715,0.004092,0.003711,0.003540,0.003269,0.002720,0.002116,0.001696,0.001392,0.001163,0.000978,0.000817,0.000679,0.000554,0.000445,0.000349,0.000267,0.000201,0.000148,0.000112
giAhh27 ftgen 0,0,4096,10, 0.000484,0.000423,0.000410,0.000587,0.001000,0.001640,0.002515,0.003647,0.005207,0.007188,0.010221,0.013861,0.021835,0.034169,0.065821,0.062536,0.031595,0.021762,0.014578,0.011706,0.009599,0.008934,0.009048,0.011008,0.014402,0.024811,0.028584,0.017346,0.010562,0.007037,0.005089,0.003642,0.002621,0.001845,0.001237,0.000781,0.000431,0.000198,0.000132,0.000128,0.000107,0.000154,0.000279,0.000469,0.000726,0.001077,0.001533,0.002136,0.002909,0.003944,0.005309,0.007297,0.010064,0.014010,0.014733,0.009883,0.005835,0.006234,0.010391,0.014244,0.013109,0.009848,0.007528,0.005944,0.004974,0.004420,0.004181,0.003659,0.002784,0.002107,0.001664,0.001346,0.001093,0.000881,0.000697,0.000538,0.000403,0.000292,0.000206,0.000145,0.000112
giAhh30 ftgen 0,0,4096,10, 0.000560,0.000482,0.000562,0.001054,0.001899,0.003144,0.004913,0.007274,0.010934,0.015824,0.026652,0.050310,0.084386,0.048694,0.027304,0.017198,0.013209,0.010896,0.010619,0.012323,0.016925,0.031534,0.030303,0.014903,0.009410,0.006192,0.004171,0.002825,0.001804,0.001089,0.000559,0.000223,0.000135,0.000159,0.000132,0.000111,0.000205,0.000408,0.000717,0.001165,0.001784,0.002644,0.003807,0.005452,0.007846,0.011507,0.016933,0.016329,0.008970,0.006585,0.011345,0.016921,0.014673,0.010437,0.007690,0.006096,0.005243,0.004888,0.003982,0.002810,0.002096,0.001620,0.001264,0.000975,0.000730,0.000526,0.000361,0.000238,0.000159,0.000123,0.000113,0.000108
giAhh33 ftgen 0,0,4096,10, 0.000645,0.000569,0.000955,0.001964,0.003652,0.006202,0.009824,0.015928,0.027700,0.054004,0.099838,0.045530,0.026122,0.017357,0.013212,0.012640,0.015734,0.025801,0.041244,0.020359,0.011360,0.006947,0.004321,0.002669,0.001503,0.000708,0.000231,0.000110,0.000186,0.000170,0.000130,0.000254,0.000564,0.001062,0.001820,0.002933,0.004538,0.006945,0.010838,0.017334,0.020883,0.011091,0.008199,0.016262,0.019876,0.013707,0.009453,0.007159,0.006126,0.005456,0.003786,0.002617,0.001920,0.001428,0.001039,0.000721,0.000469,0.000288,0.000180,0.000140,0.000132,0.000122,0.000102
giAhh36 ftgen 0,0,4096,10, 0.000741,0.000742,0.001718,0.003793,0.007172,0.012458,0.021527,0.042327,0.115172,0.060415,0.029465,0.018842,0.015053,0.016424,0.026918,0.048812,0.020626,0.010654,0.006220,0.003560,0.001819,0.000726,0.000149,0.000187,0.000214,0.000156,0.000332,0.000822,0.001662,0.003004,0.005098,0.008459,0.014399,0.024542,0.017933,0.009191,0.019377,0.022533,0.014117,0.009483,0.007465,0.006578,0.004260,0.002803,0.001970,0.001367,0.000896,0.000539,0.000299,0.000185,0.000160,0.000148,0.000121
giAhh39 ftgen 0,0,4096,10, 0.000846,0.001173,0.003275,0.007285,0.014357,0.027687,0.068261,0.124920,0.043474,0.023386,0.017847,0.021990,0.049566,0.034657,0.014061,0.007278,0.003688,0.001560,0.000396,0.000192,0.000256,0.000187,0.000558,0.001451,0.003062,0.005814,0.010610,0.020116,0.029454,0.011668,0.020774,0.026213,0.015055,0.009950,0.008362,0.005568,0.003330,0.002188,0.001396,0.000808,0.000411,0.000223,0.000189,0.000167,0.000123
giAhh42 ftgen 0,0,4096,10, 0.000963,0.002108,0.006288,0.014551,0.031661,0.100695,0.097406,0.034417,0.021810,0.024675,0.063158,0.029838,0.012403,0.005661,0.002185,0.000448,0.000270,0.000265,0.000411,0.001440,0.003583,0.007649,0.015770,0.034040,0.018012,0.022841,0.029509,0.015471,0.010557,0.008019,0.004223,0.002548,0.001472,0.000730,0.000322,0.000229,0.000199,0.000135
giAhh45 ftgen 0,0,4096,10, 0.001138,0.003929,0.012405,0.031859,0.108048,0.091053,0.034718,0.025288,0.051642,0.040725,0.013900,0.005342,0.001417,0.000220,0.000340,0.000509,0.002128,0.005880,0.013925,0.034767,0.022222,0.032638,0.027491,0.014365,0.010950,0.005254,0.002868,0.001449,0.000579,0.000282,0.000244,0.000156
giAhh48 ftgen 0,0,4096,10, 0.001482,0.007584,0.024916,0.084669,0.120825,0.037691,0.032860,0.097651,0.021318,0.007128,0.001455,0.000373,0.000311,0.001646,0.006016,0.016943,0.049158,0.018410,0.045129,0.018996,0.013181,0.005616,0.002740,0.001081,0.000371,0.000296,0.000174,0.000109
giAhh51 ftgen 0,0,4096,10, 0.002346,0.014577,0.055410,0.249952,0.046797,0.044016,0.069343,0.014565,0.003123,0.000384,0.000374,0.002906,0.011647,0.040309,0.023365,0.052509,0.019934,0.011156,0.004385,0.001619,0.000449,0.000335,0.000163,0.000126
giAhh54 ftgen 0,0,4096,10, 0.004218,0.029108,0.201426,0.068825,0.049355,0.059660,0.011314,0.000892,0.000531,0.002882,0.015302,0.068104,0.045706,0.030943,0.016039,0.005097,0.001458,0.000457,0.000270,0.000155
giAhh57 ftgen 0,0,4096,10, 0.007858,0.063721,0.182088,0.050577,0.081435,0.010683,0.000440,0.001018,0.011764,0.069560,0.065307,0.028732,0.010508,0.002898,0.000565,0.000313,0.000177
giAhh60 ftgen 0,0,4096,10, 0.015174,0.169369,0.075380,0.195316,0.014245,0.000747,0.003294,0.033895,0.036836,0.037995,0.011232,0.002161,0.000593,0.000220,0.000121
giAhh63 ftgen 0,0,4096,10, 0.029160,0.499980,0.088056,0.029134,0.000770,0.005816,0.080662,0.105049,0.022319,0.003239,0.000671,0.000252,0.000122
giAhh66 ftgen 0,0,4096,10, 0.058218,0.137644,0.119309,0.001783,0.005763,0.136221,0.061878,0.010194,0.000915,0.000309,0.000144
giAhh69 ftgen 0,0,4096,10, 0.127432,0.101146,0.021362,0.002037,0.139115,0.057450,0.005793,0.000626,0.000185
giAhh72 ftgen 0,0,4096,10, 0.338719,0.390591,0.001495,0.067786,0.075975,0.004319,0.000439,0.000166
giAhh75 ftgen 0,0,4096,10, 1.000000,0.058271,0.011635,0.210114,0.006478,0.000504,0.000138
giAhh78 ftgen 0,0,4096,10, 0.275285,0.003566,0.272436,0.020387,0.000618,0.000157
giAhh81 ftgen 0,0,4096,10, 0.202299,0.004074,0.114905,0.001251,0.000191
giAhh84 ftgen 0,0,4096,10, 0.781181,0.135572,0.008638,0.000333
giwavemapAhh    ftgen    0,0,128,-27, 0,0, 24,0,  27,1,  30,2,  33,3,  36,4,  39,5, 42,6, 45,7, 48,8, 51,9, 54,10, 57,11, 60,12, 63,13, 66,14, 69,15, 72,16, 75,17, 78,18, 81,19, 84,20, 127,20
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfAhh    ftgen    0,0,32,-2,giAhh24,giAhh27,giAhh30,giAhh33,giAhh36,giAhh39,giAhh42,giAhh45,giAhh48,giAhh51,giAhh54,giAhh57,giAhh60,giAhh63,giAhh66,giAhh69,giAhh72,giAhh75,giAhh78,giAhh81,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84
*/

giAhh24 ftgen 0,0,4096,10,0.000412,0.000370,0.000337,0.000371,0.000555,0.000859,0.001305,0.001896,0.002601,0.003585,0.004729,0.006226,0.008387,0.010759,0.015604,0.021153,0.036276,0.057553,0.053066,0.030189,0.020360,0.014722,0.011349,0.009414,0.008036,0.007520,0.007518,0.008204,0.010188,0.013445,0.021468,0.024378,0.017218,0.010300,0.007574,0.005320,0.004125,0.003105,0.002351,0.001777,0.001280,0.000908,0.000603,0.000362,0.000194,0.000113,0.000107,0.000165,0.000269,0.000410,0.000591,0.000829,0.001124,0.001497,0.001964,0.002540,0.003281,0.004214,0.005465,0.007172,0.009487,0.012221,0.012290,0.008928,0.005554,0.004575,0.006297,0.009642,0.012002,0.011210,0.008864,0.007021,0.005649,0.004715,0.004092,0.003711,0.003540,0.003269,0.002720,0.002116,0.001696,0.001392,0.001163,0.000978,0.000817,0.000679,0.000554,0.000445,0.000349,0.000267,0.000201,0.000148,0.000112
giAhh27 ftgen 0,0,4096,10,0.000484,0.000423,0.000410,0.000587,0.001000,0.001640,0.002515,0.003647,0.005207,0.007188,0.010221,0.013861,0.021835,0.034169,0.065821,0.062536,0.031595,0.021762,0.014578,0.011706,0.009599,0.008934,0.009048,0.011008,0.014402,0.024811,0.028584,0.017346,0.010562,0.007037,0.005089,0.003642,0.002621,0.001845,0.001237,0.000781,0.000431,0.000198,0.000132,0.000128,0.000107,0.000154,0.000279,0.000469,0.000726,0.001077,0.001533,0.002136,0.002909,0.003944,0.005309,0.007297,0.010064,0.014010,0.014733,0.009883,0.005835,0.006234,0.010391,0.014244,0.013109,0.009848,0.007528,0.005944,0.004974,0.004420,0.004181,0.003659,0.002784,0.002107,0.001664,0.001346,0.001093,0.000881,0.000697,0.000538,0.000403,0.000292,0.000206,0.000145,0.000112
giAhh30 ftgen 0,0,4096,10,0.000560,0.000482,0.000562,0.001054,0.001899,0.003144,0.004913,0.007274,0.010934,0.015824,0.026652,0.050310,0.084386,0.048694,0.027304,0.017198,0.013209,0.010896,0.010619,0.012323,0.016925,0.031534,0.030303,0.014903,0.009410,0.006192,0.004171,0.002825,0.001804,0.001089,0.000559,0.000223,0.000135,0.000159,0.000132,0.000111,0.000205,0.000408,0.000717,0.001165,0.001784,0.002644,0.003807,0.005452,0.007846,0.011507,0.016933,0.016329,0.008970,0.006585,0.011345,0.016921,0.014673,0.010437,0.007690,0.006096,0.005243,0.004888,0.003982,0.002810,0.002096,0.001620,0.001264,0.000975,0.000730,0.000526,0.000361,0.000238,0.000159,0.000123,0.000113,0.000108
giAhh33 ftgen 0,0,4096,10,0.000645,0.000569,0.000955,0.001964,0.003652,0.006202,0.009824,0.015928,0.027700,0.054004,0.099838,0.045530,0.026122,0.017357,0.013212,0.012640,0.015734,0.025801,0.041244,0.020359,0.011360,0.006947,0.004321,0.002669,0.001503,0.000708,0.000231,0.000110,0.000186,0.000170,0.000130,0.000254,0.000564,0.001062,0.001820,0.002933,0.004538,0.006945,0.010838,0.017334,0.020883,0.011091,0.008199,0.016262,0.019876,0.013707,0.009453,0.007159,0.006126,0.005456,0.003786,0.002617,0.001920,0.001428,0.001039,0.000721,0.000469,0.000288,0.000180,0.000140,0.000132,0.000122,0.000102
giAhh36 ftgen 0,0,4096,10,0.001076,0.000760,0.001152,0.002918,0.006013,0.010963,0.018910,0.033003,0.065564,0.123397,0.061570,0.034546,0.024544,0.022364,0.028096,0.049779,0.048235,0.023028,0.012783,0.007477,0.004231,0.002147,0.000887,0.000349,0.000355,0.000327,0.000195,0.000152,0.000288,0.000361,0.000371,0.000539,0.001217,0.002491,0.004498,0.007542,0.012318,0.020674,0.035880,0.035208,0.020677,0.019986,0.031450,0.029001,0.018014,0.012074,0.009087,0.007807,0.005927,0.003587,0.002503,0.001834,0.001291,0.000830,0.000470,0.000249,0.000194,0.000200,0.000178,0.000130
giAhh39 ftgen 0,0,4096,10,0.001221,0.000884,0.002403,0.006103,0.012641,0.024258,0.049147,0.124843,0.094249,0.043351,0.028599,0.027085,0.040952,0.068353,0.030182,0.014975,0.007910,0.003883,0.001548,0.000473,0.000421,0.000370,0.000161,0.000304,0.000426,0.000443,0.000809,0.002086,0.004462,0.008387,0.015033,0.028032,0.047931,0.029239,0.022629,0.037393,0.031509,0.018127,0.012012,0.009537,0.007224,0.004034,0.002693,0.001836,0.001138,0.000598,0.000284,0.000232,0.000231,0.000177,0.000105
giAhh42 ftgen 0,0,4096,10,0.001347,0.001378,0.005009,0.012714,0.027848,0.064949,0.174667,0.072154,0.037557,0.031873,0.051698,0.069662,0.025483,0.011788,0.005236,0.001791,0.000485,0.000510,0.000306,0.000357,0.000517,0.000580,0.001645,0.004407,0.009531,0.019083,0.040846,0.049561,0.026461,0.041849,0.036370,0.019264,0.012655,0.010146,0.005280,0.003211,0.002020,0.001088,0.000460,0.000274,0.000277,0.000199,0.000106,0.000102
giAhh45 ftgen 0,0,4096,10,0.001435,0.002814,0.010404,0.027304,0.073592,0.204283,0.065680,0.038086,0.049867,0.090768,0.028199,0.011415,0.004006,0.000835,0.000609,0.000353,0.000145,0.000530,0.000623,0.001521,0.005124,0.012704,0.029322,0.067515,0.034280,0.047183,0.040088,0.019737,0.013502,0.007873,0.004048,0.002335,0.001073,0.000384,0.000337,0.000259,0.000126,0.000121,0.000110
giAhh48 ftgen 0,0,4096,10,0.001665,0.005802,0.022639,0.067838,0.277959,0.082343,0.043643,0.072542,0.079439,0.023326,0.007411,0.001524,0.000914,0.000509,0.000186,0.000132,0.000538,0.002041,0.005636,0.014527,0.006699,0.012109,0.005270,0.003013,0.001652,0.000778,0.000392,0.000133
giAhh51 ftgen 0,0,4096,10,0.001879,0.012367,0.050272,0.249889,0.104596,0.051557,0.135917,0.047385,0.012730,0.002203,0.001070,0.000444,0.000205,0.000486,0.002508,0.008455,0.012808,0.011617,0.007544,0.003612,0.001661,0.000745,0.000277
giAhh54 ftgen 0,0,4096,10,0.002739,0.026042,0.133355,0.175744,0.061363,0.185512,0.036813,0.006813,0.001283,0.000619,0.000238,0.000104,0.001374,0.006525,0.018731,0.014280,0.007212,0.003761,0.001187,0.000430,0.000106
giAhh57 ftgen 0,0,4096,10,0.005498,0.056620,0.471547,0.079386,0.210272,0.035484,0.003858,0.001328,0.000308,0.000117,0.001906,0.011522,0.011745,0.013323,0.004796,0.001380,0.000371,0.000133
giAhh60 ftgen 0,0,4096,10,0.002967,0.046107,0.824869,0.198093,0.082301,0.006142,0.001955,0.000496,0.001429,0.017146,0.037895,0.004766,0.005960,0.003267,0.000407,0.000157,0.000155,0.000519
giAhh63 ftgen 0,0,4096,10,0.002677,0.127443,0.233422,0.181784,0.009013,0.001717,0.000284,0.007015,0.064115,0.007331,0.007952,0.002405,0.000267,0.000125,0.000600
giAhh66 ftgen 0,0,4096,10,0.010553,0.479368,0.511917,0.027219,0.002374,0.000432,0.019753,0.029194,0.005598,0.003593,0.000314,0.000115,0.000440,0.000188
giAhh69 ftgen 0,0,4096,10,0.033509,0.499985,0.127000,0.004247,0.000403,0.034562,0.013610,0.013599,0.000498,0.000136,0.000625,0.000114
giAhh72 ftgen 0,0,4096,10,0.097704,0.332922,0.011754,0.001452,0.002670,0.003963,0.017086,0.011243,0.000618,0.000448
giAhh75 ftgen 0,0,4096,10,0.266284,0.282922,0.003635,0.000202,0.006066,0.036263,0.001979,0.003494
giAhh78 ftgen 0,0,4096,10,0.976912,0.048417,0.000970,0.026534,0.032847,0.001372,0.000907
giAhh81 ftgen 0,0,4096,10,1.000001,0.008470,0.005716,0.006895,0.001809,0.000528
giAhh84 ftgen 0,0,4096,10,0.665845,0.002904,0.007927,0.022486,0.000897

giwavemapAhh    ftgen    0,0,128,-27, 0,0, 24,0,  27,1,  30,2,  33,3,  36,4,  39,5, 42,6, 45,7, 48,8, 51,9, 54,10, 57,11, 60,12, 63,13, 66,14, 69,15, 72,16, 75,17, 78,18, 81,19, 84,20, 127,20
gitabs4morfAhh    ftgen    0,0,32,-2,giAhh24,giAhh27,giAhh30,giAhh33,giAhh36,giAhh39,giAhh42,giAhh45,giAhh48,giAhh51,giAhh54,giAhh57,giAhh60,giAhh63,giAhh66,giAhh69,giAhh72,giAhh75,giAhh78,giAhh81,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84,giAhh84


;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; OOH
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
/*
giOoh24 ftgen 0,0,4096,10,0.006736,0.007744,0.009173,0.011625,0.015301,0.019467,0.030031,0.040022,0.083847,0.111438,0.078627,0.038797,0.030293,0.020098,0.016517,0.013075,0.010944,0.010263,0.009904,0.012389,0.016364,0.013953,0.007558,0.004976,0.003673,0.002791,0.002294,0.001798,0.001446,0.001130,0.000839,0.000619,0.000412,0.000255,0.000142,0.000113,0.000132,0.000128,0.000119,0.000126,0.000173,0.000228,0.000299,0.000385,0.000495,0.000633,0.000806,0.001004,0.001164,0.001174,0.001025,0.000827,0.000650,0.000509,0.000398,0.000310,0.000240,0.000183,0.000137
giOoh27 ftgen 0,0,4096,10,0.008240,0.009736,0.012406,0.017177,0.022681,0.037033,0.060187,0.125058,0.110778,0.048765,0.035061,0.022345,0.017647,0.013782,0.012248,0.011790,0.016043,0.019429,0.011945,0.006214,0.004417,0.003235,0.002505,0.001898,0.001450,0.001023,0.000710,0.000427,0.000235,0.000103,0.000106,0.000137,0.000159,0.000148,0.000130,0.000131,0.000191,0.000267,0.000368,0.000496,0.000668,0.000893,0.001169,0.001399,0.001375,0.001123,0.000853,0.000637,0.000476,0.000354,0.000259,0.000186,0.000129,0.000104,0.000147,0.000202,0.000272,0.000362,0.000479,0.000623,0.000759,0.000789,0.000681,0.000530
giOoh30 ftgen 0,0,4096,10,0.010095,0.012529,0.017608,0.024663,0.043315,0.081558,0.157661,0.088611,0.046106,0.027324,0.020478,0.015548,0.014372,0.016229,0.023376,0.013886,0.006978,0.004439,0.003346,0.002384,0.001737,0.001149,0.000718,0.000372,0.000154,0.000123,0.000167,0.000187,0.000168,0.000125,0.000135,0.000212,0.000319,0.000466,0.000663,0.000940,0.001310,0.001651,0.001590,0.001211,0.000862,0.000610,0.000429,0.000296,0.000198,0.000125,0.000140,0.000209,0.000299,0.000421,0.000586,0.000792,0.000937,0.000852,0.000643,0.000461,0.000327,0.000231,0.000160,0.000107
giOoh33 ftgen 0,0,4096,10,0.012459,0.016728,0.025997,0.045900,0.091150,0.186461,0.074200,0.042360,0.026982,0.018939,0.017010,0.022371,0.024731,0.009432,0.005684,0.003945,0.002654,0.001756,0.001058,0.000513,0.000188,0.000154,0.000216,0.000210,0.000166,0.000171,0.000292,0.000464,0.000712,0.001081,0.001602,0.002011,0.001703,0.001162,0.000770,0.000507,0.000326,0.000198,0.000111,0.000102,0.000176,0.000278,0.000423,0.000631,0.000914,0.001124,0.000963,0.000669,0.000448,0.000297,0.000192,0.000119,0.000112
giOoh36 ftgen 0,0,4096,10,0.015493,0.023256,0.038946,0.080072,0.222961,0.077625,0.040215,0.026163,0.020537,0.024794,0.027925,0.009959,0.005586,0.003599,0.002262,0.001239,0.000511,0.000154,0.000227,0.000257,0.000200,0.000105,0.000183,0.000348,0.000602,0.000995,0.001621,0.002342,0.002063,0.001309,0.000802,0.000483,0.000275,0.000140,0.000124,0.000233,0.000396,0.000642,0.001014,0.001340,0.001090,0.000692,0.000426,0.000257,0.000148,0.000123,0.000130
giOoh39 ftgen 0,0,4096,10,0.019447,0.034315,0.073971,0.249826,0.097427,0.044636,0.027534,0.023553,0.038811,0.012411,0.006462,0.003791,0.002045,0.000853,0.000206,0.000273,0.000295,0.000187,0.000173,0.000381,0.000736,0.001334,0.002335,0.002747,0.001704,0.000952,0.000519,0.000258,0.000107,0.000208,0.000405,0.000725,0.001247,0.001582,0.001061,0.000602,0.000333,0.000173,0.000145,0.000152
giOoh42 ftgen 0,0,4096,10,0.025063,0.049337,0.163181,0.177222,0.054661,0.031107,0.032482,0.027778,0.008882,0.004771,0.002299,0.000744,0.000246,0.000371,0.000246,0.000109,0.000162,0.000428,0.000938,0.001893,0.003326,0.002439,0.001229,0.000596,0.000251,0.000103,0.000284,0.000607,0.001190,0.001900,0.001302,0.000662,0.000323,0.000141,0.000101,0.000172,0.000168
giOoh45 ftgen 0,0,4096,10,0.033454,0.091804,0.372905,0.084714,0.037879,0.044758,0.018864,0.007892,0.003513,0.001026,0.000308,0.000420,0.000196,0.000129,0.000187,0.000586,0.001429,0.003217,0.003417,0.001546,0.000653,0.000221,0.000103,0.000354,0.000853,0.001843,0.001938,0.000899,0.000386,0.000141,0.000147,0.000226,0.000135
giOoh48 ftgen 0,0,4096,10,0.046516,0.160169,0.155245,0.052329,0.049605,0.019920,0.007198,0.002478,0.000308,0.000509,0.000201,0.000135,0.000127,0.000370,0.001207,0.003248,0.004134,0.001606,0.000550,0.000125,0.000106,0.000468,0.001290,0.002692,0.001390,0.000516,0.000158,0.000174,0.000259,0.000118
giOoh51 ftgen 0,0,4096,10,0.068664,0.499897,0.089313,0.047133,0.024836,0.007586,0.001706,0.000547,0.000375,0.000132,0.000136,0.000347,0.001476,0.004681,0.003416,0.001040,0.000214,0.000172,0.000813,0.002504,0.002129,0.000667,0.000162,0.000186,0.000304,0.000115
giOoh54 ftgen 0,0,4096,10,0.098674,0.354398,0.062213,0.055545,0.009543,0.001487,0.000749,0.000134,0.000212,0.000123,0.000856,0.003788,0.004876,0.001194,0.000166,0.000569,0.002379,0.002608,0.000649,0.000109,0.000205,0.000338
giOoh57 ftgen 0,0,4096,10,0.183614,0.169417,0.089526,0.015785,0.002053,0.000839,0.000154,0.000122,0.000129,0.001172,0.006436,0.003091,0.000442,0.000113,0.000708,0.003688,0.001800,0.000283,0.000294,0.000271
giOoh60 ftgen 0,0,4096,10,0.320378,0.104663,0.039843,0.004956,0.001029,0.000257,0.000735,0.006501,0.003212,0.000252,0.000215,0.002579,0.002779,0.000315,0.000143,0.000522
giOoh63 ftgen 0,0,4096,10,1.000001,0.094284,0.015175,0.001094,0.000262,0.000162,0.002953,0.006834,0.000428,0.000345,0.005011,0.001335,0.000107,0.000114,0.000609
giOoh66 ftgen 0,0,4096,10,0.708750,0.111076,0.002977,0.000267,0.000246,0.007577,0.002388,0.000170,0.004759,0.001298,0.000408,0.000196
giOoh69 ftgen 0,0,4096,10,0.338803,0.031564,0.001678,0.000244,0.000258,0.012872,0.000885,0.001416,0.003598,0.000134,0.000588,0.000112
giOoh72 ftgen 0,0,4096,10,0.209313,0.009909,0.000515,0.000104,0.013000,0.000503,0.005157,0.000630,0.000287,0.000189
giOoh75 ftgen 0,0,4096,10,0.188584,0.002188,0.000154,0.005909,0.000855,0.010025,0.000214,0.001217
giOoh78 ftgen 0,0,4096,10,0.222149,0.000533,0.000493,0.004776,0.009517,0.000194,0.000393
giOoh81 ftgen 0,0,4096,10,0.063129,0.000488,0.025748,0.002834,0.000268,0.000223
giOoh84 ftgen 0,0,4096,10,0.019817,0.000207,0.001007,0.001261,0.000379
giwavemapOoh    ftgen    0,0,128,-27, 0,0, 24,0,  27,1,  30,2,  33,3,  36,4,  39,5, 42,6, 45,7, 48,8, 51,9, 54,10, 57,11, 60,12, 63,13, 66,14, 69,15, 72,16, 75,17, 78,18, 81,19, 84,20, 127,20
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfOoh    ftgen    0,0,32,-2,giOoh24,giOoh27,giOoh30,giOoh33,giOoh36,giOoh39,giOoh42,giOoh45,giOoh48,giOoh51,giOoh54,giOoh57,giOoh60,giOoh63,giOoh66,giOoh69,giOoh72,giOoh75,giOoh78,giOoh81,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84
*/

; mixed voice types
giOoh24 ftgen 0,0,4096,10,0.002656,0.003252,0.004105,0.005622,0.007160,0.009459,0.013108,0.015856,0.025894,0.031448,0.069195,0.117293,0.097983,0.042979,0.032065,0.021929,0.018078,0.015475,0.014066,0.014550,0.016289,0.021526,0.023582,0.017583,0.010850,0.007974,0.005821,0.004584,0.003615,0.002841,0.002234,0.001707,0.001268,0.000905,0.000600,0.000361,0.000186,0.000124,0.000163,0.000187,0.000186,0.000166,0.000139,0.000101,0.000100,0.000137,0.000189,0.000263,0.000361,0.000489,0.000654,0.000858,0.001117,0.001451,0.001869,0.002471,0.003247,0.004440,0.005393,0.004629,0.002917,0.002066,0.002532,0.003852,0.005391,0.005792,0.004814,0.003675,0.002842,0.002214,0.001763,0.001401,0.001091,0.000746,0.000345,0.000127,0.000165,0.000194,0.000194,0.000185,0.000171,0.000154,0.000137,0.000119,0.000101
giOoh27 ftgen 0,0,4096,10,0.003258,0.004239,0.005824,0.008031,0.011084,0.016213,0.021193,0.035813,0.061897,0.136532,0.111390,0.042393,0.033786,0.022028,0.018961,0.016732,0.017591,0.021496,0.028305,0.022987,0.013061,0.009039,0.006327,0.004802,0.003617,0.002714,0.001972,0.001369,0.000897,0.000513,0.000245,0.000108,0.000159,0.000210,0.000226,0.000203,0.000166,0.000112,0.000138,0.000201,0.000297,0.000435,0.000622,0.000874,0.001199,0.001639,0.002216,0.003089,0.004305,0.006101,0.005978,0.003630,0.002459,0.003410,0.005563,0.006988,0.005953,0.004329,0.003190,0.002395,0.001829,0.001373,0.000891,0.000334,0.000160,0.000222,0.000234,0.000222,0.000202,0.000178,0.000153,0.000127,0.000102
giOoh30 ftgen 0,0,4096,10,0.004023,0.005567,0.008482,0.012147,0.018935,0.027436,0.043784,0.124923,0.156060,0.054359,0.038530,0.025628,0.020731,0.020302,0.024164,0.033681,0.024398,0.013158,0.008555,0.006066,0.004329,0.003073,0.002076,0.001307,0.000716,0.000311,0.000128,0.000209,0.000264,0.000252,0.000205,0.000130,0.000103,0.000161,0.000252,0.000400,0.000621,0.000939,0.001374,0.001992,0.002862,0.004244,0.006522,0.007371,0.004231,0.002979,0.004965,0.007920,0.007484,0.005169,0.003596,0.002580,0.001862,0.001227,0.000441,0.000198,0.000273,0.000271,0.000248,0.000214,0.000179,0.000142,0.000109
giOoh33 ftgen 0,0,4096,10,0.005059,0.007690,0.012187,0.020150,0.031282,0.054249,0.186526,0.120147,0.052781,0.030786,0.024379,0.024804,0.034918,0.035401,0.016807,0.010063,0.006707,0.004475,0.002912,0.001727,0.000868,0.000313,0.000184,0.000294,0.000312,0.000251,0.000145,0.000162,0.000275,0.000476,0.000802,0.001297,0.002024,0.003113,0.004960,0.008196,0.007816,0.003687,0.004946,0.009187,0.008701,0.005588,0.003662,0.002498,0.001599,0.000503,0.000264,0.000332,0.000310,0.000265,0.000215,0.000163,0.000117
giOoh36 ftgen 0,0,4096,10,0.006425,0.011190,0.018900,0.031763,0.063063,0.234533,0.085077,0.042498,0.028723,0.024364,0.029248,0.052657,0.032828,0.016019,0.009256,0.005473,0.003020,0.001411,0.000472,0.000289,0.000385,0.000324,0.000169,0.000209,0.000292,0.000313,0.000363,0.000715,0.001488,0.002773,0.004783,0.007968,0.013450,0.024276,0.028174,0.012256,0.017548,0.027382,0.016871,0.007639,0.001429,0.003903,0.002935,0.001946,0.001251,0.000808,0.000539,0.000371,0.000260,0.000189,0.000148,0.000125,0.000105
giOoh39 ftgen 0,0,4096,10,0.008376,0.015990,0.032397,0.071647,0.272662,0.083705,0.042399,0.029965,0.031725,0.060042,0.036024,0.015995,0.008610,0.004457,0.001931,0.000539,0.000377,0.000438,0.000278,0.000188,0.000329,0.000372,0.000455,0.001070,0.002432,0.004805,0.008844,0.016499,0.033376,0.023715,0.015683,0.032489,0.020044,0.007073,0.004006,0.004073,0.002568,0.001523,0.000908,0.000566,0.000367,0.000246,0.000180,0.000147,0.000118
giOoh42 ftgen 0,0,4096,10,0.011034,0.024255,0.054958,0.249960,0.107468,0.049168,0.034461,0.048258,0.060049,0.020609,0.009752,0.004351,0.001387,0.000382,0.000531,0.000321,0.000134,0.000183,0.000390,0.000442,0.000696,0.002019,0.004800,0.010033,0.021149,0.044106,0.017250,0.033814,0.026431,0.007582,0.005537,0.004027,0.002216,0.001191,0.000676,0.000405,0.000256,0.000192,0.000151,0.000111
giOoh45 ftgen 0,0,4096,10,0.015273,0.040290,0.108755,0.238831,0.059213,0.041067,0.079700,0.037954,0.014459,0.005681,0.001470,0.000525,0.000572,0.000143,0.000149,0.000276,0.000507,0.000599,0.001942,0.005592,0.013421,0.033213,0.035928,0.030674,0.033762,0.006591,0.006489,0.003618,0.001724,0.000862,0.000468,0.000280,0.000206,0.000147,0.000101
giOoh48 ftgen 0,0,4096,10,0.010601,0.034815,0.190470,0.080405,0.033305,0.058053,0.024910,0.008255,0.002415,0.000380,0.000499,0.000157,0.000134,0.000544,0.001534,0.004269,0.007067,0.006352,0.013410,0.005008,0.002977,0.001313,0.000644,0.000282
giOoh51 ftgen 0,0,4096,10,0.016431,0.063467,0.235366,0.043971,0.063965,0.024873,0.006978,0.001106,0.000633,0.000237,0.000165,0.000526,0.001871,0.006744,0.006491,0.013161,0.007320,0.003558,0.001368,0.000593,0.000178
giOoh54 ftgen 0,0,4096,10,0.024710,0.140863,0.075949,0.054103,0.032225,0.006828,0.000565,0.000591,0.000130,0.000122,0.000130,0.001184,0.004810,0.008308,0.016195,0.006829,0.003189,0.000995,0.000290
giOoh57 ftgen 0,0,4096,10,0.039150,0.500037,0.055380,0.060924,0.008930,0.000650,0.000410,0.000244,0.001590,0.009065,0.009812,0.013291,0.004577,0.001153,0.000235,0.000102
giOoh60 ftgen 0,0,4096,10,0.086407,0.267040,0.196788,0.021563,0.001095,0.000450,0.000254,0.000578,0.012590,0.058595,0.005142,0.009927,0.005483,0.000502,0.000175,0.000149,0.000775
giOoh63 ftgen 0,0,4096,10,0.153433,0.155941,0.058873,0.002749,0.000677,0.000168,0.000395,0.004570,0.084026,0.008181,0.014008,0.003788,0.000261,0.000109,0.000887
giOoh66 ftgen 0,0,4096,10,0.332841,0.193621,0.018082,0.001777,0.000274,0.000623,0.014262,0.036353,0.007910,0.005829,0.000322,0.000104,0.000472,0.000204
giOoh69 ftgen 0,0,4096,10,1.000000,0.136629,0.001978,0.000708,0.000666,0.025888,0.015536,0.029450,0.000535,0.000119,0.000710,0.000111
giOoh72 ftgen 0,0,4096,10,0.528638,0.035156,0.000814,0.000113,0.012612,0.005146,0.027163,0.002875,0.000576,0.000345
giOoh75 ftgen 0,0,4096,10,0.295700,0.004617,0.000416,0.004518,0.008174,0.059391,0.000819,0.003130
giOoh78 ftgen 0,0,4096,10,0.324009,0.003159,0.000539,0.036354,0.052795,0.000895,0.000729
giOoh81 ftgen 0,0,4096,10,0.212624,0.001375,0.025936,0.013708,0.001164,0.000397
giOoh84 ftgen 0,0,4096,10,0.070310,0.000226,0.010293,0.005749,0.000690
giwavemapOoh    ftgen    0,0,128,-27, 0,0, 24,0,  27,1,  30,2,  33,3,  36,4,  39,5, 42,6, 45,7, 48,8, 51,9, 54,10, 57,11, 60,12, 63,13, 66,14, 69,15, 72,16, 75,17, 78,18, 81,19, 84,20, 127,20
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfOoh    ftgen    0,0,32,-2,giOoh24,giOoh27,giOoh30,giOoh33,giOoh36,giOoh39,giOoh42,giOoh45,giOoh48,giOoh51,giOoh54,giOoh57,giOoh60,giOoh63,giOoh66,giOoh69,giOoh72,giOoh75,giOoh78,giOoh81,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84,giOoh84
    

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Eee
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
/*
giEee24 ftgen 0,0,4096,10, 0.010612,0.014495,0.019809,0.028477,0.041398,0.063942,0.114518,0.124942,0.078855,0.046228,0.033959,0.023394,0.018143,0.013375,0.010099,0.007577,0.005431,0.003901,0.002611,0.001632,0.000932,0.000386,0.000171,0.000289,0.000422,0.000452,0.000449,0.000398,0.000315,0.000237,0.000144,0.000104,0.000105,0.000153,0.000215,0.000290,0.000380,0.000487,0.000633,0.000816,0.001099,0.001507,0.002203,0.003126,0.003165,0.002277,0.001567,0.001153,0.000855,0.000658,0.000500,0.000378,0.000277,0.000186,0.000102,0.000154,0.000300,0.000490,0.000733,0.001033,0.001413,0.001885,0.002475,0.003256,0.004250,0.005714,0.007781,0.011083,0.015021,0.015148,0.011378,0.008165,0.006212,0.004895,0.004095,0.003568,0.003306,0.003268,0.003450,0.003938,0.004750,0.006020,0.007017,0.006500,0.005013,0.003837,0.003082,0.002641,0.002479,0.002562
giEee27 ftgen 0,0,4096,10, 0.013361,0.019632,0.028170,0.046001,0.073446,0.142620,0.135320,0.067419,0.045253,0.028671,0.021074,0.014727,0.010611,0.007252,0.004920,0.003071,0.001747,0.000807,0.000249,0.000303,0.000500,0.000540,0.000520,0.000422,0.000309,0.000180,0.000119,0.000127,0.000103,0.000168,0.000256,0.000366,0.000497,0.000673,0.000915,0.001288,0.001877,0.003013,0.003942,0.003063,0.001933,0.001339,0.000949,0.000692,0.000498,0.000349,0.000219,0.000102,0.000105,0.000283,0.000531,0.000867,0.001300,0.001875,0.002609,0.003613,0.004968,0.007064,0.010239,0.015759,0.018744,0.014260,0.009604,0.006959,0.005391,0.004464,0.003989,0.003908,0.004207,0.005028,0.006490,0.008241,0.007854,0.005766,0.004257,0.003368,0.002997,0.003039,0.003534,0.003719,0.002952,0.002107,0.001565,0.001195,0.000929,0.000719,0.000551,0.000410,0.000295,0.000202,0.000133
giEee30 ftgen 0,0,4096,10, 0.016942,0.026728,0.044993,0.073582,0.166083,0.150187,0.067471,0.042023,0.027730,0.018157,0.012270,0.007756,0.004771,0.002544,0.001124,0.000288,0.000418,0.000628,0.000639,0.000532,0.000370,0.000189,0.000133,0.000151,0.000118,0.000167,0.000288,0.000442,0.000634,0.000915,0.001325,0.002073,0.003575,0.004636,0.003020,0.001869,0.001215,0.000836,0.000565,0.000361,0.000186,0.000308,0.000658,0.001154,0.001829,0.002760,0.004060,0.005937,0.009069,0.014428,0.021876,0.018429,0.011479,0.007850,0.005939,0.004916,0.004612,0.004928,0.006092,0.008369,0.009952,0.007626,0.005224,0.003945,0.003510,0.003799,0.004447,0.003710,0.002491,0.001752,0.001285,0.000950,0.000696,0.000493,0.000332,0.000209,0.000125
giEee33 ftgen 0,0,4096,10, 0.021903,0.037076,0.070524,0.170727,0.184856,0.075853,0.042006,0.026533,0.016384,0.009694,0.005423,0.002512,0.000745,0.000367,0.000734,0.000760,0.000589,0.000358,0.000113,0.000157,0.000177,0.000123,0.000164,0.000331,0.000549,0.000839,0.001298,0.002144,0.003934,0.005481,0.003133,0.001834,0.001150,0.000725,0.000430,0.000185,0.000178,0.000585,0.001216,0.002131,0.003468,0.005492,0.008727,0.014780,0.025548,0.021159,0.012320,0.008120,0.006189,0.005514,0.005907,0.007722,0.011241,0.010580,0.006710,0.004738,0.004184,0.004875,0.005074,0.003298,0.002150,0.001485,0.001035,0.000705,0.000453,0.000265,0.000143
giEee36 ftgen 0,0,4096,10, 0.028999,0.056971,0.127925,0.249973,0.092491,0.046808,0.026763,0.015161,0.007806,0.003265,0.000772,0.000577,0.000905,0.000797,0.000474,0.000128,0.000118,0.000208,0.000175,0.000190,0.000430,0.000762,0.001269,0.002203,0.004418,0.006347,0.003143,0.001714,0.001003,0.000555,0.000205,0.000308,0.000984,0.002075,0.003788,0.006543,0.011486,0.022285,0.030464,0.016426,0.009849,0.007181,0.006579,0.007931,0.012129,0.013098,0.007735,0.005325,0.005167,0.006343,0.004251,0.002537,0.001641,0.001065,0.000654,0.000360,0.000176,0.000109,0.000108
giEee39 ftgen 0,0,4096,10, 0.039208,0.091871,0.284850,0.134665,0.057264,0.029417,0.014487,0.006136,0.001614,0.000605,0.001080,0.000843,0.000360,0.000237,0.000205,0.000101,0.000336,0.000730,0.001345,0.002573,0.006021,0.006121,0.002677,0.001384,0.000697,0.000205,0.000565,0.001734,0.003749,0.007227,0.014133,0.031533,0.028537,0.013927,0.008934,0.007821,0.010066,0.016499,0.011547,0.006746,0.006088,0.007451,0.004223,0.002395,0.001442,0.000822,0.000405,0.000174,0.000127,0.000124
giEee42 ftgen 0,0,4096,10, 0.053470,0.147208,0.300415,0.084059,0.036324,0.015517,0.005091,0.000576,0.001256,0.001064,0.000377,0.000160,0.000303,0.000150,0.000341,0.000891,0.001840,0.004161,0.009292,0.003746,0.001676,0.000724,0.000174,0.001325,0.003681,0.008171,0.018259,0.044043,0.023100,0.011960,0.009296,0.012291,0.020077,0.010540,0.007087,0.008984,0.005033,0.002599,0.001411,0.000675,0.000255,0.000150,0.000143
giEee45 ftgen 0,0,4096,10, 0.074148,0.341465,0.151691,0.053062,0.019387,0.005023,0.000735,0.001519,0.000715,0.000141,0.000355,0.000141,0.000664,0.001681,0.004296,0.010976,0.003672,0.001452,0.000370,0.001174,0.004275,0.011021,0.029674,0.042441,0.016298,0.011074,0.015522,0.021253,0.009522,0.009803,0.006632,0.002990,0.001422,0.000537,0.000189,0.000178,0.000110
giEee48 ftgen 0,0,4096,10, 0.113952,0.499961,0.093618,0.030324,0.006532,0.001153,0.001593,0.000254,0.000416,0.000182,0.000869,0.002549,0.008846,0.006282,0.002003,0.000401,0.001977,0.007592,0.023020,0.061020,0.019732,0.013187,0.024324,0.015505,0.010364,0.008518,0.003290,0.001312,0.000353,0.000214,0.000143
giEee51 ftgen 0,0,4096,10, 0.183849,0.269450,0.058864,0.012277,0.001210,0.001686,0.000115,0.000411,0.000673,0.002694,0.012061,0.005358,0.001397,0.001131,0.007512,0.028323,0.057149,0.017903,0.020182,0.023136,0.012207,0.008465,0.002892,0.000813,0.000255,0.000186
giEee54 ftgen 0,0,4096,10, 0.294425,0.168105,0.031031,0.001152,0.002129,0.000319,0.000295,0.001773,0.008324,0.007496,0.001451,0.002648,0.016348,0.088119,0.023921,0.024596,0.021078,0.017984,0.005195,0.001344,0.000303,0.000185,0.000105
giEee57 ftgen 0,0,4096,10, 0.682961,0.106120,0.010046,0.003039,0.000281,0.000282,0.003363,0.021952,0.002902,0.002350,0.022051,0.084871,0.022153,0.042507,0.019617,0.005980,0.001075,0.000355,0.000117
giEee60 ftgen 0,0,4096,10, 0.999999,0.060648,0.002312,0.000510,0.000354,0.005083,0.012579,0.000821,0.015185,0.122052,0.026381,0.031005,0.017044,0.002622,0.000436,0.000138
giEee63 ftgen 0,0,4096,10, 0.538975,0.024555,0.003372,0.000823,0.005390,0.010718,0.002266,0.056680,0.035812,0.046282,0.016933,0.001625,0.000371,0.000151
giEee66 ftgen 0,0,4096,10, 0.336196,0.002303,0.000636,0.003546,0.014990,0.005298,0.176248,0.049197,0.035968,0.002688,0.000369,0.000111
giEee69 ftgen 0,0,4096,10, 0.212220,0.006076,0.000565,0.043897,0.004701,0.169692,0.084989,0.011954,0.000713,0.000170
giEee72 ftgen 0,0,4096,10, 0.121291,0.001021,0.010166,0.001642,0.244060,0.061995,0.005244,0.000275,0.000113
giEee75 ftgen 0,0,4096,10, 0.049112,0.001647,0.021440,0.113388,0.092568,0.003249,0.000304,0.000100
giEee78 ftgen 0,0,4096,10, 0.004609,0.007093,0.010596,0.098393,0.005375,0.000222
giEee81 ftgen 0,0,4096,10, 0.012152,0.087798,0.339376,0.023913,0.000340
giEee84 ftgen 0,0,4096,10, 0.002039,0.003283,0.123990,0.000551,0.000112
giwavemapEee    ftgen    0,0,128,-27, 0,0, 24,0,  27,1,  30,2,  33,3,  36,4,  39,5, 42,6, 45,7, 48,8, 51,9, 54,10, 57,11, 60,12, 63,13, 66,14, 69,15, 72,16, 75,17, 78,18, 81,19, 84,20, 127,20
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfEee    ftgen    0,0,32,-2,giEee24,giEee27,giEee30,giEee33,giEee36,giEee39,giEee42,giEee45,giEee48,giEee51,giEee54,giEee57,giEee60,giEee63,giEee66,giEee69,giEee72,giEee75,giEee78,giEee81,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84
*/

; mixed voice types
giEee24 ftgen 0,0,4096,10,0.010612,0.014495,0.019809,0.028477,0.041398,0.063942,0.114518,0.124942,0.078855,0.046228,0.033959,0.023394,0.018143,0.013375,0.010099,0.007577,0.005431,0.003901,0.002611,0.001632,0.000932,0.000386,0.000171,0.000289,0.000422,0.000452,0.000449,0.000398,0.000315,0.000237,0.000144,0.000104,0.000105,0.000153,0.000215,0.000290,0.000380,0.000487,0.000633,0.000816,0.001099,0.001507,0.002203,0.003126,0.003165,0.002277,0.001567,0.001153,0.000855,0.000658,0.000500,0.000378,0.000277,0.000186,0.000102,0.000154,0.000300,0.000490,0.000733,0.001033,0.001413,0.001885,0.002475,0.003256,0.004250,0.005714,0.007781,0.011083,0.015021,0.015148,0.011378,0.008165,0.006212,0.004895,0.004095,0.003568,0.003306,0.003268,0.003450,0.003938,0.004750,0.006020,0.007017,0.006500,0.005013,0.003837,0.003082,0.002641,0.002479,0.002562
giEee27 ftgen 0,0,4096,10,0.013361,0.019632,0.028170,0.046001,0.073446,0.142620,0.135320,0.067419,0.045253,0.028671,0.021074,0.014727,0.010611,0.007252,0.004920,0.003071,0.001747,0.000807,0.000249,0.000303,0.000500,0.000540,0.000520,0.000422,0.000309,0.000180,0.000119,0.000127,0.000103,0.000168,0.000256,0.000366,0.000497,0.000673,0.000915,0.001288,0.001877,0.003013,0.003942,0.003063,0.001933,0.001339,0.000949,0.000692,0.000498,0.000349,0.000219,0.000102,0.000105,0.000283,0.000531,0.000867,0.001300,0.001875,0.002609,0.003613,0.004968,0.007064,0.010239,0.015759,0.018744,0.014260,0.009604,0.006959,0.005391,0.004464,0.003989,0.003908,0.004207,0.005028,0.006490,0.008241,0.007854,0.005766,0.004257,0.003368,0.002997,0.003039,0.003534,0.003719,0.002952,0.002107,0.001565,0.001195,0.000929,0.000719,0.000551,0.000410,0.000295,0.000202,0.000133
giEee30 ftgen 0,0,4096,10,0.016942,0.026728,0.044993,0.073582,0.166083,0.150187,0.067471,0.042023,0.027730,0.018157,0.012270,0.007756,0.004771,0.002544,0.001124,0.000288,0.000418,0.000628,0.000639,0.000532,0.000370,0.000189,0.000133,0.000151,0.000118,0.000167,0.000288,0.000442,0.000634,0.000915,0.001325,0.002073,0.003575,0.004636,0.003020,0.001869,0.001215,0.000836,0.000565,0.000361,0.000186,0.000308,0.000658,0.001154,0.001829,0.002760,0.004060,0.005937,0.009069,0.014428,0.021876,0.018429,0.011479,0.007850,0.005939,0.004916,0.004612,0.004928,0.006092,0.008369,0.009952,0.007626,0.005224,0.003945,0.003510,0.003799,0.004447,0.003710,0.002491,0.001752,0.001285,0.000950,0.000696,0.000493,0.000332,0.000209,0.000125
giEee33 ftgen 0,0,4096,10,0.021903,0.037076,0.070524,0.170727,0.184856,0.075853,0.042006,0.026533,0.016384,0.009694,0.005423,0.002512,0.000745,0.000367,0.000734,0.000760,0.000589,0.000358,0.000113,0.000157,0.000177,0.000123,0.000164,0.000331,0.000549,0.000839,0.001298,0.002144,0.003934,0.005481,0.003133,0.001834,0.001150,0.000725,0.000430,0.000185,0.000178,0.000585,0.001216,0.002131,0.003468,0.005492,0.008727,0.014780,0.025548,0.021159,0.012320,0.008120,0.006189,0.005514,0.005907,0.007722,0.011241,0.010580,0.006710,0.004738,0.004184,0.004875,0.005074,0.003298,0.002150,0.001485,0.001035,0.000705,0.000453,0.000265,0.000143
giEee36 ftgen 0,0,4096,10,0.018039,0.029693,0.059894,0.185737,0.154314,0.055778,0.027787,0.014854,0.007935,0.003897,0.001432,0.000187,0.000561,0.000590,0.000401,0.000185,0.000117,0.000134,0.000109,0.000405,0.000942,0.001848,0.003332,0.005831,0.010609,0.022023,0.025121,0.011959,0.006463,0.003691,0.002042,0.001000,0.000346,0.000540,0.001193,0.002221,0.003859,0.006735,0.012934,0.018831,0.010505,0.006536,0.005174,0.005305,0.007021,0.011338,0.013114,0.007641,0.004738,0.003619,0.003554,0.002391,0.001498,0.001024,0.000703,0.000458,0.000268,0.000136
giEee39 ftgen 0,0,4096,10,0.023171,0.040165,0.103046,0.249899,0.069009,0.030296,0.014964,0.006837,0.002317,0.000262,0.000693,0.000670,0.000375,0.000134,0.000157,0.000234,0.000809,0.001919,0.003902,0.007563,0.015739,0.033909,0.016874,0.008020,0.004127,0.001993,0.000762,0.000560,0.001472,0.003025,0.005784,0.011760,0.022871,0.012484,0.007312,0.006012,0.007155,0.011847,0.015854,0.008455,0.005033,0.004257,0.003348,0.001866,0.001182,0.000751,0.000433,0.000208
giEee42 ftgen 0,0,4096,10,0.030585,0.075466,0.326397,0.089842,0.037631,0.016293,0.006004,0.001189,0.000623,0.000819,0.000421,0.000187,0.000122,0.000220,0.000997,0.002690,0.006079,0.013783,0.037543,0.020537,0.008608,0.003873,0.001483,0.000243,0.000604,0.001913,0.004382,0.009617,0.024507,0.016361,0.008552,0.007239,0.010299,0.019396,0.011363,0.006002,0.005105,0.003117,0.001699,0.001004,0.000533,0.000219,0.000103,0.000108
giEee45 ftgen 0,0,4096,10,0.042426,0.110844,0.240398,0.052093,0.020251,0.006042,0.000491,0.000993,0.000619,0.000230,0.000590,0.002336,0.006356,0.016818,0.048133,0.016906,0.006376,0.002233,0.000321,0.000910,0.003109,0.007901,0.022561,0.020975,0.009763,0.009076,0.016934,0.017670,0.007601,0.006066,0.003146,0.001615,0.000823,0.000318,0.000124,0.000127
giEee48 ftgen 0,0,4096,10,0.067500,0.499957,0.084236,0.026845,0.006246,0.000524,0.001260,0.000307,0.000263,0.000154,0.000423,0.001657,0.005126,0.020030,0.007311,0.002320,0.000560,0.000313,0.001681,0.005251,0.018793,0.008143,0.003428,0.003199,0.002838,0.002897,0.002582,0.000962,0.000359
giEee51 ftgen 0,0,4096,10,0.114392,0.346891,0.051686,0.011497,0.000495,0.001393,0.000319,0.000278,0.001711,0.006671,0.024100,0.005070,0.001207,0.000395,0.002611,0.010148,0.016063,0.004678,0.003757,0.002818,0.004738,0.001749,0.000577,0.000121
giEee54 ftgen 0,0,4096,10,0.184699,0.170704,0.027773,0.001277,0.001722,0.000135,0.000260,0.000954,0.005364,0.030213,0.005416,0.000846,0.000161,0.001816,0.009486,0.018223,0.004741,0.005826,0.003571,0.002699,0.000709,0.000111
giEee57 ftgen 0,0,4096,10,0.375525,0.096426,0.009561,0.002227,0.000108,0.000215,0.002089,0.015287,0.010294,0.001315,0.000212,0.003544,0.026847,0.007999,0.006168,0.004412,0.002362,0.000444,0.000109
giEee60 ftgen 0,0,4096,10,0.227192,0.099004,0.005244,0.001344,0.001073,0.017641,0.017350,0.001409,0.000721,0.009172,0.004060,0.000250,0.000806,0.006918,0.001617,0.000117,0.000126,0.000462
giEee63 ftgen 0,0,4096,10,0.603091,0.048190,0.002514,0.000479,0.018496,0.015302,0.000391,0.003639,0.009291,0.000469,0.001181,0.008347,0.000679,0.000111,0.000537
giEee66 ftgen 0,0,4096,10,1.000000,0.014894,0.000748,0.011456,0.020880,0.000335,0.010283,0.002934,0.000513,0.010980,0.000625,0.000358,0.000173
giEee69 ftgen 0,0,4096,10,0.383427,0.002256,0.002169,0.058726,0.000416,0.019191,0.001023,0.003871,0.002038,0.000111,0.000515,0.000103
giEee72 ftgen 0,0,4096,10,0.143330,0.001607,0.002011,0.288307,0.002276,0.014994,0.010998,0.007299,0.001333,0.000996
giEee75 ftgen 0,0,4096,10,0.061863,0.001803,0.041574,0.018718,0.022519,0.023699,0.001207,0.009907,0.000135
giEee78 ftgen 0,0,4096,10,0.008237,0.002981,0.275939,0.109772,0.021334,0.000832,0.002147
giEee81 ftgen 0,0,4096,10,0.012164,0.019280,0.007395,0.003926,0.001069,0.001101
giEee84 ftgen 0,0,4096,10,0.003212,0.576616,0.029989,0.014597,0.001991
giwavemapEee    ftgen    0,0,128,-27, 0,0, 24,0,  27,1,  30,2,  33,3,  36,4,  39,5, 42,6, 45,7, 48,8, 51,9, 54,10, 57,11, 60,12, 63,13, 66,14, 69,15, 72,16, 75,17, 78,18, 81,19, 84,20, 127,20
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
gitabs4morfEee    ftgen    0,0,32,-2,giEee24,giEee27,giEee30,giEee33,giEee36,giEee39,giEee42,giEee45,giEee48,giEee51,giEee54,giEee57,giEee60,giEee63,giEee66,giEee69,giEee72,giEee75,giEee78,giEee81,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84,giEee84




;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; HORN P
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giHornP38 ftgen 0,0,4096,10,0.045737,0.148188,0.335691,0.493464,0.710665,0.765723,0.694286,0.628629,0.521398,0.428206,0.365416,0.306040,0.298023,0.254152,0.201721,0.154086,0.161771,0.146790,0.124044,0.110268,0.091099,0.076808,0.057743,0.042724,0.042724
giHornP50 ftgen 0,0,4096,10,0.280951,0.717837,0.823404,0.588908,0.394696,0.286430,0.176355,0.118365,0.064524,0.033209,0.016001,0.004104,0.004396,0.002069,0.002350,0.001903,0.001098,0.001262,0.001129,0.000906,0.000543,0.000575,0.000427,0.000355,0.000355
giHornP62 ftgen 0,0,4096,10,0.495327,0.909763,0.613398,0.368832,0.202553,0.124640,0.046586,0.011810,0.012510,0.011370,0.006659,0.004564,0.002331,0.001790,0.000667,0.000538,0.000352,0.000302,0.000558,0.000794,0.000797,0.000331,0.000342,0.000389,0.000389
giHornP74 ftgen 0,0,4096,10,0.796534,0.226262,0.060526,0.013273,0.003161,0.001015,0.001071,0.000595,0.000191,0.000337,0.000199,0.000173,0.000183,0.000187,0.000105,0.000113,0.000156,0.000119,0.000098,0.000067,0.000066,0.000064,0.000054,0.000043,0.000043
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
giwavemapHornP    ftgen    0,0,128,-27, 0,0, 38,0,  50,1,  62,2,  74,3,  127,3
gitabs4morfHornP    ftgen    0,0,4,-2,giHornP38,giHornP50,giHornP62,giHornP74

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; HORN F
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giHornF38 ftgen 0,0,4096,10,0.150333,0.205756,0.869097,0.723941,0.707935,0.116871,0.238125,0.330962,0.443031,0.325212,0.131256,0.267361,0.273576,0.146568,0.209758,0.158564,0.179271,0.084668,0.112177,0.118845,0.124993,0.096945,0.128906,0.125534,0.125534
giHornF50 ftgen 0,0,4096,10,0.348154,0.319965,0.418661,0.396206,0.046368,0.198167,0.035091,0.107400,0.035358,0.066696,0.053722,0.045581,0.066494,0.109943,0.059354,0.050001,0.123615,0.052990,0.032207,0.039448,0.051629,0.014079,0.014571,0.017523,0.017523
giHornF62 ftgen 0,0,4096,10,0.449507,0.544173,0.108516,0.300530,0.163030,0.351952,0.171674,0.202505,0.098101,0.066418,0.044492,0.071036,0.074224,0.064911,0.027430,0.041991,0.022245,0.017689,0.013130,0.020853,0.016891,0.038363,0.017524,0.016841,0.016841
giHornF74 ftgen 0,0,4096,10,0.923831,0.259141,0.262670,0.175552,0.089346,0.073056,0.055113,0.031968,0.027964,0.033656,0.035303,0.010920,0.019420,0.013447,0.012944,0.006111,0.007041,0.003864,0.005702,0.004615,0.003076,0.001589,0.002756,0.001234,0.001234
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
giwavemapHornF    ftgen    0,0,128,-27, 0,0, 38,0,  50,1,  62,2,  74,3,  127,3
gitabs4morfHornF    ftgen    0,0,4,-2,giHornF38,giHornF50,giHornF62,giHornF74

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; BASS TROMBONE (HARMON MUTE)
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giharmon33 ftgen 0,0,4096,10,0.875348,0.155770,0.111939,0.181031,0.355046,0.044216,0.039025,0.036422,0.074587,0.050492,0.042464,0.079868,0.121542,0.243299,0.253642,0.263571,0.422410,0.624259,0.742333,0.514304,0.760971,0.444426,0.456583,0.195508,0.070084,0.032239,0.029621,0.058523,0.044442,0.057324,0.077997,0.047218,0.046215,0.023786,0.017859,0.017652,0.012055,0.009595,0.009020,0.006000,0.004077,0.004950,0.004543,0.003196,0.002659,0.003533,0.004156,0.003745,0.002471,0.002264,0.003050,0.002614,0.001944,0.001709,0.001717,0.001934,0.002458,0.003618,0.004915,0.004334,0.003733,0.003804,0.003489,0.002755,0.002427,0.002273,0.002577,0.002702,0.002483,0.002059,0.002016,0.002214,0.002215,0.001947,0.001952,0.002236,0.002357,0.002417,0.002406,0.002309,0.002176,0.002227,0.002527,0.002649,0.002448,0.002165,0.002175,0.002322,0.002371,0.002202,0.001845,0.001620,0.001626,0.001927,0.002537,0.003091,0.003220,0.003231,0.003453,0.003751
giharmon34 ftgen 0,0,4096,10,0.443604,0.040301,0.041393,0.115682,0.622143,0.055550,0.073493,0.090539,0.109410,0.039052,0.138611,0.161174,0.290931,0.212197,0.410393,0.749789,0.917784,1.375368,0.934044,0.781551,0.356416,0.290222,0.181279,0.061983,0.081959,0.039112,0.044805,0.058922,0.022274,0.022731,0.038966,0.025063,0.031263,0.030753,0.032383,0.045020,0.032652,0.027144,0.033411,0.020397,0.015121,0.017071,0.014804,0.013636,0.012149,0.012930,0.014231,0.012821,0.011342,0.009001,0.008587,0.006187,0.005928,0.008836,0.012118,0.012242,0.009968,0.009445,0.006852,0.004519,0.002870,0.001975,0.001680,0.001593,0.001930,0.002744,0.003798,0.003938,0.003311,0.003174,0.003465,0.004740,0.006267,0.006620,0.006071,0.005274,0.005318,0.005795,0.006440,0.007152,0.007792,0.008192,0.007304,0.005765,0.004397,0.003369,0.002855,0.003059,0.003731,0.004377,0.004923,0.005763,0.006567,0.006834,0.006780,0.006575,0.005922,0.004714,0.003481,0.002696
giharmon36 ftgen 0,0,4096,10,0.934870,0.094639,0.263729,0.103300,0.556951,0.047384,0.167404,0.119804,0.122737,0.048833,0.154640,0.111318,0.302046,0.230750,0.517436,0.259307,0.354474,0.152230,0.109031,0.091431,0.071065,0.027335,0.037383,0.048534,0.033460,0.066084,0.041563,0.049933,0.029361,0.032804,0.017292,0.018054,0.023006,0.019576,0.018461,0.008645,0.007040,0.005924,0.005415,0.003713,0.003447,0.002816,0.003594,0.003583,0.002658,0.001561,0.001446,0.002347,0.003707,0.005343,0.007152,0.006970,0.005630,0.004404,0.002749,0.002098,0.002393,0.001708,0.001255,0.001150,0.001263,0.001445,0.001397,0.001331,0.001510,0.001783,0.001849,0.001576,0.001508,0.001400,0.001153,0.000940,0.000797,0.000734,0.000677,0.000581,0.000531,0.000644,0.000829,0.000891,0.000846,0.000864,0.000921,0.000980,0.000945,0.000994,0.001078,0.001054,0.000936,0.000863,0.000855,0.000855,0.000757,0.000683,0.000687,0.000667,0.000654,0.000639,0.000610,0.000586
giharmon38 ftgen 0,0,4096,10,0.275894,0.119750,0.302577,0.931731,0.080509,0.210351,0.194035,0.223110,0.137304,0.197845,0.327194,0.349699,0.653623,0.881895,0.939789,0.691399,0.569520,0.195895,0.129254,0.040633,0.094914,0.073078,0.145254,0.105583,0.162773,0.080818,0.095817,0.054832,0.079542,0.059357,0.055196,0.036903,0.021942,0.019084,0.020925,0.014726,0.013288,0.012653,0.010311,0.008738,0.008159,0.006852,0.004697,0.006565,0.011392,0.016788,0.018297,0.017879,0.015021,0.009463,0.005643,0.005105,0.004046,0.004010,0.005012,0.004282,0.003406,0.003104,0.003782,0.005116,0.003975,0.002459,0.001720,0.001650,0.002105,0.002882,0.003694,0.004103,0.003070,0.001947,0.001446,0.001258,0.001170,0.001349,0.001876,0.002582,0.002853,0.002391,0.002009,0.001887,0.001808,0.001514,0.001333,0.001265,0.001131,0.001082,0.001169,0.001278,0.001225,0.001070,0.000913,0.000800,0.000767,0.000799,0.000824,0.000902,0.001003,0.001061,0.001115,0.001185
giharmon39 ftgen 0,0,4096,10,0.215899,0.043569,0.155862,0.915877,0.029041,0.085938,0.096861,0.027476,0.042161,0.045780,0.077178,0.073681,0.292899,0.268475,0.052870,0.047343,0.020451,0.005226,0.004051,0.013123,0.014796,0.018766,0.016900,0.010549,0.011430,0.006502,0.006649,0.009883,0.007521,0.005298,0.002202,0.001505,0.001267,0.001744,0.001762,0.001880,0.001510,0.001300,0.001551,0.001113,0.001344,0.002682,0.004692,0.003899,0.003280,0.002211,0.001351,0.000889,0.000702,0.000681,0.000890,0.000985,0.000873,0.001082,0.000955,0.001016,0.001084,0.000930,0.000860,0.000656,0.000607,0.000724,0.000840,0.000859,0.000735,0.000640,0.000553,0.000553,0.000664,0.000694,0.000736,0.000846,0.000794,0.000763,0.000690,0.000686,0.000654,0.000659,0.000732,0.000714,0.000736,0.000748,0.000741,0.000620,0.000514,0.000484,0.000540,0.000707,0.000958,0.001141,0.001115,0.001012,0.000967,0.000906,0.000789,0.000653,0.000550,0.000512,0.000523,0.000521
giharmon40 ftgen 0,0,4096,10,0.369055,0.131998,0.314332,0.267587,0.083279,0.197505,0.246057,0.188865,0.247543,0.447904,0.311578,0.951654,0.440089,0.497270,0.248691,0.084689,0.024288,0.043815,0.092522,0.149863,0.063883,0.137365,0.089276,0.106339,0.067676,0.038080,0.044355,0.030311,0.016710,0.009404,0.009432,0.010647,0.009912,0.005595,0.004133,0.004025,0.003963,0.005248,0.007884,0.011332,0.009707,0.007295,0.004440,0.003429,0.004551,0.003060,0.002116,0.001763,0.001514,0.002027,0.002310,0.003069,0.002321,0.001811,0.001688,0.002021,0.001926,0.001770,0.001680,0.001534,0.001470,0.001491,0.001508,0.001391,0.001324,0.001260,0.001327,0.001406,0.001495,0.001338,0.001292,0.001395,0.001369,0.001408,0.001332,0.001258,0.001360,0.001502,0.001505,0.001392,0.001389,0.001469,0.001354,0.001230,0.001168,0.001229,0.001270,0.001296,0.001299,0.001384,0.001495,0.001524,0.001735,0.002015,0.002003,0.001730,0.001490,0.001313,0.001201,0.001201
giharmon41 ftgen 0,0,4096,10,0.338560,0.097872,0.404630,0.072213,0.101149,0.122271,0.088489,0.206809,0.295859,0.455726,0.802773,0.963091,0.671085,0.510166,0.125095,0.028121,0.044968,0.022808,0.033809,0.020551,0.006760,0.009093,0.007270,0.015331,0.017393,0.011093,0.005257,0.003771,0.002662,0.001862,0.002670,0.003093,0.003083,0.002636,0.001647,0.002030,0.003615,0.005828,0.005317,0.003389,0.002213,0.002189,0.002077,0.001392,0.000953,0.000899,0.000930,0.001189,0.001829,0.001616,0.001463,0.001355,0.001413,0.001368,0.001472,0.001201,0.000951,0.000804,0.000750,0.000839,0.001162,0.001540,0.001515,0.001241,0.001296,0.001396,0.001129,0.000785,0.000579,0.000562,0.000702,0.000938,0.001154,0.001155,0.001088,0.001192,0.001489,0.001693,0.001713,0.001549,0.001472,0.001606,0.001560,0.001278,0.001057,0.001106,0.001217,0.001127,0.001002,0.000959,0.000869,0.000789,0.000751,0.000712,0.000744,0.000874,0.001032,0.001112,0.001055,0.001033
giharmon43 ftgen 0,0,4096,10,0.135645,0.120943,0.979143,0.023489,0.114683,0.114072,0.130730,0.180948,0.281652,0.683720,0.964428,0.306787,0.213682,0.058648,0.025209,0.025596,0.047105,0.038846,0.017756,0.006258,0.005601,0.010206,0.010689,0.004381,0.002744,0.002982,0.002040,0.002284,0.001570,0.001528,0.002115,0.001649,0.002514,0.003304,0.005338,0.002902,0.001301,0.000713,0.000665,0.000652,0.000695,0.000698,0.000700,0.001056,0.000728,0.000485,0.000534,0.000633,0.000450,0.000429,0.000625,0.000827,0.001012,0.001160,0.001256,0.001213,0.001133,0.001362,0.001533,0.001305,0.001367,0.001616,0.001627,0.001417,0.001458,0.001637,0.001588,0.001552,0.001476,0.001327,0.001247,0.001076,0.000857,0.000699,0.000670,0.000726,0.000750,0.000854,0.000976,0.000936,0.000830,0.000768,0.000683,0.000599,0.000530,0.000491,0.000451,0.000398,0.000378,0.000368,0.000380,0.000455,0.000533,0.000535,0.000501,0.000464,0.000470,0.000496,0.000480,0.000442
giharmon45 ftgen 0,0,4096,10,0.244591,0.267325,0.126930,0.136762,0.271635,0.312002,0.223921,0.477139,0.969260,0.621811,0.276235,0.133028,0.017955,0.053396,0.048589,0.033493,0.018488,0.024467,0.024072,0.021974,0.013672,0.005364,0.005982,0.003607,0.001893,0.001866,0.001242,0.001433,0.001334,0.002488,0.002930,0.001518,0.001627,0.002275,0.001495,0.001193,0.000972,0.000737,0.000811,0.001235,0.001057,0.000838,0.001036,0.001047,0.001114,0.001012,0.000995,0.001090,0.001298,0.001286,0.001257,0.001216,0.001222,0.001045,0.001114,0.001441,0.001842,0.001582,0.001233,0.001180,0.001395,0.001374,0.001319,0.001317,0.001143,0.001024,0.001053,0.001334,0.001740,0.001862,0.001603,0.001274,0.001051,0.001022,0.000939,0.000839,0.000784,0.000794,0.000787,0.000752,0.000772,0.000742,0.000760,0.000882,0.001044,0.001103,0.001124,0.001129,0.001259,0.001523,0.001689,0.001631,0.001499,0.001332,0.001219,0.001138,0.001037,0.000952,0.000828,0.000746
giharmon46 ftgen 0,0,4096,10,0.315815,0.844525,0.164412,0.451756,0.585703,0.518759,0.681565,0.411064,0.952335,0.494514,0.366224,0.049949,0.050057,0.056689,0.053659,0.035965,0.043860,0.032015,0.019827,0.005201,0.006601,0.005746,0.007203,0.005429,0.003518,0.004758,0.003756,0.007828,0.015721,0.006418,0.003773,0.007304,0.003491,0.004073,0.002821,0.001771,0.002371,0.001798,0.002341,0.002659,0.003029,0.001998,0.001673,0.001293,0.000985,0.001051,0.001602,0.002316,0.002076,0.001608,0.001182,0.001234,0.001509,0.002038,0.001745,0.001690,0.001788,0.002285,0.002474,0.002206,0.002242,0.002438,0.002520,0.003014,0.003823,0.004834,0.005921,0.005640,0.004222,0.003103,0.002822,0.003289,0.003754,0.003426,0.002719,0.002201,0.001856,0.001927,0.002461,0.002893,0.002831,0.002430,0.002388,0.002815,0.003393,0.003964,0.004124,0.003523,0.002801,0.002546,0.002599,0.002485,0.002105,0.001812,0.001794,0.001803,0.001606,0.001468,0.001509,0.001549
giharmon48 ftgen 0,0,4096,10,0.122676,0.220788,0.073432,0.127649,0.149907,0.310663,0.320023,0.888642,0.333339,0.133323,0.020148,0.021610,0.031312,0.019058,0.010338,0.008856,0.009259,0.004665,0.008040,0.006064,0.004765,0.002412,0.001080,0.002281,0.002107,0.003516,0.001246,0.000827,0.000714,0.001050,0.000701,0.000675,0.000523,0.000934,0.001128,0.001435,0.000766,0.000427,0.000568,0.000725,0.001140,0.001379,0.000939,0.000679,0.000677,0.000639,0.000687,0.000664,0.000490,0.000511,0.000557,0.000610,0.000728,0.001052,0.001004,0.000764,0.000789,0.000916,0.000909,0.001108,0.000978,0.000656,0.000487,0.000484,0.000484,0.000521,0.000592,0.000647,0.000545,0.000494,0.000588,0.000761,0.000868,0.000902,0.000912,0.000928,0.000781,0.000667,0.000669,0.000665,0.000649,0.000608,0.000541,0.000459,0.000419,0.000435,0.000481,0.000492,0.000493,0.000550,0.000622,0.000616,0.000575,0.000571,0.000566,0.000520,0.000445,0.000380,0.000345,0.000334
giharmon50 ftgen 0,0,4096,10,0.162293,0.952587,0.107970,0.148379,0.162877,0.302263,0.378248,0.526544,0.202053,0.019075,0.035439,0.025852,0.023804,0.025676,0.028522,0.013081,0.004084,0.002802,0.001949,0.002297,0.001565,0.001596,0.004229,0.004413,0.001504,0.001251,0.001021,0.001404,0.001139,0.001161,0.000682,0.000466,0.000926,0.000495,0.000656,0.000798,0.001105,0.000691,0.000515,0.000562,0.000460,0.000566,0.000474,0.000481,0.000452,0.000510,0.000694,0.000487,0.000439,0.000584,0.000605,0.000623,0.000670,0.000916,0.000792,0.000871,0.000636,0.000514,0.000540,0.000466,0.000488,0.000461,0.000448,0.000510,0.000642,0.000795,0.000743,0.000838,0.001039,0.001142,0.000966,0.000733,0.000659,0.000657,0.000595,0.000548,0.000480,0.000411,0.000427,0.000488,0.000580,0.000723,0.000773,0.000693,0.000563,0.000471,0.000412,0.000429,0.000481,0.000468,0.000408,0.000367,0.000352,0.000329,0.000288,0.000260,0.000253,0.000249,0.000251,0.000257
giharmon52 ftgen 0,0,4096,10,0.167672,0.198778,0.227405,0.284738,0.650198,0.940684,0.975647,0.268741,0.049829,0.071939,0.024052,0.019628,0.020109,0.016231,0.003867,0.003960,0.004551,0.001438,0.002545,0.005214,0.004055,0.001727,0.002282,0.002593,0.001856,0.002206,0.001924,0.000871,0.000924,0.001558,0.001059,0.001121,0.001112,0.001260,0.001747,0.001193,0.000757,0.001035,0.000862,0.000951,0.001089,0.001598,0.001563,0.001170,0.001427,0.001409,0.001130,0.001424,0.001095,0.000785,0.000800,0.001135,0.001025,0.001093,0.000966,0.001083,0.001357,0.001272,0.001103,0.001508,0.002036,0.001512,0.001143,0.001348,0.001716,0.001528,0.001151,0.000935,0.001007,0.001124,0.000930,0.000715,0.000611,0.000658,0.000759,0.000788,0.000734,0.000745,0.000773,0.000916,0.001069,0.001056,0.000942,0.000778,0.000676,0.000670,0.000675,0.000675,0.000644,0.000597,0.000550,0.000537,0.000555,0.000559,0.000525,0.000454,0.000404,0.000407,0.000442,0.000480
giharmon53 ftgen 0,0,4096,10,0.026511,0.043338,0.103983,0.172681,0.197084,0.979650,0.257547,0.017830,0.077807,0.064492,0.041376,0.053973,0.027216,0.006355,0.007006,0.001571,0.001221,0.001837,0.007442,0.004872,0.002662,0.002280,0.002188,0.001493,0.002889,0.002534,0.002305,0.002344,0.001085,0.001468,0.002010,0.002087,0.000876,0.000665,0.000602,0.000985,0.000607,0.000521,0.000609,0.000586,0.000566,0.000481,0.000501,0.000665,0.000626,0.000653,0.000579,0.000643,0.000690,0.000651,0.000432,0.000403,0.000504,0.000532,0.000598,0.000456,0.000369,0.000393,0.000370,0.000313,0.000265,0.000292,0.000299,0.000335,0.000381,0.000415,0.000449,0.000480,0.000399,0.000307,0.000245,0.000249,0.000338,0.000468,0.000545,0.000532,0.000495,0.000471,0.000476,0.000450,0.000388,0.000331,0.000297,0.000269,0.000233,0.000202,0.000188,0.000169,0.000158,0.000175,0.000208,0.000254,0.000316,0.000380,0.000433,0.000473,0.000499,0.000504,0.000472,0.000425
giharmon55 ftgen 0,0,4096,10,0.081729,0.021968,0.138971,0.239901,0.947997,0.478402,0.152261,0.093610,0.063734,0.039617,0.032944,0.006609,0.015447,0.010428,0.006706,0.002730,0.013404,0.010554,0.003963,0.002958,0.000885,0.000799,0.001120,0.001680,0.001729,0.001274,0.001362,0.000943,0.002299,0.001254,0.001112,0.000960,0.001034,0.000866,0.000912,0.000751,0.001101,0.000945,0.000888,0.000760,0.000758,0.000764,0.000797,0.000790,0.000657,0.000779,0.000816,0.000819,0.000797,0.000793,0.000830,0.000803,0.000678,0.000695,0.000912,0.000796,0.000686,0.000833,0.000798,0.000772,0.000941,0.000835,0.000815,0.000695,0.000533,0.000448,0.000405,0.000411,0.000492,0.000554,0.000576,0.000587,0.000573,0.000542,0.000560,0.000560,0.000557,0.000544,0.000547,0.000555,0.000551,0.000537,0.000544,0.000557,0.000533,0.000512,0.000499,0.000514,0.000548,0.000548,0.000539,0.000526,0.000511,0.000513,0.000513,0.000510,0.000506,0.000503,0.000507,0.000508
giharmon57 ftgen 0,0,4096,10,0.150966,0.141943,0.169272,0.391010,0.953809,0.119929,0.039950,0.075251,0.042294,0.016963,0.005858,0.005238,0.002857,0.003841,0.008582,0.001485,0.001148,0.002596,0.001639,0.001556,0.002197,0.001142,0.001112,0.001195,0.001136,0.001245,0.000909,0.001301,0.000937,0.000941,0.001212,0.001227,0.000900,0.000862,0.000883,0.000728,0.001073,0.001100,0.000883,0.001160,0.000948,0.000936,0.001199,0.000883,0.000791,0.000945,0.000697,0.000592,0.000626,0.000588,0.000617,0.000696,0.000684,0.000803,0.000743,0.000641,0.000514,0.000453,0.000480,0.000536,0.000586,0.000581,0.000619,0.000622,0.000602,0.000589,0.000589,0.000600,0.000582,0.000552,0.000553,0.000557,0.000549,0.000526,0.000512,0.000519,0.000533,0.000535,0.000547,0.000544,0.000539,0.000534,0.000524,0.000512,0.000502,0.000502,0.000502,0.000497,0.000480,0.000468,0.000466,0.000464,0.000461,0.000459,0.000463,0.000470,0.000477,0.000482,0.000483,0.000488
giharmon60 ftgen 0,0,4096,10,0.306985,0.126374,0.301596,0.969500,0.074344,0.009729,0.006039,0.004002,0.001666,0.001675,0.002331,0.000674,0.000730,0.000729,0.000758,0.000585,0.000577,0.000687,0.000657,0.001017,0.000768,0.000581,0.000549,0.000326,0.000290,0.000463,0.000666,0.000567,0.000312,0.000634,0.000665,0.000397,0.000264,0.000232,0.000286,0.000583,0.000629,0.000663,0.000354,0.000268,0.000259,0.000263,0.000303,0.000274,0.000231,0.000322,0.000471,0.000473,0.000488,0.000569,0.000596,0.000564,0.000445,0.000430,0.000473,0.000519,0.000490,0.000446,0.000488,0.000475,0.000372,0.000301,0.000312,0.000304,0.000374,0.000526,0.000614,0.000564,0.000443,0.000349,0.000321,0.000320,0.000292,0.000243,0.000210,0.000227,0.000301,0.000393,0.000452,0.000473,0.000454,0.000415,0.000375,0.000353,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giharmon62 ftgen 0,0,4096,10,0.969106,0.095726,0.196065,0.188016,0.005133,0.010386,0.004868,0.005030,0.003515,0.002819,0.002632,0.002361,0.001260,0.000235,0.000966,0.001186,0.001018,0.000871,0.001416,0.000300,0.000430,0.000824,0.000912,0.000810,0.000516,0.000452,0.000279,0.000463,0.000690,0.000443,0.000429,0.000374,0.000579,0.000876,0.000457,0.000456,0.000449,0.000577,0.000797,0.000538,0.000571,0.000564,0.000466,0.000418,0.000306,0.000321,0.000334,0.000294,0.000244,0.000204,0.000231,0.000265,0.000340,0.000422,0.000467,0.000400,0.000291,0.000221,0.000207,0.000236,0.000249,0.000246,0.000207,0.000188,0.000203,0.000210,0.000206,0.000193,0.000167,0.000155,0.000192,0.000247,0.000284,0.000296,0.000308,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giharmon64 ftgen 0,0,4096,10,0.361663,0.582855,0.979701,0.215161,0.118878,0.052107,0.013704,0.009061,0.004379,0.007958,0.002817,0.003129,0.003908,0.003040,0.002896,0.002083,0.002113,0.004424,0.003621,0.002331,0.002616,0.002100,0.002293,0.001878,0.002282,0.002077,0.002490,0.002088,0.001811,0.001576,0.001425,0.001407,0.001400,0.001482,0.001847,0.001816,0.001426,0.001730,0.001541,0.001272,0.001398,0.001231,0.001332,0.001350,0.001199,0.001258,0.001149,0.001243,0.001247,0.001156,0.001074,0.001049,0.001142,0.001143,0.001058,0.000978,0.000854,0.000777,0.000688,0.000751,0.000914,0.001100,0.001101,0.001029,0.001044,0.001018,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giharmon65 ftgen 0,0,4096,10,0.314097,0.342771,0.973682,0.028876,0.015118,0.012713,0.002418,0.005174,0.003787,0.002218,0.002249,0.001625,0.002664,0.001830,0.002581,0.001180,0.001626,0.001619,0.001315,0.001513,0.001344,0.001175,0.000964,0.001112,0.001416,0.001129,0.001190,0.001045,0.001248,0.001029,0.000799,0.000750,0.000815,0.000965,0.000955,0.000725,0.000652,0.000709,0.000801,0.000717,0.000786,0.000648,0.000597,0.000630,0.000744,0.000667,0.000613,0.000496,0.000535,0.000596,0.000603,0.000583,0.000599,0.000593,0.000594,0.000603,0.000579,0.000588,0.000623,0.000624,0.000595,0.000573,0.000574,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
giwavemapharmon        ftgen    0,0,128,-27, 0,0, 33,0, 34,1, 36,2, 38,3, 39,3, 40,4, 41,5, 43,6, 45,7, 46,8, 48,9, 50,10,  52,11, 53,12, 55,13, 57,14, 60,15, 62,16, 64,17, 65,18, 127,18 
gitabs4morfharmon    ftgen    0,0,32,-2, giharmon33,giharmon34,giharmon36,giharmon38,giharmon39,giharmon40,giharmon41,giharmon43,giharmon45,giharmon46,giharmon48,giharmon50,giharmon52,giharmon53,giharmon55,giharmon57,giharmon60,giharmon62,giharmon64,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65,giharmon65

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; BASS TROMBONE (STRAIGHT MUTE)
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giMute122 ftgen 0,0,4096,10,0.164339,0.102551,0.069563,0.138697,0.223728,0.265220,0.280256,0.332069,0.420578,0.494911,0.577300,0.438019,0.538305,0.547959,0.857117,0.773953,0.517366,0.523329,0.250443,0.159600,0.171983,0.125233,0.061672,0.045591,0.028822,0.029244,0.026342,0.016326,0.018016,0.016474,0.009454,0.006045,0.003657,0.004270,0.008585,0.014528,0.017452,0.019652,0.016908,0.013257,0.016063,0.015947,0.010616,0.009647,0.007357,0.006942,0.005472,0.008460,0.014247,0.013954,0.009828,0.009192,0.009759,0.008585,0.009964,0.012870,0.010975,0.007757,0.007624,0.008209,0.008005,0.007979,0.007726,0.007655,0.007795,0.007616,0.006725,0.005945,0.005229,0.004482,0.003678,0.003014,0.002618,0.002393,0.002731,0.003322,0.003436,0.002905,0.002194,0.001589,0.001214,0.001156,0.001328,0.001483,0.001561,0.001609,0.001696,0.001872,0.001908,0.002069,0.002367,0.002374,0.002130,0.001840,0.001674,0.001720,0.001812,0.001802,0.001732,0.001622
giMute123 ftgen 0,0,4096,10,0.168432,0.062246,0.078354,0.110195,0.129041,0.377921,0.282536,0.228004,0.524063,0.552904,0.492429,0.533439,0.241793,0.431759,0.960231,0.527526,0.457018,0.447269,0.173908,0.188096,0.117536,0.088623,0.038960,0.034631,0.040187,0.030976,0.019737,0.017966,0.018488,0.010201,0.006147,0.003826,0.005925,0.007905,0.007681,0.008607,0.015400,0.021169,0.020768,0.019478,0.013836,0.014839,0.014657,0.013690,0.013236,0.016605,0.019334,0.015552,0.016319,0.019658,0.016907,0.011550,0.009411,0.009522,0.010270,0.008678,0.008528,0.008212,0.007305,0.006920,0.008975,0.009992,0.007897,0.006576,0.006398,0.005486,0.004674,0.004082,0.003255,0.002525,0.001877,0.001735,0.002072,0.002316,0.002265,0.001877,0.001508,0.001299,0.001160,0.001268,0.001500,0.001614,0.001649,0.001544,0.001559,0.001869,0.002444,0.002837,0.002564,0.002224,0.002193,0.002264,0.002103,0.001864,0.001829,0.001964,0.001969,0.001708,0.001368,0.001146
giMute124 ftgen 0,0,4096,10,0.142996,0.083116,0.064144,0.232641,0.363109,0.345440,0.348866,0.301465,0.496059,0.590541,0.450344,0.611392,0.691633,0.939595,0.719155,0.655138,0.609154,0.236629,0.218778,0.154757,0.077033,0.074852,0.040400,0.038516,0.034596,0.023126,0.013779,0.015070,0.008541,0.005439,0.004834,0.008763,0.016084,0.025053,0.033607,0.047747,0.045648,0.045497,0.025409,0.021094,0.022069,0.038009,0.041759,0.063224,0.070253,0.055822,0.048910,0.048219,0.052210,0.039531,0.037339,0.030915,0.028370,0.038122,0.044865,0.047418,0.042297,0.039872,0.040218,0.036388,0.033231,0.028010,0.022884,0.020191,0.018026,0.013653,0.011148,0.010124,0.011725,0.014502,0.015977,0.012802,0.008969,0.006321,0.005229,0.005789,0.006897,0.007670,0.008699,0.009835,0.010769,0.010605,0.010601,0.010721,0.010749,0.010852,0.010291,0.009559,0.008597,0.007779,0.007284,0.006683,0.005906,0.005353,0.005322,0.005542,0.005543,0.005124,0.004449,0.003603
giMute125 ftgen 0,0,4096,10,0.044114,0.061305,0.127226,0.214587,0.248138,0.322022,0.289079,0.300066,0.219345,0.389034,0.679974,0.361926,0.905776,0.756147,0.724622,0.413971,0.233101,0.180408,0.145204,0.056539,0.046805,0.060866,0.050067,0.038355,0.032608,0.021983,0.011622,0.006955,0.011848,0.020388,0.023077,0.028897,0.039329,0.040006,0.051071,0.030060,0.035671,0.024931,0.038339,0.037708,0.058492,0.053593,0.037184,0.045630,0.043341,0.034029,0.031437,0.034031,0.025773,0.028039,0.030454,0.031458,0.031318,0.029135,0.028627,0.025452,0.021115,0.018088,0.014741,0.013689,0.012102,0.010087,0.009117,0.010501,0.012107,0.011147,0.008670,0.006054,0.004653,0.004909,0.005979,0.007571,0.008945,0.009279,0.009403,0.009990,0.010831,0.011229,0.011576,0.012198,0.011522,0.009582,0.007918,0.007133,0.006815,0.006874,0.006545,0.005823,0.005215,0.004793,0.004226,0.003549,0.002763,0.002034,0.001527,0.001275,0.001284,0.001501,0.001663,0.001513
giMute126 ftgen 0,0,4096,10,0.060196,0.079208,0.165018,0.403007,0.474143,0.259060,0.390988,0.369395,0.520597,0.147418,0.294315,0.719113,0.948583,0.911775,0.688364,0.284237,0.196457,0.197963,0.098875,0.075773,0.045829,0.020036,0.034330,0.015751,0.015028,0.009926,0.008781,0.011622,0.026268,0.030349,0.027636,0.033521,0.031042,0.039392,0.034047,0.037282,0.023269,0.035915,0.033496,0.040946,0.033303,0.030305,0.029753,0.029612,0.029870,0.024821,0.021197,0.017836,0.015201,0.014801,0.017255,0.016419,0.014400,0.014847,0.014196,0.013845,0.010243,0.007414,0.006884,0.005313,0.005125,0.006712,0.007001,0.005537,0.003835,0.002755,0.002847,0.003574,0.003940,0.004035,0.004370,0.004927,0.004830,0.004569,0.004559,0.004585,0.004513,0.003828,0.003072,0.002596,0.002377,0.002294,0.002087,0.001940,0.001859,0.001843,0.002114,0.002255,0.001844,0.001305,0.000970,0.000768,0.000670,0.000707,0.000807,0.000885,0.000924,0.000852,0.000694,0.000544
giMute127 ftgen 0,0,4096,10,0.052401,0.049320,0.080590,0.183766,0.164401,0.307439,0.388671,0.284459,0.318081,0.164443,0.441431,0.928201,0.876487,0.740904,0.324363,0.142732,0.081529,0.077495,0.028649,0.043648,0.038991,0.025075,0.013227,0.010195,0.011781,0.010501,0.016602,0.024255,0.023187,0.036495,0.057654,0.054268,0.048563,0.024903,0.054174,0.048040,0.071704,0.054365,0.049624,0.064806,0.092036,0.062098,0.048326,0.054051,0.039134,0.032416,0.036465,0.055694,0.042371,0.035750,0.021224,0.022257,0.011427,0.014694,0.009479,0.004552,0.013174,0.019261,0.013315,0.005818,0.001677,0.002772,0.003021,0.006297,0.007154,0.008171,0.010310,0.008506,0.007517,0.010904,0.006062,0.012403,0.007495,0.006084,0.005664,0.006058,0.004342,0.006256,0.008205,0.007899,0.006624,0.007878,0.003449,0.001680,0.000514,0.001804,0.002081,0.003443,0.004990,0.003472,0.003035,0.002319,0.001815,0.000843,0.001081,0.003090,0.002248,0.001877,0.002589,0.003743
giMute128 ftgen 0,0,4096,10,0.027923,0.100208,0.119497,0.305740,0.241671,0.283542,0.288061,0.232034,0.826888,0.407135,0.944943,0.754001,0.650071,0.360755,0.163788,0.199546,0.064667,0.047976,0.067193,0.033442,0.022636,0.018891,0.006473,0.006036,0.009904,0.016425,0.024814,0.044941,0.038557,0.069982,0.053960,0.007689,0.060655,0.019491,0.072831,0.052666,0.042308,0.055883,0.044001,0.027661,0.043229,0.023057,0.025173,0.028614,0.031240,0.027168,0.026598,0.023386,0.018674,0.009964,0.010171,0.008959,0.004421,0.009443,0.011811,0.008975,0.005444,0.002992,0.002577,0.004840,0.005217,0.007905,0.008254,0.008604,0.004473,0.009592,0.008601,0.005583,0.008098,0.005796,0.002345,0.007586,0.005015,0.004900,0.007488,0.005743,0.006825,0.002224,0.002742,0.001117,0.001857,0.001298,0.003169,0.002324,0.002382,0.001822,0.001779,0.001092,0.000752,0.001206,0.002075,0.000843,0.001972,0.001844,0.002893,0.002804,0.002542,0.001110,0.001376,0.000804
giMute129 ftgen 0,0,4096,10,0.014669,0.032320,0.102462,0.174664,0.169222,0.111331,0.339606,0.209754,0.217866,0.343088,0.891951,0.530957,0.472318,0.164643,0.165367,0.091601,0.090801,0.015419,0.039607,0.019025,0.012281,0.004714,0.002434,0.014861,0.038522,0.020090,0.049806,0.050949,0.061040,0.054492,0.035308,0.046103,0.018911,0.085037,0.066539,0.090037,0.115123,0.114020,0.059158,0.070988,0.070450,0.068910,0.054753,0.043435,0.030742,0.021866,0.024247,0.013293,0.017833,0.010515,0.011265,0.016382,0.023641,0.010420,0.007923,0.005015,0.009328,0.008929,0.010346,0.011125,0.009063,0.009311,0.010454,0.010758,0.009200,0.014939,0.006906,0.005195,0.007827,0.007531,0.008761,0.010132,0.008289,0.010958,0.004572,0.001934,0.001818,0.002642,0.002171,0.002199,0.002528,0.001704,0.001645,0.001110,0.000567,0.001256,0.000671,0.001415,0.003604,0.003233,0.002086,0.001933,0.001526,0.000662,0.001458,0.000497,0.000845,0.000680,0.001138,0.001279
giMute130 ftgen 0,0,4096,10,0.064908,0.079846,0.127850,0.445046,0.330968,0.420691,0.480533,0.374119,0.245280,0.729167,0.936260,0.749421,0.381456,0.247718,0.207617,0.116908,0.051373,0.019507,0.008564,0.025213,0.010576,0.007009,0.011265,0.017944,0.027208,0.040488,0.021178,0.046272,0.038151,0.021208,0.044463,0.023287,0.045981,0.030790,0.020921,0.015858,0.026172,0.019160,0.017080,0.011607,0.015955,0.006307,0.009311,0.006865,0.003784,0.003961,0.002271,0.002504,0.000904,0.002864,0.002378,0.001916,0.001531,0.000626,0.001045,0.001491,0.001672,0.000890,0.001666,0.001476,0.001501,0.000957,0.003007,0.001529,0.001037,0.001154,0.001031,0.000515,0.000881,0.000327,0.000467,0.000436,0.000280,0.000329,0.000454,0.000243,0.000168,0.000282,0.000948,0.000411,0.000184,0.000452,0.000360,0.000354,0.000193,0.000243,0.000591,0.000203,0.000418,0.000245,0.000193,0.000287,0.000162,0.000386,0.000147,0.000097,0.000291,0.000161,0.000222,0.000228
giMute131 ftgen 0,0,4096,10,0.068230,0.065611,0.217451,0.232412,0.188182,0.427625,0.292296,0.554990,0.340081,0.915514,0.522787,0.307999,0.162151,0.094669,0.056724,0.027329,0.011538,0.010489,0.008261,0.001997,0.007832,0.008040,0.029670,0.021346,0.029403,0.071206,0.042160,0.041297,0.059757,0.018923,0.105664,0.038842,0.027166,0.042666,0.031872,0.028802,0.030657,0.019773,0.024186,0.023554,0.016848,0.015145,0.013926,0.008132,0.010479,0.005103,0.002808,0.010621,0.007445,0.004301,0.001100,0.003068,0.003063,0.004833,0.003047,0.003019,0.003716,0.005204,0.002390,0.002979,0.001408,0.001568,0.001539,0.001833,0.002366,0.001838,0.000682,0.001960,0.000613,0.000686,0.000652,0.000399,0.000606,0.000602,0.000521,0.000455,0.000432,0.000091,0.000136,0.000332,0.000272,0.000302,0.000126,0.000375,0.000314,0.000203,0.000126,0.000237,0.000293,0.000291,0.000185,0.000182,0.000275,0.000263,0.000195,0.000151,0.000202,0.000294,0.000205,0.000198
giMute132 ftgen 0,0,4096,10,0.072458,0.092631,0.167684,0.302817,0.140977,0.255042,0.110714,0.344468,0.928686,0.667412,0.338258,0.200769,0.147649,0.065744,0.004582,0.023573,0.011837,0.008391,0.007660,0.014753,0.031615,0.027782,0.039431,0.047372,0.049448,0.013895,0.042049,0.032148,0.026316,0.026284,0.049745,0.043954,0.026083,0.037453,0.029582,0.022591,0.018017,0.013163,0.009816,0.004994,0.005845,0.002926,0.004507,0.007678,0.002757,0.002125,0.001498,0.002105,0.002073,0.003179,0.003552,0.003030,0.003107,0.003121,0.004471,0.001227,0.002877,0.002488,0.002740,0.003051,0.002796,0.002240,0.000296,0.000464,0.000414,0.001290,0.000687,0.001086,0.000537,0.000387,0.000130,0.000370,0.000092,0.000561,0.000559,0.000109,0.000094,0.000089,0.000290,0.000073,0.000183,0.000250,0.000318,0.000509,0.000515,0.000390,0.000503,0.000327,0.000338,0.000232,0.000327,0.000154,0.000173,0.000108,0.000092,0.000271,0.000089,0.000076,0.000105,0.000132
giMute133 ftgen 0,0,4096,10,0.171302,0.215302,0.554037,0.602813,0.500383,0.551353,0.792246,0.972277,0.915826,0.832314,0.393008,0.188469,0.087171,0.051938,0.057335,0.030089,0.018181,0.012795,0.030872,0.033984,0.097553,0.117375,0.088919,0.019750,0.074370,0.074970,0.077987,0.098699,0.118454,0.054334,0.051802,0.037932,0.045722,0.044228,0.016572,0.018704,0.013524,0.013398,0.003788,0.004323,0.010291,0.003069,0.001274,0.001330,0.002466,0.002824,0.003268,0.005612,0.003975,0.000819,0.002777,0.002460,0.002046,0.001584,0.002892,0.002346,0.003012,0.001881,0.000649,0.000801,0.000646,0.001523,0.001240,0.001310,0.000390,0.000248,0.000279,0.000518,0.000341,0.000512,0.000393,0.000190,0.000358,0.000126,0.000196,0.000125,0.000251,0.000327,0.000531,0.000833,0.000907,0.000336,0.000482,0.000237,0.000687,0.000266,0.000316,0.000368,0.000104,0.000129,0.000172,0.000171,0.000167,0.000055,0.000143,0.000116,0.000132,0.000198,0.000238,0.000318
giMute134 ftgen 0,0,4096,10,0.097133,0.173820,0.374605,0.474497,0.645937,0.376089,0.435327,0.976798,0.725285,0.297676,0.120366,0.069249,0.033502,0.014976,0.014214,0.011249,0.006081,0.003570,0.011892,0.014690,0.022764,0.024186,0.025001,0.044955,0.049727,0.048851,0.047898,0.053897,0.023052,0.038993,0.025427,0.027582,0.022350,0.021270,0.010750,0.009160,0.013099,0.002909,0.008796,0.006376,0.001857,0.001716,0.003061,0.001735,0.002918,0.004325,0.002877,0.003817,0.005184,0.003931,0.001995,0.004293,0.003739,0.003678,0.003158,0.002071,0.001133,0.000232,0.000323,0.001563,0.001268,0.000658,0.000426,0.000063,0.000421,0.000195,0.000767,0.000535,0.000446,0.000289,0.000184,0.000296,0.000288,0.000200,0.000535,0.000186,0.000733,0.000229,0.000412,0.000107,0.000166,0.000355,0.000312,0.000317,0.000354,0.000185,0.000132,0.000137,0.000206,0.000233,0.000262,0.000150,0.000123,0.000145,0.000145,0.000117,0.000140,0.000106,0.000135,0.000169
giMute135 ftgen 0,0,4096,10,0.179021,0.401691,0.729330,0.982922,0.746447,0.524380,0.900583,0.585038,0.953435,0.443166,0.289448,0.098763,0.031244,0.011392,0.020066,0.020371,0.036289,0.021837,0.044660,0.116353,0.070855,0.059093,0.066299,0.083923,0.069741,0.061417,0.033091,0.045547,0.029414,0.028975,0.021471,0.021629,0.010855,0.013960,0.004240,0.008225,0.009562,0.004312,0.001601,0.002687,0.001271,0.003054,0.004689,0.003158,0.006429,0.004058,0.005468,0.002640,0.001551,0.003253,0.003142,0.003027,0.001217,0.000340,0.000216,0.000471,0.000540,0.000381,0.000432,0.000182,0.000650,0.000176,0.000758,0.000323,0.000522,0.000156,0.000289,0.000185,0.000281,0.000179,0.000391,0.000461,0.000214,0.000356,0.000189,0.000425,0.000214,0.000172,0.000322,0.000195,0.000387,0.000323,0.000373,0.000170,0.000176,0.000048,0.000079,0.000049,0.000161,0.000129,0.000141,0.000083,0.000119,0.000204,0.000318,0.000410,0.000202,0.000283,0.000292,0.000127
giMute136 ftgen 0,0,4096,10,0.084007,0.199308,0.212901,0.389112,0.310014,0.209644,0.951984,0.633937,0.286005,0.146120,0.053652,0.047694,0.020100,0.016786,0.003816,0.018198,0.008332,0.020077,0.037293,0.016675,0.022299,0.027356,0.023496,0.049061,0.039132,0.026396,0.012922,0.017081,0.012583,0.010068,0.005525,0.002721,0.003104,0.004799,0.003308,0.002047,0.001239,0.002036,0.002197,0.002242,0.003547,0.004859,0.003272,0.001567,0.002063,0.002229,0.001760,0.001286,0.001961,0.000541,0.000366,0.000986,0.000730,0.000861,0.000630,0.000616,0.000358,0.001058,0.000464,0.000438,0.000450,0.000487,0.000193,0.000559,0.000479,0.000260,0.000259,0.000887,0.000529,0.000258,0.000611,0.000375,0.000388,0.000092,0.000430,0.000337,0.000138,0.000312,0.000579,0.000492,0.000207,0.000330,0.000350,0.000212,0.000159,0.000387,0.000350,0.000158,0.000191,0.000374,0.000113,0.000150,0.000260,0.000156,0.000117,0.000208,0.000235,0.000105,0.000074,0.000268
giMute137 ftgen 0,0,4096,10,0.121187,0.301409,0.437255,0.488546,0.532406,0.556320,0.940536,0.612558,0.171145,0.078278,0.033359,0.039980,0.019198,0.003684,0.014541,0.021902,0.025869,0.028022,0.020078,0.018008,0.063141,0.040559,0.061443,0.025176,0.045882,0.030795,0.027823,0.023490,0.015819,0.011054,0.004279,0.008778,0.007683,0.002850,0.002807,0.001837,0.004594,0.006832,0.003882,0.006682,0.004472,0.002359,0.002653,0.002120,0.002271,0.001366,0.001048,0.000977,0.001563,0.000795,0.000741,0.000197,0.000435,0.000663,0.000303,0.000697,0.000460,0.000621,0.000388,0.000298,0.000103,0.000484,0.000378,0.000376,0.000296,0.000134,0.000043,0.000150,0.000155,0.000176,0.000187,0.000055,0.000111,0.000091,0.000143,0.000129,0.000153,0.000171,0.000204,0.000159,0.000206,0.000139,0.000065,0.000078,0.000107,0.000102,0.000128,0.000113,0.000157,0.000150,0.000078,0.000128,0.000144,0.000084,0.000114,0.000188,0.000043,0.000106,0.000116,0.000128
giMute138 ftgen 0,0,4096,10,0.137239,0.608302,0.333203,0.546467,0.231739,0.766708,0.950744,0.327806,0.160412,0.060869,0.019878,0.003953,0.008113,0.006915,0.028018,0.027245,0.043331,0.020302,0.014454,0.025972,0.026882,0.018782,0.017822,0.013846,0.010287,0.009995,0.005808,0.004941,0.003243,0.000812,0.002013,0.001670,0.000819,0.000666,0.000772,0.000298,0.000436,0.000772,0.000741,0.000154,0.001321,0.000422,0.000226,0.000631,0.000439,0.000344,0.000337,0.000336,0.000616,0.000238,0.000312,0.000215,0.000181,0.000430,0.000292,0.000227,0.000331,0.000311,0.000246,0.000311,0.000279,0.000324,0.000208,0.000103,0.000266,0.000302,0.000310,0.000240,0.000310,0.000296,0.000228,0.000305,0.000203,0.000181,0.000235,0.000154,0.000238,0.000155,0.000185,0.000226,0.000170,0.000202,0.000202,0.000228,0.000150,0.000178,0.000239,0.000187,0.000194,0.000253,0.000070,0.000165,0.000223,0.000127,0.000156,0.000160,0.000180,0.000163,0.000129,0.000224
giMute139 ftgen 0,0,4096,10,0.061443,0.238708,0.356688,0.269787,0.339649,0.912187,0.776022,0.165931,0.073909,0.021161,0.016878,0.008990,0.003708,0.022300,0.041143,0.038233,0.004859,0.023579,0.053718,0.052003,0.042268,0.022601,0.022660,0.029648,0.019517,0.017414,0.011974,0.004870,0.011305,0.003101,0.000546,0.002006,0.003619,0.004048,0.005624,0.003647,0.002198,0.002634,0.003291,0.001943,0.000613,0.000545,0.000250,0.000451,0.000789,0.000299,0.000410,0.000212,0.000138,0.000424,0.000415,0.000404,0.000244,0.000269,0.000335,0.000329,0.000230,0.000373,0.000400,0.000370,0.000260,0.000376,0.000293,0.000187,0.000119,0.000256,0.000177,0.000244,0.000278,0.000293,0.000302,0.000158,0.000215,0.000196,0.000203,0.000167,0.000136,0.000170,0.000168,0.000137,0.000166,0.000168,0.000160,0.000177,0.000247,0.000183,0.000197,0.000135,0.000162,0.000164,0.000186,0.000121,0.000206,0.000166,0.000144,0.000145,0.000141,0.000122,0.000131,0.000148
giMute140 ftgen 0,0,4096,10,0.171458,0.486737,0.414556,0.287720,0.507039,0.974434,0.387302,0.180263,0.044683,0.029233,0.014770,0.003589,0.009392,0.033786,0.054370,0.010033,0.024497,0.035533,0.046495,0.020949,0.024322,0.023346,0.018462,0.011212,0.009184,0.003515,0.005537,0.002905,0.001147,0.001777,0.002912,0.002534,0.003007,0.002041,0.002109,0.001352,0.001160,0.001111,0.000145,0.000433,0.000205,0.000056,0.000347,0.000268,0.000294,0.000170,0.000469,0.000118,0.000167,0.000252,0.000207,0.000132,0.000371,0.000145,0.000293,0.000072,0.000271,0.000135,0.000201,0.000141,0.000204,0.000171,0.000237,0.000126,0.000167,0.000093,0.000231,0.000074,0.000160,0.000043,0.000250,0.000055,0.000217,0.000066,0.000199,0.000066,0.000231,0.000104,0.000233,0.000061,0.000246,0.000078,0.000219,0.000099,0.000254,0.000073,0.000200,0.000053,0.000175,0.000057,0.000197,0.000066,0.000126,0.000068,0.000163,0.000069,0.000139,0.000114,0.000270,0.000106
giMute141 ftgen 0,0,4096,10,0.096700,0.501938,0.304973,0.388545,0.605784,0.973491,0.310298,0.166062,0.031823,0.020041,0.008197,0.018099,0.031406,0.032118,0.028621,0.035283,0.026152,0.031724,0.045821,0.032749,0.023933,0.009329,0.006590,0.003520,0.002035,0.005414,0.003178,0.001420,0.002462,0.001216,0.000800,0.002760,0.002395,0.001093,0.001088,0.001647,0.001527,0.000979,0.001163,0.001119,0.001071,0.001064,0.001131,0.001297,0.001070,0.001080,0.000990,0.000967,0.000985,0.001105,0.000916,0.000726,0.000869,0.000896,0.000931,0.000845,0.000797,0.000762,0.000805,0.000802,0.000683,0.000685,0.000708,0.000714,0.000719,0.000727,0.000634,0.000711,0.000686,0.000637,0.000694,0.000701,0.000592,0.000664,0.000534,0.000558,0.000681,0.000688,0.000614,0.000609,0.000647,0.000479,0.000513,0.000489,0.000396,0.000420,0.000504,0.000454,0.000492,0.000467,0.000503,0.000515,0.000470,0.000401,0.000502,0.000592,0.000567,0.000411,0.000453,0.000449
giMute142 ftgen 0,0,4096,10,0.079836,0.330689,0.372144,0.493873,0.925375,0.694151,0.118777,0.028983,0.015396,0.009622,0.001526,0.002627,0.008783,0.011024,0.010140,0.015272,0.009621,0.006265,0.004497,0.003760,0.002472,0.001108,0.000363,0.000521,0.000643,0.000834,0.000511,0.000843,0.000434,0.000735,0.001056,0.000991,0.001287,0.000997,0.000751,0.000669,0.000309,0.000891,0.000656,0.000613,0.000705,0.000663,0.000676,0.000561,0.000583,0.000611,0.000611,0.000702,0.000495,0.000478,0.000451,0.000393,0.000461,0.000506,0.000433,0.000473,0.000465,0.000454,0.000425,0.000361,0.000423,0.000458,0.000385,0.000422,0.000473,0.000380,0.000405,0.000421,0.000338,0.000379,0.000351,0.000358,0.000335,0.000353,0.000378,0.000372,0.000313,0.000331,0.000343,0.000315,0.000320,0.000280,0.000296,0.000226,0.000364,0.000296,0.000272,0.000282,0.000254,0.000289,0.000277,0.000204,0.000220,0.000269,0.000299,0.000321,0.000450,0.000259,0.000266,0.000435
giMute143 ftgen 0,0,4096,10,0.093603,0.658042,0.988507,0.565236,0.957972,0.360885,0.102049,0.051660,0.027922,0.005483,0.010335,0.012495,0.003673,0.003952,0.012964,0.008898,0.005397,0.006281,0.001252,0.006122,0.003828,0.000827,0.003226,0.002089,0.001925,0.000243,0.002872,0.002421,0.003325,0.002270,0.001755,0.001168,0.001149,0.000770,0.000928,0.000937,0.000720,0.000406,0.000688,0.000732,0.000790,0.000819,0.000398,0.000728,0.000703,0.001198,0.000151,0.000857,0.000600,0.000571,0.000660,0.000529,0.000450,0.000520,0.000520,0.000470,0.000463,0.000462,0.000454,0.000347,0.000590,0.000427,0.000210,0.000402,0.000401,0.000381,0.000525,0.000294,0.000410,0.000335,0.000296,0.000348,0.000580,0.000399,0.000186,0.000198,0.000288,0.000442,0.000411,0.000234,0.000356,0.000386,0.000271,0.000338,0.000271,0.000303,0.000385,0.000352,0.000226,0.000193,0.000103,0.000388,0.000349,0.000222,0.000208,0.000266,0.000232,0.000314,0.000425,0.000268
giMute144 ftgen 0,0,4096,10,0.127739,0.412859,0.669901,0.617303,0.972749,0.358440,0.065748,0.045528,0.010129,0.008737,0.005631,0.028396,0.004839,0.008872,0.006019,0.011096,0.006903,0.003885,0.004722,0.003204,0.000936,0.002573,0.001532,0.001369,0.001172,0.001160,0.002432,0.000544,0.000609,0.001867,0.000966,0.001039,0.001031,0.000884,0.000753,0.000848,0.000659,0.000749,0.000579,0.000769,0.000665,0.000754,0.000781,0.000490,0.000653,0.000626,0.000524,0.000572,0.000539,0.000566,0.000514,0.000414,0.000441,0.000453,0.000417,0.000478,0.000444,0.000495,0.000551,0.000346,0.000475,0.000442,0.000477,0.000269,0.000501,0.000483,0.000442,0.000267,0.000338,0.000368,0.000470,0.000382,0.000335,0.000358,0.000341,0.000354,0.000345,0.000310,0.000359,0.000228,0.000299,0.000348,0.000336,0.000319,0.000275,0.000294,0.000244,0.000271,0.000232,0.000358,0.000237,0.000269,0.000247,0.000360,0.000301,0.000208,0.000223,0.000248,0.000270,0.000258
giMute145 ftgen 0,0,4096,10,0.172584,0.253006,0.499844,0.963093,0.599448,0.063091,0.027923,0.009556,0.005430,0.007267,0.030576,0.010839,0.007632,0.008691,0.007373,0.004773,0.004338,0.006857,0.003189,0.000319,0.002787,0.001103,0.001002,0.001076,0.001129,0.001241,0.000702,0.000657,0.000919,0.000769,0.000959,0.000576,0.000715,0.000619,0.000732,0.000611,0.000499,0.000600,0.000491,0.000700,0.000580,0.000348,0.000432,0.000611,0.000489,0.000494,0.000446,0.000425,0.000537,0.000395,0.000420,0.000348,0.000423,0.000452,0.000417,0.000289,0.000444,0.000331,0.000335,0.000289,0.000366,0.000340,0.000289,0.000333,0.000336,0.000346,0.000297,0.000305,0.000318,0.000278,0.000199,0.000327,0.000258,0.000240,0.000298,0.000311,0.000247,0.000309,0.000274,0.000297,0.000343,0.000148,0.000267,0.000204,0.000326,0.000251,0.000194,0.000230,0.000250,0.000186,0.000238,0.000251,0.000196,0.000238,0.000292,0.000233,0.000162,0.000197,0.000198,0.000193
giMute146 ftgen 0,0,4096,10,0.169930,0.357871,0.478870,0.971148,0.103667,0.027577,0.012460,0.004728,0.004455,0.018143,0.018903,0.009745,0.013588,0.009707,0.007034,0.005601,0.003105,0.000981,0.000341,0.001493,0.000631,0.000446,0.000832,0.000522,0.000996,0.001277,0.000334,0.000333,0.000435,0.000326,0.000323,0.000353,0.000202,0.000214,0.000190,0.000284,0.000171,0.000339,0.000329,0.000183,0.000124,0.000342,0.000303,0.000128,0.000091,0.000224,0.000255,0.000167,0.000148,0.000089,0.000154,0.000248,0.000151,0.000141,0.000149,0.000108,0.000284,0.000121,0.000121,0.000193,0.000201,0.000123,0.000226,0.000223,0.000154,0.000094,0.000194,0.000058,0.000146,0.000232,0.000097,0.000224,0.000132,0.000062,0.000163,0.000214,0.000246,0.000418,0.000139,0.000221,0.000100,0.000095,0.000131,0.000152,0.000158,0.000051,0.000077,0.000080,0.000061,0.000131,0.000109,0.000055,0.000120,0.000091,0.000080,0.000081,0.000120,0.000053,0.000045,0.000123
giMute147 ftgen 0,0,4096,10,0.175950,0.306151,0.921267,0.803899,0.232506,0.029817,0.015312,0.002411,0.010177,0.008768,0.003962,0.004998,0.003485,0.001495,0.002390,0.001727,0.000838,0.000237,0.000572,0.000579,0.000740,0.001153,0.000286,0.000378,0.000226,0.000269,0.000314,0.000265,0.000200,0.000117,0.000250,0.000150,0.000183,0.000079,0.000168,0.000177,0.000049,0.000188,0.000111,0.000136,0.000108,0.000088,0.000122,0.000076,0.000110,0.000119,0.000091,0.000137,0.000123,0.000074,0.000144,0.000146,0.000077,0.000035,0.000056,0.000085,0.000020,0.000095,0.000092,0.000051,0.000122,0.000063,0.000147,0.000073,0.000076,0.000045,0.000098,0.000114,0.000063,0.000075,0.000037,0.000125,0.000019,0.000072,0.000077,0.000087,0.000132,0.000061,0.000112,0.000071,0.000050,0.000051,0.000054,0.000046,0.000107,0.000026,0.000027,0.000021,0.000035,0.000042,0.000030,0.000041,0.000038,0.000047,0.000015,0.000021,0.000032,0.000120,0.000147,0.000238
giMute148 ftgen 0,0,4096,10,0.546831,0.588315,0.974745,0.966438,0.134855,0.030603,0.009346,0.000932,0.030580,0.012583,0.020113,0.009766,0.016236,0.007109,0.004226,0.002316,0.001376,0.000358,0.001380,0.001135,0.000319,0.000940,0.000401,0.000703,0.000385,0.000515,0.000292,0.000252,0.000145,0.000137,0.000403,0.000278,0.000292,0.000275,0.000222,0.000110,0.000160,0.000085,0.000217,0.000270,0.000172,0.000158,0.000177,0.000095,0.000182,0.000067,0.000232,0.000159,0.000178,0.000150,0.000206,0.000086,0.000273,0.000112,0.000141,0.000195,0.000084,0.000079,0.000246,0.000201,0.000063,0.000129,0.000125,0.000117,0.000158,0.000162,0.000192,0.000121,0.000175,0.000267,0.000096,0.000179,0.000039,0.000074,0.000268,0.000160,0.000033,0.000069,0.000065,0.000055,0.000088,0.000054,0.000142,0.000087,0.000045,0.000095,0.000026,0.000039,0.000075,0.000073,0.000067,0.000054,0.000068,0.000091,0.000101,0.000092,0.000171,0.000084,0.000049,0.000055
giMute149 ftgen 0,0,4096,10,0.294629,0.975075,0.694387,0.796446,0.188314,0.013147,0.006022,0.018607,0.064160,0.026936,0.065686,0.065969,0.031796,0.020120,0.015238,0.003443,0.002738,0.001360,0.003362,0.001182,0.001135,0.002047,0.001013,0.000558,0.000429,0.000717,0.000504,0.000555,0.000624,0.000479,0.000343,0.000421,0.000606,0.000462,0.000385,0.000445,0.000230,0.000176,0.000301,0.000222,0.000323,0.000195,0.000306,0.000246,0.000307,0.000237,0.000215,0.000210,0.000380,0.000241,0.000377,0.000152,0.000491,0.000197,0.000367,0.000186,0.000204,0.000108,0.000304,0.000260,0.000093,0.000231,0.000163,0.000112,0.000185,0.000239,0.000226,0.000179,0.000200,0.000203,0.000175,0.000088,0.000156,0.000218,0.000201,0.000222,0.000100,0.000212,0.000151,0.000114,0.000168,0.000167,0.000170,0.000170,0.000116,0.000205,0.000116,0.000133,0.000139,0.000176,0.000154,0.000145,0.000119,0.000095,0.000085,0.000125,0.000145,0.000106,0.000145,0.000141
giMute150 ftgen 0,0,4096,10,0.594912,0.608509,0.961469,0.229985,0.033626,0.009789,0.002734,0.011904,0.013878,0.018699,0.018956,0.006098,0.006134,0.002889,0.000518,0.000310,0.000591,0.000264,0.000257,0.000698,0.000478,0.000286,0.000276,0.000212,0.000349,0.000257,0.000272,0.000224,0.000177,0.000174,0.000082,0.000176,0.000161,0.000172,0.000230,0.000066,0.000165,0.000161,0.000152,0.000109,0.000094,0.000130,0.000185,0.000186,0.000258,0.000200,0.000154,0.000204,0.000184,0.000157,0.000047,0.000131,0.000123,0.000097,0.000062,0.000144,0.000118,0.000149,0.000134,0.000157,0.000198,0.000063,0.000156,0.000144,0.000094,0.000105,0.000096,0.000093,0.000113,0.000081,0.000076,0.000079,0.000049,0.000061,0.000040,0.000104,0.000079,0.000022,0.000068,0.000078,0.000072,0.000063,0.000085,0.000086,0.000053,0.000080,0.000082,0.000054,0.000046,0.000081,0.000075,0.000056,0.000061,0.000025,0.000072,0.000042,0.000050,0.000054,0.000053,0.000068
giMute151 ftgen 0,0,4096,10,0.206347,0.510370,0.950258,0.127507,0.010206,0.002549,0.001810,0.012483,0.004635,0.004683,0.005187,0.003165,0.001760,0.000999,0.000554,0.000268,0.000624,0.000195,0.000027,0.000147,0.000141,0.000163,0.000117,0.000060,0.000105,0.000099,0.000067,0.000093,0.000160,0.000103,0.000140,0.000060,0.000090,0.000115,0.000051,0.000073,0.000045,0.000055,0.000143,0.000072,0.000114,0.000130,0.000048,0.000106,0.000081,0.000052,0.000096,0.000074,0.000050,0.000088,0.000108,0.000167,0.000065,0.000093,0.000168,0.000053,0.000085,0.000083,0.000061,0.000128,0.000058,0.000064,0.000040,0.000027,0.000100,0.000081,0.000054,0.000030,0.000036,0.000081,0.000048,0.000047,0.000067,0.000034,0.000037,0.000030,0.000021,0.000028,0.000043,0.000106,0.000063,0.000019,0.000039,0.000057,0.000021,0.000037,0.000026,0.000041,0.000039,0.000032,0.000024,0.000048,0.000021,0.000064,0.000037,0.000065,0.000049,0.000031,0.000048,0.000014
giMute152 ftgen 0,0,4096,10,0.718310,0.828992,0.971094,0.089148,0.007432,0.001882,0.006326,0.016275,0.013664,0.005083,0.004192,0.002987,0.000575,0.000244,0.000598,0.000195,0.000566,0.000394,0.000580,0.000162,0.000255,0.000166,0.000116,0.000132,0.000146,0.000108,0.000133,0.000237,0.000251,0.000229,0.000158,0.000127,0.000139,0.000150,0.000119,0.000043,0.000076,0.000198,0.000147,0.000151,0.000075,0.000073,0.000078,0.000098,0.000125,0.000095,0.000098,0.000053,0.000180,0.000068,0.000097,0.000168,0.000085,0.000055,0.000094,0.000084,0.000102,0.000047,0.000079,0.000083,0.000046,0.000050,0.000047,0.000066,0.000081,0.000049,0.000040,0.000040,0.000068,0.000075,0.000057,0.000054,0.000024,0.000106,0.000054,0.000084,0.000044,0.000058,0.000034,0.000045,0.000058,0.000029,0.000068,0.000046,0.000029,0.000029,0.000066,0.000045,0.000077,0.000038,0.000027,0.000035,0.000043,0.000036,0.000049,0.000024,0.000024,0.000045,0.000046,0.000034
giMute153 ftgen 0,0,4096,10,0.551061,0.501444,0.953401,0.068491,0.008520,0.012056,0.020903,0.021636,0.008585,0.007399,0.006298,0.001414,0.000788,0.000891,0.000358,0.000686,0.001110,0.000613,0.000691,0.000460,0.000664,0.000614,0.000565,0.000426,0.000472,0.000397,0.000391,0.000456,0.000378,0.000437,0.000434,0.000310,0.000368,0.000415,0.000400,0.000343,0.000353,0.000349,0.000357,0.000326,0.000313,0.000231,0.000257,0.000212,0.000281,0.000273,0.000286,0.000207,0.000174,0.000326,0.000097,0.000332,0.000115,0.000149,0.000211,0.000305,0.000186,0.000198,0.000220,0.000141,0.000131,0.000182,0.000188,0.000186,0.000182,0.000117,0.000163,0.000173,0.000185,0.000121,0.000186,0.000073,0.000203,0.000192,0.000150,0.000147,0.000131,0.000138,0.000145,0.000135,0.000111,0.000135,0.000109,0.000125,0.000126,0.000121,0.000105,0.000083,0.000102,0.000109,0.000069,0.000107,0.000148,0.000139,0.000126,0.000096,0.000101,0.000123,0.000104,0.000104
giMute154 ftgen 0,0,4096,10,0.965465,0.238341,0.801452,0.050222,0.006640,0.011475,0.012533,0.012724,0.004704,0.006848,0.004228,0.001130,0.001409,0.001463,0.000398,0.000800,0.000513,0.001091,0.001100,0.000393,0.000445,0.000310,0.000446,0.000589,0.000358,0.000505,0.000369,0.000525,0.000346,0.000322,0.000437,0.000435,0.000262,0.000286,0.000307,0.000308,0.000358,0.000350,0.000347,0.000237,0.000076,0.000385,0.000217,0.000215,0.000278,0.000200,0.000335,0.000203,0.000329,0.000194,0.000066,0.000171,0.000388,0.000178,0.000305,0.000205,0.000231,0.000150,0.000228,0.000249,0.000070,0.000179,0.000201,0.000134,0.000138,0.000151,0.000193,0.000156,0.000150,0.000137,0.000132,0.000132,0.000128,0.000118,0.000138,0.000126,0.000141,0.000133,0.000148,0.000184,0.000137,0.000131,0.000138,0.000107,0.000100,0.000116,0.000121,0.000092,0.000099,0.000106,0.000080,0.000101,0.000102,0.000073,0.000094,0.000104,0.000107,0.000106,0.000093,0.000085
giMute155 ftgen 0,0,4096,10,0.938142,0.970313,0.508740,0.021568,0.001864,0.002894,0.002724,0.002263,0.003861,0.001793,0.000443,0.001267,0.000409,0.000899,0.000541,0.000180,0.000417,0.000464,0.000377,0.000359,0.000247,0.000485,0.000254,0.000339,0.000219,0.000319,0.000168,0.000254,0.000251,0.000160,0.000021,0.000183,0.000145,0.000251,0.000161,0.000219,0.000034,0.000174,0.000112,0.000219,0.000100,0.000130,0.000289,0.000226,0.000368,0.000095,0.000094,0.000122,0.000072,0.000121,0.000126,0.000143,0.000135,0.000153,0.000050,0.000099,0.000080,0.000071,0.000089,0.000079,0.000117,0.000090,0.000082,0.000068,0.000126,0.000073,0.000077,0.000094,0.000076,0.000063,0.000056,0.000070,0.000076,0.000063,0.000036,0.000059,0.000040,0.000039,0.000031,0.000048,0.000048,0.000066,0.000072,0.000047,0.000057,0.000058,0.000066,0.000033,0.000044,0.000035,0.000051,0.000038,0.000039,0.000047,0.000046,0.000052,0.000050,0.000033,0.000052,0.000039
giMute156 ftgen 0,0,4096,10,0.625565,0.965980,0.202403,0.024063,0.002481,0.005337,0.005024,0.008205,0.003554,0.001210,0.000983,0.001570,0.000901,0.000291,0.000664,0.000883,0.000489,0.000671,0.000602,0.000605,0.000552,0.000393,0.000580,0.000544,0.000508,0.000536,0.000523,0.000420,0.000400,0.000436,0.000414,0.000508,0.000302,0.000362,0.000336,0.000489,0.000355,0.000362,0.000320,0.000350,0.000246,0.000251,0.000326,0.000214,0.000175,0.000260,0.000273,0.000224,0.000202,0.000232,0.000185,0.000237,0.000245,0.000169,0.000199,0.000160,0.000153,0.000211,0.000161,0.000157,0.000165,0.000177,0.000207,0.000214,0.000158,0.000167,0.000146,0.000138,0.000161,0.000162,0.000163,0.000138,0.000160,0.000141,0.000113,0.000139,0.000144,0.000099,0.000091,0.000160,0.000120,0.000102,0.000131,0.000108,0.000113,0.000114,0.000113,0.000111,0.000105,0.000113,0.000123,0.000107,0.000110,0.000112,0.000110,0.000106,0.000122,0.000112,0.000105,0.000101
giMute157 ftgen 0,0,4096,10,0.314475,0.959696,0.048444,0.004582,0.006574,0.012611,0.005426,0.002115,0.000604,0.000404,0.000356,0.000685,0.001096,0.000409,0.000072,0.000321,0.000329,0.000236,0.000403,0.000240,0.000158,0.000233,0.000268,0.000168,0.000154,0.000209,0.000165,0.000121,0.000111,0.000136,0.000085,0.000217,0.000086,0.000083,0.000149,0.000151,0.000115,0.000148,0.000123,0.000087,0.000095,0.000113,0.000089,0.000044,0.000137,0.000116,0.000130,0.000027,0.000101,0.000085,0.000121,0.000079,0.000106,0.000042,0.000011,0.000071,0.000080,0.000029,0.000039,0.000081,0.000041,0.000060,0.000040,0.000068,0.000066,0.000052,0.000050,0.000031,0.000056,0.000054,0.000039,0.000051,0.000037,0.000059,0.000054,0.000037,0.000046,0.000043,0.000038,0.000041,0.000031,0.000049,0.000033,0.000040,0.000038,0.000035,0.000050,0.000040,0.000043,0.000039,0.000051,0.000035,0.000045,0.000020,0.000038,0.000032,0.000026,0.000030,0.000031,0.000030
giMute158 ftgen 0,0,4096,10,0.428817,0.965152,0.013016,0.001790,0.004347,0.004997,0.005604,0.002582,0.000836,0.000569,0.000599,0.000465,0.000517,0.000563,0.000538,0.000350,0.000281,0.000361,0.000460,0.000274,0.000209,0.000199,0.000249,0.000304,0.000340,0.000213,0.000240,0.000152,0.000238,0.000157,0.000200,0.000196,0.000164,0.000118,0.000156,0.000141,0.000124,0.000105,0.000108,0.000150,0.000140,0.000124,0.000109,0.000104,0.000111,0.000096,0.000120,0.000092,0.000094,0.000103,0.000077,0.000064,0.000066,0.000111,0.000094,0.000093,0.000082,0.000078,0.000073,0.000096,0.000082,0.000079,0.000075,0.000072,0.000085,0.000068,0.000072,0.000053,0.000069,0.000054,0.000057,0.000058,0.000058,0.000065,0.000055,0.000063,0.000066,0.000063,0.000063,0.000058,0.000053,0.000052,0.000046,0.000046,0.000050,0.000049,0.000047,0.000045,0.000043,0.000050,0.000047,0.000052,0.000050,0.000049,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
giwavemapMute1        ftgen    0,0,128,-27, 0,0, 22,0, 23,1, 24,2, 25,3, 26,4, 27,5, 28,6, 29,7, 30,8, 31,9, 32,10, 33,11, 34,12, 35,13, 36,14, 37,15, 38,16, 39,17, 40,18, 41,19, 42,20, 43,21, 44,22, 45,23, 46,24, 47,25, 48,26, 49,27, 50,28, 51,29, 52,30, 53,31, 54,32, 55,33, 56,34, 57,35, 58,36, 127,36
gitabs4morfMute1    ftgen    0,0,64,-2, giMute122,giMute123,giMute124,giMute125,giMute126,giMute127,giMute128,giMute129,giMute130,giMute131,giMute132,giMute133,giMute134,giMute135,giMute136,giMute137,giMute138,giMute139,giMute140,giMute141,giMute142,giMute143,giMute144,giMute145,giMute146,giMute147,giMute148,giMute149,giMute150,giMute151,giMute152,giMute153,giMute154,giMute155,giMute156,giMute157,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158,giMute158

;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; BASS TROMBONE (OPEN)
;---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
giUnmuted22 ftgen 0,0,4096,10,0.209301,0.150536,0.083014,0.207956,0.321939,0.399510,0.327987,0.421479,0.539528,0.533091,0.721756,0.467196,0.947246,0.770314,0.912692,0.965128,0.962101,1.093136,0.708144,0.664071,0.619031,0.643414,0.371645,0.533120,0.477851,0.481481,0.451545,0.370726,0.360377,0.459325,0.288110,0.235271,0.205264,0.198828,0.206768,0.219183,0.205657,0.164230,0.168498,0.156362,0.109323,0.096989,0.105552,0.095551,0.069663,0.068933,0.067166,0.055414,0.072408,0.055978,0.033534,0.049491,0.054904,0.029956,0.018835,0.027602,0.035912,0.015979,0.021303,0.011033,0.024278,0.030411,0.020777,0.027002,0.020715,0.010245,0.010418,0.019612,0.017841,0.017176,0.012772,0.008424,0.009403,0.002947,0.006355,0.006158,0.015418,0.015477,0.011268,0.008620,0.008935,0.006072,0.005458,0.006560,0.006368,0.007559,0.004779,0.008830,0.007041,0.006334,0.002719,0.003830,0.006208,0.008425,0.006728,0.006139,0.002789,0.007133,0.005592,0.005069
giUnmuted23 ftgen 0,0,4096,10,0.129222,0.088552,0.116290,0.170792,0.176577,0.331680,0.281710,0.247957,0.379599,0.404781,0.432966,0.873924,0.774290,0.657216,0.986727,0.929760,0.832314,0.808580,0.585809,0.684034,0.674170,0.330450,0.888687,0.598954,0.599019,0.566506,0.632303,0.502653,0.431058,0.214554,0.351572,0.335693,0.277218,0.341691,0.264970,0.296760,0.293654,0.224055,0.213823,0.138880,0.188425,0.156849,0.137821,0.185206,0.172163,0.184744,0.153256,0.116401,0.138614,0.155349,0.095606,0.075646,0.091523,0.087233,0.081355,0.068920,0.094557,0.066191,0.069861,0.065915,0.072343,0.065165,0.065968,0.041851,0.063905,0.036430,0.041801,0.035516,0.038545,0.044169,0.038034,0.042219,0.032505,0.038202,0.034556,0.037548,0.036070,0.036673,0.037173,0.024089,0.029610,0.028726,0.018206,0.028118,0.023534,0.023563,0.025997,0.024870,0.021673,0.024752,0.020165,0.019645,0.016172,0.017942,0.018241,0.016560,0.014997,0.013671,0.015848,0.011383
giUnmuted24 ftgen 0,0,4096,10,0.014100,0.027948,0.037471,0.118589,0.278733,0.166216,0.237592,0.204888,0.303225,0.239198,0.590081,0.558590,0.488846,0.934503,0.791766,0.769104,0.451529,0.465344,0.363957,0.375648,0.362512,0.527955,0.584553,0.562238,0.409571,0.447397,0.547498,0.349795,0.363085,0.318352,0.305372,0.298282,0.236937,0.259059,0.251424,0.235425,0.253896,0.305811,0.227171,0.177695,0.218484,0.204418,0.227246,0.182265,0.183710,0.198997,0.141695,0.093968,0.119182,0.128975,0.118829,0.086353,0.143691,0.082113,0.096768,0.076006,0.108551,0.073045,0.080734,0.075566,0.076978,0.047507,0.057911,0.055554,0.070030,0.060662,0.054519,0.055938,0.064493,0.032839,0.043612,0.052568,0.034145,0.054702,0.041937,0.045108,0.037413,0.033601,0.044130,0.036376,0.036994,0.032488,0.031456,0.030170,0.029857,0.031038,0.027679,0.029891,0.027195,0.023506,0.027610,0.023146,0.022810,0.019058,0.018170,0.018749,0.017019,0.015480,0.016379,0.015893
giUnmuted25 ftgen 0,0,4096,10,0.075402,0.101145,0.176728,0.270616,0.291285,0.324780,0.333295,0.233831,0.290677,0.540830,0.767506,0.512758,0.980849,0.771417,0.804053,0.408122,0.631555,0.670085,0.408376,0.681153,0.911500,0.859956,0.697578,0.595419,0.593307,0.469251,0.543682,0.529757,0.367768,0.498910,0.305667,0.300578,0.386836,0.363336,0.347656,0.332606,0.322633,0.253819,0.316454,0.284584,0.227632,0.212815,0.324061,0.212387,0.161264,0.199347,0.187103,0.169679,0.152185,0.206879,0.129213,0.127022,0.158412,0.109282,0.134290,0.097383,0.119959,0.099454,0.111739,0.076911,0.100854,0.086224,0.089536,0.075617,0.075879,0.080690,0.071350,0.079834,0.089408,0.079954,0.081367,0.068297,0.067636,0.067458,0.048890,0.064138,0.053887,0.056479,0.047270,0.039648,0.049324,0.050416,0.048009,0.042951,0.034023,0.037808,0.026799,0.042091,0.023668,0.027252,0.027430,0.027822,0.030900,0.025171,0.030639,0.031093,0.018787,0.024957,0.020116,0.016682
giUnmuted26 ftgen 0,0,4096,10,0.043072,0.066480,0.103784,0.299131,0.266755,0.210417,0.254095,0.299260,0.120587,0.627381,0.502409,0.682945,0.896451,0.718962,0.798438,0.342935,0.438196,0.391451,0.571924,0.500613,0.522119,0.554206,0.351133,0.573747,0.415485,0.314183,0.377587,0.407768,0.269407,0.243405,0.358414,0.278435,0.297179,0.205957,0.278884,0.139884,0.230059,0.203655,0.273754,0.208405,0.268930,0.220588,0.093617,0.135109,0.140624,0.125541,0.127288,0.075447,0.145203,0.086283,0.144206,0.092668,0.097130,0.101259,0.096756,0.087998,0.053647,0.093505,0.075780,0.063445,0.057768,0.067925,0.062568,0.059886,0.049874,0.061576,0.047362,0.059277,0.045356,0.045205,0.049725,0.059766,0.051216,0.043048,0.047544,0.034671,0.055494,0.029430,0.038554,0.039949,0.030711,0.033481,0.031643,0.028951,0.022707,0.022752,0.019352,0.028349,0.020961,0.022731,0.022740,0.021957,0.021687,0.023511,0.015741,0.017651,0.017310,0.015487,0.015835,0.014889
giUnmuted27 ftgen 0,0,4096,10,0.116392,0.064448,0.127432,0.239485,0.257590,0.362741,0.449254,0.199901,0.348867,0.637406,0.429112,0.918400,0.936649,0.907354,0.601632,0.359938,0.492393,0.465346,0.501258,0.682788,0.539385,0.377763,0.493182,0.284353,0.308310,0.236111,0.324894,0.290903,0.171987,0.259098,0.209363,0.188311,0.165040,0.129121,0.179891,0.142395,0.192837,0.155400,0.140506,0.154670,0.085903,0.084914,0.123353,0.033350,0.085811,0.066361,0.064805,0.053334,0.059168,0.051871,0.024556,0.050781,0.036654,0.034625,0.037417,0.026993,0.036074,0.041271,0.034852,0.030858,0.035521,0.039943,0.029248,0.023157,0.034015,0.022517,0.032273,0.025478,0.021464,0.021306,0.020706,0.015205,0.017217,0.016590,0.014839,0.015475,0.014872,0.012173,0.018427,0.015918,0.010766,0.011403,0.007469,0.006837,0.010460,0.009821,0.011490,0.012841,0.009228,0.009485,0.005268,0.008016,0.007500,0.007632,0.009359,0.005060,0.006834,0.006729,0.005832,0.006610
giUnmuted28 ftgen 0,0,4096,10,0.076757,0.165970,0.231682,0.474308,0.397207,0.459106,0.300853,0.244548,0.666795,0.602133,0.902710,0.655767,0.483398,0.313870,0.349970,0.212690,0.704653,0.764853,0.648917,0.660316,0.500843,0.406945,0.437095,0.364433,0.332150,0.247819,0.335990,0.413334,0.272378,0.303040,0.347189,0.202684,0.273591,0.358844,0.326756,0.187244,0.245830,0.136904,0.149347,0.181123,0.074118,0.123133,0.099890,0.059919,0.124875,0.090488,0.085029,0.115332,0.036687,0.106389,0.065025,0.070006,0.054270,0.053720,0.068047,0.055966,0.079658,0.070141,0.046135,0.065892,0.044587,0.048301,0.044148,0.045919,0.038547,0.040422,0.052374,0.032757,0.026433,0.030900,0.031810,0.034349,0.013986,0.030399,0.015539,0.018593,0.017769,0.024225,0.019160,0.022166,0.018451,0.016544,0.010230,0.012227,0.012961,0.015889,0.015651,0.009872,0.014459,0.011942,0.015592,0.014711,0.013394,0.010871,0.008229,0.007572,0.008106,0.008828,0.011451,0.009952
giUnmuted30 ftgen 0,0,4096,10,0.102601,0.100887,0.167098,0.516781,0.330986,0.422868,0.412850,0.397849,0.628756,0.686835,0.929709,0.845577,0.935266,0.508059,0.792158,0.281350,0.378308,0.568489,0.346813,0.566233,0.413541,0.405762,0.359363,0.216505,0.366221,0.301854,0.216945,0.249969,0.168763,0.194386,0.278248,0.046419,0.257812,0.124464,0.096054,0.116201,0.059755,0.096572,0.074147,0.047405,0.054049,0.073919,0.023499,0.080643,0.079127,0.031933,0.076588,0.051990,0.032645,0.031250,0.046055,0.022654,0.038293,0.031067,0.045355,0.035515,0.043591,0.024758,0.020123,0.028097,0.025231,0.014737,0.015258,0.020891,0.010029,0.015487,0.004561,0.009797,0.007038,0.006923,0.012160,0.005577,0.012809,0.007007,0.007626,0.004440,0.008034,0.008475,0.006052,0.009027,0.006419,0.007999,0.005562,0.004974,0.008232,0.007159,0.005841,0.004633,0.005878,0.004708,0.005378,0.005775,0.004317,0.003135,0.006019,0.002682,0.003519,0.002106,0.001173,0.002911
giUnmuted31 ftgen 0,0,4096,10,0.066770,0.048141,0.313114,0.280252,0.266590,0.227818,0.195576,0.598114,0.585360,0.631625,0.922780,0.476250,0.557175,0.516114,0.284400,0.347304,0.327824,0.186298,0.429619,0.319332,0.143321,0.268362,0.179246,0.187779,0.187831,0.069696,0.157916,0.087219,0.051108,0.180413,0.090807,0.061893,0.087674,0.029616,0.053852,0.035420,0.032698,0.030786,0.031084,0.040347,0.022290,0.019770,0.033986,0.022503,0.008514,0.028204,0.013058,0.011683,0.016372,0.011838,0.005125,0.012279,0.007251,0.009685,0.008196,0.009381,0.007698,0.004608,0.004124,0.002835,0.002636,0.005858,0.004185,0.001287,0.002347,0.002862,0.004396,0.003505,0.001776,0.002038,0.003386,0.001389,0.002214,0.001394,0.002558,0.001971,0.001212,0.000993,0.001220,0.001435,0.001036,0.000315,0.000862,0.001509,0.000972,0.000423,0.000797,0.001030,0.001800,0.000491,0.000318,0.000990,0.000987,0.000840,0.000380,0.000536,0.000975,0.000576,0.000444,0.000286
giUnmuted32 ftgen 0,0,4096,10,0.153577,0.222613,0.376395,0.526417,0.515836,0.662033,0.484716,0.943689,0.936088,0.683402,0.940687,0.666224,0.399824,0.298609,0.378373,0.633100,0.213897,0.505597,0.312328,0.232655,0.144484,0.129531,0.225201,0.231697,0.116577,0.093467,0.083941,0.021798,0.081229,0.048477,0.071105,0.034486,0.057723,0.028194,0.029190,0.019811,0.027128,0.025539,0.014725,0.015158,0.019471,0.022215,0.011935,0.015510,0.012182,0.010768,0.007979,0.010226,0.004972,0.009394,0.006760,0.006326,0.005823,0.003912,0.005859,0.001846,0.002697,0.003683,0.003892,0.002554,0.002904,0.004195,0.000672,0.001703,0.001212,0.002070,0.001499,0.001454,0.002153,0.001936,0.000769,0.001242,0.001260,0.000672,0.000842,0.001179,0.001096,0.001223,0.001090,0.001644,0.001218,0.000337,0.000969,0.000736,0.000734,0.000228,0.000786,0.000788,0.000390,0.000223,0.001065,0.000755,0.000257,0.000513,0.000711,0.000538,0.000385,0.000437,0.000469,0.000430
giUnmuted33 ftgen 0,0,4096,10,0.199259,0.293112,0.678605,0.427095,0.796552,0.592920,0.832602,0.749712,0.753129,0.956136,0.706813,0.595711,0.292833,0.370551,0.334616,0.299621,0.458159,0.299313,0.277013,0.116169,0.127934,0.137952,0.174017,0.060317,0.067909,0.049889,0.073748,0.061114,0.028611,0.050743,0.022684,0.016755,0.015577,0.022867,0.023917,0.026384,0.015817,0.016226,0.020209,0.017937,0.014795,0.014935,0.011316,0.011197,0.003987,0.006306,0.007657,0.008754,0.008017,0.001430,0.002030,0.006426,0.004673,0.001697,0.003547,0.003128,0.001704,0.002225,0.003744,0.001255,0.001218,0.001706,0.000784,0.001874,0.002414,0.001450,0.001425,0.001431,0.001039,0.000512,0.001115,0.001012,0.000767,0.001368,0.000891,0.001254,0.000438,0.001473,0.000396,0.000944,0.000406,0.000645,0.000805,0.000497,0.000596,0.000632,0.000660,0.000453,0.000657,0.000437,0.000597,0.000534,0.000231,0.000579,0.000359,0.000597,0.000289,0.000515,0.000474,0.000508
giUnmuted34 ftgen 0,0,4096,10,0.156283,0.263433,0.503588,0.547226,0.458841,0.440476,0.745137,0.926771,0.703339,0.326809,0.455305,0.144506,0.578160,0.557536,0.390538,0.373849,0.349519,0.171391,0.136852,0.175680,0.170348,0.095999,0.101795,0.067277,0.069750,0.038210,0.062274,0.031281,0.045966,0.025999,0.019665,0.025196,0.019584,0.017978,0.018616,0.019068,0.009254,0.008323,0.009829,0.007950,0.009281,0.004125,0.006169,0.005516,0.008089,0.004623,0.003210,0.003435,0.002272,0.003102,0.002101,0.002482,0.000800,0.001308,0.001448,0.002671,0.000440,0.001102,0.000625,0.001246,0.000739,0.001198,0.000853,0.000982,0.000459,0.000870,0.000679,0.000793,0.000524,0.000192,0.000634,0.000314,0.000778,0.000405,0.000532,0.000235,0.000824,0.000204,0.000655,0.000308,0.000375,0.000408,0.000459,0.000698,0.000335,0.000370,0.000416,0.000463,0.000712,0.000202,0.000604,0.000194,0.000642,0.000492,0.000566,0.000304,0.000430,0.000666,0.000460,0.000725
giUnmuted35 ftgen 0,0,4096,10,0.109719,0.205679,0.327171,0.211979,0.275588,0.441424,0.909739,0.954713,0.704080,0.558176,0.412328,0.520298,0.672399,0.332547,0.403159,0.369356,0.239488,0.211806,0.231043,0.127727,0.249776,0.131377,0.104229,0.096938,0.102833,0.065079,0.066313,0.070405,0.083568,0.043975,0.062456,0.056332,0.053849,0.056551,0.038084,0.032551,0.031631,0.024900,0.019231,0.022793,0.025653,0.029848,0.012080,0.012343,0.008958,0.007429,0.009658,0.011782,0.006432,0.008668,0.004940,0.004961,0.011409,0.003677,0.005188,0.006053,0.002706,0.005660,0.005030,0.003473,0.004147,0.002260,0.003165,0.004611,0.001572,0.002973,0.001712,0.001380,0.002403,0.001241,0.001415,0.002559,0.000850,0.001416,0.000832,0.001164,0.001932,0.000912,0.001026,0.000996,0.000473,0.001056,0.000666,0.000709,0.000940,0.000380,0.000715,0.000818,0.000554,0.000733,0.000482,0.000638,0.000669,0.000419,0.000706,0.000551,0.000449,0.000615,0.000441,0.000574
giUnmuted37 ftgen 0,0,4096,10,0.163591,0.404564,0.575089,0.472316,0.486294,0.822510,0.806591,0.969411,0.362995,0.433080,0.496749,0.467537,0.467839,0.267451,0.183238,0.218200,0.199060,0.155508,0.175596,0.093844,0.131201,0.054473,0.064991,0.054622,0.028696,0.047964,0.032300,0.032288,0.027946,0.028929,0.025863,0.016471,0.015090,0.011768,0.012035,0.007896,0.009150,0.007422,0.006861,0.006328,0.006340,0.003206,0.001979,0.001196,0.003410,0.003344,0.001708,0.001304,0.001426,0.001391,0.002381,0.001136,0.000501,0.000621,0.001295,0.001362,0.001331,0.000559,0.000199,0.000503,0.000879,0.001032,0.000443,0.000370,0.000358,0.000616,0.000749,0.000331,0.000239,0.000475,0.000621,0.000552,0.000280,0.000188,0.000404,0.000434,0.000385,0.000500,0.000299,0.000339,0.000250,0.000436,0.000389,0.000319,0.000252,0.000279,0.000331,0.000496,0.000362,0.000299,0.000301,0.000389,0.000340,0.000331,0.000305,0.000321,0.000282,0.000312,0.000322,0.000305
giUnmuted38 ftgen 0,0,4096,10,0.088638,0.393952,0.219528,0.233796,0.390552,0.873953,0.973908,0.452420,0.406845,0.369372,0.418449,0.275682,0.233818,0.174596,0.176589,0.199987,0.084305,0.099145,0.066017,0.092595,0.045456,0.041233,0.057805,0.035677,0.029982,0.021224,0.023785,0.011487,0.013368,0.009656,0.009504,0.010544,0.006813,0.005152,0.006761,0.004720,0.005569,0.005772,0.003926,0.003876,0.001447,0.001937,0.001824,0.001260,0.001196,0.001056,0.001716,0.000597,0.000870,0.000129,0.001182,0.000562,0.000716,0.000558,0.000346,0.000458,0.000280,0.000472,0.000305,0.000628,0.000215,0.000532,0.000286,0.000686,0.000363,0.000433,0.000417,0.000257,0.000323,0.000182,0.000362,0.000182,0.000308,0.000090,0.000457,0.000168,0.000301,0.000207,0.000232,0.000295,0.000234,0.000253,0.000115,0.000278,0.000128,0.000246,0.000142,0.000206,0.000208,0.000168,0.000245,0.000156,0.000184,0.000153,0.000195,0.000110,0.000237,0.000120,0.000147,0.000197
giUnmuted39 ftgen 0,0,4096,10,0.072477,0.251689,0.379198,0.380111,0.725478,0.949509,0.692901,0.578175,0.268676,0.362999,0.292393,0.221151,0.135379,0.142198,0.166308,0.080654,0.080723,0.055948,0.052355,0.045652,0.019205,0.017803,0.021033,0.012636,0.010918,0.015171,0.012617,0.010832,0.009232,0.006483,0.005626,0.003851,0.002409,0.001860,0.003662,0.003232,0.003431,0.001824,0.002610,0.001564,0.001117,0.000540,0.000571,0.001003,0.001335,0.000552,0.000678,0.000584,0.001061,0.000360,0.000733,0.000138,0.000610,0.000585,0.000395,0.000329,0.000646,0.000491,0.000414,0.000402,0.000119,0.000403,0.000173,0.000409,0.000081,0.000323,0.000367,0.000312,0.000184,0.000390,0.000085,0.000295,0.000195,0.000268,0.000142,0.000267,0.000047,0.000196,0.000144,0.000201,0.000119,0.000264,0.000129,0.000192,0.000106,0.000089,0.000146,0.000206,0.000209,0.000106,0.000142,0.000140,0.000184,0.000083,0.000036,0.000158,0.000137,0.000135,0.000284,0.000115
giUnmuted40 ftgen 0,0,4096,10,0.225959,0.669389,0.383703,0.337797,0.897874,0.917315,0.679617,0.847733,0.413981,0.550916,0.384995,0.299373,0.229249,0.175955,0.251333,0.151143,0.072766,0.141216,0.038843,0.059075,0.024251,0.038600,0.034595,0.024303,0.012174,0.014682,0.023313,0.013042,0.011554,0.011010,0.010318,0.007734,0.007817,0.007700,0.004889,0.002809,0.002953,0.002732,0.002419,0.002929,0.001031,0.001758,0.002569,0.001934,0.000195,0.000657,0.000465,0.000529,0.000132,0.000495,0.000114,0.000374,0.000066,0.000383,0.000298,0.000374,0.000325,0.000203,0.000379,0.000195,0.000256,0.000055,0.000261,0.000143,0.000303,0.000225,0.000165,0.000287,0.000154,0.000362,0.000079,0.000339,0.000076,0.000304,0.000074,0.000377,0.000110,0.000203,0.000160,0.000220,0.000170,0.000246,0.000154,0.000238,0.000107,0.000233,0.000176,0.000143,0.000173,0.000149,0.000209,0.000112,0.000150,0.000111,0.000112,0.000142,0.000096,0.000126,0.000128,0.000116
giUnmuted41 ftgen 0,0,4096,10,0.097703,0.458893,0.214092,0.303131,0.643075,0.941287,0.605134,0.446467,0.333405,0.230717,0.265198,0.183918,0.159675,0.159325,0.106876,0.068197,0.087184,0.044215,0.041701,0.023995,0.024583,0.018444,0.017021,0.012909,0.009709,0.009292,0.005646,0.005169,0.005487,0.004703,0.003198,0.001953,0.002304,0.001896,0.001346,0.001214,0.000950,0.001006,0.000408,0.000899,0.000811,0.000603,0.000416,0.000186,0.000451,0.000580,0.000293,0.000134,0.000491,0.000610,0.000365,0.000202,0.000238,0.000284,0.000383,0.000318,0.000171,0.000207,0.000319,0.000262,0.000156,0.000201,0.000269,0.000277,0.000189,0.000255,0.000252,0.000251,0.000238,0.000176,0.000262,0.000221,0.000254,0.000156,0.000243,0.000273,0.000203,0.000144,0.000205,0.000227,0.000273,0.000231,0.000129,0.000148,0.000263,0.000231,0.000178,0.000240,0.000111,0.000150,0.000125,0.000253,0.000117,0.000217,0.000249,0.000167,0.000129,0.000153,0.000156,0.000166
giUnmuted42 ftgen 0,0,4096,10,0.124305,0.470693,0.408472,0.419845,0.726811,0.899803,0.320069,0.227380,0.170542,0.278327,0.087448,0.075657,0.070450,0.011278,0.027231,0.027560,0.017657,0.007864,0.003498,0.005879,0.002004,0.003338,0.002302,0.002026,0.001182,0.001269,0.001074,0.000364,0.000612,0.000235,0.000662,0.000064,0.000466,0.000427,0.000278,0.000267,0.000353,0.000292,0.000282,0.000261,0.000314,0.000191,0.000121,0.000300,0.000198,0.000184,0.000171,0.000233,0.000128,0.000158,0.000242,0.000129,0.000168,0.000158,0.000221,0.000105,0.000151,0.000134,0.000034,0.000239,0.000107,0.000190,0.000107,0.000133,0.000170,0.000077,0.000159,0.000133,0.000084,0.000139,0.000186,0.000114,0.000238,0.000054,0.000180,0.000124,0.000143,0.000260,0.000043,0.000132,0.000091,0.000030,0.000116,0.000080,0.000072,0.000057,0.000105,0.000076,0.000116,0.000065,0.000171,0.000053,0.000072,0.000074,0.000062,0.000114,0.000055,0.000095,0.000113,0.000044
giUnmuted43 ftgen 0,0,4096,10,0.125142,0.748133,0.488996,0.937672,0.810384,0.425414,0.265649,0.228407,0.170598,0.111821,0.121106,0.061239,0.019591,0.017183,0.019865,0.007807,0.001783,0.002592,0.001011,0.003527,0.002854,0.000493,0.002776,0.000788,0.001901,0.002961,0.000261,0.000981,0.000309,0.001260,0.001590,0.000419,0.000880,0.000853,0.000491,0.001106,0.000390,0.000507,0.000664,0.000732,0.000588,0.000322,0.000375,0.000612,0.000570,0.000284,0.000276,0.000346,0.000470,0.000510,0.000208,0.000313,0.000421,0.000286,0.000339,0.000330,0.000236,0.000285,0.000339,0.000333,0.000325,0.000301,0.000405,0.000289,0.000185,0.000254,0.000429,0.000220,0.000088,0.000342,0.000434,0.000323,0.000114,0.000423,0.000294,0.000189,0.000274,0.000329,0.000150,0.000193,0.000230,0.000213,0.000130,0.000222,0.000238,0.000078,0.000370,0.000313,0.000161,0.000338,0.000185,0.000246,0.000194,0.000179,0.000174,0.000154,0.000140,0.000235,0.000115,0.000258
giUnmuted44 ftgen 0,0,4096,10,0.302063,0.713050,0.452426,0.930055,0.680724,0.256993,0.187429,0.166946,0.128520,0.034480,0.011033,0.010095,0.010484,0.013378,0.003779,0.001842,0.002372,0.003682,0.002032,0.001052,0.001063,0.001158,0.000116,0.001123,0.001585,0.000539,0.001398,0.000809,0.000447,0.000282,0.000640,0.000371,0.000503,0.000484,0.000108,0.000162,0.000180,0.000319,0.000232,0.000211,0.000045,0.000246,0.000082,0.000069,0.000149,0.000105,0.000143,0.000097,0.000032,0.000091,0.000142,0.000202,0.000052,0.000133,0.000185,0.000054,0.000154,0.000137,0.000181,0.000042,0.000135,0.000129,0.000100,0.000108,0.000143,0.000065,0.000082,0.000182,0.000049,0.000189,0.000083,0.000061,0.000143,0.000078,0.000056,0.000079,0.000120,0.000037,0.000098,0.000063,0.000110,0.000162,0.000076,0.000078,0.000059,0.000095,0.000049,0.000104,0.000088,0.000117,0.000133,0.000031,0.000103,0.000040,0.000058,0.000025,0.000024,0.000077,0.000069,0.000155
giUnmuted45 ftgen 0,0,4096,10,0.233846,0.388855,0.488775,0.775865,0.964456,0.501325,0.291914,0.282804,0.153115,0.120898,0.100772,0.061525,0.021515,0.015412,0.010305,0.004386,0.004785,0.007854,0.004759,0.004311,0.003465,0.002391,0.000787,0.001491,0.000948,0.000812,0.000611,0.000750,0.000529,0.000347,0.000811,0.000462,0.000235,0.000129,0.000100,0.000270,0.000180,0.000344,0.000298,0.000283,0.000076,0.000033,0.000229,0.000057,0.000107,0.000302,0.000392,0.000250,0.000158,0.000123,0.000211,0.000176,0.000204,0.000052,0.000045,0.000073,0.000098,0.000048,0.000042,0.000062,0.000184,0.000108,0.000087,0.000108,0.000246,0.000201,0.000114,0.000035,0.000098,0.000163,0.000193,0.000214,0.000220,0.000138,0.000157,0.000201,0.000169,0.000255,0.000225,0.000166,0.000090,0.000040,0.000130,0.000094,0.000063,0.000032,0.000089,0.000092,0.000126,0.000045,0.000031,0.000089,0.000106,0.000113,0.000102,0.000097,0.000078,0.000087,0.000116,0.000073
giUnmuted46 ftgen 0,0,4096,10,0.278221,0.573695,0.445852,0.941888,0.349456,0.146739,0.401320,0.181176,0.109258,0.079677,0.027531,0.020462,0.010172,0.004976,0.008831,0.004459,0.003116,0.005925,0.003482,0.001955,0.000824,0.000849,0.000990,0.000555,0.000572,0.000248,0.000282,0.000312,0.000425,0.000324,0.000073,0.000533,0.000097,0.000455,0.000267,0.000126,0.000125,0.000100,0.000382,0.000051,0.000129,0.000240,0.000307,0.000083,0.000247,0.000140,0.000183,0.000087,0.000275,0.000383,0.000065,0.000167,0.000138,0.000204,0.000147,0.000389,0.000087,0.000087,0.000085,0.000041,0.000137,0.000224,0.000036,0.000053,0.000195,0.000205,0.000101,0.000065,0.000194,0.000064,0.000193,0.000258,0.000182,0.000073,0.000094,0.000173,0.000055,0.000075,0.000093,0.000129,0.000045,0.000085,0.000107,0.000117,0.000074,0.000107,0.000122,0.000085,0.000048,0.000110,0.000056,0.000075,0.000100,0.000057,0.000058,0.000070,0.000058,0.000062,0.000024,0.000053
giUnmuted48 ftgen 0,0,4096,10,0.452762,0.304359,0.946745,0.826937,0.631116,0.540533,0.395792,0.137892,0.089376,0.056327,0.024940,0.018700,0.015991,0.009294,0.006775,0.006495,0.001878,0.002247,0.000868,0.001248,0.001564,0.001249,0.001144,0.001326,0.001103,0.000911,0.000811,0.000899,0.000763,0.000818,0.000796,0.000855,0.000820,0.000710,0.000624,0.000684,0.000656,0.000643,0.000581,0.000503,0.000628,0.000640,0.000595,0.000471,0.000287,0.000521,0.000550,0.000603,0.000454,0.000276,0.000515,0.000524,0.000515,0.000548,0.000414,0.000417,0.000623,0.000306,0.000257,0.000314,0.000328,0.000356,0.000371,0.000363,0.000334,0.000261,0.000258,0.000294,0.000248,0.000426,0.000222,0.000250,0.000403,0.000222,0.000365,0.000158,0.000358,0.000267,0.000252,0.000328,0.000186,0.000291,0.000280,0.000198,0.000269,0.000258,0.000279,0.000252,0.000236,0.000257,0.000228,0.000248,0.000225,0.000222,0.000211,0.000262,0.000206,0.000219,0.000216,0.000223
giUnmuted49 ftgen 0,0,4096,10,0.384255,0.558888,0.714667,0.956957,0.356637,0.233667,0.190339,0.102786,0.061868,0.032079,0.017290,0.009522,0.005673,0.005487,0.002456,0.002036,0.000425,0.001796,0.002623,0.001222,0.000730,0.000805,0.000937,0.000709,0.000258,0.000626,0.000855,0.000810,0.000503,0.000376,0.000366,0.000792,0.000435,0.000475,0.000434,0.000367,0.000729,0.000327,0.000400,0.000367,0.000285,0.000247,0.000517,0.000305,0.000346,0.000356,0.000254,0.000337,0.000385,0.000231,0.000456,0.000215,0.000106,0.000201,0.000338,0.000284,0.000295,0.000166,0.000172,0.000132,0.000143,0.000401,0.000352,0.000229,0.000228,0.000272,0.000210,0.000234,0.000250,0.000182,0.000231,0.000084,0.000152,0.000170,0.000231,0.000207,0.000174,0.000130,0.000145,0.000280,0.000219,0.000185,0.000135,0.000104,0.000157,0.000222,0.000197,0.000090,0.000129,0.000142,0.000121,0.000183,0.000157,0.000120,0.000130,0.000101,0.000113,0.000151,0.000146,0.000114
giUnmuted50 ftgen 0,0,4096,10,0.446591,0.477584,0.426492,0.970753,0.374135,0.215282,0.306904,0.140851,0.070867,0.038834,0.002282,0.017849,0.007741,0.003585,0.006831,0.006003,0.003705,0.001554,0.003344,0.000137,0.001907,0.000924,0.001095,0.001011,0.000580,0.000998,0.001204,0.000696,0.001058,0.000873,0.000742,0.000768,0.000685,0.000814,0.000666,0.000827,0.000659,0.000713,0.000613,0.000584,0.000711,0.000452,0.000535,0.000517,0.000488,0.000572,0.000501,0.000577,0.000468,0.000403,0.000316,0.000555,0.000474,0.000513,0.000541,0.000421,0.000425,0.000418,0.000385,0.000321,0.000462,0.000415,0.000342,0.000312,0.000347,0.000323,0.000339,0.000320,0.000298,0.000318,0.000268,0.000265,0.000232,0.000302,0.000354,0.000263,0.000304,0.000222,0.000312,0.000311,0.000342,0.000363,0.000274,0.000300,0.000250,0.000296,0.000254,0.000287,0.000249,0.000199,0.000219,0.000195,0.000210,0.000230,0.000217,0.000181,0.000209,0.000200,0.000218,0.000224
giUnmuted51 ftgen 0,0,4096,10,0.287446,0.531276,0.971557,0.630155,0.300795,0.250731,0.098819,0.107126,0.053101,0.062126,0.011011,0.012837,0.004178,0.006909,0.004394,0.002253,0.002430,0.002140,0.001992,0.000671,0.000287,0.000471,0.000271,0.000130,0.000051,0.000046,0.000110,0.000021,0.000075,0.000192,0.000076,0.000065,0.000131,0.000125,0.000081,0.000150,0.000030,0.000017,0.000085,0.000079,0.000026,0.000096,0.000129,0.000153,0.000110,0.000071,0.000054,0.000060,0.000076,0.000043,0.000064,0.000017,0.000052,0.000117,0.000038,0.000047,0.000024,0.000058,0.000033,0.000021,0.000042,0.000074,0.000063,0.000027,0.000066,0.000039,0.000035,0.000030,0.000050,0.000068,0.000048,0.000058,0.000037,0.000069,0.000030,0.000032,0.000081,0.000067,0.000051,0.000033,0.000053,0.000067,0.000038,0.000025,0.000009,0.000037,0.000026,0.000021,0.000009,0.000039,0.000025,0.000029,0.000019,0.000007,0.000014,0.000020,0.000025,0.000019,0.000024,0.000011
giUnmuted52 ftgen 0,0,4096,10,0.780315,0.602043,0.971703,0.598540,0.419892,0.250576,0.135924,0.028914,0.007076,0.002504,0.001073,0.002881,0.002091,0.001033,0.001393,0.001759,0.002069,0.001160,0.001046,0.000119,0.000300,0.000679,0.000689,0.000161,0.000393,0.000523,0.000306,0.000179,0.000312,0.000431,0.000186,0.000165,0.000492,0.000276,0.000209,0.000416,0.000299,0.000153,0.000254,0.000350,0.000169,0.000155,0.000291,0.000165,0.000059,0.000258,0.000299,0.000257,0.000195,0.000128,0.000254,0.000142,0.000138,0.000172,0.000115,0.000086,0.000226,0.000168,0.000117,0.000109,0.000203,0.000108,0.000106,0.000096,0.000163,0.000088,0.000138,0.000083,0.000076,0.000118,0.000119,0.000130,0.000104,0.000106,0.000157,0.000102,0.000046,0.000053,0.000071,0.000098,0.000102,0.000096,0.000078,0.000080,0.000075,0.000076,0.000080,0.000077,0.000073,0.000056,0.000081,0.000076,0.000093,0.000054,0.000098,0.000069,0.000047,0.000076,0.000050,0.000039
giUnmuted53 ftgen 0,0,4096,10,0.372046,0.398945,0.933433,0.303716,0.202319,0.090480,0.045828,0.011018,0.009763,0.001157,0.003828,0.006742,0.001634,0.000320,0.001952,0.001823,0.000292,0.001455,0.000996,0.000511,0.001143,0.001002,0.000470,0.000777,0.000907,0.000648,0.000685,0.000677,0.000506,0.000454,0.000526,0.000488,0.000439,0.000473,0.000508,0.000505,0.000540,0.000447,0.000407,0.000416,0.000398,0.000364,0.000198,0.000374,0.000324,0.000391,0.000404,0.000361,0.000265,0.000399,0.000341,0.000282,0.000274,0.000277,0.000232,0.000236,0.000232,0.000234,0.000221,0.000241,0.000245,0.000215,0.000179,0.000214,0.000247,0.000215,0.000207,0.000202,0.000207,0.000195,0.000189,0.000164,0.000193,0.000172,0.000163,0.000189,0.000196,0.000183,0.000169,0.000170,0.000159,0.000177,0.000172,0.000148,0.000148,0.000174,0.000160,0.000170,0.000159,0.000159,0.000153,0.000140,0.000152,0.000124,0.000137,0.000135,0.000132,0.000111,0.000136,0.000145
giUnmuted55 ftgen 0,0,4096,10,0.553293,0.970661,0.775964,0.469479,0.247296,0.167610,0.076609,0.033410,0.020341,0.014035,0.008333,0.004948,0.005841,0.001285,0.003257,0.000737,0.001858,0.001036,0.001645,0.001036,0.001103,0.001137,0.001058,0.001046,0.000830,0.000920,0.000754,0.000986,0.000805,0.000941,0.000725,0.000785,0.000639,0.000792,0.000693,0.000710,0.000502,0.000630,0.000647,0.000611,0.000541,0.000612,0.000564,0.000535,0.000526,0.000550,0.000561,0.000452,0.000489,0.000502,0.000455,0.000494,0.000395,0.000489,0.000391,0.000455,0.000426,0.000401,0.000395,0.000424,0.000366,0.000409,0.000441,0.000420,0.000368,0.000384,0.000385,0.000372,0.000378,0.000347,0.000334,0.000364,0.000324,0.000342,0.000334,0.000331,0.000328,0.000319,0.000317,0.000303,0.000307,0.000324,0.000295,0.000316,0.000284,0.000293,0.000301,0.000292,0.000308,0.000294,0.000306,0.000291,0.000290,0.000281,0.000282,0.000284,0.000276,0.000270,0.000271,0.000271
giUnmuted56 ftgen 0,0,4096,10,0.400932,0.968520,0.498998,0.389257,0.178633,0.106598,0.036188,0.017795,0.012342,0.000119,0.001239,0.002906,0.000849,0.000166,0.001155,0.001166,0.000733,0.000431,0.000261,0.000740,0.000954,0.000788,0.000425,0.000292,0.000215,0.000111,0.000367,0.000116,0.000157,0.000202,0.000291,0.000325,0.000311,0.000291,0.000228,0.000258,0.000334,0.000245,0.000207,0.000181,0.000101,0.000272,0.000256,0.000160,0.000202,0.000191,0.000135,0.000209,0.000287,0.000145,0.000174,0.000142,0.000151,0.000158,0.000211,0.000156,0.000085,0.000205,0.000120,0.000149,0.000178,0.000110,0.000128,0.000155,0.000127,0.000078,0.000172,0.000112,0.000089,0.000134,0.000141,0.000080,0.000122,0.000145,0.000057,0.000148,0.000118,0.000076,0.000098,0.000141,0.000071,0.000118,0.000134,0.000051,0.000121,0.000094,0.000107,0.000067,0.000125,0.000091,0.000070,0.000129,0.000072,0.000073,0.000102,0.000110,0.000065,0.000125,0.000102,0.000073
giUnmuted57 ftgen 0,0,4096,10,0.291306,0.935026,0.610503,0.348987,0.214608,0.091185,0.014377,0.010820,0.009119,0.004516,0.002313,0.001529,0.002367,0.001896,0.001988,0.001276,0.001387,0.001297,0.001665,0.001145,0.001069,0.001265,0.001014,0.000912,0.000914,0.000884,0.001057,0.000657,0.000966,0.000629,0.000867,0.000822,0.000934,0.000930,0.000759,0.000705,0.000586,0.000610,0.000554,0.000522,0.000534,0.000515,0.000527,0.000475,0.000482,0.000475,0.000461,0.000481,0.000430,0.000442,0.000430,0.000434,0.000438,0.000393,0.000406,0.000411,0.000359,0.000372,0.000357,0.000351,0.000349,0.000324,0.000336,0.000324,0.000314,0.000303,0.000301,0.000285,0.000290,0.000281,0.000290,0.000278,0.000275,0.000275,0.000263,0.000266,0.000238,0.000227,0.000236,0.000240,0.000236,0.000232,0.000222,0.000228,0.000230,0.000219,0.000211,0.000210,0.000218,0.000220,0.000216,0.000205,0.000201,0.000207,0.000204,0.000198,0.000198,0.000199,0.000198,0.000204
giUnmuted58 ftgen 0,0,4096,10,0.424856,0.955425,0.300308,0.167209,0.106790,0.022221,0.009677,0.003945,0.002919,0.000972,0.002109,0.001042,0.000179,0.000802,0.000783,0.000469,0.000622,0.000452,0.000340,0.000342,0.000529,0.000442,0.000227,0.000537,0.000344,0.000279,0.000438,0.000291,0.000255,0.000281,0.000355,0.000226,0.000212,0.000190,0.000247,0.000230,0.000217,0.000203,0.000212,0.000229,0.000161,0.000199,0.000193,0.000163,0.000189,0.000156,0.000148,0.000176,0.000153,0.000136,0.000144,0.000163,0.000148,0.000142,0.000156,0.000124,0.000124,0.000128,0.000131,0.000128,0.000116,0.000119,0.000096,0.000126,0.000103,0.000119,0.000108,0.000098,0.000108,0.000100,0.000109,0.000095,0.000104,0.000104,0.000089,0.000098,0.000092,0.000092,0.000094,0.000088,0.000084,0.000083,0.000085,0.000085,0.000078,0.000083,0.000080,0.000081,0.000078,0.000077,0.000082,0.000082,0.000079,0.000080,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted59 ftgen 0,0,4096,10,0.359547,0.978853,0.629720,0.355319,0.173057,0.062497,0.018952,0.015355,0.001883,0.003244,0.000776,0.001277,0.001682,0.000930,0.001021,0.000861,0.000797,0.000784,0.000905,0.000938,0.000691,0.000529,0.000651,0.000668,0.000567,0.000394,0.000564,0.000553,0.000563,0.000374,0.000399,0.000503,0.000337,0.000498,0.000410,0.000391,0.000452,0.000316,0.000406,0.000293,0.000417,0.000301,0.000349,0.000279,0.000358,0.000270,0.000271,0.000287,0.000270,0.000252,0.000235,0.000266,0.000229,0.000223,0.000239,0.000206,0.000237,0.000201,0.000212,0.000207,0.000197,0.000204,0.000185,0.000193,0.000174,0.000185,0.000168,0.000173,0.000159,0.000158,0.000167,0.000156,0.000160,0.000158,0.000153,0.000144,0.000156,0.000152,0.000147,0.000148,0.000140,0.000141,0.000134,0.000134,0.000132,0.000133,0.000132,0.000129,0.000139,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted60 ftgen 0,0,4096,10,0.261374,0.958638,0.539283,0.255231,0.147692,0.058306,0.039086,0.029455,0.008523,0.006366,0.003368,0.001295,0.000525,0.000726,0.000847,0.000590,0.000475,0.000526,0.000505,0.000443,0.000320,0.000407,0.000362,0.000443,0.000436,0.000378,0.000327,0.000302,0.000313,0.000302,0.000287,0.000249,0.000268,0.000271,0.000288,0.000219,0.000205,0.000188,0.000206,0.000244,0.000213,0.000204,0.000153,0.000138,0.000190,0.000183,0.000175,0.000180,0.000157,0.000145,0.000159,0.000168,0.000161,0.000138,0.000144,0.000137,0.000134,0.000133,0.000132,0.000126,0.000126,0.000129,0.000119,0.000120,0.000108,0.000109,0.000110,0.000111,0.000101,0.000092,0.000091,0.000098,0.000100,0.000096,0.000094,0.000096,0.000089,0.000087,0.000090,0.000085,0.000088,0.000082,0.000081,0.000085,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted61 ftgen 0,0,4096,10,0.417597,0.966833,0.416727,0.180794,0.059064,0.009951,0.012483,0.005985,0.000936,0.001803,0.000655,0.001165,0.001044,0.000808,0.000730,0.000802,0.000431,0.000761,0.000635,0.000600,0.000672,0.000449,0.000439,0.000419,0.000423,0.000466,0.000371,0.000301,0.000340,0.000369,0.000332,0.000355,0.000303,0.000303,0.000266,0.000327,0.000298,0.000235,0.000293,0.000250,0.000196,0.000286,0.000226,0.000187,0.000270,0.000214,0.000201,0.000218,0.000195,0.000204,0.000206,0.000186,0.000183,0.000183,0.000170,0.000171,0.000168,0.000166,0.000163,0.000161,0.000161,0.000147,0.000147,0.000142,0.000139,0.000140,0.000142,0.000142,0.000138,0.000137,0.000135,0.000131,0.000129,0.000130,0.000131,0.000129,0.000128,0.000128,0.000129,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted62 ftgen 0,0,4096,10,0.619523,0.961285,0.413119,0.108674,0.024175,0.002125,0.004476,0.003247,0.001352,0.001965,0.000417,0.001471,0.001777,0.000808,0.001622,0.001287,0.001022,0.001266,0.000711,0.000939,0.000934,0.000662,0.000613,0.000589,0.000621,0.000570,0.000567,0.000537,0.000348,0.000623,0.000348,0.000581,0.000430,0.000433,0.000451,0.000404,0.000403,0.000380,0.000391,0.000385,0.000372,0.000351,0.000360,0.000320,0.000325,0.000307,0.000314,0.000267,0.000307,0.000276,0.000259,0.000269,0.000263,0.000231,0.000245,0.000235,0.000254,0.000203,0.000228,0.000209,0.000210,0.000212,0.000201,0.000201,0.000200,0.000193,0.000197,0.000181,0.000186,0.000191,0.000187,0.000179,0.000186,0.000183,0.000156,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted63 ftgen 0,0,4096,10,0.930506,0.830293,0.419689,0.124916,0.038385,0.024593,0.004794,0.004719,0.003135,0.000873,0.001282,0.002043,0.001147,0.000967,0.000988,0.000678,0.000173,0.000513,0.000302,0.000290,0.000411,0.000762,0.000371,0.000406,0.000100,0.000164,0.000359,0.000407,0.000105,0.000293,0.000391,0.000246,0.000158,0.000094,0.000092,0.000176,0.000072,0.000179,0.000079,0.000130,0.000159,0.000270,0.000068,0.000103,0.000061,0.000031,0.000141,0.000112,0.000103,0.000058,0.000084,0.000104,0.000104,0.000093,0.000106,0.000081,0.000073,0.000109,0.000082,0.000051,0.000083,0.000073,0.000065,0.000063,0.000070,0.000082,0.000059,0.000073,0.000066,0.000057,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted64 ftgen 0,0,4096,10,0.972240,0.962179,0.327494,0.131881,0.030060,0.012594,0.010159,0.006424,0.001103,0.003403,0.002598,0.002451,0.001553,0.001352,0.001133,0.001503,0.001006,0.001536,0.001309,0.001202,0.000888,0.001138,0.001263,0.001203,0.001186,0.000543,0.000766,0.000841,0.000896,0.000659,0.000695,0.000682,0.000638,0.000554,0.000597,0.000599,0.000595,0.000633,0.000510,0.000491,0.000552,0.000483,0.000426,0.000553,0.000423,0.000448,0.000409,0.000425,0.000270,0.000375,0.000420,0.000306,0.000414,0.000322,0.000332,0.000333,0.000282,0.000349,0.000328,0.000344,0.000339,0.000347,0.000316,0.000285,0.000300,0.000263,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted65 ftgen 0,0,4096,10,0.946888,0.960545,0.184774,0.018883,0.010446,0.014745,0.001268,0.004635,0.003175,0.000946,0.004142,0.002215,0.000862,0.002500,0.001973,0.001800,0.000634,0.001025,0.001812,0.001970,0.001697,0.001360,0.000855,0.000747,0.001079,0.000849,0.000880,0.000574,0.000730,0.000688,0.000981,0.000337,0.000974,0.000733,0.000756,0.000555,0.000861,0.000545,0.000749,0.000475,0.000535,0.000441,0.000525,0.000483,0.000584,0.000565,0.000414,0.000551,0.000370,0.000472,0.000237,0.000446,0.000374,0.000619,0.000426,0.000403,0.000396,0.000408,0.000394,0.000340,0.000296,0.000324,0.000299,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted67 ftgen 0,0,4096,10,0.904104,0.687203,0.074239,0.008596,0.008832,0.003406,0.003870,0.000671,0.001852,0.001635,0.001087,0.000499,0.001292,0.000964,0.000856,0.000780,0.000614,0.000711,0.000618,0.000934,0.000903,0.000581,0.000840,0.000399,0.000489,0.000328,0.000505,0.000515,0.000432,0.000365,0.000364,0.000454,0.000390,0.000377,0.000361,0.000314,0.000378,0.000357,0.000330,0.000270,0.000342,0.000244,0.000217,0.000204,0.000235,0.000181,0.000230,0.000167,0.000212,0.000212,0.000203,0.000218,0.000163,0.000210,0.000170,0.000173,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted69 ftgen 0,0,4096,10,0.944934,0.293223,0.066919,0.017808,0.007234,0.002307,0.002545,0.000739,0.000556,0.000511,0.000549,0.000723,0.000242,0.000601,0.000453,0.000316,0.000324,0.000312,0.000164,0.000264,0.000131,0.000093,0.000233,0.000211,0.000152,0.000187,0.000090,0.000233,0.000171,0.000154,0.000116,0.000082,0.000095,0.000121,0.000103,0.000074,0.000094,0.000088,0.000098,0.000122,0.000105,0.000080,0.000084,0.000079,0.000074,0.000104,0.000062,0.000071,0.000065,0.000064,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
giUnmuted70 ftgen 0,0,4096,10,0.943312,0.441802,0.052080,0.022643,0.001043,0.002473,0.002554,0.002310,0.001720,0.000881,0.001191,0.001619,0.000857,0.000858,0.001539,0.000697,0.000554,0.000498,0.000409,0.000174,0.000105,0.000147,0.000252,0.000549,0.000348,0.000262,0.000426,0.000336,0.000226,0.000305,0.000193,0.000333,0.000263,0.000183,0.000164,0.000171,0.000202,0.000221,0.000188,0.000137,0.000129,0.000098,0.000100,0.000144,0.000071,0.000094,0.000086,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,0.000000
;GEN02 TABLE CONTAINING THE FUNCTION TABLE NUMBERS OF THE TABLES BETWEEN WHICH MORPHING WILL TAKE PLACE
giwavemapUnmuted    ftgen    0,0,128,-27, 0,0, 22,0, 23,1, 24,2, 25,3, 26,4, 27,5, 28,6, 30,7, 31,8, 32,9, 33,10, 34,11, 35,12, 37,13, 38,14, 39,15, 40,16, 41,17, 42,18, 43,19, 44,20, 45,21, 46,22, 48,23, 49,24, 50,25, 51,26, 52,27, 53,28, 55,29, 56,30, 57,31, 58,32, 59,33, 60,34, 61,35, 62,36, 63,37, 64,38, 65,39, 67,40, 69,41, 70,42, 127,42
gitabs4morfUnmuted    ftgen    0,0,64,-2, giUnmuted22,giUnmuted23,giUnmuted24,giUnmuted25,giUnmuted26,giUnmuted27,giUnmuted28,giUnmuted30,giUnmuted31,giUnmuted32,giUnmuted33,giUnmuted34,giUnmuted35,giUnmuted37,giUnmuted38,giUnmuted39,giUnmuted40,giUnmuted41,giUnmuted42,giUnmuted43,giUnmuted44,giUnmuted45,giUnmuted46,giUnmuted48,giUnmuted49,giUnmuted50,giUnmuted51,giUnmuted52,giUnmuted53,giUnmuted55,giUnmuted56,giUnmuted57,giUnmuted58,giUnmuted59,giUnmuted60,giUnmuted61,giUnmuted62,giUnmuted63,giUnmuted64,giUnmuted65,giUnmuted67,giUnmuted69,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70,giUnmuted70

isaw    ftgen    0,0,4096,7,1,4096,-1

icount    =    0
loop1:
imaxh    =  sr / (2 * 440.0 * exp(log(2.0) * (icount - 69) / 12))
ifn    ftgen    1000+icount,0,4096,-30,isaw,1,imaxh
    loop_le    icount,1,127,loop1

;gisquare    ftgen    0,0,4096,10, 1,0, 1/3,0, 1/5,0, 1/7,0, 1/9,0, 1/11,0, 1/13,0, 1/15,0, 1/17,0, 1/19,0, 1/21,0, 1/23,0, 1/25,0, 1/27,0, 1/29,0, 1/31,0, 1/33,0, 1/35,0, 1/37,0, 1/39

isquare        ftgen    0,0,4096,7,1,2048,1,0,-1,2048,-1

icount    =    0
loop2:
imaxh    =  sr / (2 * 440.0 * exp(log(2.0) * (icount - 69) / 12))
ifn    ftgen    2000+icount,0,4096,-30,isquare,1,imaxh
    loop_le    icount,1,127,loop2

giorg        ftgen    0,0,4096,9, 1,1,rnd(1), 2,1/2,rnd(1), 4,1/4,rnd(1), 8,1/8,rnd(1), 16,1/16,rnd(1), 32,1/32,rnd(1), 64,1/64,rnd(1), 128,1/128,rnd(1)

gicos        ftgen    0,0,4096,11,1        ;COSINE WAVE (USED BY THE LFOS)
gieqffn        ftgen    0,0,4097,7,-1,4096,1
gieqlfn        ftgen    0,0,4097,7,-1,4096,1
gieqqfn        ftgen    0,0,4097,7,-1,4096,1

gasendL,gasendR    init    0

gkactive    init    0    ; Will contain number of active instances of instr 3 when legato mode is chosen. NB. notes in release stage will not be regarded as active. 

opcode    sspline,k,Kiii
    kdur,istart,iend,icurve    xin                                        ;READ IN INPUT ARGUMENTS
    imid    =    istart+((iend-istart)/2)                                ;SPLINE MID POINT VALUE
    isspline    ftgentmp    0,0,4096,-16,istart,4096*0.5,icurve,imid,(4096/2)-1,-icurve,iend    ;GENERATE 'S' SPLINE
    kspd    =    i(kdur)/kdur                                        ;POINTER SPEED AS A RATIO (WITH REFERENCE TO THE ORIGINAL DURATION)
    kptr    init    0                                            ;POINTER INITIAL VALUE    
    kout    tablei    kptr,isspline                                        ;READ VALUE FROM TABLE
    kptr    limit    kptr+((ftlen(isspline)/(i(kdur)*kr))*kspd), 0, ftlen(isspline)-1            ;INCREMENT THE POINTER BY THE REQUIRED NUMBER OF TABLE POINTS IN ONE CONTROL CYCLE AND LIMIT IT BETWEEN FIRST AND LAST TABLE POINT - FINAL VALUE WILL BE HELD IF POINTER ATTEMPTS TO EXCEED TABLE DURATION
        xout    kout                                            ;SEND VALUE BACK TO CALLER INSTRUMENT
endop

instr    ReadWidgets
    kporttime    linseg    0,0.001,0.1

    ;OSCILLATOR
    gkamp        chnget    "amp"
    gkNOscs        chnget    "NOscs"
    gkfmd        chnget    "fmd"
    gkmvt        chnget    "mvt"
    gkwaveform    chnget    "waveform"
    gkwaveform    init    1
    gkwidth        chnget    "width"
    gkWTableShift    chnget    "WTableShift"
    gkPBend        chnget    "PBend"
     kMOUSE_DOWN_LEFT       chnget  "MOUSE_DOWN_LEFT"
     kOff    init    0
     if trigger(kMOUSE_DOWN_LEFT,0.5,1)==1 then
       chnset    kOff,"PBend"
     endif
    gkBendRange    chnget    "BendRange"
    gkPchBend    portk    (gkPBend)*gkBendRange, kporttime

    gkWTableShift    portk    gkWTableShift, kporttime
    gkFiltLayers    chnget    "FiltLayers"

    gkFlfoType    chnget    "FlfoType"
    gkFRte        chnget    "FRte"
    gkFDep        chnget    "FDep"
    gkFDel        chnget    "FDel"
    gkFRis        chnget    "FRis"

    gkcfoct        chnget    "cf"
    gkFEnvAmt    chnget    "FEnvAmt"
    gkFAtt        chnget    "FAtt"
    gkFDec        chnget    "FDec"
    gkFSus        chnget    "FSus"
    gkFRel        chnget    "FRel"

    gkRvbMix    chnget    "RvbMix"

    gkAAtt        chnget    "AAtt"
    gkADec        chnget    "ADec"
    gkASus        chnget    "ASus"
    gkARel        chnget    "ARel"

    gklegato    chnget    "legato"
    gkLegTim    chnget    "LegTim"
    gkmono        chnget    "mono"

    gkExciterAmount    chnget    "ExciterAmount"    
    if trigger:k(gkExciterAmount,0.1,2)==1 then
     reinit UPDATE_EXCITER
    endif
    UPDATE_EXCITER:
    if i(gkExciterAmount)>0 then
     chnset    "alpha(1)", "ExciterID"
    else
     chnset    "alpha(.5)", "ExciterID"
    endif
    gkExciterFreq    chnget    "ExciterFreq"
endin

instr    1    ; triggered via midi
    gkNoteTrig    init    1    ;at the beginning of a new note set note trigger flag to '1'
    inum        notnum
    gknum        =    inum
    ivel        veloc    0,1    ;read in midi note velocity

    if i(gklegato)==0 then        ;if we are *not* in legato mode...

     ;aL,aR    subinstr    p1+1,inum    ;call voice instrument as a subinstrument. Audio will be fed back to this instrument before being sent to the outputs.
     ;    outs    aL,aR        ;send audio to outputs
     
         event_i    "i",p1+1+(inum*0.001),0,-1,inum,ivel    ; call sound producing instr
     krel    release                        ; release flag (1 when note is released, 0 otherwise)
     if krel==1 then                    ; when note is released...
      turnoff2    p1+1+(inum*0.001),4,1            ; turn off the called instrument
     endif                            ; end of conditional          
     
    else                ;otherwise... (i.e. legato mode)
     iactive    =    i(gkactive)            ;number of active notes of instr 3 (note in release are disregarded)
     if iactive==0 then                    ;...if no notes are active
      event_i    "i",p1+1,0,3600,inum,ivel        ;...start a new held note
     endif
    endif
endin

instr    2    ; triggered by instr 1
    ;kporttime    linseg    0,0.001,1        ;portamento time function rises quickly from zero to a held value
    kLegTime    =    gkLegTim;*kporttime    ;scale portamento time function with value from GUI knob widget
    
    if i(gklegato)==1 then                ;if we are in legato mode...
     krel    release                    ;sense when  note has been released
     gkactive    =    1-krel            ;if note is in release, gkactive=0, otherwise =1
     if krel==0 then
      ktrig    changed    gknum                        ;...GENERATE A TRIGGER IS A NEW NOTE NUMBER IS GENERATED (FROM INSTR. 1)
      gkNoteTrig    =    0
     endif
     gkOldNum    init    p4                                                ; OLD NOTE NUMBER = FIRST NOTE NUMBER UPON INITIAL NOTE BEING PLAYED 
     if ktrig=1 then                                                    ; IF A NEW (LEGATO) NOTE HAS BEEN PRESSED
      reinit    S_CURVE_2                                                ; BEGIN A REINITIALISATION PASS FROM LABEL
     endif                                                                ; END OF CONDITIONAL BRANCH
     S_CURVE_2:                                                            ; A LABEL. REINITIALISATION BEGINS FROM HERE.
     idiff    =    1+abs(i(gknum)-i(gkOldNum))                                ; ABSOLUTE DIFFERENCE BETWEEN OLD NOTE AND NEW NOTE IN STEPS (+ 1)
     knum    sspline    (kLegTime*idiff)+ksmps/sr,i(gkOldNum),i(gknum),1     ; CALL sspline UDO (PORTAMENTO TIME MULTIPLIED BY NOTE GAP (idiff))
     rireturn                                                            ; RETURN FROM INITIALISATION PASS
     gkOldNum    =    knum                                                ; SET OLD NUMBER CURRENT NUMBER
     kactive    active    p1-1                                            ; ...check number of active midi notes (previous instrument)
     if kactive==0 then                                                    ; if no midi notes are active...
      turnoff                                                            ; ... turn this instrument off
     endif
    else                                                                ; otherwise... (polyphonic / non-legato mode)
     knum    =    p4
    endif

    knum    init    p4

    knum    =    knum + gkPchBend        ;add pitch bend
    
    ;AMPLITUDE ENVELOPE
    aenv        linsegr    0.001,i(gkAAtt)+0.001,1,i(gkADec)+0.001,i(gkASus),i(gkARel)+0.001,0.001    ;AMPLITUDE ENVELOPE
    aenv        =    aenv-0.001
    
    ;FILTER AND FILTER ENVELOPE
    kFEnv        linsegr    0,i(gkFAtt)+0.001,1,i(gkFDec)+0.001,i(gkFSus),i(gkFRel)+0.001,0
    kFEnv        =    kFEnv * gkFEnvAmt

    ;FILTER LFO
    kdepth        linseg        0,i(gkFDel)+0.001,0,i(gkFRis)+0.001,1    ;DEPTH OF MODULATION ENVELOPE
    
    if gkFlfoType==1 then
     kFLFO        lfo        kdepth*gkFDep,gkFRte,0        ;LFO (sine)
    elseif gkFlfoType==2 then
     kFLFO        jspline        kdepth*gkFDep,gkFRte,gkFRte
    elseif gkFlfoType==3 then
     kFLFO        randomh        -kdepth*gkFDep,kdepth*gkFDep,gkFRte
     kFLFO        port        kFLFO,0.004            ;smooth out the clicks
    else
     kFLFO        lfo        kdepth*gkFDep,gkFRte,2        ;LFO (bi-square)
     kFLFO        port        kFLFO,0.004            ;smooth out the clicks
    endif

    kCFoct        limit    4+kFEnv+gkcfoct+kFLFO,4,14
    kCF        =    cpsoct(kCFoct)
      
    if i(gkwaveform)==1 then
     kwave    =    1000+knum
    elseif i(gkwaveform)==2 then
     kwave    =    2000+knum
    elseif i(gkwaveform)==3 then
     kwave    =    giorg
    elseif i(gkwaveform)==4 then
     giwavemap    =    giwavemapClar
     gitabs4morf    =    gitabs4morfClar
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==5 then
     giwavemap    =    giwavemapBClar
     gitabs4morf    =    gitabs4morfBClar
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==6 then
     giwavemap    =    giwavemapCBClar
     gitabs4morf    =    gitabs4morfCBClar
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==7 then
     giwavemap    =    giwavemapOboe
     gitabs4morf    =    gitabs4morfOboe
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==8 then
     giwavemap    =    giwavemapBassoon
     gitabs4morf    =    gitabs4morfBassoon
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==9 then
     giwavemap    =    giwavemapCBassoon
     gitabs4morf    =    gitabs4morfCBassoon
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==10 then
     giwavemap    =    giwavemapViolin
     gitabs4morf    =    gitabs4morfViolin
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==11 then
     giwavemap    =    giwavemapCello
     gitabs4morf    =    gitabs4morfCello
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==12 then
     giwavemap    =    giwavemapPiccolo
     gitabs4morf    =    gitabs4morfPiccolo
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==13 then
     giwavemap    =    giwavemapFlute
     gitabs4morf    =    gitabs4morfFlute
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==14 then
     giwavemap    =    giwavemapAFlute
     gitabs4morf    =    gitabs4morfAFlute
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==15 then
     giwavemap    =    giwavemapBFlute
     gitabs4morf    =    gitabs4morfBFlute
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==16 then
     giwavemap    =    giwavemapAhh
     gitabs4morf    =    gitabs4morfAhh
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==17 then
     giwavemap    =    giwavemapOoh
     gitabs4morf    =    gitabs4morfOoh
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==18 then
     giwavemap    =    giwavemapEee
     gitabs4morf    =    gitabs4morfEee
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==19 then
     giwavemap    =    giwavemapHornP
     gitabs4morf    =    gitabs4morfHornP
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==20 then
     giwavemap    =    giwavemapHornF
     gitabs4morf    =    gitabs4morfHornF
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==21 then
     giwavemap    =    giwavemapharmon
     gitabs4morf    =    gitabs4morfharmon
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==22 then
     giwavemap    =    giwavemapMute1
     gitabs4morf    =    gitabs4morfMute1
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    elseif i(gkwaveform)==23 then
     giwavemap    =    giwavemapUnmuted
     gitabs4morf    =    gitabs4morfUnmuted
     imorphtab    ftgentmp    0,0,4096,10,1
     kndx        limit        knum+gkWTableShift,0,127
     kftndx        tablei        kndx,giwavemap            ;ftndx REMAPPED ACCORDING TO THE WAVETABLE MAP FOR THIS INSTRUMENT
            ftmorf        kftndx, gitabs4morf, imorphtab    ;CREATE THE MORPHED TABLE ACCORDING TO THE VALUE OF THE FUNCTION TABLE INDEX CREATED ABOVE
     kwave        =        imorphtab
    endif


    ktrig    changed    gkNOscs,gkmono                ;IF ANY OF THE CONTROLS WHICH MUST BE ENTERED AS I-RATE CONTROLLERS HAVE CHANGED GENERATE A TRIGGER IMPULSE AT THE OUTPUT
    if ktrig==1 then                    ;IF ANY I-RATE CONTROL HAS CHANGED...
      reinit    UPDATE                    ;BEGIN A RE-INITILISATION PASS FROM LABEL 'UPDATE'
    endif                            
    UPDATE:                            ;BEGIN REINIT PASS FROM HERE
    kmvt    jspline    gkmvt,gkmvt,gkmvt
    kcps    =    cpsmidinn(knum)

    ; RANDOM SEED
    iSeedMode    chnget    "seed"
    if iSeedMode==1 then            ; clock mono
     iseedL    =    times:i()
     iseedR    =    iseedL
    elseif iSeedMode==2 then        ; clock stereo
     iseedL    =    times:i()
     iseedR    =    iseedL + 1000
    elseif iSeedMode==3 then        ; note mono
     iseedL    =    p4
     iseedR    =    iseedL
    elseif iSeedMode==4 then        ; note stereo
     iseedL    =    p4
     iseedR    =    iseedL + 1000
    elseif iSeedMode==5 then        ; fixed mono
     iseedL    =    1
     iseedR    =    iseedL
    elseif iSeedMode==6 then        ; fixed stereo
     iseedL    =    1
     iseedR    =    iseedL + 1000
    endif
    
    if i(gkmono)==1 then
    ;OUTPUT    OPCODE  CPS  | AMD  |    FMD     | PMD | OVERLAPS   | SEED | L1MINF  | L1MAXF  | L2MINF  | L2MAXF  | LFOMODE | EQMINF  | EQMAXF | EQMINL | EQMAXL | EQMINQ | EQMAXQ  | EQMODE | KFN  | L1FN | L2FN | EQFFN  | EQLF   |  EQQFN |  TABL  | OUTFN
     aL    oscbnk    kcps,   0,    gkfmd*kcps,    0,  i(gkNOscs),  iseedL,    0,      kmvt,      0,        0,       238,      0,       8000,      1,       1,       1,       1,       -1,   kwave, gicos, gicos, gieqffn, gieqlfn, gieqqfn
     aR    =    aL
    else
     ;TWO ITERATIONS OF oscbnk ARE CREATED, ONE FOR EACH OF THE STEREO AUDIO CHANNELS. THE OUTPUTS WILL BE DIFFERENT AS THE RANDOM NUMBER GENERATORS WILL BE SEEDED BY THE SYSTEM CLOCK
     ;OUTPUT    OPCODE  CPS  | AMD  |    FMD     | PMD | OVERLAPS   | SEED  | L1MINF  | L1MAXF  | L2MINF  | L2MAXF  | LFOMODE | EQMINF  | EQMAXF | EQMINL | EQMAXL | EQMINQ | EQMAXQ  | EQMODE | KFN  | L1FN | L2FN | EQFFN  | EQLF   |  EQQFN |  TABL  | OUTFN
     aL        oscbnk    kcps,   0,    gkfmd*kcps,   0,  i(gkNOscs),   iseedL,   0,       kmvt,      0,        0,       238,      0,       8000,      1,       1,       1,       1,       -1,   kwave, gicos, gicos, gieqffn, gieqlfn, gieqqfn
     aR        oscbnk    kcps,   0,    gkfmd*kcps,   0,  i(gkNOscs),   iseedR,   0,      -kmvt,      0,        0,       238,      0,       8000,      1,       1,       1,       1,       -1,   kwave, gicos, gicos, gieqffn, gieqlfn, gieqqfn
     aL    ntrpol    (aL+aR)*0.66,aL,gkwidth
     aR    ntrpol    (aL+aR)*0.66,aR,gkwidth
    endif
    rireturn                        ;RETURN FROM REINIT PASS

    ;aL     clfilt         aL, kCF,0,2
    ;aR     clfilt         aR, kCF,0,2
    
    if gkExciterAmount>0 then
     aEL     exciter     aL*gkExciterAmount, gkExciterFreq, 20000, 10, 10
     aER     exciter     aR*gkExciterAmount, gkExciterFreq, 20000, 10, 10
     aL        ntrpol        aL,aEL,0.5
     aR        ntrpol        aR,aER,0.5
    endif

    ktrig    changed    gkFiltLayers
    if ktrig=1 then
     reinit RESTART_TONEX
    endif
    RESTART_TONEX:
    aL     tonex         aL, kCF,i(gkFiltLayers)
    aR     tonex         aR, kCF,i(gkFiltLayers)
    rireturn
    
    aL    =    aL*aenv*gkamp*p5*0.2
    aR    =    aR*aenv*gkamp*p5*0.2
    
        outs        aL, aR    ;SEND AUDIO TO THE OUTPUTS. RESCALE EACH CHANNEL WITH NOTE VELOCITY, AMPLITUDE ENVELOPE AND AMP CONTROL WIDGET.
    gasendL    =    gasendL+(aL*gkRvbMix)
    gasendR    =    gasendR+(aR*gkRvbMix)
endin

instr    4    ;reverb
    gkRvbSize    chnget    "RvbSize"
    gkRvbCF        chnget    "RvbCF"
    if gkRvbMix==0 goto SKIP_REVERB
    aL,aR    reverbsc    gasendL,gasendR,gkRvbSize,gkRvbCF
        outs    aL,aR
    SKIP_REVERB:
        clear    gasendL,gasendR
endin

</CsInstruments>

<CsScore>
i "ReadWidgets" 0 [3600*24*7]
i 4 0 [3600*24*7]
</CsScore>

</CsoundSynthesizer>